package org.apache.flink.connector.kafka.source.enumerator.initializer;

import org.apache.flink.annotation.PublicEvolving;
import org.apache.flink.kafka.shaded.org.apache.kafka.clients.consumer.OffsetAndTimestamp;
import org.apache.flink.kafka.shaded.org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.flink.kafka.shaded.org.apache.kafka.common.TopicPartition;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;

@PublicEvolving
public interface OffsetsInitializer extends Serializable {
    Map<TopicPartition, Long> getPartitionOffsets(Collection<TopicPartition> paramCollection, PartitionOffsetsRetriever paramPartitionOffsetsRetriever);

    OffsetResetStrategy getAutoOffsetResetStrategy();

    static OffsetsInitializer committedOffsets() {
        return committedOffsets(OffsetResetStrategy.NONE);
    }

    static OffsetsInitializer committedOffsets(OffsetResetStrategy offsetResetStrategy) {
        return new ReaderHandledOffsetsInitializer(-3L, offsetResetStrategy);
    }

    static OffsetsInitializer timestamp(long timestamp) {
        return new TimestampOffsetsInitializer(timestamp);
    }

    static OffsetsInitializer earliest() {
        return new ReaderHandledOffsetsInitializer(-2L, OffsetResetStrategy.EARLIEST);
    }

    static OffsetsInitializer latest() {
        return new ReaderHandledOffsetsInitializer(-1L, OffsetResetStrategy.LATEST);
    }

    static OffsetsInitializer offsets(Map<TopicPartition, Long> offsets) {
        return new SpecifiedOffsetsInitializer(offsets, OffsetResetStrategy.EARLIEST);
    }

    static OffsetsInitializer offsets(Map<TopicPartition, Long> offsets, OffsetResetStrategy offsetResetStrategy) {
        return new SpecifiedOffsetsInitializer(offsets, offsetResetStrategy);
    }

    public static interface PartitionOffsetsRetriever {
        Map<TopicPartition, Long> committedOffsets(Collection<TopicPartition> param1Collection);

        Map<TopicPartition, Long> endOffsets(Collection<TopicPartition> param1Collection);

        Map<TopicPartition, Long> beginningOffsets(Collection<TopicPartition> param1Collection);

        Map<TopicPartition, OffsetAndTimestamp> offsetsForTimes(Map<TopicPartition, Long> param1Map);
    }
}
