package org.apache.flink.connector.kafka.source.enumerator.initializer;

import org.apache.flink.kafka.shaded.org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.flink.kafka.shaded.org.apache.kafka.common.TopicPartition;
import org.apache.flink.util.Preconditions;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

class ReaderHandledOffsetsInitializer implements OffsetsInitializer, OffsetsInitializerValidator {
    private static final long serialVersionUID = 172938052008787981L;

    private final long startingOffset;

    private final OffsetResetStrategy offsetResetStrategy;

    ReaderHandledOffsetsInitializer(long startingOffset, OffsetResetStrategy offsetResetStrategy) {
        this.startingOffset = startingOffset;
        this.offsetResetStrategy = offsetResetStrategy;
    }

    public Map<TopicPartition, Long> getPartitionOffsets(Collection<TopicPartition> partitions, PartitionOffsetsRetriever partitionOffsetsRetriever) {
        Map<TopicPartition, Long> initialOffsets = new HashMap<>();
        for (TopicPartition tp : partitions)
            initialOffsets.put(tp, Long.valueOf(this.startingOffset));
        return initialOffsets;
    }

    public OffsetResetStrategy getAutoOffsetResetStrategy() {
        return this.offsetResetStrategy;
    }

    public void validate(Properties kafkaSourceProperties) {
        if (this.startingOffset == -3L)
            Preconditions.checkState(kafkaSourceProperties
                            .containsKey("group.id"),
                    String.format("Property %s is required when using committed offset for offsets initializer", new Object[] { "group.id" }));
    }
}
