package net.wanji.datacenter.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.entity.GreenwaveCrossPO;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveCrossMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/11/07 16:02
 */
@Component
@Slf4j
public class CrossGreenWaveInfoCache implements InitializingBean {

    public static final Map<Integer, List<GreenwaveCrossPO>> greenWaveInfoMap = new HashMap<>();
    public static final Map<String, Integer> crossIdGreenIdMap = new HashMap<>();

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private GreenwaveCrossMapper greenwaveCrossMapper;

    @Override
    public void afterPropertiesSet() throws Exception {
        try {
            init();
        } catch (Exception e) {
            log.error("绿波基础信息缓存初始化失败：", e);
            throw new Exception(e);
        }
    }

    private void init() throws Exception {
        List<GreenwaveInfoPO> greenWaveInfoPOS = greenwaveInfoMapper.selectAll();
        if (!CollectionUtils.isEmpty(greenWaveInfoPOS)) {
            LocalTime now = LocalTime.now();
            for (GreenwaveInfoPO greenwaveInfoPO : greenWaveInfoPOS) {
                Integer greenWaveId = greenwaveInfoPO.getId();
                List<GreenwaveCrossPO> greenWaveCrossPOS = greenwaveCrossMapper.selectByGreenwaveId(greenWaveId);
                if (!CollectionUtils.isEmpty(greenWaveCrossPOS)) {
                    List<GreenwaveCrossPO> greenWaveCrossIds = greenWaveCrossPOS.stream()
                            .filter(po -> LocalTime.parse(po.getStartTime()).isBefore(now)
                                    && LocalTime.parse(po.getEndTime()).isAfter(now) && Objects.equals(0, po.getWeek()))
                            .sorted(Comparator.comparingInt(GreenwaveCrossPO::getSort))
                            .collect(Collectors.toList());
                    greenWaveInfoMap.put(greenWaveId, greenWaveCrossIds);

                    for (GreenwaveCrossPO greenWaveCrossPO : greenWaveCrossPOS) {
                        crossIdGreenIdMap.put(greenWaveCrossPO.getCrossId(), greenWaveId);
                    }
                }
            }
        }
    }
}
