package net.wanji.datacenter.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/11/06 16:02
 */
@Order(1)
@Component
@Slf4j
public class CrossInfoCache implements InitializingBean {

    private static final List<BaseCrossInfoPO> crossInfoList = new ArrayList<>();
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;


    public static List<String> getSignalIds(){
        List<String> signalIds = crossInfoList.stream()
                .filter(po -> Objects.equals(1, po.getIsSignal()))
                .map(BaseCrossInfoPO::getId)
                .collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(signalIds)) {
            return signalIds;
        }
        return Collections.emptyList();
    }

    public static String getName(String crossId) {
        if (!CollectionUtils.isEmpty(crossInfoList)) {
            for (BaseCrossInfoPO baseCrossInfoPO : crossInfoList) {
                if (crossId.equals(baseCrossInfoPO.getId())) {
                    return baseCrossInfoPO.getName();
                }
            }
        }
        return "龙鼎大道与西山东路交叉口";
    }
    @Override
    public void afterPropertiesSet() throws Exception {
        List<BaseCrossInfoPO> baseCrossInfoPOS = baseCrossInfoMapper.selectAll();
        if (!CollectionUtils.isEmpty(baseCrossInfoPOS)) {
            crossInfoList.addAll(baseCrossInfoPOS);
        }
    }
}
