package net.wanji.datacenter.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.LaneInfoMapper;
import net.wanji.databus.po.LaneInfoPO;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/25 18:54
 */
@Component
@Slf4j
public class CrossLaneInfoCache implements InitializingBean {

    public static final Map<String, LaneInfoPO> laneInfoMap = new HashMap<>();
    /** key crossId+dir */
    public static final Map<String,LaneInfoPO> crossIdLaneId2Map = new HashMap<>();
    /** key crossId, value dir */
    public static final Map<String, List<Integer>> crossLaneDirMap = new HashMap<>();

    @Resource
    private LaneInfoMapper laneInfoMapper;

    private void init() throws Exception {
        List<String> crossIds = CrossInfoCache.getSignalIds();
        for (String crossId : crossIds) {
            List<LaneInfoPO> laneInfoPOS = laneInfoMapper.selectBycrossId(crossId);
            if (!CollectionUtils.isEmpty(laneInfoPOS)) {
                Map<String, LaneInfoPO> crossMap = laneInfoPOS.stream().collect(Collectors.toMap(LaneInfoPO::getId, Function.identity(), (key1, key2) -> key2));
                laneInfoMap.putAll(crossMap);
                Map<Integer, List<LaneInfoPO>> dirLaneList = laneInfoPOS.stream().collect(Collectors.groupingBy(LaneInfoPO::getDir));
                for (Map.Entry<Integer, List<LaneInfoPO>> entry : dirLaneList.entrySet()) {
                    Integer dir = entry.getKey();
                    List<LaneInfoPO> value = entry.getValue();
                    for (LaneInfoPO laneInfoPO : value) {
                        String laneInfoPOId = laneInfoPO.getId();
                        String laneSortId = laneInfoPOId.substring(laneInfoPOId.length() - 2);
                        crossIdLaneId2Map.put(crossId.concat(String.valueOf(dir)).concat(laneSortId), laneInfoPO);
                    }
                }
                List<Integer> dirList = new ArrayList<>();
                for (LaneInfoPO laneInfoPO : laneInfoPOS) {
                    dirList.add(laneInfoPO.getDir());
                }
                crossLaneDirMap.put(crossId, dirList);
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        try {
            init();
        } catch (Exception e) {
            log.error("信控路口车道信息加载失败", e);
            throw new Exception(e);
        }
        log.info("信控路口车道信息加载成功");
    }
}
