package net.wanji.datacenter.common;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Objects;

/**
 * @author duanruiming
 * @date 2024/11/26 17:59
 */
@Getter
@AllArgsConstructor
public enum PhaseEmptyEnum {
    //
    EXT_NS(-100, "1,5", "2", "延长南北方向直行的绿灯"),
    EXT_EW(-101, "3,7", "2", "延长东西方向直行的绿灯"),
    EXT_NS_L(-102, "1,5", "1", " 延长南北方向左转的绿灯"),
    EXT_EW_L(-103, "3,7", "1", "延长东西方向左转的绿灯"),
    EXT_N2S(-110, "5", "2", "延长北向南直行的绿灯"),
    EXT_E2W(-112, "7", "2", "延长东向西直行的绿灯"),
    EXT_S2N(-114, "1", "2", "延长南向北直行的绿灯"),
    EXT_W2E(-116, "3", "2", "延长西向东直行的绿灯"),
    EXT_N2S_L(-120, "5", "1", "延长北向南左转的绿灯"),
    EXT_E2W_L(-122, "7", "1", "延长东向西左转的绿灯"),
    EXT_S2N_L(-124, "1", "1", "延长南向北左转的绿灯"),
    EXT_W2E_L(-126, "3", "1", "延长西向东左转的绿灯"),
    // 延长当前方向的绿灯
    EXT_CUR(-150, "0", "0", "延长当前方向的绿灯"),
    SKIP_NS(-200, "1,5", "2", "南北方向直行绿灯步进到下一个相位"),
    SKIP_EW(-201, "3,7", "2", "东西方向直行绿灯步进到下一个相位"),
    SKIP_NS_L(-202, "1,5", "1", "南北方向左转绿灯步进到下一个相位"),
    SKIP_EW_L(-203, "3,7", "1", "东西方向左转绿灯步进到下一个相位"),
    SKIP_N2S(-210, "5", "1", "北向南直行绿灯步进到下一个相位"),
    SKIP_E2W(-212, "7", "1", "东向西直行绿灯步进到下一个相位"),
    SKIP_S2N(-214, "1", "1", "南向北直行绿灯步进到下一个相位"),
    SKIP_W2E(-216, "3", "1", "西向东直行绿灯步进到下一个相位"),
    SKIP_N2S_L(-220, "5", "2", "北向南左转绿灯步进到下一个相位"),
    SKIP_E2W_L(-222, "7", "2", "东向西左转绿灯步进到下一个相位"),
    SKIP_S2N_L(-224, "1", "2", "南向北左转绿灯步进到下一个相位"),
    SKIP_W2E_L(-226, "3", "2", "西向东左转绿灯步进到下一个相位"),
    // 当前绿灯步进到下一个相位
    SKIP_CUR(-250, "3", "2", "当前绿灯步进到下一个相位"),
    NORMAL(0, "0", "0", "程序正常运行，不对信号机做调整"),
    // 获取灯态信息失败
    NO_LIGHT_STATUS(10, "0", "0", "获取灯态信息失败"),
    // 当前时刻，路口没有绿灯可通行的方向，所以不对信号机做调整
    NORMAL_NO_GREEN(11, "0", "0", "当前时刻，路口没有绿灯可通行的方向，所以不对信号机做调整"),
    PHASE_ERR(12, "0", "0", "获取或解析配时方案失败"),
    CODE_ERR(30, "0", "0", "代码出现异常"),
    // 测到灯态异常后，自动解锁成功
    ABNORMAL_UNLOCK_SUCC(40, "0", "0", "检测到灯态异常后，自动解锁成功"),
    ABNORMAL_UNLOCK_FAIL(41, "0", "0", "检测到灯态异常后，自动解锁失败"),
    // 在即将给信号机下发延长指令前检测到代码运行时间太长
    TAKE_LONG_BEFORE_EXT(60, "0", "0", "在即将给信号机下发延长指令前检测到代码运行时间太长"),
    // 在即将给信号机下发步进指令前检测到代码运行时间太长
    TAKE_LONG_BEFORE_SKIP(61, "0", "0", "在即将给信号机下发步进指令前检测到代码运行时间太长"),
    MIN_GREEN(70, "0", "0", "最小绿"),
    ABNORMAL_COUNTDOWN(9999, "0", "0", "从灯态中获取到的绿灯倒计时异常（一般是由于信号机现在受临时方案控制，比如）");

    private final Integer code;
    /**
     * 车流向
     */
    private final String dir;
    private final String turn;
    private final String desc;

    public static String getDesc(Integer code) {
        for (PhaseEmptyEnum value : PhaseEmptyEnum.values()) {
            if (Objects.equals(code, value.getCode())) {
                return value.getDesc();
            }
        }
        return null;
    }

    public static String getDir(Integer code) {
        for (PhaseEmptyEnum value : PhaseEmptyEnum.values()) {
            if (Objects.equals(code, value.getCode())) {
                return value.getDir();
            }
        }
        return null;
    }

    public static String getTurn(Integer code) {
        for (PhaseEmptyEnum value : PhaseEmptyEnum.values()) {
            if (Objects.equals(code, value.getCode())) {
                return value.getTurn();
            }
        }
        return null;
    }

}
