package net.wanji.datacenter.config;

import org.apache.kafka.clients.producer.ProducerConfig;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.ProducerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2024/04/24 9:06
 */
@Configuration
public class KafkaProducerConfig {

    @Value("${spring.kafka.producer.producer1.bootstrap-servers}")
    private String bootstrapServers1;

    @Value("${spring.kafka.producer.producer2.bootstrap-servers}")
    private String bootstrapServers2;
    @Value("${spring.kafka.producer.producer1.key-deserializer}")
    private String keySerializer1;
    @Value("${spring.kafka.producer.producer1.value-deserializer}")
    private String valueSerializer1;
    @Value("${spring.kafka.producer.producer2.key-deserializer}")
    private String keySerializer2;
    @Value("${spring.kafka.producer.producer2.value-deserializer}")
    private String valueSerializer2;

    @Bean
    public Map<String, Object> producerConfigs1() {
        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers1);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, keySerializer1);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, valueSerializer1);
        // 可以添加更多配置
        return props;
    }

    @Bean
    public Map<String, Object> producerConfigs2() {
        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers2);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, keySerializer2);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, valueSerializer2);
        // 可以添加更多配置
        return props;
    }

    @Bean
    public ProducerFactory<String, String> producerFactory1() {
        return new DefaultKafkaProducerFactory<>(producerConfigs1());
    }

    @Bean
    public ProducerFactory<String, String> producerFactory2() {
        return new DefaultKafkaProducerFactory<>(producerConfigs2());
    }
    @Primary
    @Bean("kafkaTemplate1")
    public KafkaTemplate<String, String> kafkaTemplate1() {
        return new KafkaTemplate<>(producerFactory1());
    }

    @Bean("kafkaTemplate2")
    public KafkaTemplate<String, String> kafkaTemplate2() {
        return new KafkaTemplate<>(producerFactory2());
    }
}