package net.wanji.datacenter.kafka;

import lombok.extern.slf4j.Slf4j;
import net.wanji.datacenter.service.EventDataService;
import org.apache.commons.lang3.StringUtils;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2024/11/26 15:33
 */
@Slf4j
@Component
public class EventDataConsumerHandler implements KafkaListenerErrorHandler {

    @Resource
    private EventDataService eventDataService;

    @KafkaListener(topics = {"${kafka-consumer.eventDataTopic}"}, groupId = "group")
    public void receiveEventData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String message = String.valueOf(record.value());
            if (StringUtils.isNotBlank(message)) {
                eventDataService.save(message);
            }
        } catch (Exception e) {
            log.error("事件EventData数据处理异常；", e);
            throw new Exception(e);
        }
        acknowledgment.acknowledge();
    }

    @Override
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?>
            consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }
}
