package net.wanji.datacenter.kafka;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.datacenter.service.DataProcessService;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2023/03/09 16:03
 */
@Component
@Slf4j
public class PeriodDataConsumerHandler implements KafkaListenerErrorHandler {

    @Resource
    private DataProcessService dataProcessService;

    @KafkaListener(topics = {"${kafka-consumer.lanePeriodicDataTopic}"}, groupId = "group")
    public void receiveLanePeriodicData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String originalData = String.valueOf(record.value());
        try {
            dataProcessService.laneSave(originalData);
        } catch (Exception e) {
            log.error("车道Lane周期实时数据转换异常", e);
            throw new Exception();
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.crossPeriodicDataTopic}"}, groupId = "group")
    public void receiveCrossPeriodicData(ConsumerRecord<Object, String> record, Acknowledgment acknowledgment) throws Exception {
        String originalData = String.valueOf(record.value());
        try {
            dataProcessService.crossSave(originalData);
        } catch (Exception e) {
            log.error("路口Cross实时数据转换异常", e);
            throw new Exception();
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.dirPeriodicDataTopic}"}, groupId = "group-dir")
    public void receiveDirPeriodicData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String originalData = String.valueOf(record.value());
        try {
            dataProcessService.dirSave(originalData);
        } catch (Exception e) {
            log.error("方向Dir周期实时数据转换异常", e);
            throw new Exception();
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.turnPeriodicDataTopic}"}, groupId = "group")
    public void receiveTurnPeriodicData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String originalData = String.valueOf(record.value());
        try {
            dataProcessService.turnSave(originalData);
        } catch (Exception e) {
            log.error("转向Turn周期实时数据转换异常", e);
            throw new Exception();
        }
        acknowledgment.acknowledge();
    }


    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
