package net.wanji.datacenter.pojo.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.Date;
import java.util.List;

/**
 * @author duanruiming
 * @date 2023/03/09 17:02
 */
@Data
public class CrossPeriodLaneDTO {

    /**
     * 车道事件集合
     */
    @JsonProperty("eventList")
    private List<CrossLaneDataRealTimeDTO> eventList;
    /**
     * 车道数量
     */
    private Integer laneNum;
    /**
     * 全域编号
     */
    private String orgCode;
    /**
     * 数据生成时间: yyyy-MM-dd HH:mm:ss:SSS
     */
    private String timeStamp;

    @Data
    public static class CrossLaneDataRealTimeDTO {
        // 神思-方向(0:北 1:东北 2:东 3:东南 4:南 5:西南 6:西 7:西北)
        private Integer dir;
        // 神思-车道顺序(面向路口从左往右)
        private Integer lane_num;
        // 神思-进出口类型(0:进口 1:出口 2:缓冲区)
        private Integer entry_type;
        @ApiModelProperty(
                name = "车道编号",
                notes = ""
        )
        private String id;
        @ApiModelProperty(
                name = "路口编号",
                notes = ""
        )
        private String crossId;
        @ApiModelProperty(
                name = "交通流量（辆）",
                notes = ""
        )
        private Integer flow;
        @ApiModelProperty(
                name = "平均速度（km/h）",
                notes = ""
        )
        private Double speed;
        @ApiModelProperty(
                name = "驶入速度（km/h）",
                notes = ""
        )
        private Double inSpeed;
        @ApiModelProperty(
                name = "驶出速度（km/h）",
                notes = ""
        )
        private Double outSpeed;
        @ApiModelProperty(
                name = "排队长度（米）",
                notes = ""
        )
        private Double queueLength;
        @ApiModelProperty(
                name = "停车次数",
                notes = ""
        )
        private Double stopTimes;
        @ApiModelProperty(
                name = "延误时间",
                notes = ""
        )
        private Integer delayTime;
        @ApiModelProperty(
                name = "通行能力（辆/某一时段内最大过车数）,通行能力=自然流量",
                notes = ""
        )
        private Integer capacity;
        @ApiModelProperty(
                name = "饱和度",
                notes = ""
        )
        private Double sturation;
        @ApiModelProperty(
                name = "车头间距（米）",
                notes = ""
        )
        private Double vehheadDist;
        @ApiModelProperty(
                name = "车头时距（秒）",
                notes = ""
        )
        private Double vehheadTime;
        @ApiModelProperty(
                name = "可信度（0-1）",
                notes = ""
        )
        private Double quality;
        @ApiModelProperty(
                name = "不停车率",
                notes = ""
        )
        private Double noStopRate;
        @ApiModelProperty(
                name = "一次停车率",
                notes = ""
        )
        private Double oneStopRate;
        @ApiModelProperty(
                name = "二次停车率",
                notes = ""
        )
        private Double twoStopRate;
        @ApiModelProperty(
                name = "三次停车率",
                notes = ""
        )
        private Double threeStopRate;
        @ApiModelProperty(
                name = "开始时间",
                notes = ""
        )
        private Date startTime;
        @ApiModelProperty(
                name = "采集时间（10位时间戳）",
                notes = ""
        )
        private Integer batchTime;
        @ApiModelProperty(
                name = "创建时间",
                notes = ""
        )
        private Date gmtCreate;
        @ApiModelProperty(
                name = "修改时间",
                notes = ""
        )
        private Date gmtModified;
        @ApiModelProperty(
                value = "溢流率",
                notes = ""
        )
        private Double effusionRate;
        @ApiModelProperty(
                value = "绿灯有效利用率",
                notes = ""
        )
        private Double greenLightEfficiency;
        @ApiModelProperty(
                value = "车辆负荷比",
                notes = ""
        )
        private Double vehicleNumsRatioMean;
        @ApiModelProperty(
                value = "空间长度占比",
                notes = ""
        )
        private Double vehicleLengthRatioMean;
        @ApiModelProperty(
                value = "时间占有率",
                notes = ""
        )
        private Double timeOccupancy;
        @ApiModelProperty(
                value = "非机动车流量",
                notes = ""
        )
        private int nonMotorFlow;
        @ApiModelProperty(
                value = "85位速度（km/h）",
                notes = ""
        )
        @TableField("v_85")
        private double v85;
        @ApiModelProperty(
                value = "大车流量",
                notes = ""
        )
        private int trafficFlowA;
        @ApiModelProperty(
                value = "中车流量",
                notes = ""
        )
        private int trafficFlowB;
        @ApiModelProperty(
                value = "小车流量",
                notes = ""
        )
        private int trafficFlowC;

        @ApiModelProperty(
                value = "流量",
                notes = ""
        )
        private int trafficFlow;
        @ApiModelProperty(
                value = "速度",
                notes = ""
        )
        private Double meanV;
        @ApiModelProperty(
                value = "静态最大排队",
                notes = ""
        )
        private Double staticQueueLengthMax;
        @ApiModelProperty(
                name = "通行能力（辆/某一时段内最大过车数）,通行能力=自然流量",
                notes = ""
        )
        private Integer laneCapacity;
        @ApiModelProperty(
                name = "车道饱和度",
                notes = ""
        )
        private Double laneSaturation;
        @ApiModelProperty(
                name = "不停车率",
                notes = ""
        )
        private Double laneNoStopRate;
        @ApiModelProperty(
                name = "一次停车率",
                notes = ""
        )
        private Double laneOneStopRate;
        @ApiModelProperty(
                name = "二次停车率",
                notes = ""
        )
        private Double laneTwoStopRate;
        @ApiModelProperty(
                name = "三次停车率",
                notes = ""
        )
        private Double laneThreeStopRate;
    }

}
