package net.wanji.datacenter.pojo.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2024/11/23 14:49
 * @description 神思失衡算法结果
 */
@Data
@JsonIgnoreProperties(ignoreUnknown = true)
@SuppressWarnings("all")
public class ImbalanceAlgorithmDTO {
    // 万集路口编号
    private String platformId;
    private String crossId;
    private String crossName;
    // 当前策略名称 0：无策略 1：绿灯空放 2：失衡 3：溢出
    private int currentAlgo;
    // 策略请求时间(格式：yyyy-MM-dd HH:mm:ss)
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date requestTime;
    // 策略下发时间(格式：yyyy-MM-dd HH:mm:ss)
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date issueTime;
    // 发是否成功（200成功，其他失败）
    private Integer responseCode;
    // 响应内容
    private String responseContent;
    // 配时信息
    private List<Integer> timingPlan;
    // 入库时间
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date insertTime;
    private int rtnType;
    // 延长时间（当操作为2时有值）
    private int extendTime;
    // 操作名称（0，不操作；1，切换相位；2延长相位）
    private int blockRegion;
    // 溢出类型(0:未溢出 1:即将溢出 2:路口溢出 3:溢停)
    private int blockType;
    // 当前相位剩余时长
    private int countDown;
    // 实时数据时间戳
    private String videoStamp;
    // 灯态数据时间戳
    private String signalMachineStamp;
    // 当前相位控制车流方向
    private String controlDir;
    private Integer code;
    private String msg;
    // 方案具体信息
    private SchemeData data;

    @Data
    @JsonIgnoreProperties(ignoreUnknown = true)
    public static class SchemeData {
        @JsonProperty("platform_id")
        private String platformId;
        @JsonProperty("cycle")
        private int cycle;
        @JsonProperty("green_time")
        private List<Integer> greenTime;
        @JsonProperty("cross_id")
        private String crossId;
        @JsonProperty("control_id")
        private Integer controlId;
        @JsonProperty("provider")
        private String provider;
        @JsonProperty("plan_no")
        private String planNo;
        @JsonProperty("offset")
        private Integer offset;
        @JsonProperty("ring_cnt")
        private Integer ringCnt;
        @JsonProperty("ring_set")
        private Map<String, Map<String, List<Integer>>> ringSet;
        @JsonProperty("seq_id")
        private List<Integer> seqId;
        @JsonProperty("red_time")
        private List<Integer> redTime;
        @JsonProperty("yellow_time")
        private List<Integer> yellowTime;
        @JsonProperty("green_flash")
        private List<Integer> green_flash;
        @JsonProperty("person_flash")
        private List<Integer> person_flash;
        @JsonProperty("min_green")
        private List<Integer> min_green;
        @JsonProperty("max_green")
        private List<Integer> max_green;
        @JsonProperty("direction")
        private List<Integer[][]> direction;


    }
}
