package net.wanji.datacenter.service.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.datacenter.cache.CrossLaneInfoCache;
import net.wanji.datacenter.common.TurnTypeParseEnum;
import net.wanji.datacenter.pojo.dto.CrossPeriodDataDTO;
import net.wanji.datacenter.pojo.dto.CrossPeriodDirDTO;
import net.wanji.datacenter.pojo.dto.CrossPeriodLaneDTO;
import net.wanji.datacenter.pojo.dto.CrossPeriodTurnDTO;
import net.wanji.datacenter.service.DataProcessService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/15 19:59
 */
@Service
@Slf4j
public class DataProcessServiceImpl implements DataProcessService {

    @Resource
    private CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    @Resource
    private CrossLaneDataHistMapper crossLaneDataHistMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private CrossDirDataRealtimeMapper crossDirDataRealTimeMapper;
    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;
    @Resource
    private CrossTurnDataRealtimeMapper crossTurnDataRealTimeMapper;
    @Resource
    private CrossTurnDataHistMapper crossTurnDataHistMapper;


    /**
     * 车道信息处理存库
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void laneSave(String originalData) throws Exception {
        try {
            CrossPeriodLaneDTO CrossPeriodLaneDTO = JSONObject.parseObject(originalData, CrossPeriodLaneDTO.class);
            String timeStamp = CrossPeriodLaneDTO.getTimeStamp();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
            List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO> eventList = CrossPeriodLaneDTO.getEventList();
            List<CrossLaneDataRealTimePO> insertList = getInsertLaneList(timeStamp, dateFormat, eventList);
            // 数据库操作
            if (!CollectionUtils.isEmpty(insertList)) {
                crossLaneDataRealTimeMapper.deleteBatch(insertList.stream().map(CrossLaneDataRealTimePO::getCrossId).collect(Collectors.toList()));
                crossLaneDataRealTimeMapper.insertBatch(insertList);
                List<CrossLaneDataHistPO> crossLaneDataHistPOS = new ArrayList<>(insertList.size());
                BeanListUtils.populateList(insertList, crossLaneDataHistPOS, CrossLaneDataHistPO.class);
                crossLaneDataHistMapper.insertBatch(crossLaneDataHistPOS);
            }
        } catch (Exception e) {
            log.error("车道数据处理入库异常", e);
            throw new Exception(e);
        }
    }

    private static List<CrossLaneDataRealTimePO> getInsertLaneList(String timeStamp, SimpleDateFormat dateFormat, List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO> eventList) throws ParseException {
        List<CrossLaneDataRealTimePO> insertList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(eventList)) {
            Map<String, LaneInfoPO> crossIdLaneId2Map = CrossLaneInfoCache.crossIdLaneId2Map;
            Map<String, List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO>> crossLaneMap = eventList.stream().collect(Collectors.groupingBy(CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO::getCrossId));
            // 路口级别
            for (Map.Entry<String, List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO>> entry : crossLaneMap.entrySet()) {
                String crossId = entry.getKey();
                List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO> eventListValue = entry.getValue();
                List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO> collect1 = eventListValue.stream().filter(item -> Objects.nonNull(item.getDir())).collect(Collectors.toList());
                Map<Integer, List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO>> dirLaneMap = collect1.stream().collect(Collectors.groupingBy(CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO::getDir));
                // 方向级别
                for (Map.Entry<Integer, List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO>> dirLaneEntry : dirLaneMap.entrySet()) {
                    Integer dir = dirLaneEntry.getKey();
                    List<CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO> value = dirLaneEntry.getValue();
                    // 当前方向车道数量
                    int size = value.size();
                    // 车道级别
                    int i = 0;
                    for (CrossPeriodLaneDTO.CrossLaneDataRealTimeDTO crossLaneDataRealTimeDTO : value) {
                        Integer wjLaneId = size - i + 10;
                        // 神思车道号与万集车道号镜像
                        String key = crossId.concat(String.valueOf(dir + 1)).concat(String.valueOf(wjLaneId));
                        LaneInfoPO laneInfoPO = crossIdLaneId2Map.get(key);
                        ++i;
                        if (Objects.isNull(laneInfoPO)) {
                            continue;
                        }
                        String laneId = laneInfoPO.getId();
                        crossLaneDataRealTimeDTO.setId(laneId);
                        CrossLaneDataRealTimePO crossLaneDataRealTime = new CrossLaneDataRealTimePO();
                        BeanUtils.copyProperties(crossLaneDataRealTimeDTO, crossLaneDataRealTime);
                        Date startDate = dateFormat.parse(timeStamp);
                        crossLaneDataRealTime.setStartTime(startDate);
                        Integer batchTime = Integer.valueOf(String.valueOf(startDate.getTime()).substring(0, 10));
                        crossLaneDataRealTime.setBatchTime(batchTime);
                        crossLaneDataRealTime.setFlow(crossLaneDataRealTimeDTO.getTrafficFlow());
                        crossLaneDataRealTime.setSpeed(crossLaneDataRealTimeDTO.getMeanV());
                        crossLaneDataRealTime.setQueueLength(crossLaneDataRealTimeDTO.getStaticQueueLengthMax());
                        crossLaneDataRealTime.setCapacity(crossLaneDataRealTimeDTO.getLaneCapacity());
                        crossLaneDataRealTime.setSturation(crossLaneDataRealTimeDTO.getLaneSaturation());
                        crossLaneDataRealTime.setNoStopRate(crossLaneDataRealTimeDTO.getNoStopRate());
                        crossLaneDataRealTime.setOneStopRate(crossLaneDataRealTimeDTO.getOneStopRate());
                        crossLaneDataRealTime.setTwoStopRate(crossLaneDataRealTimeDTO.getTwoStopRate());
                        crossLaneDataRealTime.setThreeStopRate(crossLaneDataRealTimeDTO.getThreeStopRate());
                        insertList.add(crossLaneDataRealTime);
                    }
                }
            }
        }
        return insertList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void crossSave(String originalData) throws Exception {
        try {
            CrossPeriodDataDTO crossPeriodDataDTO = JSONObject.parseObject(originalData, CrossPeriodDataDTO.class);
            List<CrossDataRealtimePO> eventList = crossPeriodDataDTO.getEventList();
            if (!CollectionUtils.isEmpty(eventList)) {
                List<CrossDataRealtimePO> crossDataRealtimePOS = eventList;
                crossDataRealtimeMapper.deleteBatch(crossDataRealtimePOS.stream().map(CrossDataRealtimePO::getCrossId).collect(Collectors.toList()));
                crossDataRealtimeMapper.insertBatch(crossDataRealtimePOS);
                List<CrossDataHistPO> crossDataHistPOS = new ArrayList<>(crossDataRealtimePOS.size());
                BeanListUtils.populateList(crossDataRealtimePOS, crossDataHistPOS, CrossDataHistPO.class);
                crossDataHistMapper.insertBatch(crossDataHistPOS);
            }
        } catch (Exception e) {
            log.error("路口实时数据存库处理异常：{}", e);
            throw new RuntimeException(e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void dirSave(String originalData) throws Exception {
        try {
            CrossPeriodDirDTO crossPeriodDirDTO = JSONObject.parseObject(originalData, CrossPeriodDirDTO.class);
            List<CrossDirDataRealtimePO> eventList = crossPeriodDirDTO.getEventList();
            if (!CollectionUtils.isEmpty(eventList)) {
                List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = new ArrayList<>(eventList.size());
                for (CrossDirDataRealtimePO crossDirDataRealtimePO : eventList) {
                    String crossId = crossDirDataRealtimePO.getCrossId();
                    Integer dirType = crossDirDataRealtimePO.getDirType() + 1;
                    crossDirDataRealtimePO.setId(String.join("_", crossId, String.valueOf(dirType)));
                    crossDirDataRealtimePO.setDirType(dirType);
                    crossDirDataRealtimePOS.add(crossDirDataRealtimePO);
                }
                crossDirDataRealTimeMapper.deleteBatch(crossDirDataRealtimePOS.stream().map(CrossDirDataRealtimePO::getCrossId).collect(Collectors.toList()));
                crossDirDataRealTimeMapper.insertBatch(crossDirDataRealtimePOS);
                List<CrossDirDataHistPO> list = new ArrayList<>(crossDirDataRealtimePOS.size());
                BeanListUtils.populateList(crossDirDataRealtimePOS, list, CrossDirDataHistPO.class);
                crossDirDataHistMapper.insertBatch(list);
            }
        } catch (Exception e) {
            log.error("路口实时数据存库处理异常：{}", e);
            throw new RuntimeException(e);
        }
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void turnSave(String originalData) throws Exception {
        try {
            CrossPeriodTurnDTO crossPeriodTurnDTO = JSONObject.parseObject(originalData, CrossPeriodTurnDTO.class);
            List<CrossTurnDataRealtimePO> eventList = crossPeriodTurnDTO.getEventList();
            if (!CollectionUtils.isEmpty(eventList)) {
                List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = eventList;
                List<CrossTurnDataRealtimePO> results = new ArrayList<>(eventList.size());
                for (CrossTurnDataRealtimePO crossTurnDataRealtimePO : crossTurnDataRealtimePOS) {
                    String crossId = crossTurnDataRealtimePO.getCrossId();
                    CrossTurnDataRealtimePO result = new CrossTurnDataRealtimePO();
                    BeanUtils.copyProperties(crossTurnDataRealtimePO, result);
                    Integer inDir = crossTurnDataRealtimePO.getInDir() + 1;
                    String turnType = crossTurnDataRealtimePO.getTurnType();
                    result.setInDir(inDir);
                    Integer ssCode = TurnTypeParseEnum.getCodeBySSCode(Integer.valueOf(turnType));
                    result.setTurnType(String.valueOf(ssCode));
                    String keyId = String.join("_", crossId, String.valueOf(inDir), turnType);
                    result.setId(keyId);
                    results.add(result);
                }
                List<String> crossIds = results.stream().map(CrossTurnDataRealtimePO::getCrossId).distinct().collect(Collectors.toList());
                crossTurnDataRealTimeMapper.deleteBatch(crossIds);
                crossTurnDataRealTimeMapper.insertBatch(results);
                List<CrossTurnDataHistPO> list = new ArrayList<>(results.size());
                BeanListUtils.populateList(results, list, CrossTurnDataHistPO.class);
                crossTurnDataHistMapper.insertBatch(list);
            }
        } catch (Exception e) {
            log.error("路口实时数据存库处理异常：{}", e);
            throw new RuntimeException(e);
        }
    }
}
