package net.wanji.datacenter.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveRealtimeMapper;
import net.wanji.datacenter.mapper.StrategyGreenOptHistMapper;
import net.wanji.datacenter.pojo.dto.GreenBeltKafkaDTO;
import net.wanji.datacenter.pojo.dto.GreenWavePeriodDTO;
import net.wanji.datacenter.pojo.entity.StrategyGreenOptHistEntity;
import net.wanji.datacenter.pojo.vo.GreenBeltChartVO;
import net.wanji.datacenter.pojo.vo.GreenBeltOptInfoVO;
import net.wanji.datacenter.service.GreenBeltInfoService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/19 19:04
 */
@Service
@Slf4j
public class GreenBeltInfoServiceImpl implements GreenBeltInfoService {

    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    @Resource
    private GreenwaveHistMapper greenwaveHistMapper;

    private static String greenBeltInfoData = "";

    public static void main(String[] args) throws Exception {
        String message = greenBeltInfoData;
        ObjectMapper mapper = JacksonUtils.getInstance();
        GreenBeltKafkaDTO kafkaDTO = mapper.readValue(message, GreenBeltKafkaDTO.class);
        // 正反绿波容器
        List<GreenBeltOptInfoVO.DirGreenDetail> dirGreenDetails = new ArrayList<>();
        // 构建正向绿波
        setForward(kafkaDTO, dirGreenDetails);
        // 反向绿波
        setBackward(kafkaDTO, dirGreenDetails);
        // 返回绿波实体
        GreenBeltOptInfoVO greenBeltOptInfoVO = new GreenBeltOptInfoVO();
        greenBeltOptInfoVO.setGreenId(kafkaDTO.getGreenbeltId());
        greenBeltOptInfoVO.setLength(kafkaDTO.getGreenbeltLength());
        greenBeltOptInfoVO.setCycle(kafkaDTO.getMaxCycle());
        greenBeltOptInfoVO.setControlTime(kafkaDTO.getControlTime());
        greenBeltOptInfoVO.setControlDuration(kafkaDTO.getControlDuration());
        greenBeltOptInfoVO.setType(kafkaDTO.getGreenbeltType());
        greenBeltOptInfoVO.setDynamic(kafkaDTO.getDynamic());
        greenBeltOptInfoVO.setControlMethod(kafkaDTO.getControlMethod());
        greenBeltOptInfoVO.setDirGreenDetails(dirGreenDetails);
        System.err.println(mapper.writeValueAsString(greenBeltOptInfoVO));

        GreenBeltChartVO greenBeltChartVO = calGreenChart(greenBeltOptInfoVO);
        String s = mapper.writeValueAsString(greenBeltChartVO);
        GreenBeltChartVO greenBeltChartVO1 = mapper.readValue(s, GreenBeltChartVO.class);
        System.err.println(s);
    }

    @Override
    public GreenBeltChartVO convertChartData(GreenBeltOptInfoVO vo) throws Exception {
        return calGreenChart(vo);
    }

    private static GreenBeltChartVO calGreenChart(GreenBeltOptInfoVO greenBeltOptInfoVO) throws JsonProcessingException {
        ObjectMapper mapper = JacksonUtils.getInstance();
        Map<String, List<List<Double>>> crossRedTimesMap = new TreeMap<>();
        Map<String, List<List<Double>>> backCrossRedTimesMap = new TreeMap<>();
        Map<String, Double> crossGreenStartMap = new TreeMap<>();
        Map<String, Double> backCrossGreenStartMap = new TreeMap<>();
        Map<String, Double> distanceMap = new TreeMap<>();
        // 处理绿波时序图数据
        List<GreenBeltOptInfoVO.DirGreenDetail> dirGreenDetails = greenBeltOptInfoVO.getDirGreenDetails();
        Double greenWidthTime = 0.0;
        Double backGreenWidthTime = 0.0;
        if (!CollectionUtils.isEmpty(dirGreenDetails)) {
            Integer cycle = greenBeltOptInfoVO.getCycle();
            for (GreenBeltOptInfoVO.DirGreenDetail dirGreenDetail : dirGreenDetails) {
                Integer dirType = dirGreenDetail.getDirType();
                if (dirType == 1) {
                    greenWidthTime = dirGreenDetail.getGreenWidthTime();
                    List<GreenBeltOptInfoVO.CrossGreenDetail> list = dirGreenDetail.getCrossGreenDetailList();
                    getaDouble(crossRedTimesMap, crossGreenStartMap, distanceMap, cycle, list);
                }
                if (dirType == 0) {
                    backGreenWidthTime = dirGreenDetail.getGreenWidthTime();
                    List<GreenBeltOptInfoVO.CrossGreenDetail> list = dirGreenDetail.getCrossGreenDetailList();
                    Collections.reverse(list);
                    getaDouble(backCrossRedTimesMap, backCrossGreenStartMap, distanceMap, cycle, list);

                }
            }
        }
        GreenBeltChartVO greenBeltChartVO = new GreenBeltChartVO();
        greenBeltChartVO.setGreenWidthTime(greenWidthTime);
        greenBeltChartVO.setBackGreenWidthTime(backGreenWidthTime);
        greenBeltChartVO.setCrossRedTimesMap(mapper.writeValueAsString(crossRedTimesMap));
        greenBeltChartVO.setBackCrossRedTimesMap(mapper.writeValueAsString(backCrossRedTimesMap));
        greenBeltChartVO.setGreenStartMap(mapper.writeValueAsString(crossGreenStartMap));
        greenBeltChartVO.setBackGreenStartMap(mapper.writeValueAsString(backCrossGreenStartMap));
        greenBeltChartVO.setDistanceMap(mapper.writeValueAsString(distanceMap));
        return greenBeltChartVO;
    }

    private static void getaDouble(Map<String, List<List<Double>>> crossRedTimesMap, Map<String, Double> crossGreenStartMap, Map<String, Double> distanceMap,
                                   Integer cycle, List<GreenBeltOptInfoVO.CrossGreenDetail> crossGreenDetailList) {
        if (!CollectionUtils.isEmpty(crossGreenDetailList)) {

            // 所有行程时间
            double totalTravel = crossGreenDetailList.stream().mapToDouble(GreenBeltOptInfoVO.CrossGreenDetail::getTravelTime).sum();
            GreenBeltOptInfoVO.CrossGreenDetail firstCrossGreenDetail = crossGreenDetailList.get(0);
            Double firstOffset = firstCrossGreenDetail.getOffset();
            Double firstPhaseStartTime = firstCrossGreenDetail.getPhaseStartTime();
            Double firstGreenStart = firstCrossGreenDetail.getGreenStartTime();
            double cycleSum = Math.ceil((totalTravel + firstOffset + firstPhaseStartTime + firstGreenStart) / cycle);
            Double crossGreenStartTime = firstOffset + firstPhaseStartTime + firstGreenStart;
            for (GreenBeltOptInfoVO.CrossGreenDetail crossGreenDetail : crossGreenDetailList) {
                List<List<Double>> redTimes = new ArrayList<>();
                String crossId = crossGreenDetail.getCrossId();
                Double offset = crossGreenDetail.getOffset();
                Double phaseStartTime = crossGreenDetail.getPhaseStartTime();
                Double travelTime = crossGreenDetail.getTravelTime();
                Double distance = crossGreenDetail.getDistance();
                // 相位结束时间大于周期，绿灯时时间补到周期开始位置
                Double cycleD = Double.valueOf(cycle);
                Double phaseEndTime = crossGreenDetail.getPhaseEndTime();

                for (int i = 0; i < cycleSum; i++) {
                    // 周期 + 相位差
                    Double cycleIndex = i * cycleD;
                    Double phaseReverseStartTime = phaseEndTime.intValue() - cycleD;
                    // 最后结尾要将前面相位差时间补充为周期
                    Double endCycleExt = cycleD - offset;
                    Double offsetIndex = endCycleExt;
                    // 相位结束时间 > cycle, 周期相位差后面补充周期要添加绿灯
                    if (phaseReverseStartTime > 0) {
                        phaseReverseStartTime = Math.abs(phaseReverseStartTime);
                        // cycle内红灯
                        redTimes.add(Arrays.asList(cycleIndex + phaseReverseStartTime, cycleIndex + phaseStartTime));
                        redTimes.add(Arrays.asList(cycleIndex + phaseEndTime, cycleIndex + cycleD));
                        if (endCycleExt >= phaseStartTime) {
                            // endCycleExt内红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + phaseReverseStartTime, cycleIndex + offset + phaseStartTime));
                            // offset内红灯
                            redTimes.add(Arrays.asList(phaseReverseStartTime - offsetIndex, phaseStartTime - offsetIndex));
                        } else if (endCycleExt < phaseStartTime && phaseReverseStartTime > endCycleExt) {
                            // endCycleExt全绿灯
                            // offset全绿灯
                        } else if (phaseReverseStartTime <= endCycleExt) {
                            // endCycleExt内红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + phaseReverseStartTime, cycleIndex + offset + phaseStartTime));
                            // offset内红灯
                            redTimes.add(Arrays.asList(phaseEndTime - offsetIndex, offset));
                        }
                    } else {
                        // cycle内红灯
                        redTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + phaseStartTime));
                        redTimes.add(Arrays.asList(cycleIndex + offset + phaseEndTime, cycleIndex + offset + cycleD));
                        if (endCycleExt >= phaseStartTime) {
                            // endCycleExt红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + cycleD, cycleIndex + offset + cycleD + phaseStartTime));
                            // offset红灯
                            redTimes.add(Arrays.asList(phaseEndTime - offsetIndex, offset));
                        } else {
                            // endCycleExt红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + cycleD, cycleIndex + offset + cycleD + endCycleExt));
                            // offset内红灯
                            redTimes.add(Arrays.asList(0.0, phaseStartTime - offsetIndex));
                            redTimes.add(Arrays.asList(phaseEndTime - offsetIndex, offset));
                        }
                    }

                    // 去重和排序
                    List<List<Double>> uniqueSortedRedTimes = redTimes.stream()
                            .distinct()  // 去重
                            .sorted((list1, list2) -> {
                                // 按照列表第一个元素排序
                                return Double.compare(list1.get(0), list2.get(0));
                            })
                            .collect(Collectors.toList());

                    crossRedTimesMap.put(crossId, uniqueSortedRedTimes);
                }
                crossGreenStartMap.put(crossId, crossGreenStartTime);
                crossGreenStartTime += travelTime;
                distanceMap.put(crossId, distance);
            }
        }
    }


    @Override
    public GreenBeltOptInfoVO convertData(String message) throws Exception {
        ObjectMapper mapper = JacksonUtils.getInstance();
        GreenBeltKafkaDTO kafkaDTO = mapper.readValue(message, GreenBeltKafkaDTO.class);
        if (-1 == kafkaDTO.getControlMethod()) {
            StrategyGreenOptHistEntity entity = new StrategyGreenOptHistEntity();
            entity.setGreenId(kafkaDTO.getGreenbeltId());
            entity.setLength(kafkaDTO.getGreenbeltLength());
            entity.setCycle(kafkaDTO.getMaxCycle());
            entity.setControlTime(kafkaDTO.getControlTime());
            entity.setControlDuration(kafkaDTO.getControlDuration());
            entity.setControlMethod(kafkaDTO.getControlMethod());
            entity.setType(kafkaDTO.getGreenbeltType());
            entity.setDynamic(kafkaDTO.getDynamic());
            entity.setCrossGreenDetail(JacksonUtils.getInstance().writeValueAsString(Collections.emptyList()));
            entity.setKeyCross(kafkaDTO.getKeyCross());
            entity.setResponseCode(kafkaDTO.getResponseCode());
            entity.setResponseContent(kafkaDTO.getResponseContent());
            strategyGreenOptHistMapper.insert(entity);
            return null;
        }
        // 正反绿波容器
        List<GreenBeltOptInfoVO.DirGreenDetail> dirGreenDetails = new ArrayList<>();
        // 构建正向绿波
        setForward(kafkaDTO, dirGreenDetails);
        // 反向绿波
        setBackward(kafkaDTO, dirGreenDetails);
        // 返回绿波实体
        GreenBeltOptInfoVO greenBeltOptInfoVO = new GreenBeltOptInfoVO();
        greenBeltOptInfoVO.setGreenId(kafkaDTO.getGreenbeltId());
        greenBeltOptInfoVO.setLength(kafkaDTO.getGreenbeltLength());
        greenBeltOptInfoVO.setCycle(kafkaDTO.getMaxCycle());
        greenBeltOptInfoVO.setControlTime(kafkaDTO.getControlTime());
        greenBeltOptInfoVO.setControlDuration(kafkaDTO.getControlDuration());
        greenBeltOptInfoVO.setType(kafkaDTO.getGreenbeltType());
        greenBeltOptInfoVO.setDynamic(kafkaDTO.getDynamic());
        greenBeltOptInfoVO.setControlMethod(kafkaDTO.getControlMethod());
        greenBeltOptInfoVO.setDirGreenDetails(dirGreenDetails);
        greenBeltOptInfoVO.setKeyCross(kafkaDTO.getKeyCross());
        greenBeltOptInfoVO.setResponseCode(kafkaDTO.getResponseCode());
        greenBeltOptInfoVO.setResponseContent(kafkaDTO.getResponseContent());
        return greenBeltOptInfoVO;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(GreenBeltOptInfoVO infoVO) throws Exception {
        if (Objects.nonNull(infoVO)) {
            List<GreenBeltOptInfoVO.DirGreenDetail> dirGreenDetails = infoVO.getDirGreenDetails();
            if (!CollectionUtils.isEmpty(dirGreenDetails)) {
                for (GreenBeltOptInfoVO.DirGreenDetail dirGreenDetail : dirGreenDetails) {

                    StrategyGreenOptHistEntity entity = new StrategyGreenOptHistEntity();
                    entity.setGreenId(infoVO.getGreenId());
                    entity.setLength(infoVO.getLength());
                    entity.setCycle(infoVO.getCycle());
                    entity.setControlTime(infoVO.getControlTime());
                    entity.setControlDuration(infoVO.getControlDuration());
                    entity.setControlMethod(infoVO.getControlMethod());
                    entity.setType(infoVO.getType());
                    entity.setDynamic(infoVO.getDynamic());

                    entity.setDirType(dirGreenDetail.getDirType());
                    entity.setDir(dirGreenDetail.getDir());
                    entity.setMaxSpeed(dirGreenDetail.getMaxSpeed());
                    entity.setMinSpeed(dirGreenDetail.getMinSpeed());
                    entity.setGreenWidthTime(dirGreenDetail.getGreenWidthTime());
                    entity.setCrossGreenDetail(JacksonUtils.getInstance().writeValueAsString(dirGreenDetail.getCrossGreenDetailList()));
                    entity.setKeyCross(infoVO.getKeyCross());
                    entity.setResponseContent(infoVO.getResponseContent());
                    entity.setResponseCode(infoVO.getResponseCode());
                    strategyGreenOptHistMapper.insert(entity);
                }
            }
        }
    }

    private static void setBackward(GreenBeltKafkaDTO kafkaDTO, List<GreenBeltOptInfoVO.DirGreenDetail> dirGreenDetails) {
        // 反向绿波路口列表
        List<GreenBeltOptInfoVO.CrossGreenDetail> crossGreenDetails = new ArrayList<>();
        Map<String, Double> phaseStartBackward = kafkaDTO.getPhaseStartBackward();
        for (Map.Entry<String, Double> backEntry : phaseStartBackward.entrySet()) {
            String crossId = backEntry.getKey();
            Double phaseStartTime = backEntry.getValue();
            GreenBeltOptInfoVO.CrossGreenDetail crossGreenDetail = new GreenBeltOptInfoVO.CrossGreenDetail();
            crossGreenDetail.setCrossId(crossId);
            crossGreenDetail.setPhaseStartTime(phaseStartTime);
            Double phaseEndTime = kafkaDTO.getPhaseEndBackward().get(crossId);
            crossGreenDetail.setPhaseEndTime(phaseEndTime);
            Double greenStartTime = kafkaDTO.getBeltStartBackward().get(crossId);
            crossGreenDetail.setGreenStartTime(greenStartTime);
            Double speed = kafkaDTO.getSpeedBackward().get(crossId);
            crossGreenDetail.setSpeed(speed);
            Double offset = kafkaDTO.getOffset().get(crossId);
            crossGreenDetail.setOffset(offset);

            Double distance = kafkaDTO.getDistanceBackward().get(crossId);
            crossGreenDetail.setDistance(distance);
            Double travelTime = kafkaDTO.getTravelTimeBackward().get(crossId);
            crossGreenDetail.setTravelTime(travelTime);
            crossGreenDetails.add(crossGreenDetail);
        }
        // 反向绿波
        // 反向绿波实体
        GreenBeltOptInfoVO.DirGreenDetail dirGreenDetail = new GreenBeltOptInfoVO.DirGreenDetail();
        dirGreenDetail.setDirType(0);
        String forwardDirection = kafkaDTO.getBackwardDirection();
        dirGreenDetail.setDir(forwardDirection);
        double maxSpeedForward = kafkaDTO.getMaxSpeedBackward();
        dirGreenDetail.setMaxSpeed(maxSpeedForward);
        double minSpeedForward = kafkaDTO.getMinSpeedBackward();
        dirGreenDetail.setMinSpeed(minSpeedForward);
        double greenWidthForward = kafkaDTO.getGreenWidthBackward();
        dirGreenDetail.setGreenWidthTime(greenWidthForward);
        dirGreenDetail.setCrossGreenDetailList(crossGreenDetails);
        dirGreenDetails.add(dirGreenDetail);
    }

    private static void setForward(GreenBeltKafkaDTO kafkaDTO, List<GreenBeltOptInfoVO.DirGreenDetail> dirGreenDetails) {
        // 正向绿波路口列表
        List<GreenBeltOptInfoVO.CrossGreenDetail> crossGreenDetails = new ArrayList<>();
        Map<String, Double> phaseStartForward = kafkaDTO.getPhaseStartForward();
        for (Map.Entry<String, Double> forwardEntry : phaseStartForward.entrySet()) {
            String crossId = forwardEntry.getKey();
            Double phaseStartTime = forwardEntry.getValue();
            GreenBeltOptInfoVO.CrossGreenDetail crossGreenDetail = new GreenBeltOptInfoVO.CrossGreenDetail();
            crossGreenDetail.setCrossId(crossId);
            crossGreenDetail.setPhaseStartTime(phaseStartTime);
            Double phaseEndTime = kafkaDTO.getPhaseEndForward().get(crossId);
            crossGreenDetail.setPhaseEndTime(phaseEndTime);
            Double greenStartTime = kafkaDTO.getBeltStartForward().get(crossId);
            crossGreenDetail.setGreenStartTime(greenStartTime);
            Double speed = kafkaDTO.getSpeedForward().get(crossId);
            crossGreenDetail.setSpeed(speed);
            Double offset = kafkaDTO.getOffset().get(crossId);
            crossGreenDetail.setOffset(offset);

            Double distance = kafkaDTO.getDistanceForward().get(crossId);
            crossGreenDetail.setDistance(distance);
            Double travelTime = kafkaDTO.getTravelTimeForward().get(crossId);
            crossGreenDetail.setTravelTime(travelTime);
            crossGreenDetails.add(crossGreenDetail);
        }
        // 正向绿波
        // 正向绿波实体
        GreenBeltOptInfoVO.DirGreenDetail dirGreenDetail = new GreenBeltOptInfoVO.DirGreenDetail();
        dirGreenDetail.setDirType(1);
        String forwardDirection = kafkaDTO.getForwardDirection();
        dirGreenDetail.setDir(forwardDirection);
        double maxSpeedForward = kafkaDTO.getMaxSpeedForward();
        dirGreenDetail.setMaxSpeed(maxSpeedForward);
        double minSpeedForward = kafkaDTO.getMinSpeedForward();
        dirGreenDetail.setMinSpeed(minSpeedForward);
        double greenWidthForward = kafkaDTO.getGreenWidthForward();
        dirGreenDetail.setGreenWidthTime(greenWidthForward);
        dirGreenDetail.setCrossGreenDetailList(crossGreenDetails);
        dirGreenDetails.add(dirGreenDetail);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void savePeriod(String message) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            GreenWavePeriodDTO greenWavePeriodDTO = instance.readValue(message, GreenWavePeriodDTO.class);
            List<GreenwaveRealtimePO> greenwaveRealtimePOS = greenWavePeriodDTO.getEventList();
            greenwaveRealtimePOS.forEach(item -> {
                        Integer batchTime = item.getBatchTime();
                        item.setBatchTime(batchTime);
                        item.setGmtCreate(new Date());
                        item.setGmtModified(new Date());
                        item.setStartTime(new Date(batchTime * 1000L));
                    }
            );
            if (!CollectionUtils.isEmpty(greenwaveRealtimePOS)) {
                List<Integer> collect = greenwaveRealtimePOS.stream().map(GreenwaveRealtimePO::getGreenId).collect(Collectors.toList());
                greenwaveRealtimeMapper.deleteBatch(collect);
                greenwaveRealtimeMapper.insertBatch(greenwaveRealtimePOS);
                List<GreenwaveHistPO> list = new ArrayList<>(greenwaveRealtimePOS.size());
                BeanListUtils.populateList(greenwaveRealtimePOS, list, GreenwaveRealtimePO.class);
                greenwaveHistMapper.insertBatch(list);
            }
        } catch (Exception e) {
            log.error("绿波周期数据存储异常：{}", e);
            throw new RuntimeException(e);
        }
    }

}
