//package net.wanji.datacenter.task;
//
//import com.fasterxml.jackson.core.JsonProcessingException;
//import com.fasterxml.jackson.core.type.TypeReference;
//import lombok.extern.slf4j.Slf4j;
//import net.wanji.common.enums.DateStyle;
//import net.wanji.common.framework.rest.JsonViewObject;
//import net.wanji.common.utils.tool.JacksonUtils;
//import net.wanji.databus.vo.LightsStatusVO;
//import net.wanji.datacenter.cache.CrossRealTimeCache;
//import net.wanji.datacenter.cache.CrossSchemePhaseLightsCache;
//import net.wanji.datacenter.kafka.ConsumerHandler;
//import net.wanji.datacenter.pojo.dto.CrossSchemePhaseDirTurnDTO;
//import net.wanji.datacenter.pojo.dto.CrossSnapshotDataDTO;
//import net.wanji.datacenter.service.impl.LanePeriodicDataProcessServiceImpl;
//import net.wanji.feign.service.UtcFeignClients;
//import org.apache.commons.lang.StringUtils;
//import org.springframework.scheduling.annotation.Scheduled;
//import org.springframework.stereotype.Component;
//import org.springframework.util.CollectionUtils;
//
//import javax.annotation.Resource;
//import java.text.SimpleDateFormat;
//import java.util.*;
//import java.util.concurrent.ConcurrentHashMap;
//
///**
// * @author duanruiming
// * @date 2023/03/10 14:35
// */
//@Component
//@Slf4j
//public class DataProcessTask {
//
//    @Resource
//    UtcFeignClients utcFeignClients;
//
//    private final ConcurrentHashMap<String, String> startPhaseIdMap = new ConcurrentHashMap<>();
//
//    /**
//     * 通过实时灯态计算路口相位绿灯清空率
//     * 需要注意搭接相位，目前搭接相位的两个相位当作为一个相位
//     * 延迟30s，否则Kafka中未消费但是运行定时任务获取消息
//     */
//    @Scheduled(initialDelay = 30 * 1000, fixedRate = 1000)
//    public void calculateGreenLightsClearRate() {
//        try {
//            JsonViewObject jsonViewObject = utcFeignClients.lightStatus();
//            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
//                log.error("定时任务同步信号机灯态utcService调用异常");
//                return;
//            }
//
//            List<LightsStatusVO> lightsStatusVOS = JacksonUtils.getInstance().convertValue(jsonViewObject.getContent(), new TypeReference<List<LightsStatusVO>>() {});
//            calculateData(lightsStatusVOS);
//        }catch (Exception e) {
//           // log.error("",e);
//        }
//    }
//
//    private void calculateData(List<LightsStatusVO> lightsStatusVOS) throws Exception {
//        for (LightsStatusVO lightsStatusVO : lightsStatusVOS) {
//            String crossId = lightsStatusVO.getCrossId();
//            String schemeNo = lightsStatusVO.getSchemeId();
//            String currentPhaseId = lightsStatusVO.getPhaseId(); // 当前相位红灯说明都是红灯，全场红
//            long currenSignalTime = Long.parseLong(lightsStatusVO.getSchemeStartTime());
//            ConcurrentHashMap<String, Integer> crossIdPhaseIdClearGreenMap = CrossRealTimeCache.crossPhaseCarNumsMap;
//            getClearRateCache(crossId, currentPhaseId, crossIdPhaseIdClearGreenMap);
//            Map<String, Object> dirLampGroupMap = lightsStatusVO.getDirLampGroupMap();
//            // 是否是全场红
//            boolean isAllRed = getAllRed(dirLampGroupMap);
//            // 如果全场红
//            if (isAllRed) {
//                List<CrossSchemePhaseDirTurnDTO> crossSchemePhaseDirTurnDTOList = CrossSchemePhaseLightsCache.crossSchemePhaseDirTurnDTOList;
//                int redTime = 2;
//                if (!CollectionUtils.isEmpty(crossSchemePhaseDirTurnDTOList)) {
//                    for (CrossSchemePhaseDirTurnDTO dto : crossSchemePhaseDirTurnDTOList) {
//                        String crossIdCache = dto.getCrossId();
//                        String schemeNoCache = dto.getSchemeNo();
//                        String phaseNoCache = dto.getPhaseNo();
//                        if (StringUtils.equals(crossIdCache, crossId) && StringUtils.equals(schemeNoCache, schemeNo)
//                                && StringUtils.equals(currentPhaseId, phaseNoCache)) {
//                            redTime = dto.getRedTime();
//                        }
//                    }
//                    // 剩余红灯时间
//                    if (redTime >= 1) {
//                        --redTime;
//                    }
//                    Integer carNums = getCarNums(crossId, currenSignalTime + redTime * 1000L);
//                    startPhaseIdMap.putIfAbsent(crossId, currentPhaseId);
//                    crossIdPhaseIdClearGreenMap.put(crossId.concat(currentPhaseId), carNums);
//                }
//                LanePeriodicDataProcessServiceImpl.periodHaveAllRedMap.put(crossId, 1);
//            }
//        }
//    }
//
//    private static boolean getAllRed(Map<String, Object> dirLampGroupMap) {
//        boolean isAllRed = true;
//        if (!dirLampGroupMap.isEmpty()) {
//            for (Map.Entry<String, Object> entry : dirLampGroupMap.entrySet()) {
//                Map<String, String> turnLightColorMap = (Map<String, String>) entry.getValue();
//                for (Map.Entry<String, String> colorEntry : turnLightColorMap.entrySet()) {
//                    String lightColor = colorEntry.getValue();
//                    // 全场红，如果有一个绿灯就不计算 20行人跳过
//                    String key = colorEntry.getKey();
//                    if (Objects.equals(lightColor, "green") && !StringUtils.equals(key, "20")) {
//                        isAllRed = false;
//                    }
//                }
//            }
//        }
//        return isAllRed;
//    }
//
//    private void getClearRateCache(String crossId, String currentPhaseId, Map<String, Integer> crossIdPhaseIdClearGreenMap) throws JsonProcessingException {
//        int phaseSize = 0;
//        HashMap<String, Integer> tempMap = new HashMap<>();
//        for (Map.Entry<String, Integer> entry : crossIdPhaseIdClearGreenMap.entrySet()) {
//            String crossIdPhaseNo = entry.getKey();
//            if (crossIdPhaseNo.contains(crossId)) {
//                ++phaseSize;
//                tempMap.put(crossId, entry.getValue());
//            }
//        }
//        if (startPhaseIdMap.containsValue(currentPhaseId) && phaseSize >= 2) {
//            double clearRate = 0.0;
//            for (Map.Entry<String, Integer> entry : tempMap.entrySet()) {
//                int value = entry.getValue() > 0 ? 1 : 0;
//                clearRate += (double) value / phaseSize;
//            }
//            startPhaseIdMap.remove(crossId);// 只计算一次
//            log.info("绿灯清空率为：" + (1 - clearRate) * 100);
//            CrossRealTimeCache.crossPhaseIdClearGreenMap.put(crossId, 1 - clearRate);
//        }
//    }
//
//    /**
//     * 通过路口编号获取主题消息中的车辆数量
//     * @param crossId
//     * @return java.lang.Integer
//     * @description
//     */
//    private Integer getCarNums(String crossId, Long currenSignalTime) throws Exception {
//        Map<String, List<CrossSnapshotDataDTO>> crossSnapshotDataMap = ConsumerHandler.crossSnapshotDataMap;
//        if (!crossSnapshotDataMap.isEmpty()) {
//            List<CrossSnapshotDataDTO> crossSnapshotDataDTOS = crossSnapshotDataMap.get("JN050001CrossSnapshotData");
//            if (!CollectionUtils.isEmpty(crossSnapshotDataDTOS)) {
//                for (CrossSnapshotDataDTO crossSnapshotDataDTO : crossSnapshotDataDTOS) {
//                    String timeStamp = crossSnapshotDataDTO.getTimeStamp();
//                    Date msgDate = new SimpleDateFormat(DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()).parse(timeStamp);
//                    Long aLong = Long.valueOf(String.valueOf(currenSignalTime).substring(0, 10));
//                    if (aLong == msgDate.getTime() / 1000) {
//                        List<CrossSnapshotDataDTO.EventList> eventList = crossSnapshotDataDTO.getEventList();
//                        for (CrossSnapshotDataDTO.EventList element : eventList) {
//                            String currentCrossId = element.getCrossId();
//                            if (StringUtils.equals(crossId, currentCrossId)) {
//                                return element.getCarNums();
//                            }
//                        }
//                    }
//                }
//            }
//        }
//        return 0;
//    }
//}
