package net.wanji.datacenter.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.datacenter.common.Constants;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.index.reindex.DeleteByQueryRequest;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author duanruiming
 * @date 2023/10/27 17:11
 */
@Component
@Slf4j
public class LaneSnapShotDataDelTask {

    @Resource
    private RestHighLevelClient client;

    @Scheduled(fixedRate = 1000 * 3600)
    public void delLast24HourData() throws Exception {
        try {
            DeleteByQueryRequest deleteByQueryRequest = new DeleteByQueryRequest(Constants.LANE_SNAPSHOT_DATA_ES_INDEX);
            Date date = new Date();
            long currentTime = date.getTime() / 1000;
            long lastDay = currentTime - 86400L;
            RangeQueryBuilder lte = QueryBuilders.rangeQuery("timeStamp").lte(lastDay);
            deleteByQueryRequest.setQuery(lte);
            client.deleteByQuery(deleteByQueryRequest, RequestOptions.DEFAULT);
        } catch (Exception e) {
            log.error("定时删除elastic前一天数据失败", e);
            throw new Exception(e);
        }
    }
}
