/**
 *
 */
package com.wanji.indicators.cache;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.vividsolutions.jts.geom.Geometry;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.mapper.BaseCrossTurnInfoMapper;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;


@Component
public class BaseLaneCache {

    //车道基础信息
    private static Map<String, BaseLaneInfo> baseLaneInfoMap = new ConcurrentHashMap<>();

    //同方向进口道对应的出口车道集
    private static Map<String, List<BaseLaneInfo>> ridSameDirOutLaneMap = new ConcurrentHashMap<>();

    private static Map<String, Integer> segmentLaneNumberMap = new ConcurrentHashMap<>();

    private static Map<String, Double[]> ridLaneLengthMap = new ConcurrentHashMap<>();

    private static Map<String, String> refLaneIdMap = new ConcurrentHashMap<>();

    private static Map<String, List<BaseCrossTurnInfo>> crossTurnInfoMap = new ConcurrentHashMap<>();

    private static Map<String, Map<String, Double>> freeSpeedMap = new ConcurrentHashMap<>();

    private static BaseLaneCache instance;
    static PropertiesHelper property = PropertiesHelper.getInstance();


    private BaseLaneCache() {
    }

    public static synchronized BaseLaneCache getInstance() {
        if (instance == null) {
            instance = new BaseLaneCache();
            loadData();

            loadTurnRelation();

            loadFreeSpeed();

        }
        return instance;
    }

    private static void loadData() {
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        BaseLaneInfoService baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);
        String crossList = property.getProperties().getProperty("full.area.cross.list");

        Map<String, BaseLaneInfo> map = baseLaneInfoService.findLaneInfo(crossList);
        baseLaneInfoMap.putAll(map);

        ridSameDirOutLaneMap.putAll(baseLaneInfoService.findRidSameDirLaneInfo(map.values()));

        //车道衔接
        Map<String, String> refMap = baseLaneInfoService.findLaneJoinRef();
        refLaneIdMap.putAll(refMap);

        Collection<BaseLaneInfo> collection = map.values();
        Map<String, List<BaseLaneInfo>> segmentLaneList = collection.stream().filter(o -> Objects.nonNull(o.getSegmentId())).collect(Collectors.groupingBy(o -> o.getSegmentId()));
        for (Map.Entry<String, List<BaseLaneInfo>> entry : segmentLaneList.entrySet()) {
            segmentLaneNumberMap.put(entry.getKey(), entry.getValue().size());
        }

        Map<String, List<BaseLaneInfo>> ridLaneList = collection.stream().collect(Collectors.groupingBy(o -> o.getRid() + "_" + o.getType()));
        for (Map.Entry<String, List<BaseLaneInfo>> entry : ridLaneList.entrySet()) {

            List<BaseLaneInfo> value = entry.getValue();
            double totalLength = value.stream().mapToDouble(o -> o.getLength()).summaryStatistics().getSum();

            //路段上车道总长度
            ridLaneLengthMap.put(entry.getKey(), new Double[]{totalLength, (double) value.size()});
        }
    }

    private static void loadTurnRelation() {

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        BaseCrossTurnInfoMapper baseCrossTurnInfoMapper = beanConf.getBean(BaseCrossTurnInfoMapper.class);

        List<BaseCrossTurnInfo> dataList = baseCrossTurnInfoMapper.findRidTurnInfo(null);
        if (dataList != null && !dataList.isEmpty()) {
            //按出口道分组
            Map<String, List<BaseCrossTurnInfo>> groupByTRid = dataList.stream().collect(Collectors.groupingBy(o -> o.getOutRid()));

            for (Map.Entry<String, List<BaseCrossTurnInfo>> entry : groupByTRid.entrySet()) {
                List<BaseCrossTurnInfo> value = entry.getValue();
                for (BaseCrossTurnInfo info : value) {
                    //转换为数字型字典
                    info.setTurnType(TurnEnum.getValue(info.getTurnType()));
                }
                crossTurnInfoMap.put(entry.getKey(), value);
            }
        }
    }

    private static void loadFreeSpeed() {
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        BaseLaneInfoService baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);

        String crossList = property.getProperties().getProperty("full.area.cross.list");

        Double defaultFreeSpeed = Double.parseDouble(property.getProperties().getProperty("rid.default.free.speed"));

        Map<String, Map<String, Double>> ridFreeSpeed = baseLaneInfoService.getRidFreeSpeed(crossList, defaultFreeSpeed);

        freeSpeedMap.putAll(ridFreeSpeed);
    }

    public Map<String, BaseLaneInfo> getBaseLaneInfoMap() {
        return baseLaneInfoMap;
    }

    public static Map<String, List<BaseCrossTurnInfo>> getCrossTurnInfoMap() {
        return crossTurnInfoMap;
    }

    public Map<String, Integer> getSegmentLaneNumberMap() {
        return segmentLaneNumberMap;
    }

    public Map<String, String> getRefLaneIdMap() {
        return refLaneIdMap;
    }

    public Map<String, List<BaseLaneInfo>> getRidSameDirOutLaneMap() {
        return ridSameDirOutLaneMap;
    }

    public static Map<String, Double[]> getRidLaneLengthMap() {
        return ridLaneLengthMap;
    }

    public   Map<String, Map<String, Double>> getFreeSpeedMap() {
        return freeSpeedMap;
    }
}
