package com.wanji.indicators.constant;

import java.util.Arrays;
import java.util.List;

/**
 * 事件等级
 */
public enum EventGradeEnum {

    DEDUCT_POINTS(1,"扣分"),
    TICKET(2,"罚款"),
    WARN(3,"警告");


    int level;
    String desc;

    EventGradeEnum(int b,String desc) {
        this.level = b;
        this.desc = desc;
    }

    public int getLevel() {
        return level;
    }

    public String getDesc() {
        return desc;
    }

    public static int getLevelByEventType(EventAbnormalEnum eventType){
        if(eventType == null){
            return 0;
        }
        return getLevelByEventType(eventType.getType());

    }


    // 扣分
    public static final List<EventAbnormalEnum> DEDUCT_POINTS_Events = Arrays.asList(new EventAbnormalEnum[]{
            EventAbnormalEnum.VEHICLERUNREDLIGHT,  // 机动车闯红灯
            EventAbnormalEnum.MOTO_NO_YIEDLD_PEDESTRIAN,  // 机动车未礼让行人
            EventAbnormalEnum.RETROGRADE,  // 机动车 逆行
            EventAbnormalEnum.MOTO_PARK_ON_LINE  // 机动车停车越线
    });
    // 罚款
    public static final List<EventAbnormalEnum> TICKET_Events = Arrays.asList(new EventAbnormalEnum[]{
            EventAbnormalEnum.MOTO_RUN_WRONG_DIRECTION,  // 机动车不按车道线行驶
            EventAbnormalEnum.MOTO_RUN_IN_OTHER_LINE,  // 机动车 占用专用车道
            EventAbnormalEnum.OCCUPYBUSLANE,  // 机动车 占用公交车道
            EventAbnormalEnum.RESTRICTION_LICENSE_PLATE  // 机动车 限号出行
    });
    // 警告
    public static final List<EventAbnormalEnum> WARN_Events = Arrays.asList(new EventAbnormalEnum[]{
            EventAbnormalEnum.NONMOTORETROGRADE,  // 非机动车逆行
            EventAbnormalEnum.NONMOTO_RUN_REDLIGHT,  // 非机动车闯红灯
            EventAbnormalEnum.NONMOTO_RUN_IN_OTHER_LINE,  // 非机动车不按车道线行驶
            EventAbnormalEnum.NONMOTO_PARK_ON_LINE,  // 非机动车越线停车
            EventAbnormalEnum.NONMOTO_CARRY_PEOPLE  // 非机动车载人
    });


    public static int getLevelByEventType(String eventType){
        // 扣分
        if(DEDUCT_POINTS_Events.stream().anyMatch(event -> event.getType().equals(eventType))){
            return DEDUCT_POINTS.getLevel();
        }

        // 罚款
        if(TICKET_Events.stream().anyMatch(event -> event.getType().equals(eventType))){
            return TICKET.getLevel();
        }

        // 警告
        if(WARN_Events.stream().anyMatch(event -> event.getType().equals(eventType))){
            return WARN.getLevel();
        }
        // 其他返回 0
        return 0;
    }

}
