package com.wanji.indicators.constant;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public enum TurnEnum {

    //******************************************************//
    LEFT("l","1","左"),
    STRAIGHT("s","2","直"),
    RIGHT("r","3","右"),
    UTURN("u","4","掉头");
    //******************************************************//
    private String key;
    private String value;
    private String name;
    TurnEnum(String key, String value,String name) {
        this.key = key;
        this.value = value;
        this.name = name;
    }

    public static String getValue(String key){
        for(TurnEnum typeEnum: TurnEnum.values()){
            if(Objects.equals(typeEnum.key,key)){
                return typeEnum.value;
            }
        }
        return null;
    }

    public static String getKey(TurnEnum key) {
        for(TurnEnum typeEnum: TurnEnum.values()) {
            if(Objects.equals(typeEnum,key)){
                return typeEnum.key;
            }
        }
        return null;
    }

    /**
     * 转换车道功能转向[l,s,r]
     *
     * @param function 车道功能 1左转；2直行；3右转；4掉头；5直左；6直右；7左直右；8左右；9左转掉头；10直行掉头；11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头
     * @return
     */
    public static String[] getLaneTurnEn(int function) {
        String[] turnArr = new String[4];
        if (function == 1) {
            turnArr[0] = TurnEnum.LEFT.getKey();
        } else if (function == 2) {
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
        } else if (function == 3) {
            turnArr[2] = TurnEnum.RIGHT.getKey();
        } else if (function == 4) {
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 5) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
        } else if (function == 6) {
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
            turnArr[2] = TurnEnum.RIGHT.getKey();
        } else if (function == 7) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
            turnArr[2] = TurnEnum.RIGHT.getKey();
        } else if (function == 8) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[2] = TurnEnum.RIGHT.getKey();
        } else if (function == 9) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 10) {
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 11) {
            turnArr[2] = TurnEnum.RIGHT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 12) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 13) {
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
            turnArr[2] = TurnEnum.RIGHT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 14) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[1] = TurnEnum.STRAIGHT.getKey();
            turnArr[2] = TurnEnum.RIGHT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        } else if (function == 15) {
            turnArr[0] = TurnEnum.LEFT.getKey();
            turnArr[2] = TurnEnum.RIGHT.getKey();
            turnArr[3] = TurnEnum.UTURN.getKey();
        }
        List<String> turnList = Arrays.asList(turnArr.clone()).stream().filter(o->Objects.nonNull(o)).collect(Collectors.toList());
        return turnList.toArray(new String[]{});
    }

    /**
     * 转换车道功能转向[1,2,3]
     *
     * @param function 车道功能 1左转；2直行；3右转；4掉头；5直左；6直右；7左直右；8左右；9左转掉头；10直行掉头；11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头
     * @return
     */
    public static String[] getLaneTurnValue(int function) {
        String[] turnArr = new String[4];
        if (function == 1) {
            turnArr[0] = TurnEnum.LEFT.getValue();
        } else if (function == 2) {
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
        } else if (function == 3) {
            turnArr[2] = TurnEnum.RIGHT.getValue();
        } else if (function == 4) {
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 5) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
        } else if (function == 6) {
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
            turnArr[2] = TurnEnum.RIGHT.getValue();
        } else if (function == 7) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
            turnArr[2] = TurnEnum.RIGHT.getValue();
        } else if (function == 8) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[2] = TurnEnum.RIGHT.getValue();
        } else if (function == 9) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 10) {
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 11) {
            turnArr[2] = TurnEnum.RIGHT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 12) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 13) {
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
            turnArr[2] = TurnEnum.RIGHT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 14) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[1] = TurnEnum.STRAIGHT.getValue();
            turnArr[2] = TurnEnum.RIGHT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        } else if (function == 15) {
            turnArr[0] = TurnEnum.LEFT.getValue();
            turnArr[2] = TurnEnum.RIGHT.getValue();
            turnArr[3] = TurnEnum.UTURN.getValue();
        }
        List<String> turnList = Arrays.asList(turnArr.clone()).stream().filter(o->Objects.nonNull(o)).collect(Collectors.toList());
        return turnList.toArray(new String[]{});

    }

    /**
     * 转换车道功能转向[]
     *
     * @param function 车道功能 1左转；2直行；3右转；4掉头；5直左；6直右；7左直右；8左右；9左转掉头；10直行掉头；11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头
     * @return
     */
    public static String getLaneFunction(int function) {
        String[] turnArr = TurnEnum.getLaneTurnValue(function);
        //车道功能组装
        String laneFuncS = Arrays.asList(turnArr).stream().filter(o->Objects.nonNull(o)).collect(Collectors.joining(","));
         return laneFuncS;
    }

    public String getKey() {
        return key;
    }

    public void setKey(String key) {
        this.key = key;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }
}
