package com.wanji.indicators.constant;

import com.google.common.collect.Maps;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName VehicleTypeEnum
 * @Description
 * @Date 2021/4/14 16:56
 * @Version 1.0
 */
public enum VehicleTypeEnum {
    //未知
    UNKNOWN((short) 0),

    //小客车
    CAR((short) 1),
    //大货车
    TRUCK((short) 2),
    //大巴车
    BUS((short) 3),
    //行人
    PEDESTRIAN((short) 4),
    //自行车
    BIKE((short) 5),
    //电动车
    ELECTROMOBILE((short) 6),
    //中巴车
    MEDIUMBUS((short) 7),
    //危化品
    DANGERCAR((short) 8),
    //遗撒物
    LITTER((short) 9),
    //小货车
    VAN((short) 10),
    //中货车
    MIDTRUCK((short) 11),
    //救护车
    AMBULANCE((short) 12),
    //消防车
    FIREVEHICEL((short) 13),
    //机动摩托车
    MOTOR((short) 14),
    //三轮车【作为机动车看待】
    TRICYCLE((short) 15),
    //外卖车
    TACKEOUT((short) 150),
    //外卖车-美团
    TACKEOUT_MEITUAN((short) 151),
    //外卖车-饿了么
    TACKEOUT_ELEME((short) 152),
    //外卖车-肯德基
    TACKEOUT_KFC((short) 153),
    //外卖车-麦当劳
    TACKEOUT_MCDONALD((short) 154),
    //快递车
    EXPRESS((short) 160),
    //快递车-京东
    EXPRESS_JD((short) 161),
    //快递车-顺丰
    EXPRESS_SF((short) 162),
    //快递车-申通
    EXPRESS_STO((short) 163),
    //快递车-圆通
    EXPRESS_YTO((short) 164),
    //快递车-中通
    EXPRESS_ZTO((short) 165),
    //快递车-韵达
    EXPRESS_YUNDA((short) 166);

    short type;

    VehicleTypeEnum(short b) {
        this.type = b;
        //命令字
    }

    public short getType() {
        return type;
    }

    /**
     * 根据大类获取小类
     * @param type
     * @return
     */
    public static List<Integer> getOriginalTypeList(String type) {
        List<Integer> originalTypeList = new ArrayList<>();
        if (Objects.isNull(type))
            return originalTypeList;

        String[] sps = type.split(",");
        for (String tp : sps) {
            if (Objects.equals(ParticipantEnum.MOTOR.getType(), tp)) {
                List<Integer> list = VehicleTypeEnum.VehicleType.stream().map(o -> (int)o.getType()).collect(Collectors.toList());
                originalTypeList.addAll(list);
            }
            if (Objects.equals(ParticipantEnum.NON_MOTOR.getType(), tp)) {
                List<Integer> list = VehicleTypeEnum.NonMotorVehicleType.stream().map(o -> (int)o.getType()).collect(Collectors.toList());
                originalTypeList.addAll(list);
            }
            if (Objects.equals(ParticipantEnum.PED.getType(), tp)) {
                List<Integer> list = VehicleTypeEnum.PedestrianType.stream().map(o -> (int)o.getType()).collect(Collectors.toList());
                originalTypeList.addAll(list);
            }
        }
        return originalTypeList;
    }

    // 机动车类型
    public static final List<VehicleTypeEnum> VehicleType = Arrays.asList(new VehicleTypeEnum[]{
            VehicleTypeEnum.CAR,
            VehicleTypeEnum.TRUCK,
            VehicleTypeEnum.BUS,
            VehicleTypeEnum.MEDIUMBUS,
            VehicleTypeEnum.DANGERCAR,
            VehicleTypeEnum.VAN,
            VehicleTypeEnum.MIDTRUCK,
            VehicleTypeEnum.AMBULANCE,
            VehicleTypeEnum.FIREVEHICEL,
            VehicleTypeEnum.MOTOR
    });





    /**
     * 是否为机动车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isMotorVehicles(short type) {
        return VehicleTypeEnum.CAR.type == type
                || VehicleTypeEnum.TRUCK.type == type
                || VehicleTypeEnum.BUS.type == type
                || VehicleTypeEnum.MEDIUMBUS.type == type
                || VehicleTypeEnum.DANGERCAR.type == type
                || VehicleTypeEnum.VAN.type == type
                || VehicleTypeEnum.MIDTRUCK.type == type
                || VehicleTypeEnum.AMBULANCE.type == type
                || VehicleTypeEnum.FIREVEHICEL.type == type
                || VehicleTypeEnum.MOTOR.type == type
                || VehicleTypeEnum.TRICYCLE.type == type
                ;
    }

    // 非机动车类型
    public static final List<VehicleTypeEnum> NonMotorVehicleType = Arrays.asList(new VehicleTypeEnum[] {
            VehicleTypeEnum.BIKE,
            VehicleTypeEnum.ELECTROMOBILE,
            VehicleTypeEnum.TACKEOUT,
            VehicleTypeEnum.TACKEOUT_MEITUAN,
            VehicleTypeEnum.TACKEOUT_ELEME,
            VehicleTypeEnum.TACKEOUT_KFC,
            VehicleTypeEnum.TACKEOUT_MCDONALD,
            VehicleTypeEnum.EXPRESS,
            VehicleTypeEnum.EXPRESS_JD,
            VehicleTypeEnum.EXPRESS_SF,
            VehicleTypeEnum.EXPRESS_STO,
            VehicleTypeEnum.EXPRESS_YTO,
            VehicleTypeEnum.EXPRESS_ZTO,
            VehicleTypeEnum.EXPRESS_YUNDA
    });

    /**
     * 是否为非机动车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isNonMotorVehicles(short type) {
        return VehicleTypeEnum.BIKE.type == type
                || VehicleTypeEnum.ELECTROMOBILE.type == type
                || isTakeout(type)
                || isExpress(type);
    }

    // 非机动车类型
    public static final List<VehicleTypeEnum> PedestrianType = Arrays.asList(new VehicleTypeEnum[] {
            VehicleTypeEnum.PEDESTRIAN
    });
    /**
     * 是否为行人
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isPedestrian(short type) {
        return VehicleTypeEnum.PEDESTRIAN.type == type;
    }

    /**
     * 是否为 小汽车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isCar(short type) {
        return VehicleTypeEnum.CAR.type == type;
    }

    /**
     * 是否为 大货车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isTruck(short type) {
        return VehicleTypeEnum.TRUCK.type == type;
    }

    /**
     * 是否为 中货车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isMediumTruck(short type) {
        return VehicleTypeEnum.MIDTRUCK.type == type;
    }

    /**
     * 是否为 小货车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isSmallTruck(short type) {
        return VehicleTypeEnum.VAN.type == type;
    }

    /**
     * 是否为 大巴车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isBus(short type) {
        return VehicleTypeEnum.BUS.type == type;
    }

    /**
     * 是否为 中巴车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isMediumBus(short type) {
        return VehicleTypeEnum.MEDIUMBUS.type == type;
    }

    /**
     * 是否为 危化品车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isDangerCar(short type) {
        return VehicleTypeEnum.DANGERCAR.type == type;
    }

    /**
     * 是否为 救护车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isAmbulance(short type) {
        return VehicleTypeEnum.AMBULANCE.type == type;
    }

    /**
     * 是否为 消防车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isFireEngine(short type) {
        return VehicleTypeEnum.FIREVEHICEL.type == type;
    }

    /**
     * 是否为 外卖车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isTakeout(short type) {
        return VehicleTypeEnum.TACKEOUT.type == type
                || VehicleTypeEnum.TACKEOUT_MEITUAN.type == type
                || VehicleTypeEnum.TACKEOUT_ELEME.type == type
                || VehicleTypeEnum.TACKEOUT_KFC.type == type
                || VehicleTypeEnum.TACKEOUT_MCDONALD.type == type;
    }

    /**
     * 是否为 外卖车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isTakeout_MEITUAN(short type) {
        return VehicleTypeEnum.TACKEOUT_MEITUAN.type == type;
    }

    /**
     * 是否为 外卖车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isTakeout_ELEME(short type) {
        return VehicleTypeEnum.TACKEOUT_ELEME.type == type;
    }

    /**
     * 是否为 快递车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isExpress(short type) {
        return VehicleTypeEnum.EXPRESS.type == type
                || VehicleTypeEnum.EXPRESS_JD.type == type
                || VehicleTypeEnum.EXPRESS_SF.type == type
                || VehicleTypeEnum.EXPRESS_STO.type == type
                || VehicleTypeEnum.EXPRESS_YTO.type == type
                || VehicleTypeEnum.EXPRESS_ZTO.type == type
                || VehicleTypeEnum.EXPRESS_YUNDA.type == type;
    }

    /**
     * 是否为 顺丰快递车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isExpress_SF(short type) {
        return VehicleTypeEnum.EXPRESS_SF.type == type;
    }


    /**
     * 是否为 京东快递车
     *
     * @param type 车辆类型
     * @return
     */
    public static boolean isExpress_JD(short type) {
        return VehicleTypeEnum.EXPRESS_JD.type == type;
    }

}
