package com.wanji.indicators.event.cross.accident;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.event.cross.accident.funcnew.*;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.accident.AccidentShowModel;
import com.wanji.indicators.sink.AccidentEventFileSink;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.source.ConflictTurnSource;
import com.wanji.indicators.source.CongestionSource;
import com.wanji.indicators.source.FirdsOfRightTridSource;
import com.wanji.indicators.task.pedestrian.service.func.TrafficLightStatusFlatMap;
import com.wanji.indicators.task.pedestrian.service.func.TrafficLightStatusInfoCoFlatMap;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.ConnectedStreams;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.ProcessingTimeSessionWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;

import java.io.Serializable;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * 疑似异常停车/事故计算
 *
 * @author fengyi
 */
public class AccidentEventMainNew implements Serializable {

    private final StreamExecutionEnvironment env;
    private final PropertiesHelper instance;
    private final Properties p;
    private String path;
    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private AccidentEventMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    public static AccidentEventMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new AccidentEventMainNew(env, path, isDebug);
    }

    private AccidentEventMainNew(StreamExecutionEnvironment env) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
    }

    private AccidentEventMainNew(StreamExecutionEnvironment env, String path) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
    }

    public static AccidentEventMainNew init(StreamExecutionEnvironment env) {
        return new AccidentEventMainNew(env);
    }

    public static AccidentEventMainNew init(StreamExecutionEnvironment env, String path) {
        return new AccidentEventMainNew(env, path);
    }

    /**
     * 开始计算
     */
    public void run(DataStream<CrossFrameModel> thinningDataStream, DataStream<String> lightStatusStream) {

        //初始化kafka生产者
        FlinkKafkaProducer producer = new FlinkKafkaProducer<>("analysis.cross.event",
                new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);
        //异常停车
        FlinkKafkaProducer abnormalStopProducer = new FlinkKafkaProducer<>("analysis.cross.abnormal.stop",
                new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);


        //解析数据
        SingleOutputStreamOperator<AccidentFrameModel> accidentFrameModelFilter = null;

        //有灯态
        if (lightStatusStream != null) {
            SingleOutputStreamOperator<Map<String, List<CrossRidTurnLampStatusModel>>> turnLightStatusStream =
                    lightStatusStream.flatMap(new TrafficLightStatusFlatMap()).setParallelism(1).name("实时灯态信息");

            //连接灯态数据源
            ConnectedStreams<CrossFrameModel, Map<String, List<CrossRidTurnLampStatusModel>>> connStream =
                    thinningDataStream.connect(turnLightStatusStream);

            SingleOutputStreamOperator<CrossFrameModel> bindLightStatusStream =
                    connStream.flatMap(new TrafficLightStatusInfoCoFlatMap()).setParallelism(1).name("全域轨迹帧数据-路口灯态绑定");

            accidentFrameModelFilter = bindLightStatusStream.flatMap(new AccidentFlatMap()).setParallelism(1).name("事故检测-解析数据");
        } else {
            //无灯态
            accidentFrameModelFilter = thinningDataStream.flatMap(new AccidentFlatMap()).setParallelism(1).name("事故检测-解析数据");
        }

//        SingleOutputStreamOperator<AccidentFrameModel> travelDataProcess = accidentFrameModelFilter
//                .keyBy(AccidentFrameModel::getCrossId)
//                .process(new AbnormalCrossingTravelDataProcess())
//                .setParallelism(1)
//                .name("事故检测-计算车辆在路口内行驶距离及旅行时间");


        //匹配拥堵检测区域
        SingleOutputStreamOperator<AccidentFrameModel> checkCongestionArea = accidentFrameModelFilter
                .connect(env.addSource(new CongestionSource()).broadcast())
                .flatMap(new AccidentCheckCongestionAreaCoFlatMap())
                .setParallelism(1)
                .name("事故检测-匹配拥堵检测区域");

        //加载冲突相位信息
        SingleOutputStreamOperator<AccidentFrameModel> loadConflictSourceProcess = checkCongestionArea
                .connect(env.addSource(new ConflictTurnSource()).broadcast())
                .flatMap(new AbnormalDeadlockConflictSlotCoFlatMap())
                .setParallelism(1)
                .name("事故检测-加载冲突相位信息");

        //匹配右转出口道 可进入的其他方向rid
        SingleOutputStreamOperator<AccidentFrameModel> rightVehicleTurnRef = loadConflictSourceProcess
                .connect(env.addSource(new FirdsOfRightTridSource()).broadcast())
                .flatMap(new AccidentRightVehicleRidRefFlatMap())
                .setParallelism(1)
                .name("事故检测-右转出口道-其他方向可进入进口道匹配");

        //路口冲突检测
        SingleOutputStreamOperator<AccidentFrameModel> conflictDirectProcess = rightVehicleTurnRef
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AbnormalDeadlockConflictCheckKeyedProcess())
                .setParallelism(1)
                .name("事故检测-路口冲突转向车辆检测");

        //计算是否疑似拥堵 若有拥堵特征则 过 "滤掉" 从拥堵rid驶出的所有车辆
        SingleOutputStreamOperator<AccidentFrameModel> checkCongestionKeyedProcess = conflictDirectProcess
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AccidentCongestionKeyedProcess())
                .setParallelism(1)
                .name("事故检测-计算是否路口溢出");

        //计算是否有同向行驶的车辆
        SingleOutputStreamOperator<AccidentFrameModel> checkSameDirKeyedProcess = checkCongestionKeyedProcess
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AccidentSameDriDriveKeyedProcess())
                .setParallelism(1)
                .name("事故检测-检测停车车辆是否存在相同转向运行车辆");

        //计算停驶车辆和停车持续时间
        SingleOutputStreamOperator<AccidentFrameModel> checkStopKeyedProcess = checkSameDirKeyedProcess
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AccidentStopCheckKeyedProcess())
                .setParallelism(1)
                .name("事故检测-计算停驶车辆和停车持续时间");

        //计算有灯态的情况下  绿灯车辆停车时间
        SingleOutputStreamOperator<AccidentFrameModel> checkLampStateProcess = checkStopKeyedProcess
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AccidentLampStateKeyedProcess())
                .setParallelism(1)
                .name("事故检测-有灯态则计算绿灯车辆停车时间");

        //右转车，无灯态时，计算停车时长
        SingleOutputStreamOperator<AccidentFrameModel> checkRightVehiclProcess = checkLampStateProcess
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AccidentRightVehicleKeyedProcess())
                .setParallelism(1)
                .name("事故检测-右转无灯态停车时间计算");

        //根据事故持续时间计算事故结果
        SingleOutputStreamOperator<AccidentFrameModel> accidentFrameFlatMap = checkRightVehiclProcess
                .flatMap(new AccidentCheckTimeFlatMap())
                .setParallelism(1)
                .name("事故检测-根据事故持续时间计算事故结果");

        //事故逻辑去重
        SingleOutputStreamOperator<AccidentFrameModel> duplicateRemoveKeyedProcess = accidentFrameFlatMap
                .keyBy(AccidentFrameModel::getCrossId)
                .process(new AccidentDuplicateRemoveKeyedProcess())
                .setParallelism(1)
                .name("事故检测-计算事故结果逻辑去重");

        SingleOutputStreamOperator<AccidentFrameModel> eventAccidentsResult = duplicateRemoveKeyedProcess
                .keyBy(o->o.getCrossId()+"_"+o.getAccidentType())
                .window(ProcessingTimeSessionWindows.withGap(Time.milliseconds(60*1000)))
                .process(new AccidentDistinguishProcess())
                .name("事故检测-事故数据合并");

        //事故辅助定责
        SingleOutputStreamOperator<AccidentShowModel> accidentShowModel = eventAccidentsResult
                .flatMap(new AccidentEncapsulationFlatMap())
                .setParallelism(1)
                .name("事故检测-计算事故辅助定责");

        //事故数据转换
        SingleOutputStreamOperator<String> accidentOutputStream = accidentShowModel
                .filter(o->!o.isAbnormalStop())
                .flatMap(new FlatMapFunction<AccidentShowModel, String>() {
                    @Override
                    public void flatMap(AccidentShowModel model, Collector<String> out) {
                        out.collect(JSONObject.toJSONString(model));
                    }
                }).name("事故检测-数据格式转换结果封装")
                .setParallelism(1);

        //非事故异常停车数据转换
//        SingleOutputStreamOperator<String> abnormalStopOutputStream = accidentShowModel
//                .filter(o->o.isAbnormalStop())
//                .flatMap(new FlatMapFunction<AccidentShowModel, String>() {
//                    @Override
//                    public void flatMap(AccidentShowModel model, Collector<String> out) {
//                        out.collect(JSONObject.toJSONString(model));
//                    }
//                }).name("异常停车-非事故异常停车数据转换")
//                .setParallelism(1);

        if (!isDebug) {
            accidentOutputStream.addSink(producer).name("事故计算-数据发送至kafka").setParallelism(1);
            accidentOutputStream.name("事故计算-结果打印");

//            abnormalStopOutputStream.addSink(abnormalStopProducer).name("异常停车计算-数据发送至kafka").setParallelism(1);
//            abnormalStopOutputStream.name("异常停车计算-结果打印");
        } else {
            accidentShowModel.addSink(new AccidentEventFileSink(path)).setParallelism(1).name("事故计算-结果输出至文件");

//            String path = this.path + "analysis.cross.abnormal.stop";
//            abnormalStopOutputStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("异常停车-结果输出至文件");
        }

    }


//    /**
//     * 事故持续时间/影响程度计算
//     */
//    private void accidentLastsFor(SingleOutputStreamOperator<EventCross> eventCrossStream) {
//
//        SingleOutputStreamOperator<AccidentTimingModel> accidentTimingModel = eventCrossStream
//                .flatMap(new AccidentInfluenceDegreeFlatMap())
//                .setParallelism(1)
//                .name("事故计算-持续时间/交通影响严重程度");
//
//        SingleOutputStreamOperator<AccidentTimingModel> reduceAccidentTiming = accidentTimingModel
//                .keyBy(AccidentTimingModel::getId)
//                .window(TumblingProcessingTimeWindows.of(Time.seconds(1)))
//                .reduce(new AccidentTimingReduce())
//                .setParallelism(1)
//                .name("事故持续时间+交通影响严重程度聚合-开窗1秒聚合");
//
////        reduceAccidentTiming.addSink(new AccidentDurationRedisSink(instance))
////                .setParallelism(1)
////                .name("事故持续时间数据输出至redis");
//
//        SingleOutputStreamOperator<AccidentTimingModel> result = reduceAccidentTiming
//                .keyBy(AccidentTimingModel::getId)
//                .window(ProcessingTimeSessionWindows.withGap(Time.seconds(30)))
//                .reduce(new ReduceFunction<AccidentTimingModel>() {
//                    @Override
//                    public AccidentTimingModel reduce(AccidentTimingModel a, AccidentTimingModel b) throws Exception {
//                        return b;
//                    }
//                }).setParallelism(1)
//                .name("事故消散状态计算");
//
//        SingleOutputStreamOperator<String> accidentTimingToES = result.flatMap(new FlatMapFunction<AccidentTimingModel, String>() {
//            @Override
//            public void flatMap(AccidentTimingModel model, Collector<String> out) {
//                String id = model.getId();
//                model.setId(id.replace("duration_accident_info:", Constant.EMPTY_STRING));
//                String s = JSONObject.toJSONString(model);
//                out.collect(s);
//            }
//        }).setParallelism(1).name("事故状态结果集封装");
//
//        FlinkKafkaProducer producer = new FlinkKafkaProducer<>("cross.event.data.accident.time",
//                new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
//                instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);
//
//        accidentTimingToES.addSink(producer).setParallelism(1).name("事故持续时间数据输出至kafka");
//        accidentTimingToES.print("事故持续时间-结果打印").name("事故持续时间-结果打印");
//
//    }

}
