package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadLockStatusModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadlockResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 计算车辆在路口内行驶距离、行程时间
 */
public class AbnormalCrossingTravelDataProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AbnormalCrossingTravelDataProcess.class);

    private MapState<Integer, AccidentFrameModel.CarTrack> mapState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_car_travel_data", Integer.class, AccidentFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(AccidentFrameModel crossFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        ConfigModel configModel = crossFrameModel.getConfigModel();
        long nowTime = crossFrameModel.getTime();

        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
        List<AccidentFrameModel.CarTrack> trackList = crossFrameModel.getTrackList();
        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            Integer id = carTrack.getId();
            AccidentFrameModel.CarTrack agoCar = mapState.get(id);
            if (agoCar != null) {
                //路口区域内行驶距离
                double areaDist = GeomsConvertUtil.getDistance(agoCar.getLongitude(), agoCar.getLatitude(), carTrack.getLongitude(), carTrack.getLatitude());
                carTrack.setAreaDist(agoCar.getAreaDist() + areaDist);
                carTrack.setTravelTime(nowTime - agoCar.getGlobalTimeStamp());
            } else {
                carTrack.setAreaDist(carTrack.getStopLineDist());
            }
            // 筛选行驶越过斑马线
            if (carTrack.getAreaDist() > configModel.getAbnormalParkZebraCrossLength()) {
                list.add(carTrack);
            }
            mapState.put(id, carTrack);
        }
        crossFrameModel.setTrackList(list);
        collector.collect(crossFrameModel);

        /*******************过期数据清理********************************************/
        Set<Integer> expireKeys = new HashSet<>();
        Iterator<Map.Entry<Integer, AccidentFrameModel.CarTrack>> iterator = mapState.iterator();
        while (iterator.hasNext()) {
            Map.Entry<Integer, AccidentFrameModel.CarTrack> entry = iterator.next();
            AccidentFrameModel.CarTrack carTrack = entry.getValue();
            long difTime = nowTime - carTrack.getGlobalTimeStamp();
            if (difTime > 30 * 1000) {
                expireKeys.add(entry.getKey());
            }
        }
        for (Integer key : expireKeys) {
            mapState.remove(key);
        }
        /*******************过期数据清理********************************************/

    }
}
