package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadLockStatusModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadlockResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * 检测是否存在冲突运行的车辆
 */
public class AbnormalDeadlockConflictCheckKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AbnormalDeadlockConflictCheckKeyedProcess.class);

    private MapState<String, CrossDeadLockStatusModel> mapState;
    private MapState<String, CrossDeadlockResultModel> resultMapState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAbnormalDeadlock_stop_1", String.class, CrossDeadLockStatusModel.class));
        resultMapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAbnormalDeadlock_result_1", String.class, CrossDeadlockResultModel.class));
    }

    @Override
    public void processElement(AccidentFrameModel deadLockFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        ConfigModel configModel = deadLockFrameModel.getConfigModel();
        long nowTime = deadLockFrameModel.getTime();

        List<AccidentFrameModel.CarTrack> trackList = deadLockFrameModel.getTrackList();
        List<AccidentFrameModel.CarTrack> conflictTrackList =  getConflictCarList(deadLockFrameModel);
        //去重
        conflictTrackList = conflictTrackList.stream().distinct().collect(Collectors.toList());
        //死锁状态 true时死锁
        boolean stateFlg = false;
        if (!conflictTrackList.isEmpty()) {
            //List<Integer> ridList = conflictTrackList.stream().map(AccidentFrameModel.CarTrack::getTrackID).distinct().collect(Collectors.toList());
            //log.info("异常停车路口车辆冲突行驶，时间：{},车辆数：{},冲突车辆轨迹ID:{}",DateUtil.toDateTime(nowTime,"yyyy-MM-dd HH:mm:ss"),conflictTrackList.size(),ridList.toString());
            stateFlg = deadLockState(nowTime, deadLockFrameModel.getCrossId(), configModel, conflictTrackList);
        }
        deadLockFrameModel.setDeadLock(stateFlg);

        //key:cross-rid-ridDir-turn value:crossid-rid-ridDir-冲突转向
        Map<String, List<String>> conflictMap = deadLockFrameModel.getConflictSlotMap();
        //按RID分组
        Map<String, List<AccidentFrameModel.CarTrack>> groupList = trackList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));

        if (groupList.size() >= 2) {
            //存放按RID+方向+转向分组后的车辆集合
            Map<String, List<AccidentFrameModel.CarTrack>> ridTurnCarList = new HashMap<>();
            for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry : groupList.entrySet()) {
                List<AccidentFrameModel.CarTrack> value = entry.getValue();

                for (AccidentFrameModel.CarTrack ridCar : value) {
                    String laneFunc = TurnEnum.getLaneFunction(ridCar.getRoadnet().getTurn());
                    String[] sps = laneFunc.split(Constant.COMMA);

                    for (String func : sps) {
                        String ridTurnKey = this.getKey(ridCar,func);

                        List<AccidentFrameModel.CarTrack> tmplist = ridTurnCarList.get(ridTurnKey);
                        if (Objects.isNull(tmplist)) {
                            tmplist = new ArrayList<>();
                        }
                        tmplist.add(ridCar);
                        ridTurnCarList.put(ridTurnKey, tmplist);
                    }
                }
            }
            //List<AccidentFrameModel.CarTrack> stopCarList = trackList.stream().filter(o -> o.isStop()).collect(Collectors.toList());
            for (AccidentFrameModel.CarTrack carTrack : trackList) {
//                String time = "2024-02-02 09:03:10";
//                long ts = DateUtil.StringToMillis(time, "yyyy-MM-dd HH:mm:ss");
//                String dt = DateUtil.toDateTime(carTrack.getGlobalTimeStamp(), "yyyy-MM-dd HH:mm:ss");
//                if ("鲁A7D9P0".equals(carTrack.getPicLicense()) && carTrack.getGlobalTimeStamp() >= ts) {
//                     System.out.println();
//                 }
//                if (carTrack.getId()==7607768){
//                    System.out.println();
//                }
                String[] laneFunc = TurnEnum.getLaneTurnValue(carTrack.getRoadnet().getTurn());
                for (String func : laneFunc) {
                    String ridTurnKey = this.getKey(carTrack,func);
                    //获取冲突进口及转向
                    List<String> mapList = conflictMap.get(ridTurnKey);
                    if (Objects.nonNull(mapList)) {
                        for (String ckey : mapList) {
                            //检测冲突方向是否有车辆
                            if (ridTurnCarList.containsKey(ckey)) {
                                //冲突方向运行车流
                                boolean isMatched = ridTurnCarList.get(ckey).stream().anyMatch(o -> o.getSpeed() >= 0);
                                if (isMatched) {
                                    carTrack.setHasConflictCar(true);
                                }
                            }
                        }
                    }
                }
            }
            deadLockFrameModel.setTrackList(trackList);
        }
        collector.collect(deadLockFrameModel);

    }


    private String getKey(AccidentFrameModel.CarTrack carTrack, String func) {
        return carTrack.getRoadnet().getCrossId() +
                Constant.MARK +
                carTrack.getRoadnet().getRid() +
                Constant.MARK +
                carTrack.getRoadnet().getRidDir8()+
                Constant.MARK +  func
                ;
    }

    /**
     * 死锁检测
     */
    private boolean deadLockState(long nowTime, String crossId, ConfigModel configModel, List<AccidentFrameModel.CarTrack> conflictTrackList) {
        //记录当前路口是否处于死锁状态
        boolean isDeadLock = false;
        //死锁消散
        boolean isDissp = false;
        try {
            List<String> ridList = conflictTrackList.stream().map(o -> o.getRoadnet().getRid()).distinct().collect(Collectors.toList());

            int allSize = conflictTrackList.size();
            //停止车辆数
            List<AccidentFrameModel.CarTrack> stopList = conflictTrackList.stream().filter(o -> o.getSpeed() <= configModel.getAbnormalParkDeadlockStopCarSpeed()).collect(Collectors.toList());
            double stopCarRate = stopList.size() / Double.valueOf(allSize);
//            String ftime = DateUtil.toDateTime(nowTime,"yyyy-MM-dd HH:mm:ss.SSS");
//            if (ftime.equals("2022-02-17 18:43:18.804")){
//                System.out.println();
//            }
            if (ridList.size() > 1 && !stopList.isEmpty() && allSize > configModel.getAbnormalParkDeadlockMinCarNums() && stopCarRate > configModel.getAbnormalParkDeadlockStopCarRate()) {

                long duration = 0;
                CrossDeadLockStatusModel statusModel = mapState.get(crossId);
                if (Objects.isNull(statusModel)) {
                    statusModel = new CrossDeadLockStatusModel();
                    statusModel.setCrossId(crossId);
                    statusModel.setStartTime(nowTime);
                    statusModel.setEndTime(nowTime);
                    statusModel.setDuration(0);
                    statusModel.setTime(nowTime);

                    mapState.put(crossId, statusModel);
                } else {
                    long difTime = Math.abs(nowTime - statusModel.getEndTime());
                    duration = nowTime - statusModel.getStartTime();
                    statusModel.setDuration(duration);
                    statusModel.setEndTime(nowTime);
                    statusModel.setTime(nowTime);
                    mapState.put(crossId, statusModel);
                }
                if (duration > configModel.getAbnormalParkDeadlockStopCarDuration()) {
                    CrossDeadlockResultModel resultModel = new CrossDeadlockResultModel();
                    BeanUtils.copyProperties(statusModel, resultModel);
                    resultMapState.put(resultModel.getCrossId(), resultModel);
                    isDeadLock = true;
                    log.info("路口内出现死锁现象=》crossId={},startTime={},endTime={},持续时长：{}，停车数量：{},总车辆数：{}", resultModel.getCrossId(), DateUtil.toDateTime(resultModel.getStartTime(), "yyyy-MM-dd HH:mm:ss.SSS"), DateUtil.toDateTime(resultModel.getEndTime(), "yyyy-MM-dd HH:mm:ss.SSS"), duration, stopList.size(), allSize);

                    //List<AccidentFrameModel.CarTrack> runList = conflictTrackList.stream().filter(o -> o.getSpeed() > configModel.getAbnormalParkDeadlockStopCarSpeed()).collect(Collectors.toList());
                    //List<Integer> stoptrackidList = conflictTrackList.stream().map(AccidentFrameModel.CarTrack::getTrackId).distinct().collect(Collectors.toList());
                    //List<Integer> runtrackidList = runList.stream().map(AccidentFrameModel.CarTrack::getTrackId).distinct().collect(Collectors.toList());
                    //log.info("路口内出现死锁现象=》停车轨迹ID:{},运行车辆轨迹ID:{}",stoptrackidList.toString(),runtrackidList.toString());
                } else {
                    isDissp = dissp(crossId, configModel, nowTime);
                }
            } else {//消散清理
                isDissp = dissp(crossId, configModel, nowTime);
            }
            if (!isDeadLock && isDissp) {
                //死锁解除并且已经消散
                return false;
            }

        } catch (Exception e) {
            log.error("", e);
        }
        return isDeadLock;

    }

    //消散清理
    private boolean dissp(String crossId, ConfigModel configModel, long nowTime) throws Exception {
        CrossDeadlockResultModel disspModel = resultMapState.get(crossId);
        if (Objects.isNull(disspModel))
            return true;

        if (disspModel != null) {//消散清理
            long disspDuration = nowTime - disspModel.getEndTime();
            if (disspDuration > configModel.getAbnormalParkDeadlockDissipationDuration()) {
                mapState.remove(crossId);
                resultMapState.remove(crossId);
                return true;
                //log.info("路口死锁事件消散，清理缓存,当前帧时间{},死锁事件终止i时间{}",DateUtil.toDateTime(nowTime,"yyyy-MM-dd HH:mm:ss"),DateUtil.toDateTime(disspModel.getEndTime(),"yyyy-MM-dd HH:mm:ss"));
            }
        } else {
            CrossDeadLockStatusModel statusModel = mapState.get(crossId);
            if (statusModel != null) {
                long difTime = Math.abs(nowTime - statusModel.getEndTime());
                if (difTime > 300) {//跳帧或迟帧情
                    mapState.remove(crossId);
                    resultMapState.remove(crossId);
                }
            }
        }
        return false;
    }

    private double calculateConlictCarMinDist(List<AccidentFrameModel.CarTrack> carList, List<AccidentFrameModel.CarTrack> carConflictList) {
        double minDist = Double.MAX_VALUE;
        for (AccidentFrameModel.CarTrack carTrack1 : carList) {
            double x1 = carTrack1.getLongitude();
            double y1 = carTrack1.getLatitude();
            for (AccidentFrameModel.CarTrack carTrack2 : carConflictList) {
                double x2 = carTrack2.getLongitude();
                double y2 = carTrack2.getLatitude();
                double distance = PtInPolyUtil.getDistance(x1, y1, x2, y2);
                if (distance < minDist) {
                    minDist = distance;
                }
            }
        }
        return minDist;
    }


    /**
     * 获取所有冲突方向上的车辆
     *
     * @return
     */
    private List<AccidentFrameModel.CarTrack> getConflictCarList(AccidentFrameModel deadLockFrameModel) {
        ConfigModel configModel = deadLockFrameModel.getConfigModel();

        List<AccidentFrameModel.CarTrack> conflictTrackList = new ArrayList<>();

        List<AccidentFrameModel.CarTrack> trackList = deadLockFrameModel.getTrackList();
        ////key:cross-rid-ridDir-turn value:List<crossid-rid-ridDir-冲突转向>
        Map<String, List<String>> conflictMap = deadLockFrameModel.getConflictSlotMap();
        //按RID分组
        Map<String, List<AccidentFrameModel.CarTrack>> groupList = trackList.stream()
                .filter(o->o.getRoadnet().getInCrossFlag()==Constant.CROSSING)
                .collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));

        if (groupList.size() >= 2) {
            //按RID+转向分组
            Map<String, List<AccidentFrameModel.CarTrack>> ridTurnCarList = new HashMap<>();

            for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry : groupList.entrySet()) {
                String rid = entry.getKey();

                List<AccidentFrameModel.CarTrack> value = entry.getValue();

                for (AccidentFrameModel.CarTrack ridCar : value) {
//                    String laneFunc = ridCar.getRoadnet().getTurn().toString();
                    Integer ridDir = ridCar.getRoadnet().getRidDir8();
                    String laneFunc = TurnEnum.getLaneFunction(ridCar.getRoadnet().getTurn());
                    String[] sps = laneFunc.split(Constant.COMMA);
                    for (String func : sps) {
                        String ridTurnKey = null;
                        if (func.contains(Constant.LEFT_TURN_CODE)) {
                            ridTurnKey = deadLockFrameModel.getCrossId()+ Constant.MARK +rid + Constant.MARK+ ridDir+Constant.MARK  + Constant.LEFT_TURN_CODE;
                        } else if (func.contains(Constant.STRAIGHT_LINE_CODE)) {
                            ridTurnKey = deadLockFrameModel.getCrossId() + Constant.MARK +rid + Constant.MARK + ridDir+ Constant.STRAIGHT_LINE_CODE;
                        }
                        List<AccidentFrameModel.CarTrack> tmplist = ridTurnCarList.get(ridTurnKey);
                        if (Objects.isNull(tmplist)) {
                            tmplist = new ArrayList<>();
                        }
                        tmplist.add(ridCar);
                        ridTurnCarList.put(ridTurnKey, tmplist);
                    }
                }
            }

            for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry : ridTurnCarList.entrySet()) {
                String ridTurnKey = entry.getKey();
                //冲突进口及转向
                List<String> mapList = conflictMap.get(ridTurnKey);
                if (Objects.nonNull(mapList)) {
                    for (String cmap : mapList) {
                        String rid = cmap.split(Constant.MARK)[1];
                        String turn = null;
                        try {
                            turn = cmap.split(Constant.MARK)[3];
                        }catch (Exception e) {
                            e.printStackTrace();
                        }
                        String ckey = rid + Constant.MARK + turn;
                        //检测冲突方向是否有车辆
                        boolean isMatched = ridTurnCarList.containsKey(ckey);
                        if (isMatched) {
                            List<AccidentFrameModel.CarTrack> carList = entry.getValue();
                            //conflictTrackList.addAll(carList);
                            List<AccidentFrameModel.CarTrack> carConflictList = ridTurnCarList.get(ckey);
                            //当前方向停止的车辆数
                            long curDirStopSize =  carList.stream().filter(o -> o.getSpeed() < configModel.getAbnormalParkDeadlockStopCarSpeed()).count();
                            //冲突方向停车车辆数
                            long conflictDirStopSize =  carConflictList.stream().filter(o -> o.getSpeed() < configModel.getAbnormalParkDeadlockStopCarSpeed()).count();
                            //冲突之间最小距离
                            double minDist = this.calculateConlictCarMinDist(carList, carConflictList);

                            long conflictMinStopCarNums = configModel.getAbnormalParkDeadlockConflictMinStopCarNums();
                            long conflictCarMaxDist = configModel.getAbnormalParkDeadlockConflictCarMaxDistance();
                            //每个冲突方向至少停车数量，车辆间最小间距满足阈值
                            if (curDirStopSize >= conflictMinStopCarNums && conflictDirStopSize >= conflictMinStopCarNums && minDist < conflictCarMaxDist) {
                                conflictTrackList.addAll(carList);
                            }
                        }
                    }
                }
            }
        }
        return conflictTrackList;
    }

}
