package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

public class AccidentCheckTimeFlatMap implements FlatMapFunction<AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentCheckTimeFlatMap.class);

    @Override
    public void flatMap(AccidentFrameModel accidentFrameModel, Collector<AccidentFrameModel> collector) throws Exception {
        if (accidentFrameModel.getConfigModel().getAccidentIsOpen() == Constant.OPEN) {
            ConfigModel configModel = accidentFrameModel.getConfigModel();

            List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
            List<AccidentFrameModel.CarTrack> listAccident = new ArrayList<>();
            List<AccidentFrameModel.CarTrack> listStop = new ArrayList<>();
            for (AccidentFrameModel.CarTrack carTrack : trackList) {
                if (carTrack.isStop()) {
                    if (carTrack.isHasSameTurnCar() && carTrack.getSameTurnStopDuration() > configModel.getAbnormalParkHaveSameFlowStopDuration()) {
                        //有同行车流且无冲突车流时持续时长运行时、判断停车时间是否超过阈值 超过阈值则加入疑似事故车辆集合
                        listAccident.add(carTrack);
                        log.info("LOG_EVENT_ACCIDENT_CHECK_TIME >>> 无灯态 有同行车流运行时、异常停车超过阈值 " + carTrack.getRoadnet().getCrossId() +
                                "-" + carTrack.getId() + "-" + carTrack.getPicLicense() +
                                " stopDuration: " + carTrack.getStopDuration() +
                                " stopTime: " + DateUtil.toDateTime(carTrack.getStopTime(), "yyyy-MM-dd HH:mm:ss.SSS") +
                                " 有同行车流车辆停车持续时长: " + carTrack.getSameTurnStopDuration() +
                                " 是否有冲突方向车辆：" + carTrack.isHasConflictCar()
                        );
                    } else if (carTrack.getStopDuration() > configModel.getAbnormalParkNoSameFlowStopDuration()) {
                        //无相同方向
                       // listAccident.add(carTrack);
                    }
                    //加入停驶车辆集合
                    listStop.add(carTrack);
                }
            }
            //路口内停驶车辆组合事故
            List<AccidentFrameModel.CarTrack> carTracks = checkDis(listAccident, listStop, accidentFrameModel.getConfigModel());
            int size = carTracks.size();
            if (size > 0) {
                accidentFrameModel.setTrackList(carTracks);
                if (size == 1) {
                    //单车事故
                    accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
                } else if (size == 2) {
                    //双车事故
                    accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_TWO_YS.getType());
                } else {
                    //多车事故
                    accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_MORE_YS.getType());
                }
                collector.collect(accidentFrameModel);
            } else {
                //单车事故
                accidentFrameModel.setTrackList(listAccident);
                accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
                collector.collect(accidentFrameModel);
            }
        }
    }


    /**
     * 计算车辆之间的距离是否达到要求
     *
     * @param listAccident
     * @param listStop
     * @return
     */
    private List<AccidentFrameModel.CarTrack> checkDis(List<AccidentFrameModel.CarTrack> listAccident, List<AccidentFrameModel.CarTrack> listStop, ConfigModel configModel) {
        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
        Set<String> set = new HashSet<>();
        for (AccidentFrameModel.CarTrack accident : listAccident) {
            int trackIdA = accident.getId();
            long stopTimeA = accident.getStopTime();

            double distanceMin = Double.MAX_VALUE;
            for (AccidentFrameModel.CarTrack stop : listStop) {
                int trackIdS = stop.getId();
                long stopTimeS = stop.getStopTime();
                //排除当前车辆
                if (trackIdA != trackIdS) {
                    double distance = PtInPolyUtil.getDistance(accident.getLongitude(), accident.getLatitude(), stop.getLongitude(), stop.getLatitude());
                    if (distance < 10.0) {
                        //小于10米
                        long stopDu = stop.getStopDuration();
                        //疑似事故车辆与停车车辆之间的停车时间差
                        long tx = Math.abs(stopTimeA - stopTimeS);

                        if (stopDu >= configModel.getAbnormalParkStopBaseDuration()) {
                            if (set.add(trackIdA + "-" + trackIdS) && set.add(trackIdS + "-" + trackIdA)) {
                                //如果距离小于上一车辆
                                if (distanceMin > distance) {
                                    distanceMin = distance;
                                    accident.setAccidentTrackId(stop.getId());
                                    accident.setAccidentDis(distance);

                                    stop.setAccidentTrackId(accident.getId());
                                    stop.setAccidentDis(distance);
                                    list.add(accident);
                                    list.add(stop);
//                                    System.err.println(trackIdA + "-"+accident.getPicLicense() + " >> " + trackIdS +"-"+ stop.getPicLicense() +
//                                            "  dis:" + distance + "  stA: " + stopTimeA + "  stS: " + stop.getStopTime() +
//                                            "  tx:" + tx + "  accidentDu:" + accident.getStopDuration() +
//                                            "  stopDu:" + stop.getStopDuration());
                                }
                            }
                        }
                    }
                }
            }
        }
        if (!list.isEmpty()) {
            list = list.stream().sorted(Comparator.comparing(o->o.getGlobalTimeStamp())).collect(Collectors.toList());
        }
        return list;
    }




    /**
     * 排序 -- 时间戳 -- 升序
     *
     * @param list
     * @return
     */
    public List<AccidentFrameModel.CarTrack> sort(List<AccidentFrameModel.CarTrack> list) {
        try {
            Collections.sort(list, new Comparator<AccidentFrameModel.CarTrack>() {
                @Override
                public int compare(AccidentFrameModel.CarTrack o1, AccidentFrameModel.CarTrack o2) {
                    long o1x = o1.getId();
                    long o2x = o2.getId();
                    if (o1x > o2x) {
                        return 1;
                    }
                    if (o1x == o2x) {
                        return 1;
                    }
                    return -1;
                }
            });
            return list;
        } catch (Exception e) {
            return list;
        }
    }

    /**
     * 计算行车角度是否同向
     *
     * @param angleA
     * @param angleB
     * @return
     */
    private static boolean isSyntropy(double angleA, double angleB) {
        double angleX = Math.abs(angleA - angleB);
        if (angleX > 180.0) {
            angleX = 360.0 - angleX;
        }
        if (angleX <= 60.0) {
            return true;
        } else {
            return false;
        }
    }


}
