package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentCongestionModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AccidentCongestionKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentCongestionKeyedProcess.class);

    private MapState<String, AccidentCongestionModel> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccidentCongestion_f0c1", String.class, AccidentCongestionModel.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {
        ConfigModel configModel = accidentFrameModel.getConfigModel();
        long timeNow = accidentFrameModel.getTime();
        //按rid分组当前帧的车辆
        Map<String, List<AccidentFrameModel.CarTrack>> ridMap = keyByRidMap(accidentFrameModel);
        for (String ridKey : ridMap.keySet()) {
            Map<String, Integer> congestionInfo = accidentFrameModel.getCongestionInfo();
            if (congestionInfo != null && congestionInfo.get(ridKey) != null) {
                Integer lane_num = congestionInfo.get(ridKey);
                List<AccidentFrameModel.CarTrack> list = ridMap.get(ridKey);
                AccidentCongestionModel congestionAgo = mapState.get(ridKey);
                if (congestionAgo != null) {
                    //已经拥堵的情况下计算当前帧是否持续拥堵
                    boolean congestion = isCongestion(list, configModel.getAbnormalParkCongestDissipationCarNum(), configModel.getAbnormalParkCongestSpeed());
                    //计算时间差
                    long timeX = timeNow - congestionAgo.getTime();
                    //获取上一帧累计的拥堵持续时间
                    long duration = congestionAgo.getDuration();
                    if (timeX > 0L && timeX < 3000L) {
                        duration += timeX;
                    }
                    AccidentCongestionModel model = new AccidentCongestionModel();
                    model.setKey(ridKey);
                    model.setStartTime(congestionAgo.getStartTime());
                    model.setTime(accidentFrameModel.getTime());
                    model.setDuration(duration);
                    if (congestion) {
                        //若当前帧拥堵则将当前帧时间缓存更新至状态  更新了拥堵的持续时间
                        mapState.put(ridKey, model);
                    } else {
                        //若当前计算到不拥堵了 则延迟3s清除拥堵缓存标识
                        if (timeX > configModel.getAbnormalParkCongestDissipationDuration()) {
                            mapState.remove(ridKey);
                            //判断路口当前停驶车辆是否满足畅通逻辑
//                            if (!isCrossCongestion(configModel, accidentFrameModel.getTrackList(), configModel.getAbnormalParkCrossCongestRate(), configModel.getAbnormalParkCongestSpeed())) {
//                                mapState.remove(ridKey);
//                                log.info("LOG_EVENT_ACCIDENT_CONGESTION >>> 事故检测 拥堵消散: " + ridKey + "  du:" + duration +
//                                        " " + DateFormatUtils.format(model.getStartTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
//                                        " | " + DateFormatUtils.format(timeNow,DateUtil.YYYY_MM_DD_HH_MM_SS) +
//                                        " - " + DateFormatUtils.format(congestionAgo.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
//                                        " timeX:" + timeX);
//                            }
                        } else {
                            //若当前帧拥堵则将当前帧时间缓存至状态  更新了拥堵的持续时间
                            mapState.put(ridKey, model);
                        }
                    }
                } else {
                    //计算当前帧是否拥堵
                    boolean congestion = isCongestion(list, lane_num, configModel.getAbnormalParkCongestSpeed());
                    if (congestion) {
                        AccidentCongestionModel model = new AccidentCongestionModel();
                        model.setKey(ridKey);
                        model.setStartTime(accidentFrameModel.getTime());
                        model.setTime(accidentFrameModel.getTime());
                        model.setDuration(0L);
                        //若当前帧拥堵则将当前帧时间缓存至状态
                        mapState.put(ridKey, model);
                    }
                }
            }
        }
        List<AccidentFrameModel.CarTrack> listResult = new ArrayList<>();
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        boolean isOk = true;
        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            if (!isOk) {
                //若拥堵超过3秒则不输出当前路口的所有轨迹数据
                continue;
            }
            String congestionOutRid = carTrack.getCongestionOutRid();
            //获取当前rid是否发生路口溢出 若发生拥堵则过滤当前路口对应的数据
            AccidentCongestionModel congestionModel = mapState.get(congestionOutRid);
            if (congestionModel != null) {
                //当前rid路口溢出持续3秒后则不输出当前路口的所有轨迹数据
                if (congestionModel.getDuration() > configModel.getAbnormalParkCongestDuration()) {
                    isOk = false;
//                    log.info("LOG_EVENT_ACCIDENT_CONGESTION >>> 事故检测 发现路口溢出: " + congestionOutRid +
//                            " " + DateFormatUtils.format(congestionModel.getStartTime(), DateUtil.YYYY_MM_DD_HH_MM_SS) +
//                            " - " + DateFormatUtils.format(timeNow, DateUtil.YYYY_MM_DD_HH_MM_SS));
                } else {
                    //当前rid路口溢出持续时间不足8秒则输出轨迹数据
                    listResult.add(carTrack);
                }
            } else {
                //没有路口溢出时则输出当前路口所有轨迹数据
                listResult.add(carTrack);
            }
        }
//        if (isOk && !listResult.isEmpty()  && !accidentFrameModel.isDeadLock()) {
//            accidentFrameModel.setTrackList(listResult);
//            collector.collect(accidentFrameModel);
//        }
        accidentFrameModel.setTrackList(listResult);
        accidentFrameModel.setOverFlow(isOk == false ? true : false);
        accidentFrameModel.setDeadLock(accidentFrameModel.isDeadLock());
        collector.collect(accidentFrameModel);
    }

    /**
     * 计算当前帧是否拥堵
     *
     * @param list     当前帧车辆集合
     * @param checkNum 拥堵车辆判定数量阈值
     * @return
     */
    private boolean isCongestion(List<AccidentFrameModel.CarTrack> list, long checkNum, double checkSpeed) {
        if (checkNum < 2) {
            checkNum = 2;
        }
        long congestionNum = 0;
        for (AccidentFrameModel.CarTrack carTrack : list) {
            double waitLaneDistance = 0;// carTrack.getWaitLaneDistance() + 1.0;
            double areaDist = carTrack.getAreaDist();
            //计算车辆是否驶出待转区 若驶出待转区则 areaDist 大于 waitLaneDistance
            if (areaDist > waitLaneDistance) {
                //计算旅行时间超过10s且车速低于10km/h的车辆
                if (carTrack.getSpeed() < checkSpeed) {
                    congestionNum++;
                }
            }
        }
        //行驶缓慢的车辆数超过传值的阈值则认为路口发生路口溢出
        if (congestionNum > checkNum) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 计算当前帧路口中是否拥堵
     *
     * @param list       当前帧车辆集合
     * @param checkRatio 拥堵车辆判定比例阈值
     * @return
     */
    private boolean isCrossCongestion(ConfigModel configModel, List<AccidentFrameModel.CarTrack> list, double checkRatio, double checkSpeed) {
        int congestionNum = 0;
        int countCrossing = 0;
        for (AccidentFrameModel.CarTrack carTrack : list) {
            double waitLaneDistance =  1;// carTrack.getWaitLaneDistance() + 1.0;
            double areaDist = carTrack.getAreaDist();
            //计算车辆是否驶出待转区 若驶出待转区则 areaDist 大于 waitLaneDistance
            if (areaDist > waitLaneDistance) {
                if (carTrack.getRoadnet().getInCrossFlag() == Constant.CROSSING) {
                    //计算旅行时间超过10s且车速低于10km/h的车辆
                    if (carTrack.getSpeed() < checkSpeed) {
                        congestionNum++;
                    }
                    countCrossing++;
                }

            }
        }
        //路口内 所有车辆总数
        if (countCrossing < configModel.getAbnormalParkUseCrossCongestCarNum()) {
            //路口内 缓慢车辆总数
            if (congestionNum < configModel.getAbnormalParkCrossSlowCarNum()) {
                return false;
            } else {
                return true;
            }
        } else {
            double ratio = congestionNum * 1.0 / countCrossing;
            //行驶缓慢的车辆数超过传值的阈值则认为路口发生拥堵
            log.info("====>>>> 比例: " + ratio + "  " + congestionNum + "  " + countCrossing);
            if (ratio > checkRatio || congestionNum > 4) {
                return true;
            } else {
                return false;
            }
        }
    }


    /**
     * 按照rid分组计算当前帧的所有车辆
     *
     * @param accidentFrameModel
     * @return
     */
    private Map<String, List<AccidentFrameModel.CarTrack>> keyByRidMap(AccidentFrameModel accidentFrameModel) {
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        Map<String, List<AccidentFrameModel.CarTrack>> listMap = new HashMap<>();
        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            String congestionOutRid = carTrack.getCongestionOutRid();
            if (StringUtils.isNotBlank(congestionOutRid)) {
                List<AccidentFrameModel.CarTrack> carTrackRidList = listMap.get(congestionOutRid);
                if (carTrackRidList == null) {
                    carTrackRidList = new ArrayList<>();
                }
                carTrackRidList.add(carTrack);
                listMap.put(congestionOutRid, carTrackRidList);
            }
        }
        return listMap;
    }

}
