package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * 事故连续上报，通过session窗口等待，1分钟内未收到数据，认为事故已结束，此时输出事故信息
 */
@Slf4j
public class AccidentDistinguishProcess extends ProcessWindowFunction<AccidentFrameModel, AccidentFrameModel, String, TimeWindow> {

    @Override
    public void process(String s, Context ctx, Iterable<AccidentFrameModel> iterable, Collector<AccidentFrameModel> out) throws Exception {

        List<AccidentFrameModel> list = StreamSupport.stream(iterable.spliterator(), false).collect(Collectors.toList());

        list = list.stream().sorted(Comparator.comparing(o -> o.getTime())).collect(Collectors.toList());

        List<AccidentFrameModel.CarTrack> accidentOutList = new ArrayList<>();

        AccidentFrameModel outModel = new AccidentFrameModel();
        //outModel.setSysTime(outModel.getSysTime());
        outModel.setSysTimeStr(DateUtil.toDateTime(outModel.getTime(),"yyyy-MM-dd HH:mm:ss.SSSS"));

        if (!list.isEmpty()) {
            outModel = list.get(list.size()-1);

            List<AccidentFrameModel.CarTrack> accidentCarList = new ArrayList<>();
            for (AccidentFrameModel model : list) {
                List<AccidentFrameModel.CarTrack> trackList = model.getTrackList();
                accidentCarList.addAll(trackList);
            }
            Map<Integer, List<AccidentFrameModel.CarTrack>> groupById = accidentCarList.stream().collect(Collectors.groupingBy(o -> o.getId()));
            for (Map.Entry<Integer, List<AccidentFrameModel.CarTrack>> entry : groupById.entrySet()) {
                List<AccidentFrameModel.CarTrack> value = entry.getValue();
                value = value.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());

                AccidentFrameModel.CarTrack first = value.get(0);

                int size = value.size();
                if (size > 1) {
                    //多次记录合并成一条输出
                    AccidentFrameModel.CarTrack last = value.get(size - 1);
                    last.setStopTime(first.getStopTime());
                    last.setStop(first.isStop());
                    last.setStopDuration(last.getGlobalTimeStamp()-first.getStopTime());
                    first = last;
                }
                accidentOutList.add(first);
            }
        }
        outModel.setTrackList(accidentOutList);
        out.collect(outModel);

    }
}
