package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 *
 */
public class AccidentDuplicateRemoveKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentDuplicateRemoveKeyedProcess.class);

    private MapState<String, AccidentFrameModel.CarTrack> mapState;
    private ValueState<AccidentFrameModel> valueState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("accidentQCMapState_01011101", String.class, AccidentFrameModel.CarTrack.class));
        valueState = getRuntimeContext().getState(new ValueStateDescriptor<>("abnormalCheckMinDurationValueState_02i131", AccidentFrameModel.class));

    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {
        String accidentType = accidentFrameModel.getAccidentType();
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();

        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            int trackId = carTrack.getId();
            String crossId = carTrack.getRoadnet().getCrossId();
            String key = crossId + Constant.MARK + trackId + Constant.MARK + accidentType;
            mapState.put(key, carTrack);
            //同一个ID先检测为单车事故，后又检测为多车事故，删除单车类型保留多车类型
            if (!EventAbnormalEnum.ACCIDENT_ONE_YS.getType().equals(accidentType)) {
                String delkey = crossId + Constant.MARK + trackId + Constant.MARK + EventAbnormalEnum.ACCIDENT_ONE_YS.getType();
                AccidentFrameModel.CarTrack valueCheck = mapState.get(delkey);
                if (valueCheck != null) {
                    mapState.remove(delkey);

                    carTrack.setStop(valueCheck.isStop());
                    carTrack.setStopTime(valueCheck.getStopTime());
                    carTrack.setStopDuration(carTrack.getGlobalTimeStamp()-carTrack.getStopTime());

                    mapState.put(key, carTrack);

                }
            }
        }
//        List<AccidentFrameModel.CarTrack> list =  StreamSupport.stream(mapState.values().spliterator(), false).collect(Collectors.toList());
//        accidentFrameModel.getTrackList().clear();
//        accidentFrameModel.setTrackList(list);
//        AccidentFrameModel agoFrameModel = valueState.value();
//        if (agoFrameModel == null) {
//            valueState.update(accidentFrameModel);
//            collector.collect(accidentFrameModel);
//            context.timerService().registerProcessingTimeTimer(System.currentTimeMillis() + 1800000L);
//        }
        accidentFrameModel.setSysTimeStr(DateUtil.toDateTime(System.currentTimeMillis(),"yyyy-MM-dd HH:mm:ss.SSS"));
        //输出结果数据
        collector.collect(accidentFrameModel);
        //清除状态缓存
        clearStateCache(accidentFrameModel);

    }

    @Override
    public void onTimer(long timestamp, KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel>.OnTimerContext ctx, Collector<AccidentFrameModel> out) throws Exception {
        valueState.clear();
    }


    /**
     * 定期清除状态中的数据
     *
     * @param accidentFrameModel
     * @throws Exception
     */
    private void clearStateCache(AccidentFrameModel accidentFrameModel) throws Exception {
        Iterator<AccidentFrameModel.CarTrack> iterator = mapState.values().iterator();
        List<AccidentFrameModel.CarTrack> clear = new ArrayList<>();
        while (iterator.hasNext()) {
            AccidentFrameModel.CarTrack next = iterator.next();
            clear.add(next);
        }
        //遍历并清除事故
        for (AccidentFrameModel.CarTrack carTrack1 : clear) {
            long x = Math.abs(carTrack1.getGlobalTimeStamp() - accidentFrameModel.getTime());
            if (x > 3600000) {
                mapState.remove(carTrack1.getRoadnet().getCrossId() + Constant.MARK + carTrack1.getId() + Constant.MARK + EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
                mapState.remove(carTrack1.getRoadnet().getCrossId() + Constant.MARK + carTrack1.getId() + Constant.MARK + EventAbnormalEnum.ACCIDENT_MORE_YS.getType());
                mapState.remove(carTrack1.getRoadnet().getCrossId() + Constant.MARK + carTrack1.getId() + Constant.MARK + EventAbnormalEnum.ACCIDENT_TWO_YS.getType());
            }
        }
    }
}
