package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.constant.EventLevelEnum;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.accident.AccidentShowModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 */
public class AccidentEncapsulationFlatMap implements FlatMapFunction<AccidentFrameModel, AccidentShowModel> {

    @Override
    public void flatMap(AccidentFrameModel accidentFrameModel, Collector<AccidentShowModel> collector) throws Exception {
        String accidentType = accidentFrameModel.getAccidentType();
        AccidentShowModel accidentShowModel = null;
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        if (trackList != null && !trackList.isEmpty()) {
            if (EventAbnormalEnum.ACCIDENT_ONE_YS.getType().equals(accidentType)) {
                //单车事故
                accidentShowModel = oneVehicleAccidentShow(accidentFrameModel);
            } else {
                //多车事故
                accidentShowModel = someVehicleAccidentShow(accidentFrameModel);
            }
            if (accidentShowModel != null) {
                if (StringUtils.isBlank(accidentShowModel.getPlateNumber())) {
                    accidentShowModel.setPlateNumber(Constant.TARGET_VEHICLE);
                }
                accidentShowModel.setStartTimeStr(DateUtil.toDateTime(accidentShowModel.getStartTime(),DateUtil.YYYY_MM_DD_HH_MM_SS));
                accidentShowModel.setEndTimeStr(DateUtil.toDateTime(accidentShowModel.getEndTime(),DateUtil.YYYY_MM_DD_HH_MM_SS));
                 accidentShowModel.setSysTimeStart(accidentFrameModel.getSysTimeStr());
                accidentShowModel.setSysTimeEnd(DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));
                collector.collect(accidentShowModel);
            }
        }
    }

    //多车事故
    private AccidentShowModel someVehicleAccidentShow(AccidentFrameModel accidentFrameModel) {
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        EventAbnormalEnum eventAbnormalEnum = null;
        if (trackList.size()==2) {
            eventAbnormalEnum = EventAbnormalEnum.ACCIDENT_TWO_YS;
        } else if (trackList.size() > 2) {
            eventAbnormalEnum = EventAbnormalEnum.ACCIDENT_MORE_YS;
        }
        if (trackList.size() > 1) {
            String runingDir = Constant.EMPTY_STRING;
            AccidentFrameModel.CarTrack carA = trackList.get(0);
            AccidentFrameModel.CarTrack carB = trackList.get(1);
            String ridA = carA.getRoadnet().getRid();
            String ridB = carB.getRoadnet().getRid();
            if (StringUtils.isNotBlank(ridA) && StringUtils.isNotBlank(ridB)) {
                if (ridA.equals(ridB)) {
                    //相同方向进入路口
                    runingDir = Constant.CAUSE_OF_TROUBLE_A;
                } else {
                    //不同方向进入路口
                    runingDir = Constant.CAUSE_OF_TROUBLE_C;
                }
            } else {
                //行驶路线冲突导致事故
                runingDir = Constant.CAUSE_OF_TROUBLE_B;
            }
            //计算事故碰撞类型
            String collisionType = collisionType(carA, carB);
            //计算主责车辆
            List<AccidentFrameModel.CarTrack> responsibilityList = getResponsibility(collisionType, carA, carB);
            AccidentShowModel showModel = null;
            if (!responsibilityList.isEmpty()) {
                //正常定则
                showModel =  getAccidentResult(collisionType, Constant.RESPONSIBILITY_A, runingDir, responsibilityList.get(0), responsibilityList.get(1),eventAbnormalEnum);
            } else {
                //无法定责
                showModel =  getAccidentResult(collisionType, Constant.RESPONSIBILITY_B, runingDir, carA, carB,eventAbnormalEnum);
            }
            showModel.setOtherCarList(trackList.stream().filter(o-> !Objects.equals(carA.getId(),o.getId())).collect(Collectors.toList()));
            showModel.setAbnormalStop(carA.isAbnormalStop());
            return  showModel;
        }
        return null;
    }

    private AccidentShowModel getAccidentResult(String collisionType,
                                                String responsibility,
                                                String runingDir,
                                                AccidentFrameModel.CarTrack carTrack0,
                                                AccidentFrameModel.CarTrack carTrack1,
                                                EventAbnormalEnum eventAbnormalEnum) {
        AccidentShowModel m = new AccidentShowModel();
        m.setEventType(EventAbnormalEnum.ACCIDENT_YS.getType());
        m.setTypeCode(eventAbnormalEnum.getType());
        m.setEventClass(eventAbnormalEnum.getDesc());
        m.setCrossId(carTrack0.getRoadnet().getCrossId());
        m.setTrackIds(carTrack0.getId() + Constant.COMMA + carTrack1.getId());
        m.setTrackId(carTrack0.getId());
        m.setPlateNumber(carTrack0.getPicLicense());
        m.setVehicleType(String.valueOf(carTrack0.getOriginalType()));
        m.setDuration(carTrack0.getStopDuration());
        m.setStartTime(carTrack0.getStopTime());
        m.setEndTime(carTrack0.getGlobalTimeStamp());

        m.setLocation(new double[] {carTrack0.getLongitude(), carTrack0.getLatitude()});
        m.setTimeSlot(DateUtil.currentTimeSlot(carTrack0.getStopTime()));
        m.setPlace(1);
        m.setVehicleNum(2);
        m.setHasNonMotorVehicle(0);
        m.setHasPedestrian(0);
        m.setInfluenceDegree(2);
        m.setX(carTrack0.getLongitude());
        m.setY(carTrack0.getLatitude());
        m.setCategory(Integer.valueOf(EventAbnormalEnum.ACCIDENT_YS.getType()));
        m.setPictureTime1(carTrack0.getStopTime() - 10000);
        m.setPictureTime2(carTrack0.getStopTime());
        m.setPictureTime3(carTrack0.getStopTime() + 10000);
        m.setAccidentLocation("未识别");
        m.setLaneId(carTrack0.getRoadnet().getLaneId());
        m.setHasConflictCar(carTrack0.isHasConflictCar());
        m.setHasSameTurnCar(carTrack0.isHasSameTurnCar());
        m.setSameTurnStopDuration(carTrack0.getSameTurnStopDuration());
        if (carTrack0.isGreenStop()) {
            m.setGreenStopDuration(carTrack0.getGreenStopDuration());
            m.setGreenStop(carTrack0.isGreenStop());
            m.setGreenStopStartTime(DateUtil.toDateTime(carTrack0.getGreenStopStartTime(), "yyyy-MM-dd HH:mm:ss.SSS"));
        }
        //中文 - 事故分析
        m.setAuxiliaryInfo(m.getPlateNumber() + Constant.SPACE + collisionType + Constant.SPACE + carTrack1.getPicLicense() + Constant.SEMICOLON + Constant.SPACE + m.getPlateNumber() + Constant.POSITION);
        //英文 - 事故分析
        m.setAuxiliaryInfoEn(m.getPlateNumber() + Constant.SPACE + Constant.mapAccident.get(collisionType) + Constant.SPACE + carTrack1.getPicLicense() + Constant.SPACE + Constant.mapAccident.get(Constant.POSITION));
        //中文 - 定责建议
        m.setResponsibility(m.getPlateNumber() + responsibility);//Constant.RESPONSIBILITY_A
        //英文 - 定责建议
        m.setResponsibilityEn(m.getPlateNumber() + Constant.mapAccident.get(responsibility));//Constant.EN_RESPONSIBILITY_A
        if (eventAbnormalEnum.equals(EventAbnormalEnum.ACCIDENT_MORE_YS)) {
            //中文 - 事故描述
            m.setAccidentDescribe(Constant.MULTI_CAR_ACCIDENT + m.getPlateNumber() + Constant.AND + carTrack1.getPicLicense() + Constant.SPACE + runingDir);
            //英文 - 事故描述
            m.setAccidentDescribeEn(Constant.EN_MULTI_CAR_ACCIDENT + m.getPlateNumber() + Constant.EN_AND + carTrack1.getPicLicense() + Constant.SPACE + Constant.mapAccident.get(runingDir));
        } else if (eventAbnormalEnum.equals(EventAbnormalEnum.ACCIDENT_TWO_YS)) {
            //中文 - 事故描述
            m.setAccidentDescribe(Constant.TWO_CAR_ACCIDENT + m.getPlateNumber() + Constant.AND + carTrack1.getPicLicense() + Constant.SPACE + runingDir);
            //英文 - 事故描述
            m.setAccidentDescribeEn(Constant.EN_TWO_CAR_ACCIDENT + m.getPlateNumber() + Constant.EN_AND + carTrack1.getPicLicense() + Constant.SPACE + Constant.mapAccident.get(runingDir));
        }
        m.setId();

        return m;
    }

    /**
     * 计算事故碰撞类型
     */
    private String collisionType (AccidentFrameModel.CarTrack carA, AccidentFrameModel.CarTrack carB) {
        String entryLaneA = carA.getRoadnet().getLaneId();
        String entryLaneB = carB.getRoadnet().getLaneId();
        double driveAngleA = carA.getCourseAngle();
        double driveAngleB = carB.getCourseAngle();
        String ridA = carA.getRoadnet().getRid();
        String ridB = carB.getRoadnet().getRid();
        if (ridA.equals(ridB)){
            //是否追尾 车道不为空 且 进入路口行驶的车道为同一车道则为追尾
            if (StringUtils.isNotBlank(entryLaneA) && entryLaneA.equals(entryLaneB)) {
                //追尾
                return Constant.REAR_END_COLLISION;
            } else {
                //刮蹭
                return Constant.SCRATCH;
            }
        } else {
            //计算辆车角度是否接近90度 是则为侧碰 否则为碰撞
            double angleX = Math.abs(driveAngleA - driveAngleB);
            if (angleX > 180.0) {
                angleX = 360.0 - angleX;
            }
            double abs = Math.abs(angleX - 90);
            if (abs < 30.0) {
                //侧碰
                return Constant.SIDE_IMPACT;
            } else {
                //碰撞
                return Constant.SUSPECTED_COLLISION;
            }
        }
    }

    /**
     * 获取责任人  返回集合  第0个元素为主责车  第1个元素为另一辆车  空集合则无法定责
     */
    private List<AccidentFrameModel.CarTrack> getResponsibility(String collisionType, AccidentFrameModel.CarTrack carA, AccidentFrameModel.CarTrack carB) {
        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
        if (Constant.REAR_END_COLLISION.equals(collisionType) || Constant.SCRATCH.equals(collisionType)) {
            double areaDistA = carA.getAreaDist();
            double areaDistB = carB.getAreaDist();
            if (areaDistA > areaDistB) {
                //B车为主责车
                list.add(carB);
                list.add(carA);
            } else {
                //A车为主责车
                list.add(carA);
                list.add(carB);
            }
            return list;
        } else {
            double angleAB = PtInPolyUtil.getAngle(carA.getLongitude(), carA.getLatitude(), carB.getLongitude(), carB.getLatitude());
            double angleBA = PtInPolyUtil.getAngle(carB.getLongitude(), carB.getLatitude(), carA.getLongitude(), carA.getLatitude());
            boolean isSyntropyBA = isSyntropy(angleBA, carB.getCourseAngle());
            boolean isSyntropyAB = isSyntropy(angleAB, carA.getCourseAngle());
            if (isSyntropyBA && !isSyntropyAB) {
                list.add(carB);
                list.add(carA);
            }
            if (isSyntropyAB && !isSyntropyBA) {
                list.add(carA);
                list.add(carB);
            }
            return list;
        }

    }

    /**
     * 计算行车角度是否同向
     *
     * @param angleA
     * @param angleB
     * @return
     */
    private static boolean isSyntropy(double angleA, double angleB) {
        double angleX = Math.abs(angleA - angleB);
        if (angleX > 180.0) {
            angleX = 360.0 - angleX;
        }
        if (angleX <= 30.0) {
            return true;
        } else {
            return false;
        }
    }


    //单车事故
    private AccidentShowModel oneVehicleAccidentShow(AccidentFrameModel accidentFrameModel) {
        AccidentFrameModel.CarTrack model = accidentFrameModel.getTrackList().get(0);
        AccidentShowModel m = new AccidentShowModel();
        m.setAbnormalStop(m.isAbnormalStop());

        m.setEventType(EventAbnormalEnum.ACCIDENT_YS.getType());
        m.setTypeCode(EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
        m.setEventClass(EventAbnormalEnum.ACCIDENT_ONE_YS.getDesc());

        m.setStartTime(model.getStopTime());
        m.setEndTime(model.getGlobalTimeStamp());

        m.setCrossId(model.getRoadnet().getCrossId());
        m.setTrackId(model.getId());
        m.setTrackIds(model.getId() + Constant.EMPTY_STRING);
        m.setLaneId(model.getRoadnet().getLaneId());
        m.setMatchLane(model.getRoadnet().getLaneId());
        m.setEntryLane(model.getRoadnet().getLaneId());
        m.setRid(model.getRoadnet().getRid());
        m.setPlateNumber(model.getPicLicense());
        m.setVehicleType(String.valueOf(model.getOriginalType()));

        m.setHasConflictCar(model.isHasConflictCar());
        m.setHasSameTurnCar(model.isHasSameTurnCar());
        m.setSameTurnStopDuration(model.getSameTurnStopDuration());
        if (model.isGreenStop()) {
            m.setGreenStopDuration(model.getGreenStopDuration());
            m.setGreenStop(model.isGreenStop());
            m.setGreenStopStartTime(DateUtil.toDateTime(model.getGreenStopStartTime(), "yyyy-MM-dd HH:mm:ss.SSS"));
        }
        //事故描述
        m.setAccidentDescribe(Constant.SINGLE_CAR_ACCIDENT + model.getPicLicense());
        //定责信息
        m.setResponsibility(model.getPicLicense() + Constant.RESPONSIBILITY_A);
        //辅助信息
        m.setAuxiliaryInfo(Constant.TRAJECTORY_ANALYSIS_A + (model.getPicLicense()==null?model.getId():model.getPicLicense()) + Constant.TRAJECTORY_ANALYSIS_B);
        //英语版本 - 事故描述
        m.setAccidentDescribeEn(Constant.EN_SINGLE_CAR_ACCIDENT + model.getPicLicense());
        //英语版本 - 定责信息
        m.setResponsibilityEn(model.getPicLicense() + Constant.EN_RESPONSIBILITY_A);
        //英语版本 - 辅助信息
        m.setAuxiliaryInfoEn(Constant.EN_TRAJECTORY_ANALYSIS_A + model.getPicLicense() + Constant.EN_TRAJECTORY_ANALYSIS_B);
        m.setTimeSlot(DateUtil.currentTimeSlot(model.getStopTime()));
        m.setPlace(1);
        m.setVehicleNum(1);
        m.setHasNonMotorVehicle(0);
        m.setHasPedestrian(0);
        m.setInfluenceDegree(2);
        m.setX(model.getLongitude());
        m.setY(model.getLatitude());
        m.setCategory(Integer.valueOf(EventAbnormalEnum.ACCIDENT_YS.getType()));

        m.setDuration(model.getStopDuration());
        m.setId();
        return m;
    }

}
