package com.wanji.indicators.event.cross.accident.funcnew;

import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.wanji.indicators.cache.PedestrianDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.event.WaitingAreaDo;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @auther fengyi
 */
@Slf4j
public class AccidentFlatMap implements FlatMapFunction<CrossFrameModel, AccidentFrameModel> {


    @Override
    public void flatMap(CrossFrameModel frameModel, Collector<AccidentFrameModel> collector) throws Exception {

        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
        //灯态
        List<CrossRidTurnLampStatusModel> lampStatusList = frameModel.getRidTurnLampList();
        try {
            List<CarTrackModel> trackList = frameModel.getTrackList();

            //筛选行人、非机动车目标
            List<CarTrackModel> nonMotorList = new ArrayList<>();

            for (CarTrackModel carTrack : trackList) {
                CarTrackModel.RoadNet roadNet = carTrack.getRoadnet();

                String rid = roadNet.getRid();
                Integer ridDir = roadNet.getRidDir8();
                Integer laneFunction = roadNet.getTurn();
                String[] turns = TurnEnum.getLaneTurnValue(laneFunction);
                List<String> turnList = Arrays.asList(turns);
                //进口道本车道灯态
                List<CrossRidTurnLampStatusModel> dirLampList = lampStatusList.stream().filter(o -> o.getDir().equals(ridDir) && turnList.contains(o.getTurn())).collect(Collectors.toList());
                carTrack.setRidTurnLampStatusModel(dirLampList);

                boolean isMotor = VehicleTypeEnum.isMotorVehicles(carTrack.getOriginalType().shortValue());
                int inCrossingFlag = carTrack.getRoadnet().getInCrossFlag();
                //路口内机动车
                if (inCrossingFlag == Constant.CROSSING) {
//                    if (carTrack.getId()==11438667) {
//                        System.out.println();
//                    }
                    if (isMotor) {
                        if (roadNet.getTurn() > 4) {
                            carTrack.setCarTurn(CommonUtil.getCarTurn(carTrack));
                        } else {
                            carTrack.setCarTurn(roadNet.getTurn());
                        }

                        AccidentFrameModel.CarTrack car = new AccidentFrameModel.CarTrack();
                        BeanUtils.copyProperties(carTrack, car);
                        String laneEndPoint = car.getRoadnet().getLaneEndPoint();
                        if (Objects.isNull(laneEndPoint)) {
                            continue;
                        }
                        String[] sps = laneEndPoint.split(",");
                        Double angeleToLane = PtInPolyUtil.getAngle(carTrack.getLongitude(), carTrack.getLatitude(), Double.parseDouble(sps[0]), Double.parseDouble(sps[1]));
                        //通过车辆与车道终点坐标得出的角度与车道方位角度判断是否驶出停止线
                        double angleX = Math.abs(angeleToLane - carTrack.getRoadnet().getLaneAngle());
                        boolean isReverseDir = false;
                        if (angleX > 160.0 && angleX < 200.0) {
                            isReverseDir = true;
                        }
                        //距离停止线距离
                        double stopLineDist = GeomsConvertUtil.getDistance(car.getLongitude(), car.getLatitude(), Double.parseDouble(sps[0]), Double.parseDouble(sps[1]));
                        car.setStopLineDist(stopLineDist);

                        boolean isInWaitArea = CommonUtil.isInWaitTurnArea(rid, carTrack);
                        //排除掉待转区车辆
                        if (!isInWaitArea) {
                            list.add(car);
                        }
                    } else {
                        nonMotorList.add(carTrack);
                    }

                }
                if (!list.isEmpty()) {
                    AccidentFrameModel model = new AccidentFrameModel();
                    model.setCrossId(frameModel.getCrossId());
                    model.setTime(frameModel.getGlobalTimeStamp());
                    model.setTrackList(list);
                    model.setRidTurnLampList(frameModel.getRidTurnLampList());
                    model.setConfigModel(frameModel.getConfigModel());

                    this.checkFrontObstacleCount(list, nonMotorList);

                    collector.collect(model);
                }
            }
        } catch (Exception e) {
            log.error("", e);
        }

    }

    //车辆前方左右50度，半径5m的扇形区域内是否有非机动车、行人目标
    private void checkFrontObstacleCount(List<AccidentFrameModel.CarTrack> motorList, List<CarTrackModel> nonMotorList) {
        for (AccidentFrameModel.CarTrack motor : motorList) {
            String date = "2024-05-20 08:29:15:000";
            long sd = DateUtil.StringToMillis(date,"yyyy-MM-dd HH:mm:ss:SSS");
            if (sd < motor.getGlobalTimeStamp() && motor.getId().equals(11438502)){
                System.out.println();
            }
            Polygon polygon = CommonUtil.getCarFrontSector(motor);
            int obstacleCount = 0;
            for (CarTrackModel nonMotor : nonMotorList) {
                Point point = GeomsConvertUtil.createPoint(nonMotor.getLongitude() + "," + nonMotor.getLatitude());
                if (point.within(polygon)) {
                    //log.info("机动车ID:{},非机动车ID:{}，时间：{},视野区域：{}",motor.getId(),nonMotor.getId(),DateUtil.toDateTime(motor.getGlobalTimeStamp(),"yyyy-MM-dd HH:mm:ss.SSS"),polygon.toText());
                    obstacleCount++;
                }
            }
            motor.setObstacleCount(obstacleCount);
        }
    }
}
