package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.LightStatusEnum;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.accident.AccidentLampStateModel;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 2021/11/19 17:54
 *
 * @auther fengyi
 */
public class AccidentLampStateKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentLampStateKeyedProcess.class);

    private MapState<String, AccidentLampStateModel> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccidentLampState_x01", String.class, AccidentLampStateModel.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {
        //若无灯态接入则直接跳过此算子

        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();

        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            //对停驶车辆进行灯态验证
            String key = getKey(carTrack);
            if (carTrack.isStop()) {
                List<CrossRidTurnLampStatusModel> lampStateList = carTrack.getRidTurnLampStatusModel();
                //对在绿灯下停驶的车辆进行计时
                if (CommonUtil.isGreen(lampStateList)) {
                    AccidentLampStateModel lampStateModelAgo = mapState.get(key);
                    //若上一帧车辆绿灯停车 则使用当前帧与上一帧数据做计算
                    AccidentLampStateModel model = new AccidentLampStateModel();
                    if (lampStateModelAgo != null) {
                        long timeAgo = lampStateModelAgo.getTime();
                        long timeNow = accidentFrameModel.getTime();
                        long timeX = timeNow - timeAgo;
                        //获取上一帧累计的绿灯停驶持续时间
                        long duration = lampStateModelAgo.getDuration();
                        if (timeX > 0L && timeX < 1000L) {
                            duration += timeX;
                        }
                        model.setKey(key);
                        model.setStartTime(lampStateModelAgo.getStartTime());
                        model.setTime(accidentFrameModel.getTime());
                        model.setDuration(duration);
                        mapState.put(key, model);
//                        log.info("LOG_EVENT_ACCIDENT_GREEN_STOP >>> " + DateFormatUtils.format(carTrack.getTime(), DateUtil.YYYY_MM_DD_HH_MM_SS) +
//                                " greenStop: "  + carTrack.getPlateNumber() +
//                                " greenDu: " + duration +
//                                " timeX: " + timeX +
//                                " laneFunc: " + carTrack.getLaneFunc() +
//                                " laneId: " + carTrack.getEntryLane());
                    } else {
                        //若上一帧车辆红灯停车 则将当前帧数据存入状态
                        model.setKey(key);
                        if (lampStateModelAgo != null) {
                            model.setStartTime(lampStateModelAgo.getStartTime());
                        } else {
                            model.setStartTime(accidentFrameModel.getTime());
                        }
                        model.setTime(accidentFrameModel.getTime());
                        model.setDuration(0);
                        mapState.put(key, model);
                    }
                } else {
                    // 灯态变为红灯  暂时不做操作
                }
            } else {
                //车辆已开走  需要清除缓存的状态
                mapState.remove(key);
            }
        }

        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            String key = getKey(carTrack);
            AccidentLampStateModel model = mapState.get(key);
//            if (carTrack.getId()==7607695){
//                System.out.println();
//            }
            if (model != null) {
                //绿灯停车  写入绿停持续时间
                carTrack.setGreenStop(true);
                carTrack.setGreenStopDuration(model.getDuration());
                carTrack.setGreenStopStartTime(model.getStartTime());
            } else {
                //正常行驶/停驶车辆 正常输出
                carTrack.setGreenStop(false);
                carTrack.setGreenStopDuration(0);
                carTrack.setGreenStopStartTime(0);
            }
            list.add(carTrack);
        }

        accidentFrameModel.setTrackList(list);
        collector.collect(accidentFrameModel);
    }

    /**
     * 获取key
     * @param carTrack
     * @return
     */
    private String getKey(AccidentFrameModel.CarTrack carTrack) {
        return carTrack.getRoadnet().getCrossId() + Constant.MARK + carTrack.getId();
    }

    /**
     * 验证灯态是否为绿
     * @param lampStateList
     * @return
     */
    private boolean isGreen(List<CrossRidTurnLampStatusModel> lampStateList) {
        if (lampStateList != null){
            for (CrossRidTurnLampStatusModel lampState : lampStateList){
                String lamp = lampState.getLampState();
                if (LightStatusEnum.GREEN.getType().equals(lamp)){
                    return  true;
                }
            }
        }
        return false;
    }

}
