package com.wanji.indicators.event.cross.accident.funcnew;

import com.google.common.base.Strings;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/* *
 * 判断无灯态是否正常停车
 * @Description
 * @Author fengyi
 * @Date 17:05 2022/1/20
 **/
@Slf4j
public class AccidentNoLampNomalStopCarKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private MapState<String, AccidentFrameModel.CarTrack> mapStateOfNormalStop;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapStateOfNormalStop = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateNormalStopCar", String.class, AccidentFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();

        //无灯态时计算路口内停车车辆是否属于正常停车
        if (accidentFrameModel.getRidTurnLampList().isEmpty()) {
            String crossId = accidentFrameModel.getCrossId();
            ConfigModel configModel = accidentFrameModel.getConfigModel();
            //按进口RID分组
            Map<String, List<AccidentFrameModel.CarTrack>> groupList = trackList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));

            for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry : groupList.entrySet()) {
                List<AccidentFrameModel.CarTrack> value = entry.getValue();
                //停车车辆
                List<AccidentFrameModel.CarTrack> stopTrackList = value.stream().filter(o -> o.isStop()).collect(Collectors.toList());
                //同方向行驶的车辆
                List<AccidentFrameModel.CarTrack> runTrackList = value.stream().filter(o -> !o.isStop()).collect(Collectors.toList());

                for (AccidentFrameModel.CarTrack carTrack : stopTrackList) {

                    String laneFunc = TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn());
                    String[] sps = laneFunc.split(Constant.COMMA);

                    boolean isNormalStop = false;
                    //是否有同转向车道车辆
                    boolean isHasSameTurnCar = false;
                    for (String func : sps) {
                        //筛选停止线前的包含路口内停车车辆车道功能的车辆 value【stop+run】
                        List<AccidentFrameModel.CarTrack> stopLineAgoCarList = value.stream().filter(o ->TurnEnum.getLaneFunction(o.getRoadnet().getTurn()).contains(func) && o.getStopLineDist()>0).collect(Collectors.toList());
                        //按车道分组，按路口内行驶距离排序
                        Map<String, List<AccidentFrameModel.CarTrack>> groupByLane = stopLineAgoCarList.stream().sorted(Comparator.comparing(o -> o.getStopLineDist())).collect(Collectors.groupingBy(o -> o.getRoadnet().getLaneId()));
                        /* *********************************************************** */
                        Map<String, Double> turnStopRateMap = this.getStopLaneRate(groupByLane, configModel);
                        /* ********************************************************** */
                        //遍历RID方向上每个车道驶出的车辆
                        for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry1 : groupByLane.entrySet()) {
                            //按到停止线距离排序
                            List<AccidentFrameModel.CarTrack> laneCarList = entry1.getValue().stream().sorted(Comparator.comparing(o -> o.getStopLineDist())).collect(Collectors.toList());
                            AccidentFrameModel.CarTrack firstCar = laneCarList.get(0);

                            String gkey = crossId + Constant.MARK + firstCar.getRoadnet().getRid() + Constant.MARK + func;
                            double stopRate = 0;
                            if (turnStopRateMap.containsKey(gkey))
                                stopRate = turnStopRateMap.get(gkey);

                            isHasSameTurnCar = true;
                            //通过停车车道数比例来规避车辆在车道出现跳变情况
                            if (stopRate >= configModel.getAbnormalParkNoLampStopLaneRate()) {
                                isNormalStop = true;
                                log.info("NO LAMPSTATE 判定路口内正常停车车辆 plateNo=" + carTrack.getPicLicense() + ",laneFunc=" + laneFunc + "，停止线前车辆：" + firstCar.toString());
                                break;
                            }
                        }
                        if (isNormalStop) {
                            break;
                        }
                    }
                    carTrack.setHasSameTurnCar(isHasSameTurnCar);
                    carTrack.setNormalStopOfNoLampstate(isNormalStop);
                }
            }
        }
        accidentFrameModel.setTrackList(trackList);
        collector.collect(accidentFrameModel);
    }

    /**
     * 相同转向车道停车的占比
     * @param groupByLane 按车道分组驶出停止线的所有车辆
     * @param configModel
     * @return
     */
    private Map<String, Double> getStopLaneRate(Map<String, List<AccidentFrameModel.CarTrack>> groupByLane, ConfigModel configModel) {
        Map<String, Double> turnStopRateMap = new HashMap<>();
        //记录各转向上出现停车的车道总数
        Map<String, Double> turnStopCountMap = new HashMap<>();
        //每个转向关联的车道数
        Map<String, Integer> turnLaneCountMap = new HashMap<>();
        //groupByLane驶出停止线的所有车辆
        for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry1 : groupByLane.entrySet()) {

            List<AccidentFrameModel.CarTrack> value1 = entry1.getValue().stream().collect(Collectors.toList());
            //每个车道驶出的第一辆车
            AccidentFrameModel.CarTrack firstCar = value1.get(0);

            String[] funcSps =TurnEnum.getLaneFunction(firstCar.getRoadnet().getTurn()).split(Constant.COMMA);

            for (String func : funcSps) {
                String gkey = firstCar.getRoadnet().getCrossId() + Constant.MARK + firstCar.getRoadnet().getRid() + Constant.MARK + func;
                Integer turnLaneCount = turnLaneCountMap.get(gkey);
                if (turnLaneCount == null) {
                    turnLaneCount = 1;
                } else {
                    turnLaneCount++;
                }
                turnLaneCountMap.put(gkey, turnLaneCount);

                if (firstCar.getSpeed() <= configModel.getAbnormalParkStopSpeed()) {
                    Double count = turnStopCountMap.get(gkey);
                    if (count == null) {
                        count = 1D;
                    } else {
                        count++;
                    }
                    turnStopCountMap.put(gkey, count);
                }
            }
        }

        for (Map.Entry<String, Double> entry1 : turnStopCountMap.entrySet()) {
            String key = entry1.getKey();
            //转向停车数/相同转向的车道数量
            double rate = entry1.getValue() / turnLaneCountMap.get(key);
            turnStopRateMap.put(key, rate);
        }

        return turnStopRateMap;
    }


    /**
     * 获取key
     *
     * @param carTrack
     * @return
     */
    private String getKey(AccidentFrameModel.CarTrack carTrack) {
        return carTrack.getRoadnet().getCrossId() + Constant.MARK + carTrack.getId();
    }

}
