package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.accident.AccidentLampStateModel;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

/* *
 *
 * @Description
 * @Author fengyi
 * @Date 17:05 2022/1/20
 **/
public class AccidentRightVehicleKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentRightVehicleKeyedProcess.class);

    private MapState<String, AccidentLampStateModel> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccidentRightLampState_x01", String.class, AccidentLampStateModel.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();

        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            //对停驶车辆进行灯态验证
            String key = getKey(carTrack);
            if (carTrack.isStop()) {
                List<CrossRidTurnLampStatusModel> lampState = carTrack.getRidTurnLampStatusModel();
                //车道功能
                String laneFunc = TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn());
                //右转车无灯态时
                if (Objects.nonNull(laneFunc) && laneFunc.contains(Constant.RIGHT_TURN_CODE) &&
                        (Objects.isNull(lampState))) {
                    String rid = carTrack.getRoadnet().getRid();
                    Map<String,String> ridTurnRefMap = carTrack.getRidTurnRefMap();
                    boolean isAllRed = isAllRedOfOtherDirLampState(ridTurnRefMap,accidentFrameModel.getRidTurnLampList(),rid);
                    //其他关联方向全部红灯时开始停车时长计算
                    if (isAllRed) {
                        AccidentLampStateModel lampStateModelAgo = mapState.get(key);
                        //若上一帧车辆绿灯停车 则使用当前帧与上一帧数据做计算
                        AccidentLampStateModel model = new AccidentLampStateModel();
                        if (lampStateModelAgo != null) {
                            long timeAgo = lampStateModelAgo.getTime();
                            long timeNow = accidentFrameModel.getTime();
                            long timeX = timeNow - timeAgo;
                            //获取上一帧累计的绿灯停驶持续时间
                            long duration = lampStateModelAgo.getDuration();
                            if (timeX > 0L && timeX < 1000L) {
                                duration += timeX;
                            }
                            model.setKey(key);
                            model.setStartTime(lampStateModelAgo.getStartTime());
                            model.setTime(accidentFrameModel.getTime());
                            model.setDuration(duration);
                            mapState.put(key, model);
                            log.info("右转车无灯态停车计算：关联进口全红灯时停车，停车开始时间：{},持续时间：{}ms",model.getStartTime(),model.getStartTime());
                        } else {
                            //若上一帧车辆红灯停车 则将当前帧数据存入状态
                            model.setKey(key);
                            if (lampStateModelAgo != null) {
                                model.setStartTime(lampStateModelAgo.getStartTime());
                            } else {
                                model.setStartTime(accidentFrameModel.getTime());
                            }
                            model.setTime(accidentFrameModel.getTime());
                            model.setDuration(200);
                            mapState.put(key, model);
                        }
                    } else {
                        //绿灯放行时，清理缓存
                        AccidentLampStateModel model = mapState.get(key);
                        if (model != null) {
                            mapState.remove(key);
                        }
                    }
                }
            } else {
                //车辆已开走  需要清除缓存的状态
                mapState.remove(key);
            }
        }

        List<AccidentFrameModel.CarTrack> newlist = new ArrayList<>();

        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            String key = getKey(carTrack);
            AccidentLampStateModel model = mapState.get(key);
            if (model != null) {
                //绿灯停车  写入绿停持续时间
                carTrack.setGreenStop(true);
                carTrack.setGreenStopDuration(model.getDuration());
                carTrack.setGreenStopStartTime(model.getStartTime());

                List<CrossRidTurnLampStatusModel> virtualState = new ArrayList<>();
                CrossRidTurnLampStatusModel vs= new CrossRidTurnLampStatusModel();
                vs.setRid(carTrack.getRoadnet().getRid());
                vs.setLampState(Constant.GREEN);
                //右转设置虚拟绿灯
                carTrack.setRidTurnLampStatusModel(virtualState);
            }
            newlist.add(carTrack);
        }

        accidentFrameModel.setTrackList(newlist);
        collector.collect(accidentFrameModel);
    }
    /**
     *
     * @Description 右转出口道 其他入口方向是否开始绿灯放行
     * @Author fengyi
     * @Date 19:45 2022/1/20
     * @Param [accidentFrameModel, currentRid]
     * @return boolean
     **/
    private boolean isAllRedOfOtherDirLampState(Map<String,String> ridTurnRefMap,List<CrossRidTurnLampStatusModel> ridTurnLampStatusList,String currentRid) {
        boolean otherLampStateFlg = false;

        String otherDirFrid = null;
        if (!ridTurnRefMap.isEmpty()){
            //frid1;turn_dir_no1,frid2;turn_dir_no2,...
            otherDirFrid = ridTurnRefMap.get(currentRid);

            if (Objects.nonNull(otherDirFrid)){
                String[] sps = otherDirFrid.split(",");
                for (String fridTurn : sps){
                    //frid1;turn_dir_no1
                    String[] sps1 = fridTurn.split(";");
                    String ofrid = sps1[0];
                    String oturn = sps1[1];
                    //其他关联方向可进入右转出口道的进口道转向灯态
                    Optional<CrossRidTurnLampStatusModel> lampStatusModel =  ridTurnLampStatusList.stream()
                            .filter(o->Objects.equals(Constant.RED,o.getLampState())
                                    && Objects.equals(ofrid,o.getRid())
                                    && Objects.equals(oturn,o.getTurn())
                                    )
                            .findFirst();
                    if (lampStatusModel.isPresent()){
                        otherLampStateFlg = true;
                    }else{
                        otherLampStateFlg = false;
                        break;
                    }
                }
            }
        }
        log.info("右转无灯态停车计算-灯态，关联rid转向:{},路口灯态数据：{}，关联灯态是否全红：{}",otherDirFrid,ridTurnLampStatusList.toString(),otherLampStateFlg);

        return otherLampStateFlg;
    }


    /**
     * 获取key
     * @param carTrack
     * @return
     */
    private String getKey(AccidentFrameModel.CarTrack carTrack) {
        return carTrack.getRoadnet().getCrossId() + Constant.MARK + carTrack.getId();
    }

}
