package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentDetourModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @auther fengyi
 */
public class AccidentSameDriDriveKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentSameDriDriveKeyedProcess.class);


    @Override
    public void open(Configuration parameters) throws Exception {
     }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {
        ConfigModel configModel = accidentFrameModel.getConfigModel();

        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();

        List<AccidentFrameModel.CarTrack> trackRun = trackList.stream().filter(o-> o.getSpeed() > configModel.getAbnormalParkStopSpeed()).collect(Collectors.toList());
        Map<String,List<AccidentFrameModel.CarTrack>> runGroup = trackRun.stream().collect(Collectors.groupingBy(o->o.getRoadnet().getRid()));

        List<AccidentFrameModel.CarTrack> trackStop = trackList.stream().filter(o->o.getSpeed() <= configModel.getAbnormalParkStopSpeed()).collect(Collectors.toList());
        Map<String,List<AccidentFrameModel.CarTrack>> stopGroup = trackStop.stream().collect(Collectors.groupingBy(o->o.getRoadnet().getRid()));


        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();

        for (Map.Entry<String,List<AccidentFrameModel.CarTrack>> entry : stopGroup.entrySet()) {
            String rid = entry.getKey();
            //同方向上运行的车辆
            List<AccidentFrameModel.CarTrack> runCarList = runGroup.get(rid);
            //同方向上停车车辆
            List<AccidentFrameModel.CarTrack> stopCarList = entry.getValue();
            for (AccidentFrameModel.CarTrack carTrack : stopCarList) {
                String laneFunc = TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn());
                String[] sps = laneFunc.split(Constant.COMMA);
                boolean isHasSameTurnCar = false;
                //遍历车道功能【混合车道有多个转向】
                for (String func : sps) {
                    try {
                        if (Objects.nonNull(runCarList)) {
                            List<AccidentFrameModel.CarTrack> sameTurnCarList = runCarList.stream().filter(o -> TurnEnum.getLaneFunction(o.getRoadnet().getTurn()).contains(func)).collect(Collectors.toList());
                            //判断停车车辆是否有从相同转向运行的车辆，有同向行驶车辆加入到列表传递给下一个算子
                            if (!sameTurnCarList.isEmpty()) {
                                isHasSameTurnCar = true;
                                break;
                            }
                        }
                    }catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                carTrack.setHasSameTurnCar(isHasSameTurnCar);
                list.add(carTrack);

            }
        }

        //发送数据
        if (!list.isEmpty()) {
            accidentFrameModel.setTrackList(list);
            collector.collect(accidentFrameModel);
        }

    }


}
