package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.*;


/**
 * @auther fengyi
 */
@Slf4j
public class AccidentStopCheckKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private MapState<String, AccidentFrameModel.CarTrack> mapState;
    private MapState<String, AccidentFrameModel.CarTrack> mapStateStop;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_stop_1", String.class, AccidentFrameModel.CarTrack.class));
        mapStateStop = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_stop_2", String.class, AccidentFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        if (accidentFrameModel.isOverFlow() || accidentFrameModel.isDeadLock()) {
            //溢出拥堵或死锁时清除缓存，下次重新开始计算
            mapState.clear();
            mapStateStop.clear();
            accidentFrameModel.getTrackList().clear();

//            log.info("路口ID:{}，发送溢出或死锁状态时停车车辆不作计算，溢出状态：{}，死锁状态：{},时间：{}",accidentFrameModel.getCrossId(),
//                    accidentFrameModel.isOverFlow(),accidentFrameModel.isDeadLock(),DateUtil.toDateTime(accidentFrameModel.getTime(),"yyyy-MM-dd HH:mm:ss.SSS"));
        } else {
            String key = null;
            ConfigModel configModel = accidentFrameModel.getConfigModel();
            long nowTime = accidentFrameModel.getTime();
            Set<String> nowKeys = new HashSet<>();
            List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
            List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
            for (AccidentFrameModel.CarTrack carTrack : trackList) {
                key = getKey(carTrack);
                double speed = carTrack.getSpeed();

                AccidentFrameModel.CarTrack stopPoint = mapStateStop.get(key);
                if (stopPoint != null) {
                    AccidentFrameModel.CarTrack carTrackBefore = mapState.get(key);
                    //当前帧的时间必须在停车时间之后 否则 不予计算 (过滤乱序数据)
                    if (nowTime > stopPoint.getGlobalTimeStamp() && nowTime > carTrackBefore.getGlobalTimeStamp()) {
//                        if (carTrack.getId()==5546643){
//                            System.out.println();
//                        }
                        //计算上一帧数据和当前数据时间差
                        long frameTimeX = nowTime - carTrackBefore.getGlobalTimeStamp();
                        if (frameTimeX > 0 && frameTimeX < 1000L) {
                            //计算停车持续时间，单位毫秒
                            long stopDuration = nowTime - stopPoint.getStopTime();
                            stopPoint.setStopDuration(stopDuration);
                            if (carTrack.isHasSameTurnCar() && stopPoint.isHasSameTurnCar() && !carTrack.isHasConflictCar()) {
                                //计算有相同方向车流且无冲突车流时持续时长
                                long sameTurnStopTime = nowTime - stopPoint.getHasSameTurnTime();
                                stopPoint.setSameTurnStopDuration(sameTurnStopTime);
                            }
                            if (!stopPoint.isHasSameTurnCar() && carTrack.isHasSameTurnCar() && !carTrack.isHasConflictCar()) {
                                //有相同转向、且没有发生冲突车流的开始时刻
                                stopPoint.setHasSameTurnTime(nowTime);
                            }
                            stopPoint.setHasSameTurnCar(carTrack.isHasSameTurnCar());

                            mapStateStop.put(key, stopPoint);

                            //大于此停车车速阈值(外部配置文件设置)的车辆且位移一定距离后移除对其的跟踪，【过滤停车抖动车】
                            if (speed > configModel.getAbnormalParkStopShakeSpeed()) {
                                double distance = PtInPolyUtil.getDistance(carTrack.getLongitude(), carTrack.getLatitude(), stopPoint.getLongitude(), stopPoint.getLatitude());
                                //位移超过 5.0m 且车速超过 3.6km/h 的才算正常行驶
                                if (distance > configModel.getAbnormalParkStopShakeRange()) {
                                    mapStateStop.remove(key);
                                    log.info( "distance 清除抖动移动车辆 id="+carTrack.getId()+", plateNum="+carTrack.getPicLicense()+ " distance: " + distance + "  speed:" + speed + " 阈值:" + configModel.getAbnormalParkStopShakeRange());
                                }
                            }
                            String dt = DateUtil.toDateTime(nowTime,"yyyy-MM-dd HH:mm:ss.SSS");
                            if (speed > configModel.getAbnormalParkStopSpeed()) {
                                //车速超过 9km/h 的算正常行驶
                                mapStateStop.remove(key);
//                                            log.info( "speed 清除正常车速车辆 - key: " + speed);
                            }
//                            if (carTrack.getId()==11438502 && stopDuration>10000) {
//                                System.out.println();
//                            }

                            if (carTrack.isHasConflictCar()) {
                                //剔除因冲突方向车流导致的停车
                                if (carTrack.getStopDuration()< configModel.getAbnormalParkHaveSameFlowStopDuration()) {
                                    mapStateStop.remove(key);
                                    log.info( "剔除因冲突方向车流导致的停车{}，时间：{}" ,(carTrack.getPicLicense()==null?carTrack.getId():carTrack.getPicLicense())+"@"+ carTrack.getRoadnet().toString(),DateUtil.toDateTime(accidentFrameModel.getTime(),"yyyy-MM-dd HH:mm:ss.SSS"));
                                }
                            }
                            if (carTrack.getObstacleCount()>1) {
                                //前方有行人、非机动车阻碍减速停车情况
                                mapStateStop.remove(key);
//                                log.info("LOG_EVENT_STOP_DURATION_TIME >>> 礼让行人、非机动车导致的停车，清理计算停车时长缓存 " + carTrack.getRoadnet().getCrossId() +
//                                        "-" + carTrack.getId() + "-" + carTrack.getPicLicense() +
//                                        " ObstacleCount: " + carTrack.getObstacleCount()
//                                );
                            }
                        }
                    }
                } else if (stopPoint == null && speed <= configModel.getAbnormalParkStopSpeed()) {
                    //判定车辆停驶的车速阈值  车辆小于等于此阈值时认为车辆停驶
                    carTrack.setStop(true);
                    carTrack.setStopTime(nowTime);
                    if (carTrack.isHasSameTurnCar()) {
                        carTrack.setHasSameTurnTime(nowTime);
                    }
                    if (!carTrack.isHasConflictCar()) {
                        mapStateStop.put(key, carTrack);
                        //log.info( "剔除因冲突方向车流导致的停车: " +carTrack.getPicLicense()+"@"+ carTrack.getRoadnet().toString());
                    }
                }
                nowKeys.add(key);
                mapState.put(key, carTrack);
                list.add(carTrack);
            }
            //清除过期数据
            List<AccidentFrameModel.CarTrack> listNowFrame = new ArrayList<>();
            Iterator<AccidentFrameModel.CarTrack> iterator = mapState.values().iterator();
            while (iterator.hasNext()) {
                listNowFrame.add(iterator.next());
            }
            for (AccidentFrameModel.CarTrack next : listNowFrame) {
                key = getKey(next);
                if (!nowKeys.contains(key)) {
                    long timeX = nowTime - next.getGlobalTimeStamp();
                    //计算此key消失的时间  超过1.7秒的做删除处理  轨迹消失后超时移除状态的时间
                    if (timeX > configModel.getAbnormalParkStopDissipateRemoveTime() && timeX < 3600000L) {
//                    log.info( " 清除消失车辆 - keyRemove: " + keyRemove + " 超时: " + timeX);
                        mapState.remove(key);
                        mapStateStop.remove(key);
                    }
                }
            }

            //发送数据
            if (!list.isEmpty()) {
                for (AccidentFrameModel.CarTrack carTrack : list) {
                    key = getKey(carTrack);
//                    if (carTrack.getId()==7607695){
//                        System.out.println();
//                    }
                    AccidentFrameModel.CarTrack carTrackStop = mapStateStop.get(key);
                    if (carTrackStop != null) {
                        carTrack.setStop(true);
                        carTrack.setStopDuration(nowTime - carTrackStop.getGlobalTimeStamp());
                        carTrack.setStopTime(carTrackStop.getGlobalTimeStamp());
                        carTrack.setGlobalTimeStamp(nowTime);
                        carTrack.setSameTurnStopDuration(carTrackStop.getSameTurnStopDuration());
                        carTrack.setHasSameTurnTime(carTrackStop.getHasSameTurnTime());
                    } else {
                        carTrack.setStop(false);
                        carTrack.setStopDuration(0);

                    }
                }
                accidentFrameModel.setTrackList(list);
                collector.collect(accidentFrameModel);
            }
        }
    }

    /**
     * 获取key
     *
     * @param carTrack
     * @return
     */
    private static String getKey(AccidentFrameModel.CarTrack carTrack) {
        return carTrack.getRoadnet().getCrossId() + Constant.MARK + carTrack.getId();
    }


}
