package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.accident.AccidentStraightLeftThroughtModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * 2021/11/19 17:54
 *
 * @auther fengyi
 */
public class AccidentStraightLeftThroughKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentStraightLeftThroughKeyedProcess.class);
    //用于计算冲突方向头车到停车车辆最小距离及相应的持续时间
    private MapState<String, AccidentStraightLeftThroughtModel> mapState;
    //缓存有冲突车辆时停车5s次数、停车时长
    private MapState<String, AccidentStraightLeftThroughtModel> hasConflictCarMapState;
    //缓存无冲突车辆时停车时长
    private MapState<String, AccidentStraightLeftThroughtModel> noConflictCarMapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateStraightLeftThroughLampState", String.class, AccidentStraightLeftThroughtModel.class));
        hasConflictCarMapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateHasConflictCarStatisticResult", String.class, AccidentStraightLeftThroughtModel.class));
        noConflictCarMapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateNoConflictCarStatisticResult", String.class, AccidentStraightLeftThroughtModel.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        long nowTime = accidentFrameModel.getTime();

        this.process(accidentFrameModel);

        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            String[] laneFunc = TurnEnum.getLaneTurnValue(carTrack.getRoadnet().getTurn());
            boolean isSatisfy = false;
            for (String func : laneFunc) {//复合车道时分割转向，每个转向都要满足输出阈值才输出时间
                String key = this.getKey(carTrack, func);
                AccidentStraightLeftThroughtModel model = hasConflictCarMapState.get(key);
                if (Objects.nonNull(model)) {
                    carTrack.setStraightLeftThrough(true);
                    carTrack.setHasCflictCarStopCount(model.getGreenStopCount());
                    carTrack.setHasCflictCarStopDuration(model.getGreenStopDuration());
                    carTrack.setNextPhaseStopCount(model.getNextPhaseStopCount());
                    carTrack.setNextPhaseStopDuration(model.getNextPhaseStopDuration());
                    carTrack.setNextPhase(model.isNextPhase());
                }
                AccidentStraightLeftThroughtModel noConflictCarModel = noConflictCarMapState.get(key);
                if (Objects.nonNull(noConflictCarModel)) {
                    carTrack.setStraightLeftThrough(true);
                    //无冲突流停车时长
                    carTrack.setNoCflictCarStopDuration(noConflictCarModel.getGreenStopDuration());
                    carTrack.setNextPhaseStopCount(noConflictCarModel.getNextPhaseStopCount());
                    carTrack.setNextPhaseStopDuration(noConflictCarModel.getNextPhaseStopDuration());
                    carTrack.setNextPhase(noConflictCarModel.isNextPhase());
                }
                if (carTrack.isStraightLeftThrough())
                    isSatisfy = isSatisfyStraightLeftEvent(accidentFrameModel.getConfigModel(), carTrack);
            }
            carTrack.setSatisfyStraightLeftEvent(isSatisfy);
        }
        collector.collect(accidentFrameModel);
        //缓存清理
        this.clearMapState(nowTime, mapState);
        this.clearMapState(nowTime, noConflictCarMapState);
        this.clearMapState(nowTime, hasConflictCarMapState);
    }

    /**
     * 检测直左通行是否满足事件输出条件
     *
     * @param carTrack
     * @return
     */
    private boolean isSatisfyStraightLeftEvent(ConfigModel configModel, AccidentFrameModel.CarTrack carTrack) {
        boolean flag = false;
        long noCflictCarStopDuration = carTrack.getNoCflictCarStopDuration();//无冲突车辆绿灯持续时间
        int hasCflictCarStopCount = carTrack.getHasCflictCarStopCount();//有冲突车流绿灯内停车次数
        long hasCflictCarStopDuration = carTrack.getHasCflictCarStopDuration();//有冲突车流绿灯内停车时长
        long nextPhaseStopDuration = carTrack.getNextPhaseStopDuration();//下个相位绿灯停车时长
        long nextPhaseStopCount = carTrack.getNextPhaseStopCount(); //下个相位绿灯停车次数
        boolean isNextPhase = carTrack.isNextPhase();//是否停止到下个相位
        //绿灯停车时长阈值
        long greenStopMaxDuration = configModel.getAbnormalParkStraightLeftStopGreenDuration();

        if (hasCflictCarStopCount >= 2) {
            flag = true;
        } else if (hasCflictCarStopCount == 1) {
            if (noCflictCarStopDuration >= greenStopMaxDuration) {
                flag = true;
            } else if (hasCflictCarStopDuration < greenStopMaxDuration || hasCflictCarStopCount == 0) {
                //监测下个相位
                if (isNextPhase && (nextPhaseStopDuration >= greenStopMaxDuration || nextPhaseStopCount > 0)) {
                    flag = true;
                }
            }
        } else if (hasCflictCarStopCount == 0) {
            //监测下个相位
            if (isNextPhase && (nextPhaseStopDuration >= greenStopMaxDuration || nextPhaseStopCount > 0)) {
                flag = true;
            }
        } else if (noCflictCarStopDuration >= greenStopMaxDuration) {
            //车停对面冲突方向无车流
            flag = true;
        }
        return flag;
    }


    private void process(AccidentFrameModel deadLockFrameModel) throws Exception {
        ConfigModel configModel = deadLockFrameModel.getConfigModel();
        List<AccidentFrameModel.CarTrack> trackList = deadLockFrameModel.getTrackList();
        //冲突基础数据  key:cross-rid-ridDir-turn value:crossid-rid-ridDir-冲突转向
        Map<String, List<String>>  conflictMap = deadLockFrameModel.getConflictSlotMap();
        //按RID分组（包含路口内及到达出口的车辆）
        Map<String, List<AccidentFrameModel.CarTrack>> groupList = trackList.stream()
                .filter(o -> o.getStopLineDist() < 0)
                .collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));

        //复合车道出来的车辆，例如直左，会给直行和左转都赋予该车辆
        Map<String, List<AccidentFrameModel.CarTrack>> ridTurnCarList = this.getRidTurnCarList(groupList);

        for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry : ridTurnCarList.entrySet()) {
            String ridTurnKey = entry.getKey();
            if (StringUtils.isBlank(ridTurnKey)) {
                continue;
            }
            String[] turnSps = ridTurnKey.split(Constant.MARK);
            String ridTurn = turnSps[1];

            //复合车道相同车辆会循环两次，分别当作直行、左转来循环计算
            for (AccidentFrameModel.CarTrack carTrack : entry.getValue()) {
                boolean isStop = carTrack.isStop();
                boolean isGreen = CommonUtil.isGreen(carTrack.getRidTurnLampStatusModel());
                String laneFunc = TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn());
//                if (laneFunc.contains(Constant.COMMA)) {
//                    //多转向暂时不处理
//                    continue;
//                }
                String key = this.getKey(carTrack, ridTurn);
                if (isStop && isGreen) {//1）当前停车车辆处于绿灯相位灯态
                    //冲突进口及转向
                    List<String> mapList = conflictMap.get(ridTurnKey);
                    if (Objects.nonNull(mapList)) {
                        for (String cmap : mapList) {
                            String[] sps = cmap.split(Constant.MARK);
                            String rid = sps[1];
                            String turn = sps[3];
                            String ckey = rid + Constant.MARK + turn;
                            //冲突转向上的车辆
                            List<AccidentFrameModel.CarTrack> conflictDirCarList = ridTurnCarList.get(ckey);

                            List<CrossRidTurnLampStatusModel> lampStateList = deadLockFrameModel.getRidTurnLampList();
                            Optional<CrossRidTurnLampStatusModel> optional = lampStateList.stream().filter(o -> Objects.equals(rid, o.getRid()) && Objects.equals(turn, o.getTurn())).findAny();
                            if (optional.isPresent()) {
                                CrossRidTurnLampStatusModel lampStatusModel = optional.get();
                                //冲突转向也处于绿灯灯态
                                if (this.isGreen(lampStatusModel.getLampState())) {
                                    //计算冲突方向车辆与当前停车车辆间最小距离、持续时间
                                    this.calToStopCarMinDistDuration(configModel, carTrack, conflictDirCarList, ridTurn);
                                }
                            }

                        }
                    }
                } else if (!isGreen) {
                    //相位绿灯结束，设置标记
                    AccidentStraightLeftThroughtModel tmp = mapState.get(key);
                    if (Objects.nonNull(tmp)) {
                        tmp.setOverGreen(true);
                        mapState.put(key, tmp);
                    }
                    AccidentStraightLeftThroughtModel noConflictCarModel = noConflictCarMapState.get(key);
                    if (Objects.nonNull(noConflictCarModel)) {
                        noConflictCarModel.setOverGreen(true);
                        noConflictCarMapState.put(key, noConflictCarModel);
                    }
                    AccidentStraightLeftThroughtModel hasConflictCarModel = hasConflictCarMapState.get(key);
                    if (Objects.nonNull(hasConflictCarModel)) {
                        hasConflictCarModel.setOverGreen(true);
                        hasConflictCarMapState.put(key, hasConflictCarModel);
                    }
                } else {//车辆行驶清理缓存
                    mapState.remove(key);
                    hasConflictCarMapState.remove(key);
                    noConflictCarMapState.remove(key);
                }
            }
        }

    }

    /**
     * 计算冲突方向车辆与当前停车车辆间最小距离、持续时间
     *
     * @param stopCar            停车车辆
     * @param conflictDirCarList 冲突方向车辆
     */
    private void calToStopCarMinDistDuration(ConfigModel configModel, AccidentFrameModel.CarTrack stopCar, List<AccidentFrameModel.CarTrack> conflictDirCarList, String stopCarTurn) throws Exception {

        String key = this.getKey(stopCar, stopCarTurn);

        if (Objects.nonNull(conflictDirCarList) && !conflictDirCarList.isEmpty()) {//冲突方向有车流
            AccidentFrameModel.CarTrack conflictCarTmp = null;
            //车头时距
            List<Double> timeHeadList = new ArrayList<>();
            //冲突方向有车流
            for (AccidentFrameModel.CarTrack conflictCar : conflictDirCarList) {
                //停车车辆与冲突方向车辆之间的距离
                double dist = PtInPolyUtil.getDistance(stopCar.getLongitude(), stopCar.getLatitude(), conflictCar.getLongitude(), conflictCar.getLatitude());
                double timeHead = 0;
                if (conflictCarTmp != null) {
                    double headDist = PtInPolyUtil.getDistance(conflictCarTmp.getLongitude(), conflictCarTmp.getLatitude(), conflictCar.getLongitude(), conflictCar.getLatitude());
                    //车头时距
                    timeHead = headDist / (conflictCar.getSpeed() * 1000 / 3600);
                    if (timeHead > configModel.getAbnormalParkStraightLeftSafeTimeHead())
                        timeHeadList.add(timeHead);
                }
                if (conflictCar.getRoadnet().getInCrossFlag()==Constant.CROSSING) {//路口内
                    AccidentStraightLeftThroughtModel stateModel = mapState.get(key);
                    if (Objects.isNull(stateModel)) {
                        stateModel = new AccidentStraightLeftThroughtModel();
                        stateModel.setDistance(dist);
                        stateModel.setStartTime(stopCar.getGlobalTimeStamp());
                        stateModel.setTime(stopCar.getGlobalTimeStamp());
                        stateModel.setKey(key);
                        mapState.put(key, stateModel);
                    } else {
                        if (stateModel.isOverGreen()) {//运行到下一次相位
                            stateModel = new AccidentStraightLeftThroughtModel();
                            stateModel.setDistance(dist);
                            stateModel.setStartTime(stopCar.getGlobalTimeStamp());
                            stateModel.setTime(stopCar.getGlobalTimeStamp());
                            stateModel.setKey(key);
                            stateModel.setNextPhase(true);
                            stateModel.setOverGreen(false);
                            mapState.put(key, stateModel);
                        } else {
                            double agoDist = stateModel.getDistance();
                            long timex = stopCar.getGlobalTimeStamp() - stateModel.getTime();
                            //绿灯内停车时长
                            stateModel.setGreenStopDuration(stateModel.getTime() - stateModel.getStartTime());
                            if (dist < agoDist && Objects.isNull(conflictCarTmp)) {//头车
                                //计算冲突车辆到停止车辆最小距离时持续时长
                                stateModel.setDuration(stateModel.getTime() - stateModel.getStartTime());
                                stateModel.setDistance(dist);
                            }
                            stateModel.setTime(stopCar.getGlobalTimeStamp());
                            stateModel.setTimeHeadwayList(timeHeadList);
                            mapState.put(key, stateModel);
                        }
                    }
                } else {
                    boolean isNextPhase = false;
                    //驶入到出口
                    AccidentStraightLeftThroughtModel stateModel = mapState.get(key);
                    if (Objects.nonNull(stateModel)) {
                        stateModel.setToExitLane(true);
                        isNextPhase = stateModel.isNextPhase();

                        long headerPassTime = stateModel.getDuration();
                        int timeHeadSize = stateModel.getTimeHeadwayList().size();

                        AccidentStraightLeftThroughtModel model = hasConflictCarMapState.get(key);
                        if (Objects.isNull(model)) {
                            model = new AccidentStraightLeftThroughtModel();
                        }
                        if (!isNextPhase) {
                            model.setKey(key);
                            model.setGreenStopDuration(stateModel.getGreenStopDuration());
                            model.setStartTime(stateModel.getStartTime());
                            model.setTime(stateModel.getTime());
                            if (headerPassTime > configModel.getAbnormalParkStraightLeftSafeTimeHead() || timeHeadSize > 0) {
                                model.setGreenStopCount(model.getGreenStopCount() + 1);
                            }
                            //缓存有冲突流时统计结果
                            hasConflictCarMapState.put(key, model);
                        } else {
                            model.setKey(key);
                            model.setOverGreen(false);
                            model.setNextPhase(true);
                            model.setNextPhaseStopDuration(stateModel.getGreenStopDuration());
                            if (headerPassTime > configModel.getAbnormalParkStraightLeftSafeTimeHead() || timeHeadSize > 0) {
                                model.setNextPhaseStopCount(model.getGreenStopCount() + 1);
                            }
                            hasConflictCarMapState.put(key, model);
                        }
                    }
                }
                conflictCarTmp = conflictCar;
            }
        } else {
            //冲突方向无车流
            AccidentStraightLeftThroughtModel noCflictFlowModel = noConflictCarMapState.get(key);
            if (Objects.isNull(noCflictFlowModel)) {
                noCflictFlowModel = new AccidentStraightLeftThroughtModel();
                noCflictFlowModel.setStartTime(stopCar.getGlobalTimeStamp());
                noCflictFlowModel.setTime(stopCar.getGlobalTimeStamp());
                noCflictFlowModel.setGreenStopDuration(0);
            } else {
                long timex = stopCar.getGlobalTimeStamp() - noCflictFlowModel.getTime();
                if (noCflictFlowModel.isOverGreen()) {
                    noCflictFlowModel.setStartTime(stopCar.getGlobalTimeStamp());
                    noCflictFlowModel.setTime(stopCar.getGlobalTimeStamp());
                    noCflictFlowModel.setGreenStopDuration(0);
                    noCflictFlowModel.setNextPhase(true);
                    noCflictFlowModel.setOverGreen(false);
                }
                long du = stopCar.getGlobalTimeStamp() - noCflictFlowModel.getStartTime();
                if (!noCflictFlowModel.isNextPhase()) {
                    noCflictFlowModel.setGreenStopDuration(du);
                } else {
                    noCflictFlowModel.setNextPhaseStopDuration(du);
                }
            }
            noConflictCarMapState.put(key, noCflictFlowModel);

        }

    }

    /**
     * 获取rid转向上的车辆
     *
     * @param groupList
     */
    private Map<String, List<AccidentFrameModel.CarTrack>> getRidTurnCarList(Map<String, List<AccidentFrameModel.CarTrack>> groupList) {
        //按RID+转向分组
        Map<String, List<AccidentFrameModel.CarTrack>> ridTurnCarList = new HashMap<>();

        for (Map.Entry<String, List<AccidentFrameModel.CarTrack>> entry : groupList.entrySet()) {
            String ridLaneFunKey = entry.getKey();
            String rid = ridLaneFunKey.split(Constant.MARK)[0];

            for (AccidentFrameModel.CarTrack ridCar : entry.getValue()) {

                String[] sps = TurnEnum.getLaneTurnValue(ridCar.getRoadnet().getTurn());
                for (String func : sps) {
                    if (Objects.equals(Constant.RIGHT_TURN_CODE,func))
                        continue;
                    String ridTurnKey = getRidTurnKey(rid, func);
                    List<AccidentFrameModel.CarTrack> tmplist = ridTurnCarList.get(ridTurnKey);
                    if (Objects.isNull(tmplist)) {
                        tmplist = new ArrayList<>();
                    }
                    tmplist.add(ridCar);
                    ridTurnCarList.put(ridTurnKey, tmplist);
                }
            }
        }
        return ridTurnCarList;
    }

    private String getRidTurnKey(String rid, String laneFunc) {
        String ridTurnKey = null;
        if (laneFunc.contains(Constant.LEFT_TURN_CODE)) {
            ridTurnKey = rid + Constant.MARK + Constant.LEFT_TURN_CODE;
        } else if (laneFunc.contains(Constant.STRAIGHT_LINE_CODE)) {
            ridTurnKey = rid + Constant.MARK + Constant.STRAIGHT_LINE_CODE;
        }
        return ridTurnKey;
    }

    /**
     * 获取key
     *
     * @param carTrack
     * @return
     */
    private String getKey(AccidentFrameModel.CarTrack carTrack, String turn) {
        return carTrack.getRoadnet().getCrossId() + Constant.MARK + carTrack.getId() + Constant.MARK + turn;
    }

    /**
     * 验证灯态是否为绿
     *
     * @param lampState
     * @return
     */
    public static boolean isGreen(String lampState) {
        if (StringUtils.isNotBlank(lampState) && lampState.contains(Constant.GREEN)) {
            return true;
        }
        return false;
    }

    //缓存清理
    private void clearMapState(long nowTime, MapState<String, AccidentStraightLeftThroughtModel> mapState) throws Exception {
        if (!mapState.isEmpty()) {
            //缓存清理
            Iterable<AccidentStraightLeftThroughtModel> iterable = mapState.values();
            List<AccidentStraightLeftThroughtModel> list = StreamSupport.stream(iterable.spliterator(), false).collect(Collectors.toList());
            list = list.stream().filter(o -> (nowTime - o.getTime()) > 10 * 60 * 1000).collect(Collectors.toList());
            for (AccidentStraightLeftThroughtModel model : list) {
                mapState.remove(model.getKey());
            }
        }
    }

}
