package com.wanji.indicators.event.cross.conflictpoint;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.event.cross.accident.funcnew.AbnormalDeadlockConflictCheckKeyedProcess;
import com.wanji.indicators.event.cross.accident.funcnew.AbnormalDeadlockConflictSlotCoFlatMap;
import com.wanji.indicators.event.cross.conflictpoint.func.*;
import com.wanji.indicators.model.*;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.model.event.confilct.EventCross;
import com.wanji.indicators.sink.EventFileSink;
import com.wanji.indicators.source.ConflictTurnSource;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.source.WaitingAreaSource;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;

import java.io.Serializable;
import java.util.Properties;

/**
 * 冲突点计算
 */
public class ConflictEventMain implements Serializable {

    private StreamExecutionEnvironment env;
    private PropertiesHelper instance;
    private Properties p;
    private boolean isDebug;
    private String path;

    private ConflictEventMain(StreamExecutionEnvironment env) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
    }

    public static ConflictEventMain init(StreamExecutionEnvironment env) {
        return new ConflictEventMain(env);
    }
    private ConflictEventMain(StreamExecutionEnvironment env, String path,boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    public static ConflictEventMain init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new ConflictEventMain(env, path, isDebug);
    }

    public void run(DataStream<CrossFrameModel> streamSource) {

        /**
         * 转换数据为实体类 并筛选需要计算的数据（路口面 上的数据）
         */
        SingleOutputStreamOperator<CrossFrameModel> eventStream = streamSource
                .flatMap(new ConflictFrameFlatMap())
                .setParallelism(1)
                .name("冲突点-解析数据");

        /**
         * 读取路口待转区配置 并过滤待转区
         */
        SingleOutputStreamOperator<CrossFrameModel> flatMapWaitingArea = eventStream
                 .flatMap(new ConflictWaitingAreaCoFlatMap())
                 .setParallelism(1)
                 .name("冲突点-匹配待转区并过滤其数据");

        //加载冲突相位信息
        SingleOutputStreamOperator<CrossFrameModel> loadConflictSourceProcess = flatMapWaitingArea
                .connect(env.addSource(new ConflictTurnSource()).broadcast())
                .flatMap(new ConflictSlotCoFlatMap())
                .setParallelism(1)
                .name("冲突点检测-加载冲突相位信息");

        //路口冲突检测
        SingleOutputStreamOperator<EventCross> processWindowStopAndRuning = loadConflictSourceProcess
                .keyBy(CrossFrameModel::getCrossId)
                .process(new ConflictCheckKeyedProcess())
                .setParallelism(1)
                .name("冲突点检测-路口冲突转向车辆检测");

        /**
         *
         */
//        SingleOutputStreamOperator<EventCross> processWindowStopAndRuning = flatMapWaitingArea
//                .flatMap(new ConflictCheckConflictFlatMap())
//                .setParallelism(1)
//                .name("冲突点-计算是否发生冲突");

        SingleOutputStreamOperator<EventCross> durationStream = processWindowStopAndRuning
                .keyBy("crossId","trackId")
                .process(new ConflictDurationKeyedProcess())
                .setParallelism(1)
                .name("冲突点-筛选满足持续时间的冲突点");

        /**
         * 去重
         */
        SingleOutputStreamOperator<EventCross> eventMapState = durationStream
                .keyBy("crossId","trackId")
                .process(new ConflictDeduplicationKeyedProcess())
                .setParallelism(1)
                .name("冲突点-结果去重");

        /**
         * 构建结果集
         */
        SingleOutputStreamOperator<String> eventTestSingleOutput = eventMapState
                .flatMap(new ConflictEncapsulationResultFlatMap())
                .setParallelism(1)
                .name("冲突点-封装结果数据");

        if (isDebug) {
            /**
             * 转换数据
             */
            SingleOutputStreamOperator<BaseEventResultModel> stringSingleOutputStream = eventTestSingleOutput
                    .flatMap(new FlatMapFunction<String, BaseEventResultModel>() {
                        @Override
                        public void flatMap(String jsonData, Collector<BaseEventResultModel> collector) throws Exception {
                            ConflictPointModelBase result = JSONObject.parseObject(jsonData,ConflictPointModelBase.class);
                            collector.collect(result);
                        }
                    })
                    .setParallelism(1);
            stringSingleOutputStream.addSink(new EventFileSink(this.path));
        } else {
            /**
             * 安全专题数据产出
             */
            FlinkKafkaProducer producer1 = new FlinkKafkaProducer<>(
                    p.getProperty("cross.event.data.topic"),
                    new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                    instance.getProducerProperties(),
                    FlinkKafkaProducer.Semantic.NONE);
            eventTestSingleOutput.addSink(producer1).setParallelism(1).name("冲突点-结果输出至kafka");
            eventTestSingleOutput.print("冲突点-结果").name("冲突点-结果打印");
        }

    }


}
