package com.wanji.indicators.event.cross.conflictpoint.func;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.event.confilct.EventCross;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ConflictCheckConflictFlatMapNew implements FlatMapFunction<CrossFrameModel, EventCross> {
    @Override
    public void flatMap(CrossFrameModel frameModel, Collector<EventCross> collector) throws Exception {
        ConfigModel configModel = new ConfigModel();
        //判断该车道内的是否同时存在行驶车辆和停驶车辆 如果同时存在则 set.size = 2 否则 set.size = 1
        Set<Integer> keyStop = new HashSet<>();
        Set<Integer> keyRun = new HashSet<>();
        //存放停止的车辆
        List<EventCross> listStop = new ArrayList<>();
        //存放运行的车辆
        List<EventCross> listRun = new ArrayList<>();

        List<CarTrackModel> trackList = frameModel.getTrackList();
        for (CarTrackModel carTrack : trackList) {
            double speed = carTrack.getSpeed();
            int trackId = carTrack.getId();
            EventCross eventCross = toEventCross(carTrack,frameModel.getGlobalTimeStamp());
            eventCross.setConfigModel(configModel);
            if (speed <= 5.0) {
                if (keyStop.add(trackId)) {
                    listStop.add(eventCross);
                }
            } else {
                if (keyRun.add(trackId)) {
                    listRun.add(eventCross);
                }
            }
        }

        for (EventCross stop : listStop) {
            int trackIdA = stop.getTrackId();
            String ridA = stop.getRid();
            //停止车辆的运行角度
            double driveAngleA = stop.getDriveAngle();
            double lngA = stop.getLng();
            double latA = stop.getLat();
            boolean isContinue = false;

            for (CarTrackModel carTrack : trackList) {
                if (stop.getTrackId() != carTrack.getId() &&
                        stop.getLaneFunction().contains(Constant.STRAIGHT_LINE_CODE) &&
                        stop.getLaneFunction().contains(Constant.LEFT_TURN_CODE) &&
                        StringUtils.isNotBlank(stop.getRid()) &&
                        stop.getRid().equals(carTrack.getRoadnet().getRid())) {
                    //判定停止车辆前是否有同行车辆
                    double distance = PtInPolyUtil.getDistance(stop.getLng(), stop.getLat(), carTrack.getLongitude(), carTrack.getLatitude());
                    if (distance < configModel.getConflictFilterSameDirTravelDist()) {
                        double angle = PtInPolyUtil.getAngle(stop.getLng(), stop.getLat(), carTrack.getLongitude(), carTrack.getLatitude());
                        //计算a车行驶方向和前车连线方向是否一致 确定前车是否在正前方
                        double angleX = Math.abs(stop.getDriveAngle() - angle);
                        if (angleX < 10.0 || angleX > 350.0) {
                            isContinue = true;
                            break;
                        }
                    }
                }
            }
            //从直左混合车道驶出的停车车辆
            if (isContinue) {
                continue;
            }
            for (EventCross run : listRun) {
                int trackIdB = run.getTrackId();
                String ridB = run.getRid();
                boolean isRun = true;
                //两方向车道功能都是直行的且行车方向相反则排除
                if (Constant.STRAIGHT_LINE_CODE.equals(stop.getLaneFunction()) && Constant.STRAIGHT_LINE_CODE.equals(run.getLaneFunction())) {
                    double angleX = Math.abs(stop.getDriveAngle() - run.getDriveAngle());
                    if (angleX > 160.0 && angleX < 200.0) {
                        isRun = false;
                    }
                }
                //两方向车道功能都是左转的且行车方向相反则排除
                if (Constant.LEFT_TURN_CODE.equals(stop.getLaneFunction()) && Constant.LEFT_TURN_CODE.equals(run.getLaneFunction())) {
                    double angleX = Math.abs(stop.getDriveAngle() - run.getDriveAngle());
                    if (angleX > 160.0 && angleX < 200.0) {
                        isRun = false;
                    }
                }

                if (isRun && trackIdA != trackIdB && !ridA.equals(ridB)) {
                    double lngB = run.getLng();
                    double latB = run.getLat();
                    //车头->延长1.2后的点作为车头坐标
                    double[] head = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(driveAngleA, lngA, latA, configModel.getConflictStopCarAlongLengOfHead());//延长1.2m
                    double distance = PtInPolyUtil.getDistance(head[0], head[1], lngB, latB);//车头位置与行驶车辆的距离
                    double driveAngleB = run.getDriveAngle();
                    double angleX = Math.abs(driveAngleA - driveAngleB);
                    //默认条件：两车距离小于5米，两车视野角度在(60,120)范围内
                    if (distance < configModel.getConflictTwoCarMaxDist()
                            && angleX > configModel.getConflictTwoCarMinAngle()
                            && angleX < configModel.getConflictTwoCarMaxAngle()) {//停车车辆与行驶车辆两车角度差计算范围
                        //A 反向延长2.5m后的坐标作为车尾坐标
                        double[] doublesA = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(getBackAngle(driveAngleA), lngA, latA, configModel.getConflictStopCarAlongLengOfRear());
                        //A前 停车车辆向前延长线
                        double[] doublesAS = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(driveAngleA, lngA, latA, configModel.getConflictStopCarForwardAlongLength());
                        //A后 停车车辆向后延长线
                        double[] doublesAE = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(getBackAngle(driveAngleA), lngA, latA, configModel.getConflictStopCarBackAlongLength());
                        //B 行驶车辆长度阈值
                        double[] doublesB = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(getBackAngle(driveAngleB), lngB, latB, configModel.getConflictTravelCarLength());
                        //B前 行驶车辆向前延长线长度
                        double[] doublesBS = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(driveAngleB, lngB, latB, configModel.getConflictTravelCarForwardAlongLength());
                        //B后 行驶车辆向后延长线长度
                        double[] doublesBE = PtInPolyUtil.calLocationByDistanceAndLocationAndDirection(getBackAngle(driveAngleB), lngB, latB, configModel.getConflictTravelCarBackAlongLength());
                        //A前 交 B后
                        int isIntersectA = PtInPolyUtil.segIntersect(lngA, latA, doublesAS[0], doublesAS[1], doublesB[0], doublesB[1], doublesBE[0], doublesBE[1]);
                        //B前 交 A后
                        int isIntersectB = PtInPolyUtil.segIntersect(doublesA[0], doublesA[1], doublesAE[0], doublesAE[1], lngB, latB, doublesBS[0], doublesBS[1]);
                        if (isIntersectA != 1 && isIntersectB != 1) {
                            double angle = PtInPolyUtil.getAngle(head[0], head[1], lngB, latB);
                            double abs = Math.abs(angle - driveAngleA);
                            if (abs < configModel.getConflictResultAngleDif()) {
                                //停止车辆车头位置与行驶车辆角度差
                                stop.setTrackIdNear(stop.getTrackId() + Constant.COMMA + run.getTrackId());
                                stop.setNearTrackObj(run);
                                collector.collect(stop);
                            }
                        }
                        doublesA = null;
                        doublesB = null;
                        doublesAS = null;
                        doublesBS = null;
                        doublesAE = null;
                        doublesBE = null;
                    }
                }
            }
        }
        keyStop = null;
        keyRun = null;
        listStop = null;
        listRun = null;
    }

    /**
     * 计算反向角度
     * @param angle
     * @return
     */
    private static double getBackAngle(double angle) {
        if (angle >= 180) {
            return angle - 180;
        } else {
            return angle + 180;
        }
    }

    private static EventCross toEventCross(CarTrackModel carTrack,long timeTamp){
        EventCross e = new EventCross();


        e.setCrossId(carTrack.getRoadnet().getCrossId());
        e.setRid(carTrack.getRoadnet().getRid());
        e.setEntryLane(carTrack.getRoadnet().getLaneId());
        e.setLaneFunction(TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn()));
        e.setTrackId(carTrack.getId());
        e.setLng(carTrack.getLongitude());
        e.setLat(carTrack.getLatitude());

        e.setSpeed(carTrack.getSpeed());
        e.setDriveAngle(carTrack.getCourseAngle());
        e.setPlateNumber(carTrack.getPicLicense());
        e.setVehicleType(carTrack.getOriginalType());
        e.setTime(timeTamp);

        if (VehicleTypeEnum.isMotorVehicles((short) e.getVehicleType())) {
            e.setCategory(1);
        } else if (VehicleTypeEnum.isNonMotorVehicles((short) e.getVehicleType())) {
            e.setCategory(2);
        } else {
            e.setCategory(3);
        }

        return e;
    }
}
