package com.wanji.indicators.event.cross.conflictpoint.func;

import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.confilct.EventCross;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.java.tuple.Tuple;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

public class ConflictDurationKeyedProcess extends KeyedProcessFunction<Tuple, EventCross, EventCross> {

    MapState<String, EventCross> mapState = null;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("eventMapStat-DurationC", String.class, EventCross.class));
    }

    @Override
    public void processElement(EventCross event, Context ctx, Collector<EventCross> out) throws Exception {
        String key = event.getCrossId() + "-" + event.getTrackId();
        EventCross value = mapState.get(key);
        if (value == null) {
            mapState.put(key, event);
            ctx.timerService().registerProcessingTimeTimer(System.currentTimeMillis() + 180000L);
        } else {
            EventCross agoModel = mapState.get(key);
            long agoTime = agoModel.getTime();
            long nowTime = event.getTime();
            ConfigModel configModel = event.getConfigModel();
            long duration = nowTime - agoTime;
            if (duration > configModel.getConflictDuration()){
                event.setDuration(duration);
                //持续3秒输出
                out.collect(event);
            }
        }
        key = null;
    }

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<EventCross> out) throws Exception {
        mapState.clear();
    }
}
