package com.wanji.indicators.event.cross.conflictpoint.func;

import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ConflictLaneInfoCoFlatMap implements CoFlatMapFunction<CrossFrameModel, Map<String, BaseLaneInfo>, CrossFrameModel> {

    private Map<String, BaseLaneInfo> map = new HashMap();

    @Override
    public void flatMap1(CrossFrameModel frameModel, Collector<CrossFrameModel> collector) throws Exception {

        List<CarTrackModel> trackList = frameModel.getTrackList();

        List<CarTrackModel> resList = new ArrayList<>();

        for (CarTrackModel carTrack : trackList) {
            String entryLane = carTrack.getRoadnet().getLaneId();

            BaseLaneInfo baseLaneInfo = map.get(entryLane);
            if (baseLaneInfo != null) {
                int ridDir = baseLaneInfo.getDir();
                String laneLngLat = baseLaneInfo.getWkt();
                String[] xys = laneLngLat.split(";");
                String laneStartPoint = xys[0];
                Double fx = Double.parseDouble(laneStartPoint.split(",")[0]);
                Double fy = Double.parseDouble(laneStartPoint.split(",")[1]);

                String laneEndPoint = xys[xys.length-1];
                Double ex = Double.parseDouble(laneEndPoint.split(",")[0]);
                Double ey = Double.parseDouble(laneEndPoint.split(",")[1]);

                double laneAngle = PtInPolyUtil.getAngle(fx,fy,ex,ey);

                Integer type = baseLaneInfo.getType();
                carTrack.getRoadnet().setLaneType(type);
                carTrack.getRoadnet().setLaneStartPoint(laneStartPoint);
                carTrack.getRoadnet().setLaneEndPoint(laneEndPoint);
                carTrack.getRoadnet().setTurn(baseLaneInfo.getTurn());
                carTrack.getRoadnet().setRidDir8(ridDir);
                carTrack.getRoadnet().setLaneAngle(laneAngle);
                carTrack.getRoadnet().setCrossId(baseLaneInfo.getCrossId());
            }
            resList.add(carTrack);
        }

        frameModel.setTrackList(resList);

        collector.collect(frameModel);
    }

    @Override
    public void flatMap2(Map<String, BaseLaneInfo> hashMap, Collector<CrossFrameModel> collector) throws Exception {
        this.map = hashMap;
    }
}
