package com.wanji.indicators.event.cross.congestion;

import com.wanji.indicators.event.cross.congestion.funcnew.*;
import com.wanji.indicators.model.BaseEventResultModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.event.congestion.CongestionFrameModel;
import com.wanji.indicators.model.event.congestion.CongestionModel;
import com.wanji.indicators.model.event.congestion.CongestionOverflowModel;
import com.wanji.indicators.source.ConsulAreaSource;
import com.wanji.indicators.source.RidDirSource;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.util.Properties;

/**
 * 拥堵溢出检测
 * @author fengyi
 */
public class CongestionEventMainNew implements Serializable {

    private static final Logger log = LoggerFactory.getLogger(CongestionEventMainNew.class);

    private StreamExecutionEnvironment env;
    private PropertiesHelper instance;
    private Properties p;
    private String path;
    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private CongestionEventMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }
    public static CongestionEventMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new CongestionEventMainNew(env, path,isDebug);
    }

    private CongestionEventMainNew(StreamExecutionEnvironment env) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
    }

    public static CongestionEventMainNew init(StreamExecutionEnvironment env) {
        return new CongestionEventMainNew(env);
    }

    public void run(DataStream<FrameModel> thinningDataStream) {

        FlinkKafkaProducer<String> kafkaProducer = new FlinkKafkaProducer<>(
                p.getProperty("cross.event.data.topic"),
                new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);

        //解析数据
        SingleOutputStreamOperator<CongestionFrameModel> accidentFrameModelFilter = null;
//                thinningDataStream
//                .flatMap(new CongestionFlatMap())
//                .setParallelism(1)
//                .name("路口溢出-解析数据");

        //计算车辆在路口内的旅行时间
        SingleOutputStreamOperator<CongestionFrameModel> checkDataKeyedProcess = accidentFrameModelFilter
                .keyBy(CongestionFrameModel::getCrossId)
                .process(new CongestionCheckDataKeyedProcess())
                .setParallelism(1)
                .name("路口溢出-计算车辆在路口内的旅行时间");

        //匹配待转区
//        SingleOutputStreamOperator<CongestionFrameModel> checkWaitLaneKeyedProcess = checkDataKeyedProcess
//                .connect(env.addSource(new WaitLaneSource()).broadcast())
//                .flatMap(new CongestionCheckWaitLaneCoFlatMap())
//                .setParallelism(1)
//                .name("路口溢出-匹配待转区");

        //匹配拥堵检测区域
        SingleOutputStreamOperator<CongestionFrameModel> checkCongestionArea = checkDataKeyedProcess
                .connect(env.addSource(new ConsulAreaSource()).broadcast())
                .flatMap(new CongestionAreaCheckCoFlatMap())
                .setParallelism(1)
                .name("事故检测-匹配拥堵检测区域");

        //匹配对应拥堵出口rid的方向
        SingleOutputStreamOperator<CongestionFrameModel> checkDirCoFlatMap = checkCongestionArea
                .connect(env.addSource(new RidDirSource()).broadcast())
                .flatMap(new CongestionCheckDirCoFlatMap())
                .setParallelism(1)
                .name("路口溢出-匹配车辆对应的出口rid的方向");

        //计算是否疑似路口溢出 开始/消散
        SingleOutputStreamOperator<CongestionModel> checkCongestionKeyedProcess = checkDirCoFlatMap
                .keyBy(CongestionFrameModel::getCrossId)
                .process(new CongestionKeyedProcess())
                .setParallelism(1)
                .name("路口溢出-计算是否路口溢出");

        //读取路口形状并根据 "车道转向功能" 和 "进口rid" 匹配对应的  "出口rid"
//        SingleOutputStreamOperator<CongestionModel> checkRidInCoFlatMap = checkCongestionKeyedProcess
//                .connect(env.addSource(new CrossShapeSource()).broadcast())
//                //.flatMap(new CongestionCheckOutRidCoFlatMap())
//                //.setParallelism(1)
//                .name("路口溢出-匹配车辆对应的出口rid");

        //路口溢出-逻辑去重
        SingleOutputStreamOperator<CongestionOverflowModel> congestionOverflowStream = checkCongestionKeyedProcess
                .keyBy("crossId", "ridOut", "startTime")
                .process(new CongestionValueStateKeyedProcess())
                .setParallelism(1)
                .name("路口溢出-逻辑去重");

        if (!isDebug) {
            SingleOutputStreamOperator<String> stringSingleOutputStreamOperator = congestionOverflowStream
                    .flatMap(new CongestionToStringFlatMap())
                    .setParallelism(1)
                    .name("路口溢出-封装结果集");

            stringSingleOutputStreamOperator.addSink(kafkaProducer).setParallelism(1).name("路口溢出-结果输出至kafka");
            stringSingleOutputStreamOperator.print("拥堵溢出检测-结果").name("路口溢出-结果打印");
        } else {

            SingleOutputStreamOperator<BaseEventResultModel> baseEventResultModelStream = congestionOverflowStream.flatMap(new FlatMapFunction<CongestionOverflowModel, BaseEventResultModel>() {
                @Override
                public void flatMap(CongestionOverflowModel congestionOverflowModel, Collector<BaseEventResultModel> collector) throws Exception {
                    collector.collect(congestionOverflowModel);
                }
            });

           // baseEventResultModelStream.addSink(new EventFileSink(this.path));


        }




    }

}
