package com.wanji.indicators.event.cross.congestion.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.congestion.CongestionAreaDo;
import com.wanji.indicators.model.event.congestion.CongestionFrameModel;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;

/**
 * 2022/3/17 15:52
 *
 * @auther fengyi
 */
public class CongestionAreaCheckCoFlatMap implements CoFlatMapFunction<CongestionFrameModel, HashMap<String, Map<String, CongestionAreaDo>>, CongestionFrameModel> {

    private HashMap<String, Map<String, CongestionAreaDo>> hashMap = new HashMap();

    @Override
    public void flatMap1(CongestionFrameModel congestionFrameModel, Collector<CongestionFrameModel> collector) throws Exception {
        List<CongestionFrameModel.CarTrack> trackList = congestionFrameModel.getTrackList();
        ConfigModel configModel = congestionFrameModel.getConfigModel();
        List<CongestionFrameModel.CarTrack> list = new ArrayList<>();
        Set<Integer> set = new HashSet<>();
        Map<String, CongestionAreaDo> congestionAreaDoMap = hashMap.get(congestionFrameModel.getCrossId());
        if (congestionAreaDoMap != null) {
            Map<String, Integer> map = new HashMap<>();
            for (CongestionAreaDo congestionAreaDo : congestionAreaDoMap.values()) {
                int laneNum = congestionAreaDo.getLaneNum();
                if (laneNum > 0) {
                    //存入各出口rid对应的车道数
                    map.put(congestionAreaDo.getRid(), laneNum);
                }
            }
            congestionFrameModel.setCongestionInfo(map);

            for (CongestionFrameModel.CarTrack carTrack : trackList) {
                String entryLane = carTrack.getEntryLane();
                if (carTrack.getCrossing() == Constant.CROSSING && carTrack.getSpeed() < configModel.getAbnormalParkCongestSpeed() && StringUtils.isNotBlank(entryLane)) {
                    for (CongestionAreaDo congestionAreaDo : congestionAreaDoMap.values()) {
                        String ridIn = entryLane.substring(0, 23);
                        //判断是否属于该方向验证的入口rid数据
                        if (congestionAreaDo.getRidInArr().contains(ridIn)) {
                            PtInPolyUtil.Point point = new PtInPolyUtil.Point(carTrack.getLng(), carTrack.getLat());
                            boolean isIn = PtInPolyUtil.isPtInPoly(point, congestionAreaDo.getPoints());
                            if (isIn) {
                                if (set.add(carTrack.getTrackId())) {
                                    carTrack.setRidOut(congestionAreaDo.getRid());
                                    list.add(carTrack);
                                }
                            }
                        }
                    }
                }
            }
            for (CongestionFrameModel.CarTrack carTrack : trackList) {
                if (!set.contains(carTrack.getTrackId())) {
                    list.add(carTrack);
                }
            }
            congestionFrameModel.setTrackList(list);
        }
        collector.collect(congestionFrameModel);
    }

    @Override
    public void flatMap2(HashMap<String, Map<String, CongestionAreaDo>> stringCongestionAreaDoHashMap, Collector<CongestionFrameModel> collector) throws Exception {
        this.hashMap = stringCongestionAreaDoHashMap;
    }


}
