package com.wanji.indicators.event.cross.congestion.funcnew;

import com.wanji.indicators.model.event.congestion.CongestionFrameModel;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.*;

/**
 * 2021/11/18 16:47
 *
 * @auther fengyi
 */
public class CongestionCheckDataKeyedProcess extends KeyedProcessFunction<String, CongestionFrameModel, CongestionFrameModel> {

    private MapState<Integer, CongestionFrameModel.CarTrack> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateCongestion_ridCheck_11x", Integer.class, CongestionFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(CongestionFrameModel congestionFrameModel, Context context, Collector<CongestionFrameModel> collector) throws Exception {
        long timeNow = congestionFrameModel.getTime();
        Map<Integer,CongestionFrameModel.CarTrack> map = new HashMap<>();
        List<CongestionFrameModel.CarTrack> trackList = congestionFrameModel.getTrackList();
        List<CongestionFrameModel.CarTrack> list = new ArrayList<>();
        for (CongestionFrameModel.CarTrack carTrack : trackList) {
            int trackId = carTrack.getTrackId();
            map.put(trackId, carTrack);
            CongestionFrameModel.CarTrack carTrackFirst = mapState.get(trackId);
            if  (carTrackFirst == null) {
                carTrack.setTravelTime(0);
                mapState.put(trackId, carTrack);
            } else {
                long duration = carTrack.getTime() - carTrackFirst.getTime();
                if (duration > 0L) {
                    carTrack.setTravelTime(duration/1000.0);
                }
            }
            list.add(carTrack);
        }

        //清除过期数据
        Iterator<CongestionFrameModel.CarTrack> iterator = mapState.values().iterator();
        List<CongestionFrameModel.CarTrack> listMapState = new ArrayList<>();
        while (iterator.hasNext()) {
            CongestionFrameModel.CarTrack next = iterator.next();
            listMapState.add(next);
        }
        for (CongestionFrameModel.CarTrack next : listMapState) {
            int trackId = next.getTrackId();
            CongestionFrameModel.CarTrack carTrack = map.get(trackId);
            //查询当前帧数据中没有此trackId 说明该目标 上一帧有 当前帧没有
            if (carTrack == null) {
                //计算当前此目标与时间差多少毫秒 若超过10秒则清除该目标
                long timeX = timeNow - next.getTime();
                if (timeX > 10000L) {
                    mapState.remove(trackId);
//                    System.out.println(" 数据过期 " + trackId + " " + timeX + " " + list.size());
                }
            }
        }

        //发送数据
        if (!list.isEmpty()) {
            congestionFrameModel.setTrackList(list);
            collector.collect(congestionFrameModel);
        }
    }
}
