package com.wanji.indicators.event.cross.congestion.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.congestion.AccidentCongestionModel;
import com.wanji.indicators.model.event.congestion.CongestionFrameModel;
import com.wanji.indicators.model.event.congestion.CongestionModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 2021/11/19 17:54
 *
 * @auther fengyi
 */
public class CongestionKeyedProcess extends KeyedProcessFunction<String, CongestionFrameModel, CongestionModel> {

    private static final Logger log = LoggerFactory.getLogger(CongestionKeyedProcess.class);

    private MapState<String, AccidentCongestionModel> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateCongestion_01x33330", String.class, AccidentCongestionModel.class));
    }

    @Override
    public void processElement(CongestionFrameModel congestionFrameModel, Context context, Collector<CongestionModel> collector) throws Exception {
        long timeNow = congestionFrameModel.getTime();
        ConfigModel configModel = congestionFrameModel.getConfigModel();
        Map<String, Integer> congestionInfo = congestionFrameModel.getCongestionInfo();
        //按rid分组当前帧的车辆
        Map<String, List<CongestionFrameModel.CarTrack>> ridMap = keyByRidMap(congestionFrameModel);
        for (String ridKey : ridMap.keySet()) {
            List<CongestionFrameModel.CarTrack> list = ridMap.get(ridKey);
            AccidentCongestionModel congestionAgo = mapState.get(ridKey);
            if (congestionAgo != null) {
                //计算时间差
                long timeX = timeNow - congestionAgo.getTime();
                //获取上一帧累计的拥堵持续时间
                long duration = congestionAgo.getDuration();
                if (timeX > 0L && timeX < 10000L) {
                    duration += timeX;
                }
                //构建缓存数据
                AccidentCongestionModel model = getStateModel(ridKey, duration, timeNow, congestionAgo);
                //构建结果数据
                CongestionModel result = getResult(model, congestionFrameModel, list.get(0).getRid());

                //读取拥堵三色预警时间阈值配置，单位秒
                long minDuration = 30;//configModel.getMinDuration(EventType.OVERFLOW_CONGESTION.getCode());
                //计算当前帧是否拥堵 拥堵状态是否持续
                boolean congestion = isCongestion(list, configModel.getOverFlowCongestDissipationCarNums(), configModel);
                if (congestion) {
                    //若当前帧拥堵则将当前帧时间缓存更新至状态  更新了拥堵的持续时间
                    mapState.put(ridKey, model);
                    //判定是否输出路口溢出事件
                    if  (model.getDuration() > minDuration) {
                        collector.collect(result);
                        log.info("LOG_EVENT_CONGESTION >>> 拥堵持续: " + ridKey +
                                " " + DateFormatUtils.format(model.getStartTime(), DateUtil.YYYY_MM_DD_HH_MM_SS) +
                                " - " + DateFormatUtils.format(model.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                                "  du: " + model.getDuration() + "  ridOut: " + ridKey +
                                "  dir:" + list.get(0).getRidOutDir() + "  checkTime: " + minDuration);
                    }
                } else {
                    //计算是否消散 未消散则持续输出数据
                    if (dissipation(timeNow, timeX, model.getDuration(), minDuration, ridKey, congestionFrameModel, congestionAgo, model)) {
                        collector.collect(result);
                    }
                }
            } else {
                if (congestionInfo != null && congestionInfo.get(ridKey) != null) {
                    Integer lane_num = congestionInfo.get(ridKey);
                    //计算当前帧是否拥堵 拥堵则将状态放入缓存
                    checkCongestion(ridKey, congestionFrameModel, list, lane_num);
                }
            }
        }
    }

    /**
     * 构建缓存数据结构  中间缓存存储的数据
     * @param ridKey
     * @param duration
     * @param nowTime
     * @param congestionAgo
     * @return
     */
    private AccidentCongestionModel getStateModel(String ridKey, long duration, long nowTime, AccidentCongestionModel congestionAgo) {
        AccidentCongestionModel model = new AccidentCongestionModel();
        model.setKey(ridKey);
        model.setStartTime(congestionAgo.getStartTime() / 1000 * 1000);
        model.setTime(nowTime);
        model.setDuration(duration);
        model.setRidOut(congestionAgo.getRidOut());
        model.setRidOutDir(congestionAgo.getRidOutDir());
        model.setLng(congestionAgo.getLng());
        model.setLat(congestionAgo.getLat());
        model.setRecoveryCross(congestionAgo.getRecoveryCross());
        return model;
    }

    /**
     * 构建结果集
     * @param model
     * @param congestionFrameModel
     * @return
     */
    private CongestionModel getResult(AccidentCongestionModel model, CongestionFrameModel congestionFrameModel, String ridIn) {
        CongestionModel result = new CongestionModel();
        result.setCrossId(congestionFrameModel.getCrossId());
        result.setRidIn(ridIn);
        result.setStartTime(model.getStartTime());
        result.setEndTime(model.getTime());
        long duReal = model.getTime() - model.getStartTime();
        if (duReal - model.getDuration() > 120000L)  {
            result.setDuration(duReal);
        } else {
            result.setDuration(model.getDuration());
        }
        result.setRidOut(model.getRidOut());
        result.setDirection(model.getRidOutDir());
        result.setLng(model.getLng());
        result.setLat(model.getLat());
        if (StringUtils.isNotBlank(result.getDirection()) && !"null".equals(result.getDirection())) {
            result.setDirection(result.getDirection() + "出口");
        } else {
            result.setDirection("");
        }
        result.setRecoveryCross(model.getRecoveryCross());
        result.setConfigModel(congestionFrameModel.getConfigModel());
        return result;
    }


    /**
     * 计算消散状态
     * @param timeNow
     * @param timeX
     * @param duration
     * @param minDuration
     * @param ridKey
     * @param congestionFrameModel
     * @param congestionAgo
     * @param model
     * @return
     * @throws Exception
     */
    private boolean dissipation(long timeNow, long timeX, long duration, long minDuration, String ridKey, CongestionFrameModel congestionFrameModel,
                             AccidentCongestionModel congestionAgo, AccidentCongestionModel model) throws Exception {
        long du = timeNow - congestionAgo.getStartTime();
        ConfigModel configModel = congestionFrameModel.getConfigModel();
        //若当前计算到不拥堵了 则延迟60s清除拥堵缓存标识
        if (timeX > configModel.getOverFlowCongestDissipationDuration() || du > configModel.getOverFlowCongestSplitMaxDuration()) {
            mapState.remove(ridKey);
            mapState.remove(congestionFrameModel.getCrossId());
            log.info("LOG_EVENT_CONGESTION >>> 拥堵消散: " + ridKey + "  du:" + duration +
                    " " + DateFormatUtils.format(model.getStartTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                    " - " + DateFormatUtils.format(model.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                    " | " + DateFormatUtils.format(timeNow,DateUtil.YYYY_MM_DD_HH_MM_SS) +
                    " - " + DateFormatUtils.format(congestionAgo.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                    " timeX:" + timeX);

        } else {
            //若当前帧拥堵则将当前帧时间缓存至状态  更新了拥堵的持续时间
            mapState.put(ridKey, model);
            //判定是否输出路口溢出事件
            if  (duration > minDuration) {
                return true;
            } else {
                //若当前帧拥堵持续时间未满足
                if (timeX > configModel.getOverFlowCongestUnsatisfiedDurationClear()) {
                    mapState.remove(ridKey);
                    mapState.remove(congestionFrameModel.getCrossId());
                    log.info("LOG_EVENT_CONGESTION >>> 不足30秒-清除缓存-拥堵消散: " + ridKey + "  du:" + duration +
                            " " + DateFormatUtils.format(model.getStartTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                            " - " + DateFormatUtils.format(model.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                            " | " + DateFormatUtils.format(timeNow,DateUtil.YYYY_MM_DD_HH_MM_SS) +
                            " - " + DateFormatUtils.format(congestionAgo.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                            " timeX:" + timeX);
                }
            }
        }
        return false;
    }

    /**
     * 计算拥堵状态
     * @param ridKey
     * @param congestionFrameModel
     * @param list
     * @throws Exception
     */
    private void checkCongestion (String ridKey, CongestionFrameModel congestionFrameModel,
                                  List<CongestionFrameModel.CarTrack> list, int congestCarNum) throws Exception {
        boolean congestion = isCongestion(list, congestCarNum, congestionFrameModel.getConfigModel());
        if (congestion) {
            AccidentCongestionModel crossCongestion = mapState.get(congestionFrameModel.getCrossId());
            if (crossCongestion == null) {
                CongestionFrameModel.CarTrack carTrack = list.get(0);
                AccidentCongestionModel model = new AccidentCongestionModel();
                model.setKey(ridKey);
                model.setStartTime(congestionFrameModel.getTime() / 1000 * 1000);
                model.setTime(congestionFrameModel.getTime());
                model.setDuration(0L);
                model.setRidOut(ridKey);
                model.setRidOutDir(carTrack.getRidOutDir());
                model.setLng(carTrack.getLng());
                model.setLat(carTrack.getLat());
                model.setRecoveryCross(carTrack.getRecoveryCross());

                //若当前帧拥堵则将当前帧时间缓存至状态
                mapState.put(ridKey, model);
                mapState.put(congestionFrameModel.getCrossId(), model);
                log.info("LOG_EVENT_CONGESTION >>> 发现拥堵: " + ridKey +
                        " " + DateFormatUtils.format(model.getStartTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                        " - " + DateFormatUtils.format(model.getTime(),DateUtil.YYYY_MM_DD_HH_MM_SS) +
                        "  ridOut: " + carTrack.getRidOut() + "  dir:" + carTrack.getRidOutDir());
            }
        }
    }

    /**
     * 计算当前帧是否拥堵
     * @param list      当前帧车辆集合
     * @param checkNum  拥堵车辆判定数量阈值
     * @return
     */
    private boolean isCongestion(List<CongestionFrameModel.CarTrack> list, long checkNum,ConfigModel configModel) {
        long congestionNum = 0;
        for (CongestionFrameModel.CarTrack carTrack : list) {
            double waitLaneDistance = carTrack.getWaitLaneDistance() + 2.5;
            double areaDist = carTrack.getAreaDist();
            //计算车辆是否驶出待转区 若驶出待转区则 areaDist 大于 waitLaneDistance
            if (areaDist > waitLaneDistance) {
                //计算旅行时间超过10s且车速低于10km/h的车辆
                if (carTrack.getTravelTime() > configModel.getOverFlowCongestCarTravelTime()
                        && carTrack.getSpeed() < configModel.getOverFlowCongestSpeed()) {
                    congestionNum++;
                }
            }
        }
        //行驶缓慢的车辆数超过传值的阈值则认为路口发生路口溢出
        if (congestionNum > checkNum) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 按照rid分组计算当前帧的所有车辆
     *
     * @param accidentCongestionFrameModel
     * @return
     */
    private Map<String, List<CongestionFrameModel.CarTrack>> keyByRidMap(CongestionFrameModel accidentCongestionFrameModel) {
        List<CongestionFrameModel.CarTrack> trackList = accidentCongestionFrameModel.getTrackList();
        Map<String, List<CongestionFrameModel.CarTrack>> listMap = new HashMap<>();
        for (CongestionFrameModel.CarTrack carTrack : trackList) {
            boolean isOk = false;
            String ridOut = carTrack.getRidOut();
            //出口车道第一个渠化段
            if(carTrack.getCrossing() != Constant.CROSSING && carTrack.getStopLineDist() == -1 && StringUtils.isNotBlank(carTrack.getMatchLane())){
                String laneNo = carTrack.getMatchLane().substring(23, 26);
                if("100".equals(laneNo)) {
                    isOk = true;
                    if (StringUtils.isBlank(ridOut)) {
                        ridOut = carTrack.getMatchLane().substring(0,23);
                    }
                }
            }
            if (StringUtils.isNotBlank(ridOut) && carTrack.getCrossing() == Constant.CROSSING) {
                isOk = true;
            }
            if(isOk) {
                List<CongestionFrameModel.CarTrack> carTrackRidList = listMap.get(ridOut);
                if (carTrackRidList == null) {
                    carTrackRidList = new ArrayList<>();
                }
                carTrackRidList.add(carTrack);
                listMap.put(ridOut, carTrackRidList);
            }
        }
        return listMap;
    }

}
