package com.wanji.indicators.event.cross.congestion.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.DirectionFromCnToEn;
import com.wanji.indicators.model.event.congestion.CongestionModel;
import com.wanji.indicators.model.event.congestion.CongestionOverflowModel;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.api.java.tuple.Tuple;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

/**
 * 2022/1/12 17:48
 *
 * @auther fengyi
 */
public class CongestionValueStateKeyedProcess extends KeyedProcessFunction<Tuple, CongestionModel, CongestionOverflowModel> {

    private ValueState<CongestionModel> valueState;

    @Override
    public void open(Configuration parameters) {
        valueState = getRuntimeContext().getState(new ValueStateDescriptor<>("congestionOverFlowValueState_1111III_ox",CongestionModel.class));
    }

    @Override
    public void processElement(CongestionModel congestionModel, Context context, Collector<CongestionOverflowModel> collector) throws Exception {
        CongestionModel value = valueState.value();
        if (value == null) {
            CongestionOverflowModel model = new CongestionOverflowModel();
            model.setCrossId(congestionModel.getCrossId());
            model.setRid(congestionModel.getRidIn());
            model.setRidOut(congestionModel.getRidOut());
            model.setTrackId(0);
//            model.setEventClass(EventType.EVENT.getName());
//            model.setTypeCode(EventType.OVERFLOW_CONGESTION.getCode());
//            model.setEventType(EventType.OVERFLOW_CONGESTION.getName());
            model.setStartTime(congestionModel.getStartTime());
            model.setEndTime(congestionModel.getEndTime());
            model.setTime(congestionModel.getStartTime());
            model.setDesc("路口溢出: 路口" + congestionModel.getDirection() + "车辆行驶缓慢，疑似路口溢出。");
            model.setRecoveryCross(congestionModel.getRecoveryCross());
            if (StringUtils.isNotBlank(congestionModel.getDirection())) {
                String dirCnDesc = congestionModel.getDirection();
                String dirCn = dirCnDesc.substring(0,dirCnDesc.length()-2);

                String dirEn = DirectionFromCnToEn.getEnDir(dirCn);
                model.setDescEn("Overflow congestion: vehicles at the "+dirEn+" exit of the intersection drive slowly, which is suspected of overflow congestion");

                model.setPlateNumber(congestionModel.getDirection());
            } else {
                model.setPlateNumber("路口溢出");
            }
            model.setVehicleType("");
            model.setX(congestionModel.getLng());
            model.setY(congestionModel.getLat());
            model.setLocation(new double[]{congestionModel.getLng(), congestionModel.getLat()});
            model.setDuration(congestionModel.getDuration());
            model.setDir(congestionModel.getDirection());
            model.setPictureTime1(model.getStartTime() - 3000L);
            model.setPictureTime2(model.getStartTime());
            model.setPictureTime3(model.getStartTime() + 3000L);
            model.setCheckOutTime(model.getDuration());
            model.setId(model.getCrossId() + Constant.MARK + model.getRidOut() + Constant.MARK + model.getTypeCode() + Constant.MARK + model.getStartTime());
            collector.collect(model);
            valueState.update(congestionModel);
            context.timerService().registerProcessingTimeTimer(System.currentTimeMillis() + 7200000L);
        }
    }

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<CongestionOverflowModel> out) throws Exception {
        valueState.clear();
    }

}
