package com.wanji.indicators.event.cross.deadlock.func;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.event.deadlock.DeadLockFrameModel;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.*;

/**
 * 处理RID挂接、车道功能转换
 */
public class DeadlockCheckDataKeyedProcess extends KeyedProcessFunction<String, DeadLockFrameModel, DeadLockFrameModel> {

    private MapState<Integer, DeadLockFrameModel.CarTrack> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateDeadlock_ridCheck_1", Integer.class, DeadLockFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(DeadLockFrameModel deadLockFrameModel, Context context, Collector<DeadLockFrameModel> collector) throws Exception {
        long timeNow = deadLockFrameModel.getTime();
        List<DeadLockFrameModel.CarTrack> trackList = deadLockFrameModel.getTrackList();
        Map<Integer, DeadLockFrameModel.CarTrack> map = new HashMap<>();
        List<DeadLockFrameModel.CarTrack> list = new ArrayList<>();
        for (DeadLockFrameModel.CarTrack carTrack : trackList) {
            map.put(carTrack.getTrackId(), carTrack);
            DeadLockFrameModel.CarTrack carTrackState = mapState.get(carTrack.getTrackId());
            String matchLane = carTrack.getMatchLane();
            if (StringUtils.isNotBlank(matchLane)) {
                if (carTrackState == null) {
                    carTrack.setEntryLane(matchLane);
                    carTrack.setRid(carTrack.getMatchLane().substring(0, 23));
                    carTrack.setLaneFunc(CommonUtil.getLaneFunc(carTrack.getMatchLaneName()));
                    mapState.put(carTrack.getTrackId(), carTrack);
                } else {
                    carTrack.setMatchLane(carTrackState.getMatchLane());
                    carTrack.setEntryLane(carTrackState.getEntryLane());
                    carTrack.setRid(carTrackState.getRid());
                    carTrack.setLaneFunc(carTrackState.getLaneFunc());
                }
            } else {
                if (carTrackState == null) {
                    String entryLane = carTrack.getEntryLane();
                    if (StringUtils.isNotBlank(entryLane)) {
                        carTrack.setRid(entryLane.substring(0, 23));
                        carTrack.setLaneFunc(Constant.STRAIGHT_LINE_CODE);
                        double stopLineDist = carTrack.getStopLineDist();
                        if (stopLineDist > 0.0 && stopLineDist < 6.0) {
                            carTrack.setRid(entryLane.substring(0, 23));
                            carTrack.setLaneFunc(CommonUtil.getLaneFunc(carTrack.getMatchLaneName()));
                            mapState.put(carTrack.getTrackId(), carTrack);
                        }
                    }
                } else {
                    if (StringUtils.isNotBlank(carTrack.getEntryLane())) {
                        carTrack.setRid(carTrack.getEntryLane().substring(0, 23));
                        carTrack.setLaneFunc(carTrackState.getLaneFunc());
                    }
                }
            }
            if (StringUtils.isNotBlank(carTrack.getLaneFunc())) {
                list.add(carTrack);
            } else {
                //补录无车道属性的数据
                if (carTrack.getCrossing() == Constant.CROSSING) {
                    carTrack.setMatchLane(Constant.RID_MARK_LANE);
                    carTrack.setEntryLane(Constant.RID_MARK_LANE);
                    carTrack.setRid(Constant.RID_MARK);
                    carTrack.setLaneFunc(Constant.STRAIGHT_LINE_CODE);
                    list.add(carTrack);
                }
            }
        }

        //清除过期数据
        Iterator<DeadLockFrameModel.CarTrack> iterator = mapState.values().iterator();
        List<DeadLockFrameModel.CarTrack> listMapState = new ArrayList<>();
        while (iterator.hasNext()) {
            DeadLockFrameModel.CarTrack next = iterator.next();
            listMapState.add(next);
        }
        for (DeadLockFrameModel.CarTrack next : listMapState) {
            int trackId = next.getTrackId();
            DeadLockFrameModel.CarTrack carTrack = map.get(trackId);
            //查询当前帧数据中没有此trackId 说明该目标 上一帧有 当前帧没有
            if (carTrack == null) {
                //计算当前此目标与时间差多少毫秒 若超过3秒则清除该目标
                long timeX = timeNow - next.getTime();
                if (timeX > 3000L) {
                    mapState.remove(trackId);
                }
            }
        }
        //发送数据
        if (!list.isEmpty()) {
            deadLockFrameModel.setTrackList(list);
            collector.collect(deadLockFrameModel);
        }
    }
}
