package com.wanji.indicators.event.cross.deadlock.func;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.deadlock.DeadLockFrameModel;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 检测是否存在冲突运行的车辆
 */
public class DeadlockConflictCheckKeyedProcess extends KeyedProcessFunction<String, DeadLockFrameModel, DeadLockFrameModel> {

    @Override
    public void open(Configuration parameters) {
    }

    @Override
    public void processElement(DeadLockFrameModel deadLockFrameModel, Context context, Collector<DeadLockFrameModel> collector) throws Exception {

        ConfigModel configModel = deadLockFrameModel.getConfigModel();
        long nowTime = deadLockFrameModel.getTime();

        List<DeadLockFrameModel.CarTrack> trackList = deadLockFrameModel.getTrackList();
        List<DeadLockFrameModel.CarTrack> conflictTrackList = new ArrayList<>();
        //冲突基础数据key:rid-turn
        Map<String, List<String>> conflictMap = deadLockFrameModel.getConflictSlotMap();
        //按RID分组
        Map<String, List<DeadLockFrameModel.CarTrack>> groupList = trackList.stream().collect(Collectors.groupingBy(o -> o.getRid()));

        if (groupList.size() >= 2) {
            //按RID+转向分组
            Map<String, List<DeadLockFrameModel.CarTrack>> ridTurnCarList = new HashMap<>();

            for (Map.Entry<String, List<DeadLockFrameModel.CarTrack>> entry : groupList.entrySet()) {
                String rid = entry.getKey();
                List<DeadLockFrameModel.CarTrack> value = entry.getValue();

                for (DeadLockFrameModel.CarTrack ridCar : value) {
                    String laneFunc = ridCar.getLaneFunc();
                    String[] sps = laneFunc.split(Constant.COMMA);
                    for (String func : sps) {
                        String ridTurnKey = null;
                        if (func.contains(Constant.LEFT_TURN_CODE)) {
                            ridTurnKey = rid + Constant.MARK + Constant.LEFT_TURN_CODE;
                        } else if (func.contains(Constant.STRAIGHT_LINE_CODE)) {
                            ridTurnKey = rid + Constant.MARK + Constant.STRAIGHT_LINE_CODE;
                        }

                        List<DeadLockFrameModel.CarTrack> tmplist = ridTurnCarList.get(ridTurnKey);
                        if (Objects.isNull(tmplist)) {
                            tmplist = new ArrayList<>();
                        }
                        tmplist.add(ridCar);
                        ridTurnCarList.put(ridTurnKey, tmplist);
                    }
                }
            }

            for (Map.Entry<String, List<DeadLockFrameModel.CarTrack>> entry : ridTurnCarList.entrySet()) {
                String ridTurnKey = entry.getKey();
                //冲突进口及转向
                List<String> maplist = conflictMap.get(ridTurnKey);
                if (Objects.nonNull(maplist)) {
                    for (String cmap : maplist) {
                        String[] sps = cmap.split(Constant.MARK);
                        String rid = sps[1];
                        String turn = sps[2];
                        String ckey = rid + Constant.MARK + turn;

                        //检测冲突方向是否有车辆
                        boolean isMatched = ridTurnCarList.containsKey(ckey);
                        if (isMatched) {
                            List<DeadLockFrameModel.CarTrack> carList = entry.getValue();
                            List<DeadLockFrameModel.CarTrack> carConflictList = ridTurnCarList.get(ckey);

                            long curDirStopSize = carList.stream().filter(o -> o.getSpeed() < configModel.getDeadlockStopCarSpeed()).count();
                            long conflictDirStopSize = carConflictList.stream().filter(o -> o.getSpeed() < configModel.getDeadlockStopCarSpeed()).count();
                            double minDist = this.calculateConlictCarMinDist(carList, carConflictList);

                            long conflictMinStopCarNums = configModel.getDeadlockConflictMinStopCarNums();
                            long conflictCarMaxDist = configModel.getDeadlockConflictCarMaxDistance();
                            //每个冲突方向至少停车数量，所有冲突车辆间最小距离满足阈值
                            if (curDirStopSize >= conflictMinStopCarNums && conflictDirStopSize >= conflictMinStopCarNums && minDist < conflictCarMaxDist) {
                                conflictTrackList.addAll(carList);
                            }
                        }
                    }
                }
            }
        }
        deadLockFrameModel.setTrackList(conflictTrackList);
        collector.collect(deadLockFrameModel);
    }

    //计算所有冲突车辆的最小距离
    private double calculateConlictCarMinDist(List<DeadLockFrameModel.CarTrack> carList, List<DeadLockFrameModel.CarTrack> carConflictList) {
        double minDist = Double.MAX_VALUE;
        for (DeadLockFrameModel.CarTrack carTrack1 : carList) {
            double x1 = carTrack1.getLng();
            double y1 = carTrack1.getLat();
            for (DeadLockFrameModel.CarTrack carTrack2 : carConflictList) {
                double x2 = carTrack2.getLng();
                double y2 = carTrack2.getLat();
                double distance = PtInPolyUtil.getDistance(x1, y1, x2, y2);
                if (distance < minDist) {
                    minDist = distance;
                }
            }
        }
        return minDist;
    }

    /**
     * 获取key
     *
     * @param carTrack
     * @return
     */
    private static String getKey(DeadLockFrameModel.CarTrack carTrack) {
        return carTrack.getCrossId() + Constant.MARK + carTrack.getTrackId();
    }


}
