package com.wanji.indicators.event.cross.deadlock.func;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.event.deadlock.CrossDeadlockResultModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.api.java.tuple.Tuple;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 2022/1/12 17:48
 *
 * @auther fengyi
 */
public class DeadlockDistinctProcess extends KeyedProcessFunction<Tuple, CrossDeadlockResultModel, CrossDeadlockResultModel> {
    private static final Logger log = LoggerFactory.getLogger(DeadlockDistinctProcess.class);
    private ValueState<CrossDeadlockResultModel> valueState;

    @Override
    public void open(Configuration parameters) {
        valueState = getRuntimeContext().getState(new ValueStateDescriptor<>("DeadlockDistinctProcess",CrossDeadlockResultModel.class));
    }

    @Override
    public void processElement(CrossDeadlockResultModel deadlockResult, Context context, Collector<CrossDeadlockResultModel> collector) throws Exception {
        CrossDeadlockResultModel value = valueState.value();
        if (value == null) {
            deadlockResult.setId(deadlockResult.getCrossId() + Constant.MARK + deadlockResult.getCrossId() + Constant.MARK + deadlockResult.getTypeCode() + Constant.MARK + deadlockResult.getStartTime());
            log.info("路口发生死锁事件：crossId={},startTime={},endTime={},持续时长：{}ms",deadlockResult.getCrossId(), DateUtil.toDateTime(deadlockResult.getStartTime(),"yyyy-MM-dd HH:mm:ss"),DateUtil.toDateTime(deadlockResult.getEndTime(),"yyyy-MM-dd HH:mm:ss"),deadlockResult.getDuration());
            collector.collect(deadlockResult);
            valueState.update(deadlockResult);
            context.timerService().registerProcessingTimeTimer(System.currentTimeMillis() + 7200000L);
        }
    }

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<CrossDeadlockResultModel> out) throws Exception {
        valueState.clear();
    }

}
