package com.wanji.indicators.event.cross.deadlock.func;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadLockStatusModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadlockResultModel;
import com.wanji.indicators.model.event.deadlock.DeadLockFrameModel;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * 检测是否存在冲突运行的车辆
 */
public class DeadlockStopCheckKeyedProcess extends KeyedProcessFunction<String, DeadLockFrameModel, CrossDeadlockResultModel> {
    private static final Logger log = LoggerFactory.getLogger(DeadlockStopCheckKeyedProcess.class);
    private MapState<String, CrossDeadLockStatusModel> mapState;
    private MapState<String, CrossDeadlockResultModel> resultMapState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateDeadlock_stop_1", String.class, CrossDeadLockStatusModel.class));
        resultMapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateDeadlock_result_1", String.class, CrossDeadlockResultModel.class));
    }

    @Override
    public void processElement(DeadLockFrameModel deadLockFrameModel, Context context, Collector<CrossDeadlockResultModel> collector) throws Exception {

        ConfigModel configModel = deadLockFrameModel.getConfigModel();
        long nowTime = deadLockFrameModel.getTime();

        List<DeadLockFrameModel.CarTrack> conflictTrackList = deadLockFrameModel.getTrackList();
        //去重
        conflictTrackList = conflictTrackList.stream().distinct().collect(Collectors.toList());


        List<String> ridList = conflictTrackList.stream().map(DeadLockFrameModel.CarTrack::getRid).distinct().collect(Collectors.toList());

        int allSize = conflictTrackList.size();
        //停止车辆数
        List<DeadLockFrameModel.CarTrack> stopList = conflictTrackList.stream().filter(o -> o.getSpeed() < configModel.getDeadlockStopCarSpeed()).collect(Collectors.toList());
        double stopCarRate = stopList.size() / Double.valueOf(allSize);

        if (ridList.size() > 1 && !stopList.isEmpty() && allSize > configModel.getDeadlockMinCarNums() && stopCarRate > configModel.getDeadlockStopCarRate()) {
            //log.info(deadLockFrameModel.getCrossId() + "=>满足条件帧时间：" + DateUtil.toDateTime(nowTime, "yyyy-MM-dd HH:mm:ss.SSS"));
            long duration = 0;
            String effectRids = String.join(",",ridList);
            CrossDeadLockStatusModel statusModel = mapState.get(deadLockFrameModel.getCrossId());
            if (Objects.isNull(statusModel)) {
                statusModel = new CrossDeadLockStatusModel();
                statusModel.setCrossId(deadLockFrameModel.getCrossId());
                statusModel.setStartTime(nowTime / 1000 * 1000);
                statusModel.setEndTime(nowTime);
                statusModel.setDuration(0);
                statusModel.setTime(nowTime);
                statusModel.setEffectRids(effectRids);
                mapState.put(deadLockFrameModel.getCrossId(), statusModel);
            } else {
                duration = nowTime - statusModel.getStartTime();
                statusModel.setDuration(duration);
                statusModel.setEndTime(nowTime);
                statusModel.setTime(nowTime);
                mapState.put(deadLockFrameModel.getCrossId(), statusModel);
            }

            if (duration > configModel.getDeadlockStopCarDuration()) {
                CrossDeadlockResultModel resultModel = new CrossDeadlockResultModel();
                BeanUtils.copyProperties(statusModel, resultModel);
//                resultModel.setEventClass(EventType.EVENT.getName());
//                resultModel.setEventType(EventType.CROSS_DEADLOCK.getName());
//                resultModel.setTypeCode(EventType.CROSS_DEADLOCK.getCode());
//                resultModel.setRecoveryCross(stopList.get(0).getRecoveryCross());
                resultModel.setDesc("死锁事件：路口内存在冲突方向车辆缓慢运行，疑似出现死锁现象");
                resultModel.setDescEn("Deadlock event: there are vehicles running slowly in the conflicting direction at the intersection, which is suspected of deadlock");
                CrossDeadlockResultModel disspModel = resultMapState.get(deadLockFrameModel.getCrossId());
                if (disspModel == null) {
                    resultModel.setCheckOutTime(duration);
                }else{
                    resultModel.setCheckOutTime(disspModel.getCheckOutTime());
                }
                resultModel.setDuration(duration);
                //resultModel.setRid(stopList.get(0).getRid());
                resultModel.setLocation(new double[]{stopList.get(0).getLng(), stopList.get(0).getLat()});
                resultModel.setId(statusModel.getCrossId() + Constant.MARK + statusModel.getCrossId() + Constant.MARK + /*EventType.CROSS_DEADLOCK.getCode() +*/ Constant.MARK + statusModel.getStartTime());
//                List<String> xyList = conflictTrackList.stream().map(o->o.getLng()+","+o.getLat()).distinct().collect(Collectors.toList());
//                String xys = String.join(";",xyList);
//                Point point = GeomsConvertUtil.genGeometry(xys).getCentroid();
//                resultModel.setLocation(new double[]{point.getX(),point.getY()});

                resultMapState.put(resultModel.getCrossId(), resultModel);
                collector.collect(resultModel);
                //log.info("路口发生死锁事件：crossId={},startTime={},endTime={},持续时长：{},停车车辆数：{},总车辆数：{}",resultModel.getCrossId(),DateUtil.toDateTime(resultModel.getStartTime(),"yyyy-MM-dd HH:mm:ss"),DateUtil.toDateTime(resultModel.getEndTime(),"yyyy-MM-dd HH:mm:ss"),duration,stopList.size(),allSize);
            } else {
                dissp(collector, deadLockFrameModel, configModel, nowTime);
            }
        } else {//消散清理
            //log.info(deadLockFrameModel.getCrossId() + "=>未满足条件帧时间：" + DateUtil.toDateTime(nowTime, "yyyy-MM-dd HH:mm:ss.SSS"));
            dissp(collector, deadLockFrameModel, configModel, nowTime);
        }

        if (!mapState.isEmpty()) {
            //缓存清理
            Iterable<CrossDeadLockStatusModel> iterable = mapState.values();
            List<CrossDeadLockStatusModel> list = StreamSupport.stream(iterable.spliterator(),false).collect(Collectors.toList());
            list = list.stream().filter(o->(nowTime-o.getTime())>5000).collect(Collectors.toList());
            //缓存超过5秒没有新数据，
            for (CrossDeadLockStatusModel model : list){
                mapState.remove(model.getCrossId());
            }
        }

    }

    //消散清理
    private void dissp(Collector<CrossDeadlockResultModel> collector, DeadLockFrameModel deadLockFrameModel, ConfigModel configModel, long nowTime) throws Exception {
        CrossDeadlockResultModel disspModel = resultMapState.get(deadLockFrameModel.getCrossId());
        if (disspModel != null) {//消散清理
            long disspDuration = nowTime - disspModel.getEndTime();
            if (disspDuration > configModel.getDeadlockDissipationDuration()) {
                mapState.remove(deadLockFrameModel.getCrossId());
                resultMapState.remove(deadLockFrameModel.getCrossId());
                disspModel.setIsDissipate(1);
                collector.collect(disspModel);
                //log.info("路口死锁事件消散，清理缓存,当前帧时间{},死锁事件终止i时间{}",DateUtil.toDateTime(nowTime,"yyyy-MM-dd HH:mm:ss"),DateUtil.toDateTime(disspModel.getEndTime(),"yyyy-MM-dd HH:mm:ss"));
            }
        } else {
            CrossDeadLockStatusModel statusModel = mapState.get(deadLockFrameModel.getCrossId());
            if (statusModel != null) {
                long difTime = Math.abs(nowTime - statusModel.getEndTime());
                //未满足持续时间清理状态时间阈值
                if (difTime > configModel.getDeadlockStateUnsatisfiedDurationClear()) {
                    mapState.remove(deadLockFrameModel.getCrossId());
                    resultMapState.remove(deadLockFrameModel.getCrossId());
                }
            }
        }
    }

    /**
     * 获取key
     *
     * @param carTrack
     * @return
     */
    private static String getKey(DeadLockFrameModel.CarTrack carTrack) {
        return carTrack.getCrossId() + Constant.MARK + carTrack.getTrackId();
    }


}
