package com.wanji.indicators.event.stream;

import com.wanji.indicators.event.cross.accident.AccidentEventMainNew;
import com.wanji.indicators.event.cross.conflictpoint.ConflictEventMain;
import com.wanji.indicators.event.cross.conflictpoint.func.ConflictLaneInfoCoFlatMap;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.event.stream.func.RefBaseRidInfoCoFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.RidTrafficStateMain;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.contrib.streaming.state.EmbeddedRocksDBStateBackend;
import org.apache.flink.runtime.state.filesystem.FsStateBackend;
import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
import org.apache.flink.streaming.api.CheckpointingMode;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;

import java.util.Properties;
import java.util.UUID;

public class CrossTrafficEventMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final String GROUP_ID = "FlinkConsumerGroup-V56-TRACK_CROSS_TRAFFIC_EVENT";

    public static void main(String[] args) {

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        String topic = properties.getProperty("consumer.topic");

        boolean isStartLightStatus = false;
        ParameterTool paramTool = ParameterTool.fromArgs(args);
        String startLightStatusFlag = paramTool.get("isStartLightStatus", "false");
        isStartLightStatus = Boolean.parseBoolean(startLightStatusFlag);

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
//        env.getCheckpointConfig().disableCheckpointing();
//        env.enableCheckpointing(2 * 60 * 1000);
//        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);
//        env.getCheckpointConfig().setCheckpointTimeout(10 * 60 * 1000);
//        env.getCheckpointConfig().setCheckpointStorage(new FileSystemCheckpointStorage(properties.getProperty("check.point.uri")));
//        env.setStateBackend(new FsStateBackend(properties.getProperty("check.point.uri")));
//        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.DELETE_ON_CANCELLATION);
//        env.getCheckpointConfig().setCheckpointingMode(CheckpointingMode.EXACTLY_ONCE);


        String groupId  =GROUP_ID;
        String groupChangeFlag = properties.getProperty("consumer.group.change.flag");
        if (Boolean.parseBoolean(groupChangeFlag)){
            groupId += DateUtil.toDateTime(System.currentTimeMillis(),"yyyyMMdd");
        }

        DateTime currentDateTime = new DateTime();
        DateTime dateTime = currentDateTime.withMillisOfDay(0).minusHours(-4);
        long startTimeStamp = DateUtil.StringToMillis("2023-10-23 20:00:00");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("enable.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("auto.offset.reset", "latest")
//                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(groupId+"-event_track_data")
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-event");

        SingleOutputStreamOperator<CrossFrameModel> thinningDataStream =
                stream
                        .flatMap(new CrossFrameFlatMap())
                        .setParallelism(1)
                        .name("全域轨迹帧数据-按路口分组输出");

        //抽帧
        SingleOutputStreamOperator<CrossFrameModel> thinningStream = thinningDataStream
                .keyBy(CrossFrameModel::getCrossId)
                .countWindow(1)
                .maxBy("globalTimeStamp")
                .name("数据抽帧");

//        SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = thinningStream
//                .connect(env.addSource(new LaneInfoSource()).broadcast())
//                .flatMap(new ConflictLaneInfoCoFlatMap())
//                .setParallelism(1)
//                .name("全域轨迹帧数据-车道属性数据绑定");

        SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = thinningStream
                .flatMap(new RefBaseRidInfoCoFlatMap())
                .setParallelism(1)
                .name("全域轨迹帧数据-车道属性数据绑定");


        try {
            //路口内冲突点计算
            ConflictEventMain.init(env).run(joinLaneStream);

            DataStream<String> lightStatusStream = null;
            if (isStartLightStatus) {
                KafkaSource<String> lightStatusSource = KafkaSource.<String>builder()
                        .setProperties(instance.getConsumerProperties())
                        .setProperty("auto.offset.commit", "true")
                        .setProperty("auto.commit.interval.ms", "1000")
                        .setProperty("commit.offsets.on.checkpoint", "false")
                        .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                        .setTopics("cross_lights_status")
                        .setGroupId(groupId + "-event_light")
                        .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                        .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                        .build();
                lightStatusStream = env
                        .fromSource(lightStatusSource, WatermarkStrategy.noWatermarks(), "cross_lights_status-data-source");
                //事故计算
                AccidentEventMainNew.init(env).run(joinLaneStream, lightStatusStream);
            } else {
                //事故计算
                AccidentEventMainNew.init(env).run(joinLaneStream, null);
            }

            env.execute("安全专题计算-工作流");
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
