package com.wanji.indicators.event.stream;

import com.wanji.indicators.event.cross.accident.AccidentEventMainNew;
import com.wanji.indicators.event.cross.conflictpoint.ConflictEventMain;
import com.wanji.indicators.event.cross.conflictpoint.func.ConflictLaneInfoCoFlatMap;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.event.stream.func.RefBaseRidInfoCoFlatMap;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrame;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrameDtFile;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 离线计算入口
 * @author
 */
public class OfflineEventTestMain {

    private static final Logger log = LoggerFactory.getLogger(OfflineEventTestMain.class);

    public static void main(String[] args) {
        //初始化环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        env.setParallelism(1);

        //加载配置项 并获取工作路径
        String inPath = "D:\\WanJi-Work\\07 项目\\02 济南\\数据";
        String outPath = "d:/flink/out/";
        String startTime = "2024-02-25_08:30:00";
        String endTime = "2024-02-25_09:00:00";

         startTime = "2024-05-20_08:28:00";
         endTime = "2024-05-20_08:30:00";

        //读取离线数据
        DataStreamSource<String> streamSource = env.addSource(new DebugSourceGlobalFrameDtFile(inPath,startTime,endTime));


        SingleOutputStreamOperator<CrossFrameModel> crossDataStream =
                streamSource
                        .flatMap(new CrossFrameFlatMap())
                        .setParallelism(1)
                        .name("全域轨迹帧数据-按路口分组输出");

        SingleOutputStreamOperator<CrossFrameModel> thinningDataStream = crossDataStream
                .keyBy(CrossFrameModel::getCrossId)
                .countWindow(1)
                .maxBy("globalTimeStamp")
                .name("数据抽帧");

        //        SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = thinningStream
//                .connect(env.addSource(new LaneInfoSource()).broadcast())
//                .flatMap(new ConflictLaneInfoCoFlatMap())
//                .setParallelism(1)
//                .name("全域轨迹帧数据-车道属性数据绑定");

        SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = thinningDataStream
                .flatMap(new RefBaseRidInfoCoFlatMap())
                .setParallelism(1)
                .name("全域轨迹帧数据-车道属性数据绑定");

        //离线调试模式
        boolean isDebug = true;
        try {
//            ConflictEventMain.init(env,outPath,isDebug).run(joinLaneStream);
            AccidentEventMainNew.init(env,outPath,isDebug).run(joinLaneStream,null);
            env.execute("事故事件测试");
        } catch (Exception e) {
            log.error("事件测试 - 计算任务异常 : " + e);
        }
    }
}
