package com.wanji.indicators.event.stream;

import com.wanji.indicators.event.cross.accident.AccidentEventMainNew;
import com.wanji.indicators.event.cross.conflictpoint.ConflictEventMain;
import com.wanji.indicators.event.cross.conflictpoint.func.ConflictLaneInfoCoFlatMap;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Properties;

public class TrafficEventMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);

    private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);

    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();


    private TrafficEventMain(StreamExecutionEnvironment env) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = this.instance.getProperties();
    }


    public static TrafficEventMain init(StreamExecutionEnvironment env) {
        return new TrafficEventMain(env);
    }

    public void trafficEventMain(DataStream<String> streamSource) {

        try {

            SingleOutputStreamOperator<CrossFrameModel> thinningDataStream =
                    streamSource
                            .flatMap(new CrossFrameFlatMap())
                            .setParallelism(1)
                            .name("全域轨迹帧数据-按路口分组输出");

            //抽帧
            SingleOutputStreamOperator<CrossFrameModel> thinningStream = thinningDataStream
                    .keyBy(CrossFrameModel::getCrossId)
                    .countWindow(1)
                    .maxBy("globalTimeStamp")
                    .name("数据抽帧");

            SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = thinningStream
                    .connect(env.addSource(new LaneInfoSource()).broadcast())
                    .flatMap(new ConflictLaneInfoCoFlatMap())
                    .setParallelism(1)
                    .name("全域轨迹帧数据-车道属性数据绑定");

            //路口内冲突点计算
            ConflictEventMain.init(env).run(joinLaneStream);
            //事故计算
           // AccidentEventMainNew.init(env).run(joinLaneStream);

        } catch (Exception e) {
            log.error("全域安全专题计算任务异常 :   " + e);
        }
    }

}
