package com.wanji.indicators.event.stream.func;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.*;
import com.wanji.indicators.model.event.confilct.EventCross;
import com.wanji.indicators.task.trajectory.func.CarDataValidatorFlatMap;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;

@Slf4j
public class CrossFrameFlatMap implements FlatMapFunction<String, CrossFrameModel> {
    @Override
    public void flatMap(String value, Collector<CrossFrameModel> out) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(value);
        long dataTime = jsonObject.getLongValue("globalTimeStamp");
        String timeStamp = jsonObject.getString("timeStamp");
        JSONArray track = jsonObject.getJSONArray("e1FrameParticipant");
        try {
            if (!track.isEmpty()) {
                Map<String, CrossFrameModel> crossFrameModelMap = new HashMap<>();

                for (Object obj : track) {
                    CarTrackModel carTrack = JSONObject.parseObject(obj.toString(), CarTrackModel.class);
                    //校验路网信息是否齐全
                    CarTrackModel.RoadNet roadnet = carTrack.getRoadnet();
                    if (Objects.nonNull(roadnet)) {
                        boolean qualifiedCrossId = CarDataValidatorFlatMap.isQualified(roadnet.getCrossId());
                        if (qualifiedCrossId) {
                            carTrack.setGlobalTimeStamp(dataTime);

                            String crossId = carTrack.getRoadnet().getCrossId();
                            //福地街路口ID被认为修改为转山西路【因使用同一个信号机】、计算事件此处进行路口ID复原
                            BaseLaneInfo baseLaneInfo = BaseLaneCache.getInstance().getBaseLaneInfoMap().get(roadnet.getLaneId());
                            if (baseLaneInfo != null) {
                                crossId = baseLaneInfo.getCrossId();
                            }

                            CrossFrameModel crossFrameModel = crossFrameModelMap.get(crossId);
                            if (Objects.isNull(crossFrameModel)) {
                                crossFrameModel = new CrossFrameModel();
                            }
                            crossFrameModel.setTimeStamp(timeStamp);
                            crossFrameModel.setGlobalTimeStamp(dataTime);
                            crossFrameModel.getTrackList().add(carTrack);
                            crossFrameModel.setCrossId(crossId);

                            crossFrameModelMap.put(crossId, crossFrameModel);
                        }
                    }
                }
                for (Map.Entry<String, CrossFrameModel> entry : crossFrameModelMap.entrySet()) {
                    out.collect(entry.getValue());
                }
            }
        }catch (Exception e) {
            log.error("",e);
        }

    }

    /**
     * 计算反向角度
     * @param angle
     * @return
     */
    private static double getBackAngle(double angle) {
        if (angle >= 180) {
            return angle - 180;
        } else {
            return angle + 180;
        }
    }

    private static EventCross toEventCross(CarTrackModel carTrack){
        EventCross e = new EventCross();

        e.setTrackId(carTrack.getId());
        e.setLng(carTrack.getLongitude());
        e.setLat(carTrack.getLatitude());

        e.setSpeed(carTrack.getSpeed());
        e.setDriveAngle(carTrack.getCourseAngle());
        e.setPlateNumber(carTrack.getPicLicense());
        e.setVehicleType(carTrack.getOriginalType());
        e.setTime(carTrack.getGlobalTimeStamp());

        return e;
    }
}
