package com.wanji.indicators.event.stream.func;

/**
 * @author fengyi
 * @ 2023/10/30
 * @dedatescription
 */

import com.vividsolutions.jts.algorithm.Angle;
import com.vividsolutions.jts.geom.Coordinate;
import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.cache.PedestrianDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;

/**
 * 行人轨迹基础
 */
@Slf4j
public class RefBaseRidInfoCoFlatMap implements FlatMapFunction<CrossFrameModel, CrossFrameModel> {

    //private Map<String, BaseLaneInfo> map = new HashMap();

    @Override
    public void flatMap(CrossFrameModel frameModel, Collector<CrossFrameModel> collector) throws Exception {

        Map<String, BaseLaneInfo> map = BaseLaneCache.getInstance().getBaseLaneInfoMap();

        Map<String, String> refLaneMap = BaseLaneCache.getInstance().getRefLaneIdMap();

        Map<String, List<BaseCrossTurnInfo>> turnRelationMap = BaseLaneCache.getInstance().getCrossTurnInfoMap();

        List<CarTrackModel> trackList = frameModel.getTrackList();

        List<CarTrackModel> resList = new ArrayList<>();

        for (CarTrackModel carTrack : trackList) {
            String laneId = carTrack.getRoadnet().getLaneId();
            String mainLaneId = refLaneMap.get(laneId);
            if (StringUtils.isNotEmpty(mainLaneId)) {
                laneId = mainLaneId;
            }
            carTrack.getRoadnet().setLaneId(laneId);
            BaseLaneInfo baseLaneInfo = map.get(laneId);
            //扩展属性绑定
            CommonUtil.boundLaneExtProperty(carTrack, baseLaneInfo, turnRelationMap);
            if (baseLaneInfo != null) {
                frameModel.setCrossId(baseLaneInfo.getCrossId());
            }

            resList.add(carTrack);
        }
        frameModel.getTrackList().clear();
        frameModel.setTrackList(resList);

        collector.collect(frameModel);
    }

//    @Override
//    public void flatMap2(Map<String, BaseLaneInfo> hashMap, Collector<CrossFrameModel> collector) throws Exception {
//        this.map = hashMap;
//    }
}