package com.wanji.indicators.model;

import com.vividsolutions.jts.geom.Geometry;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import lombok.Data;

import java.io.Serializable;
import java.util.List;

/**
 * @author fengyi
 * @date 2023/3/1
 * @description
 */
@Data
public class CarTrackModel implements Serializable {
    private static final long serialVersionUID = 1;
    /*******************************************/
    /*全域ID*/
    private String orgCode = "";
    /*车辆ID*/
    private Integer id;
    /*车牌号*/
    private String picLicense;
    /*车牌颜色*/
    private Integer licenseColor;
    /*车辆颜色*/
    private Integer originalColor;
    /*车辆类型*/
    private Integer originalType;
    /*路网数据*/
    private RoadNet roadnet;
    /*参与者大类别 1机动车 2非机动车 3行人*/
    private Integer category;
    /*******************************************/

    /*格式化时间戳：yyyy-MM-dd HH:mm:ss.SSS*/
    private String timeStamp;
    /*数值时间戳*/
    private long globalTimeStamp;
    /*经度*/
    private Double longitude;
    /*纬度*/
    private Double latitude;
    /*速度*/
    private Double speed;
    /*航向角*/
    private Double courseAngle;

    //是否位于指定区域内 默认0：不在指定区域内 1:位于指定区域内
    private int isInSpecificArea = 0;

    /*******************************************/

    @Data
    public static class RoadNet implements Serializable {
        private static final long serialVersionUID = 1;
        /*路口ID*/
        private String crossId;
        /*路段ID*/
        private String rid;
        /*车道ID*/
        private String laneId;
        private Integer laneNo;
        /*渠化ID*/
        private String segmentId;
        /*是否在路口范围内*/
        private int inCrossFlag;
        /*车道功能转向*/
        //车道转向：1左转；2直行；3右转；4掉头；5直左；6直右；7左直右；8左右；9左转掉头；10直行掉头；11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头
        private Integer turn=0;
        /*车道终点坐标:x,y*/
        private String laneEndPoint;
        /*车道起点坐标:x,y*/
        private String laneStartPoint;
        /*车道类型 2：进口车道 3：出口车道*/
        private Integer laneType;
        /*rid 8方向*/
        private Integer ridDir8;
        /*rid 4方向*/
        private Integer ridDir4;
        /*rid 自由流速度*/
        private Double freeSpeed;
        /*车道角度*/
        private Double laneAngle;
        /*道路等级*/
        private int ridLevel;
        /*路段长度*/
        private double ridLength;
        /*绑定的转向关系*/
        private List<BaseCrossTurnInfo> turnRelationList;
        private double laneLength;
        private Geometry geometry;
    }

    /***********扩展属性*********************************/
    //路口内旅行时间，单位ms
    private long travelTime=0;
    //待转区车道长度，单位米
    private Double waitingTurnLaneDist=0D;
    //路口内距离停止线距离
    double stopLineDist ;
    //路口内行驶距离
    private double areaDist;
    //区域自由流速度 km/h
    private double areaFreeSpeed;
    //灯态
    private List<CrossRidTurnLampStatusModel> ridTurnLampStatusModel;
    //车辆转向 1：左转 2：直行 3：右转
    private Integer carTurn = 0;
}
