package com.wanji.indicators.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

import java.util.List;

/**
 * @author duanruiming
 * @date 2023/03/09 17:02
 */
@Data
public class LanePeriodicDataModel {

    /**
     * 车道事件集合
     */
    @JsonProperty("eventList")
    private List<EventList> eventList;
    /**
     * 车道数量
     */
    private Integer laneNum;
    /**
     * 全域编号
     */
    private String orgCode;
    /**
     * 数据生成时间: yyyy-MM-dd HH:mm:ss:SSS
     */
    private String timeStamp;

    @Data
    public static class EventList {

        /**
         * 最大动态排队长度
         */
        @JsonProperty("dynamic_queue_length_max")
        private double dynamicQueueLengthMax;
        /**
         * 最小动态排队长度
         */
        @JsonProperty("dynamic_queue_length_min")
        private double dynamicQueueLengthMin;
        /**
         * 绿灯有效利用率
         */
        @JsonProperty("green_light_efficiency")
        private double greenLightEfficiency;
        /**
         * 车道号(路网数据)
         */
        @JsonProperty("laneId")
        private String laneId;
        /**
         * 车道通行能力
         */
        @JsonProperty("lane_capacity")
        private int laneCapacity;
        /**
         * 车道流率
         */
        @JsonProperty("lane_flow_rate")
        private double laneFlowRate;
        /**
         * 不停车率
         */
        @JsonProperty("lane_no_stop_rate")
        private double laneNoStopRate;
        /**
         * 一次停车率
         */
        @JsonProperty("lane_one_stop_rate")
        private double laneOneStopRate;
        /**
         * 车道饱和度
         */
        @JsonProperty("lane_saturation")
        private double laneSaturation;
        /**
         * 车道饱和流率
         */
        @JsonProperty("lane_saturation_flow_rate")
        private int laneSaturationFlowRate;
        /**
         * 三次停车率
         */
        @JsonProperty("lane_three_stop_rate")
        private double laneThreeStopRate;
        /**
         * 二次停车率
         */
        @JsonProperty("lane_two_stop_rate")
        private double laneTwoStopRate;
        /**
         * 绿灯结束排队长度
         */
        @JsonProperty("light_green_finish_queue_length")
        private double lightGreenFinishQueueLength;
        /**
         * 绿灯起亮排队长度
         */
        @JsonProperty("light_green_start_queue_length")
        private double lightGreenStartQueueLength;
        /**
         * 平均延误
         */
        @JsonProperty("mean_delay")
        private double meanDelay;
        /**
         * 平均停车次数
         */
        @JsonProperty("mean_stops_number")
        private int meanStopsNumber;
        /**
         * 平均点速度
         */
        @JsonProperty("mean_v")
        private double meanV;
        /**
         * 平均车长(米)
         */

        @JsonProperty("mean_len")
        private double meanLen;
        /**
         * 最大静态排队长度
         */
        @JsonProperty("static_queue_length_max")
        private double staticQueueLengthMax;
        /**
         * 最小静态排队长度
         */
        @JsonProperty("static_queue_length_min")
        private double staticQueueLengthMin;
        /**
         * 时间占有率
         */
        @JsonProperty("time_occupancy")
        private double timeOccupancy;
        /**
         * 自然流量
         */
        @JsonProperty("traffic_flow")
        private int trafficFlow;
        @JsonProperty("traffic_flow_A")
        private int trafficFlowA;
        @JsonProperty("traffic_flow_B")
        private int trafficFlowB;
        @JsonProperty("traffic_flow_C")
        private int trafficFlowC;
        /**
         * 小车当量
         */
        @JsonProperty("trolley_equivalent")
        private int trolleyEquivalent;
        /**
         * 平均空间密度（长度占比）
         */
        @JsonProperty("vehicle_length_ratio_mean")
        private double vehicleLengthRatioMean;
        /**
         * 平均空间密度（车辆负荷比）
         */
        @JsonProperty("vehicle_nums_ratio_mean")
        private double vehicleNumsRatioMean;
        /*
         * 平均车头时距(s)
         */
        @JsonProperty("time_headway_section_mean")
        private double timeHeadwaySectionMean;
        /**
         * 路况信息（1:畅通2:缓行３:拥堵４:未知）
         */
        @JsonProperty("traffic_information")
        private int trafficInformation;
        /*
         * 溢流率
         */
        @JsonProperty("overflow_rate")
        private double overflowRate;
        /**
         * 非机动车流量
         */
        @JsonProperty("non_motor_flow")
        private int nonMotorFlow;
        /**
         * 85位速度（km/h）
         */
        @JsonProperty("v_85")
        private double v85;
    }
}
