package com.wanji.indicators.model.event.congestion;


import com.wanji.indicators.model.ConfigModel;
import lombok.Data;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

@Data
public class CongestionFrameModel implements Serializable {

    private String crossId;
    private String rid;
    private long time;
    private List<CarTrack> trackList;
    private ConfigModel configModel;
    private Map<String,Integer> congestionInfo;


    public CongestionFrameModel() {
    }

    public CongestionFrameModel(String crossId, long time, List<CarTrack> trackList) {
        this.crossId = crossId;
        this.time = time;
        this.trackList = trackList;
    }

    public CongestionFrameModel(String crossId, String rid, long time, List<CarTrack> trackList) {
        this.crossId = crossId;
        this.rid = rid;
        this.time = time;
        this.trackList = trackList;
    }

    @Data
    public static class CarTrack implements Serializable {
        //待转区长度 >> 待转区停止线距离入口车道停止线的距离
        private int waitLaneDistance;
        //是否拥堵
        private boolean isCongestion;
        //拥堵开始时间
        private long congestionStartTime;
        //拥堵结束时间
        private long congestionEndTime;
        //根据车道功能和进口rid匹配的出口rid
        private String ridOut;
        //出口rid方向
        private String ridOutDir;
        //路口当前rid方向行车距离长度
        private double crossLength;
        private String recoveryCross;//设备名称
        /**
         * 轨迹数据字段
         */
        private String crossId;//: "10I4M0830K0"
        private int crossing;// 1
        private String lampState;// G
        private int trackId;//: 408732
        private int objectType;//: 1
        private int vehicleColor;//: 4
        private int vehicleType;//: 1
        private String vehicleBrand;//: "\346\234\254\347\224\260"
        private String vehicleSubbrand;//: "\351\233\205\351\230\201"
        private String vehicleYearBrand;//: "2003,2004,2007,2008,2010,2011,2012,2013,2014"
        private String entryLane;//: "10I6S082VM010I4M0830K0090012"
        private String exitLane;//: "10I4M0830K010I360830D0010011"
        private String matchLane;//: "10I4M0830K010I360830D0020011"
        private String matchLaneName;//: "\350\245\277  1"
        private String laneFunc;
        private String rid;
        private double driveAngle;//: 262.88
        private double lat;//: 26.52352862096375
        private double lng;//: 106.71488115347212
        private int plateColor;//: 6
        private String plateNumber;//: "\350\264\265CEA922"
        private int plateType;//: 2
        private double speed;//: 49.37
        private double at;//: -0.09
        private double areaDist;//: 61.68
        private double stopLineDist;//: -1.0
        private double delayTime;//: 0.72
        private double travelTime;//: 10.1
        private long time;//: 1617279936374
        private String plateBgColor;//: "#0000ff"
        private String plateTextColor;//: "#ffffff"
    }

    @Override
    public String toString() {
        return "FrameModel{" +
                "crossId='" + crossId + '\'' +
                ", time=" + time +
                '}';
    }
}
