package com.wanji.indicators.model.event.deadlock;

import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import lombok.Data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 2021/11/9 14:53
 *
 * @auther ke.han
 */
@Data
public class DeadLockFrameModel implements Serializable {

    private int accidentType;
    private String crossId;
    private String rid;
    private long time;
    private List<CarTrack> trackList;
    private List<CarTrack> conflictTrackList;
    private Map<String,Integer> congestionInfo;
    private ConfigModel configModel;
    //路口各进口道灯态数据
    private List<CrossRidTurnLampStatusModel> ridTurnLampList = new ArrayList<>();
    //冲突相位信息
    private Map<String, List<String>> conflictSlotMap = new HashMap<>();


    public DeadLockFrameModel() {
    }

    public DeadLockFrameModel(String crossId, long time, List<CarTrack> trackList) {
        this.crossId = crossId;
        this.time = time;
        this.trackList = trackList;
    }

    public DeadLockFrameModel(String crossId, String rid, long time, List<CarTrack> trackList) {
        this.crossId = crossId;
        this.rid = rid;
        this.time = time;
        this.trackList = trackList;
    }
    @Data
    public static class CarTrack implements Serializable {
        /**
         * 辅助计算字段
         */
        //当前帧是否停车 true停车  false行驶
        private boolean isStop;
        //停车持续时间 单位毫秒
        private long stopDuration;
        //停车开始时间 时间戳
        private long stopTime;
        //车辆绕行累计持续时间
        private long detourDuration;
        //绕行车辆数
        private int detourNum;
        //待转区长度 >> 待转区停止线距离入口车道停止线的距离
        private int waitLaneDistance;
        //是否拥堵
        private boolean isCongestion;
        //拥堵开始时间
        private long congestionStartTime;
        //拥堵结束时间
        private long congestionEndTime;
        //拥堵区域出口rid
        private String congestionOutRid;
        //是否紧急制动
        private boolean isEmergencyBraking;
        //紧急制动最大加速度
        private double emergencyBrakingAt;
        //紧急制动车速
        private double emergencyBrakingSpeed;
        //ttc碰撞预警
        private boolean isTTC;
        //ttc碰撞预警时的车速
        private double ttcSpeed;
        //ttc检测到的相交车辆trackId
        private int ttcTrackId;
        //事故车辆组合后匹配的最近距离trackId
        private int accidentTrackId;
        //事故车辆组合后与其他车辆的距离
        private double accidentDis;
        //停车轨迹稳定持续时间
        private long stopStableDuration;
        //停车时间阈值
        private long checkTime;
        //是否绿灯停车
        private boolean isGreenStop;
        //绿灯停车持续时间
        private long greenStopDuration;
        //绿灯停车开始时间
        private long greenStopStartTime;
        //分组key  = crossId + trackId
        private String key;
        //key:frid value:右转出口道可进入的其他方向进口道，格式：rid;turn_dir_no
        private Map<String,String> ridTurnRefMap = new HashMap<>();
        //三色预警蓝色预警开关
        private String isOpenBlue;
        /**
         * 轨迹数据字段
         */
        private String crossId;//: "10I4M0830K0"
        private int crossing;// 1
        private String lampState;// G
        private int trackId;//: 408732
        private int objectType;//: 1
        private int vehicleColor;//: 4
        private int vehicleType;//: 1
        private String vehicleBrand;//: "\346\234\254\347\224\260"
        private String vehicleSubbrand;//: "\351\233\205\351\230\201"
        private String vehicleYearBrand;//: "2003,2004,2007,2008,2010,2011,2012,2013,2014"
        private String entryLane;//: "10I6S082VM010I4M0830K0090012"
        private String exitLane;//: "10I4M0830K010I360830D0010011"
        private String matchLane;//: "10I4M0830K010I360830D0020011"
        private String matchLaneName;//: "\350\245\277  1"
        private String laneFunc;
        private String rid;
        private double driveAngle;//: 262.88
        private double lat;//: 26.52352862096375
        private double lng;//: 106.71488115347212
        private int plateColor;//: 6
        private String plateNumber;//: "\350\264\265CEA922"
        private int plateType;//: 2
        private double speed;//: 49.37
        private double at;//: -0.09
        private double areaDist;//: 61.68
        private double stopLineDist;//: -1.0
        private double delayTime;//: 0.72
        private double travelTime;//: 10.1
        private long time;//: 1617279936374
        private String plateBgColor;//: "#0000ff"
        private String plateTextColor;//: "#ffffff
        /**
         * 路口名称
         */
        private String recoveryCross;
    }

    @Override
    public String toString() {
        return "FrameModel{" +
                "crossId='" + crossId + '\'' +
                ", time=" + time +
                '}';
    }
}
