package com.wanji.indicators.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.vividsolutions.jts.geom.LineString;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.mapper.BaseLaneInfoMapper;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import com.wanji.indicators.util.Tools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 车道基础信息 服务实现类
 * </p>
 *
 * @author fengyi
 * @since 2023-04-06
 */
@Service
@Slf4j
public class BaseLaneInfoServiceImpl extends ServiceImpl<BaseLaneInfoMapper, BaseLaneInfo> implements BaseLaneInfoService {

    @Resource
    private BaseLaneInfoMapper baseLaneInfoMapper;

    @Override
    public List<BaseLaneInfo> findLaneRidInfo() {
        List<BaseLaneInfo> list = null;//baseLaneInfoMapper.findAllBaseLaneInfo();
        for (BaseLaneInfo model : list) {
            String wkt = model.getRidWkt();
            LineString lineString = GeomsConvertUtil.getLineString(wkt);
            double ridLength = lineString.getLength();
            model.setRidLength(ridLength);
        }
        return list;
    }

    @Override
    public Map<String, BaseLaneInfo> findLaneInfo(String crossList) {
        String[] arr = StringUtils.split(crossList, ",");
        List<String> crosslist = Arrays.asList(arr);
        Map<String, Object> params = new HashMap<>();
        //params.put("crossList", crosslist);

        List<BaseLaneInfo> list = this.baseLaneInfoMapper.findCrossLaneInfo(params);
        // t.cross_id,t.road_id,t.rid,t.type,t.dir,count(*) lane_num,t.road_wkt

        Map<String, BaseLaneInfo> laneInfoMap = new HashMap<>();
        for (BaseLaneInfo baseLaneInfo : list) {
            Integer category = baseLaneInfo.getCategory();
            if (Objects.equals(2,category)) {
                continue;
            }

            String startPoint = baseLaneInfo.getStartPoint();
            String[] p1 = startPoint.split(",");
            String endPoint = baseLaneInfo.getEndPoint();
            String[] p2 = endPoint.split(",");

            double length = PtInPolyUtil.getDistance(Double.parseDouble(p1[0]), Double.parseDouble(p1[1]), Double.parseDouble(p2[0]), Double.parseDouble(p2[1]));
            baseLaneInfo.setLength(length);

            String ridWkt = baseLaneInfo.getRidWkt();
            double ridLength = GeomsConvertUtil.getSegmentTotalLeng(ridWkt);
            baseLaneInfo.setRidLength(ridLength);

            Double endAngle = baseLaneInfo.getEndAngle();
            //计算4方位方向
            int ridDir4 = CommonUtil.getAngleDir(endAngle);
            baseLaneInfo.setRidDir4(ridDir4);

            laneInfoMap.put(baseLaneInfo.getLaneId(), baseLaneInfo);
        }
        return laneInfoMap;
    }

    @Override
    public Map<String, List<String>> getRidTurnConflictInfo(String crossList) {
        Map<String, List<String>> conflictTurnMap = new HashMap<>();

        String[] arr = StringUtils.split(crossList, ",");
        List<String> crosslist = Arrays.asList(arr);
        Map<String, Object> params = new HashMap<>();
        params.put("crossList", crosslist);

        List<Map<String, Object>> list = baseLaneInfoMapper.findRidInfo(params);
        //按路口分组
        Map<String, List<Map<String, Object>>> groupList = list.stream().collect(Collectors.groupingBy(o -> o.get("cross_id").toString()));
        for (Map.Entry<String, List<Map<String, Object>>> entry : groupList.entrySet()) {

            List<Map<String, Object>> value = entry.getValue();
            //按角度排序
            value = value.stream().sorted(Comparator.comparing(o -> Double.parseDouble(o.get("end_angle").toString()))).collect(Collectors.toList());

            Map<Integer, String> ridDirMap = new TreeMap<>();

            Set<Integer> dirSets = new HashSet<>();

            for (Map<String, Object> dataMap : value) {
                String crossId = Tools.getMapValue("cross_id", dataMap);
                String rid = Tools.getMapValue("rid", dataMap);
                String ridDir8 = Tools.getMapValue("rid_dir", dataMap);
                String angle = Tools.getMapValue("end_angle", dataMap);
                //计算4方位方向
                int ridDir4 = CommonUtil.getAngleDir(Double.parseDouble(angle));
                if (dirSets.contains(ridDir4)) {
                    String cRid1 = ridDirMap.get(ridDir4);
                    String[] sps = cRid1.split(Constant.MARK);
                    if (!Objects.equals(sps[2],ridDir8)) {
                        ridDir4 = ridDir4 + 1;
                    }
                }
                dirSets.add(ridDir4);
                String crid = crossId + Constant.MARK + rid + Constant.MARK + ridDir8;
                ridDirMap.put(ridDir4, crid);
            }

            for (Map.Entry<Integer, String> entry1 : ridDirMap.entrySet()) {
                Integer dir4 = entry1.getKey();
                String crid = entry1.getValue();
                //顺时针方向
                int clockwise = (dir4 + 1) % 4 == 0 ? 4 : (dir4 + 1) % 4;
                //逆时针直行相位
                int antiClockWise = (dir4 - 1) % 4 == 0 ? 4 : (dir4 - 1) % 4;
                //对向
                int across = ((dir4 + 2) % 4 == 0 ? 4 : (dir4 + 2) % 4);

                Integer[] turns = new Integer[]{1, 2};
                for (Integer turn : turns) {
                    String key = crid + Constant.MARK + turn;
                    List<String> conflictTurnList = new ArrayList<>();
                    if (turn == 2) {
                        if (ridDirMap.containsKey(clockwise)) {
                            //与顺时针方向直行冲突
                            String clockwiseConflictStraightTurn = ridDirMap.get(clockwise) + Constant.MARK + "2";
                            conflictTurnList.add(clockwiseConflictStraightTurn);
                        }
                        if (ridDirMap.containsKey(antiClockWise)) {
                            //与逆时针方向直行冲突
                            String antiConflictStraightTurn = ridDirMap.get(antiClockWise) + Constant.MARK + "2";
                            conflictTurnList.add(antiConflictStraightTurn);
                        }
                        if (ridDirMap.containsKey(across)) {
                            //与对向左转冲突
                            String acrossConflictLeftTurn = ridDirMap.get(across) + Constant.MARK + "1";
                            conflictTurnList.add(acrossConflictLeftTurn);
                        }
                        if (ridDirMap.containsKey(antiClockWise)) {
                            //与逆时针左转冲突
                            String antiClockWiseConflictLeftTurn = ridDirMap.get(antiClockWise) + Constant.MARK + "1";
                            conflictTurnList.add(antiClockWiseConflictLeftTurn);
                        }
                    } else if (turn == 1) {
                        if (ridDirMap.containsKey(clockwise)) {
                            //与顺时针方向直行冲突
                            String clockwiseConflictStraightTurn = ridDirMap.get(clockwise) + Constant.MARK + "2";
                            conflictTurnList.add(clockwiseConflictStraightTurn);
                        }
                        if (ridDirMap.containsKey(clockwise)) {
                            //与顺时针方向左转冲突
                            String antiConflictLeftTurn = ridDirMap.get(clockwise) + Constant.MARK + "1";
                            conflictTurnList.add(antiConflictLeftTurn);
                        }
                        if (ridDirMap.containsKey(across)) {
                            //与对向直行冲突
                            String acrossConflictStraightTurn = ridDirMap.get(across) + Constant.MARK + "2";
                            conflictTurnList.add(acrossConflictStraightTurn);
                        }
                        if (ridDirMap.containsKey(antiClockWise)) {
                            //与逆时针左转冲突
                            String antiClockWiseConflictLeftTurn = ridDirMap.get(antiClockWise) + Constant.MARK + "1";
                            conflictTurnList.add(antiClockWiseConflictLeftTurn);
                        }
                    }
                    // log.info("冲突基础数据，冲突方向：{}，冲突转向：{}", key, conflictTurnList);
                    conflictTurnMap.put(key, conflictTurnList);
                }

            }
        }
        return conflictTurnMap;
    }

    @Override
    public double getAreaFreeSpeed(String crossList, Double defaultFreeSpeed) {
        Map<String, Map<String, Double>> ridFreeSpeed = this.getRidFreeSpeed(crossList, defaultFreeSpeed);
        Collection<Map<String, Double>> collection = ridFreeSpeed.values();
        return collection.stream().mapToDouble(o -> o.get("freeSpeed")).summaryStatistics().getAverage();
    }

    @Override
    public Map<String, Map<String, Double>> getRidFreeSpeed(String crossList, Double defaultFreeSpeed) {
        Map<String, Map<String, Double>> retmap = new HashMap<>();
        String[] sps = crossList.split(",");
        Map<String, Object> params = new HashMap<>();
        params.put("crossList", Arrays.asList(sps));
        List<Map<String, Object>> list = baseLaneInfoMapper.findRidFreeSpeed(params);

        for (Map<String, Object> map : list) {
            String crossId = Tools.getMapValue("cross_id", map);
            String rid = Tools.getMapValue("rid", map);
            String ridDir = Tools.getMapValue("rid_dir", map);
            String fs = Tools.getMapValue("free_speed", map);
            fs = fs == null ? defaultFreeSpeed.toString() : fs;

            String level = Tools.getMapValue("level", map);
            String wkt = Tools.getMapValue("wkt", map);
            double ridLength = GeomsConvertUtil.getSegmentTotalLeng(wkt);
            double freeSpeed = Double.parseDouble(fs);

            String key = rid;

            Map<String, Double> dataMap = new HashMap<>();
            dataMap.put("level", Double.parseDouble(level));
            dataMap.put("ridLength", ridLength);
            dataMap.put("freeSpeed", freeSpeed);
            dataMap.put("ridDir",Double.valueOf(ridDir));

            retmap.put(key, dataMap);
        }
        return retmap;
    }

    @Override
    public Map<String, String> findLaneJoinRef() {
        Map<String, String> retMap = new HashMap<>();

        List<Map<String,Object>> list = this.baseLaneInfoMapper.findLaneJoinRef(null);
        for (Map<String,Object> map : list ){
            String mainLaneId = Tools.getMapValue("lane_id",map);
            String joinLaneId = Tools.getMapValue("join_lane_id",map);
            retMap.put(joinLaneId,mainLaneId);
        }

        return retMap;
    }


    @Override
    public Map<String, List<BaseLaneInfo>> findRidSameDirLaneInfo(Collection<BaseLaneInfo> laneList) {
        Map<String, List<BaseLaneInfo>> retList = new HashMap<>();

        Map<String, List<BaseLaneInfo>> groupByDir =  laneList.stream().collect(Collectors.groupingBy(o->o.getCrossId()+"_"+o.getDir()));
        for (Map.Entry<String, List<BaseLaneInfo>> entry : groupByDir.entrySet()) {
            List<BaseLaneInfo> dirLaneList = entry.getValue();

            List<BaseLaneInfo> inList = dirLaneList.stream().filter(o->o.getType().equals(2)).collect(Collectors.toList());
            List<BaseLaneInfo> outList = dirLaneList.stream().filter(o->o.getType().equals(3)).collect(Collectors.toList());

            if (!inList.isEmpty() && !outList.isEmpty())
                retList.put(inList.get(0).getRid(),outList);

            if (!outList.isEmpty() && !inList.isEmpty())
                retList.put(outList.get(0).getRid(),inList);
        }

        return retList;
    }
}
