package com.wanji.indicators.source;

import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 自由流速度
 */
@Service
public class AreaFreeSpeedSource extends RichParallelSourceFunction<Map<String, Map<String,Double>>> {

    private static final Logger log = LoggerFactory.getLogger(AreaFreeSpeedSource.class);

    private volatile boolean running = true;

    Properties properties = null;

    BaseLaneInfoService baseLaneInfoService;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

    }


    @Override
    public void run(SourceContext<Map<String, Map<String,Double>>> sourceContext) throws Exception {

        try {
            while (running) {
                String crossList = properties.getProperty("full.area.cross.list");
                Double defaultFreeSpeed = Double.parseDouble(properties.getProperty("rid.default.free.speed"));

                Map<String, Map<String, Double>> ridFreeSpeed = baseLaneInfoService.getRidFreeSpeed(crossList, defaultFreeSpeed);
                sourceContext.collect(ridFreeSpeed);
                TimeUnit.SECONDS.sleep(5);
            }
        } catch (Exception e) {
            log.error("查询车道信息异常: " + e.getMessage(), e);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }

}
