package com.wanji.indicators.source;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 查询 rid 转向数据 对应的冲突转向
 */
public class ConflictTurnSource extends RichParallelSourceFunction<Map<String, List<String>>> {

    private static final Logger log = LoggerFactory.getLogger(ConflictTurnSource.class);

    private volatile boolean running = true;

    Properties properties = null;

    BaseLaneInfoService baseLaneInfoService;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

    }

    public void run(SourceContext<Map<String, List<String>>> sourceContext) throws Exception {
        //key:cross-rid-ridDir-turn value:crossid-rid-ridDir-冲突转向
        while (running) {
            try {
                String crossList = properties.getProperty("full.area.cross.list");
                Map<String, List<String>> retMap = baseLaneInfoService.getRidTurnConflictInfo(crossList);
                sourceContext.collect(retMap);
                TimeUnit.MINUTES.sleep(1);
            } catch (Exception e) {
                log.error("查询 rid 冲突信息数据异常: " + e.getMessage(), e);
            }
        }
    }

    @Override
    public void cancel() {
        running = false;
    }

    /**
     * 理论冲突相位数据，
     * 相位标号定义 以北方向为基准定义 1:北直行相位 2：东直行相位 3：南直行相位 4：西直行相位 5：北左转相位 6：东左转 7：南左转 8：西左转
     */
    public Map<Integer, List<Integer>> createConflictSlots() {

        Map<Integer, List<Integer>> maplist = new HashMap<Integer, List<Integer>>();

        for (int i = 1; i < 9; i++) {
            Integer sidewalkDir = null;
            List<Integer> conflictList = maplist.get(i);
            if (Objects.isNull(conflictList)) {
                conflictList = new ArrayList<>();
            }
            if (i < 5) { // 直行相位
                // 直行相位的冲突相位点
                // 顺时针直行相位
                int clockwise = (i + 1) % 4 == 0 ? 4 : (i + 1) % 4;
                // 逆时针直行相位
                int antiClockWise = (i - 1) % 4 == 0 ? 4 : (i - 1) % 4;
                // 对向左转冲突点
                int acrossLeft = ((i + 2) % 4 == 0 ? 4 : (i + 2) % 4) + 4;
                // 逆时针左转冲突相位点
                int antiLeft = ((i - 1) % 4 == 0 ? 4 : (i - 1) % 4) + 4;

                conflictList.add(clockwise);
                conflictList.add(antiClockWise);
                conflictList.add(acrossLeft);
                conflictList.add(antiLeft);
            } else {// 左转相位冲突相位
                // 顺时针直行相位
                int antiClockWiseStraight = (i + 1) % 4 == 0 ? 4 : (i + 1) % 4;
                // 对向直行冲突点
                int acrossStraight = (i + 2) % 4 == 0 ? 4 : (i + 2) % 4;
                // 顺时针左转相位
                int clockwiseLeft = ((i + 1) % 4 == 0 ? 4 : (i + 1) % 4) + 4;
                // 逆时针左转相位
                int antiClockWiseLeft = ((i - 1) % 4 == 0 ? 4 : (i - 1) % 4) + 4;

                conflictList.add(antiClockWiseStraight);
                conflictList.add(acrossStraight);
                conflictList.add(clockwiseLeft);
                conflictList.add(antiClockWiseLeft);
            }
            maplist.put(i, conflictList);
        }

        return maplist;

    }


}
