package com.wanji.indicators.source;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.CongestionArea;
import com.wanji.indicators.mapper.CongestionAreaMapper;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 查询 溢出 检测区域 数据
 */
public class CongestionSource extends RichParallelSourceFunction<HashMap<String, Map<String, CongestionArea>>> {

    public static void main(String[] args) {
        System.out.println(DateUtil.stringToDate("2023-11-27 17:30:00","yyyy-MM-dd HH:mm:ss").getTime());
        System.out.println(DateUtil.stringToDate("2023-11-27 07:10:00","yyyy-MM-dd HH:mm:ss").getTime());
    }

    private static final Logger log = LoggerFactory.getLogger(CongestionSource.class);

    private volatile boolean running = true;

    Properties properties = null;

    CongestionAreaMapper congestionAreaMapper;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        congestionAreaMapper = beanConf.getBean(CongestionAreaMapper.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

    }


    @Override
    public void run(SourceContext<HashMap<String, Map<String, CongestionArea>>> sourceContext) throws Exception {
        HashMap<String, Map<String, CongestionArea>> outMap = new HashMap<>();
        while (running) {
            try {
                String sql = "SELECT cross_id AS crossId,rid,lane_num AS laneNum,rid_in_arr AS ridInArr,wkt FROM t_congestion_area ";
                List<CongestionArea> dataList = congestionAreaMapper.findAllCongestionArea();// new JdbcUtil().excuteQueryToList(CongestionAreaDo.class, sql);
                if (dataList != null && !dataList.isEmpty()) {
                    outMap.clear();
                    for (CongestionArea data : dataList) {
                        Map<String, CongestionArea> congestionAreaDoMap = outMap.get(data.getCrossId());
                        if (congestionAreaDoMap == null) {
                            congestionAreaDoMap = new HashMap<>();
                        }
                        String wkt = data.getWkt();
                        JSONArray jsonArray = JSONArray.parseArray(wkt);
                        JSONObject geometryJsonObj = jsonArray.getJSONObject(0).getJSONObject("geometry");
                        JSONArray coordinatesArray = geometryJsonObj.getJSONArray("coordinates");

                        StringBuilder stringBuilder = new StringBuilder();
                        for (int j = 0; j < coordinatesArray.size(); j++) {
                            JSONArray pointArr = coordinatesArray.getJSONArray(j);
                            for (int k=0; k<pointArr.size();k++) {
                                double lng = pointArr.getJSONArray(k).getDouble(0);
                                double lat = pointArr.getJSONArray(k).getDouble(1);
                                stringBuilder.append(lng + "," + lat);
                                if (k != pointArr.size() - 1) {
                                    stringBuilder.append(";");
                                }
                            }
                        }
                        String[] wktPointArr = stringBuilder.toString().split(Constant.SEMICOLON);
                        List<PtInPolyUtil.Point> points = new ArrayList<>();
                        for (String p : wktPointArr) {
                            String[] xyArr = p.split(Constant.COMMA);
                            PtInPolyUtil.Point point = new PtInPolyUtil.Point(Double.valueOf(xyArr[0]), Double.valueOf(xyArr[1]));
                            points.add(point);
                        }
                        data.setPoints(points);
                        congestionAreaDoMap.put(data.getOutRid(), data);
                        outMap.put(data.getCrossId(), congestionAreaDoMap);
                    }
                }
                if (outMap.size() > 0) {
                    sourceContext.collect(outMap);
                }
                TimeUnit.MINUTES.sleep(1);
            } catch (Exception e) {
                log.error("查询溢出检测区域: " + e.getMessage() , e);
            }
//            Thread.sleep(1000*30);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}
