package com.wanji.indicators.source;

import com.alibaba.fastjson.JSONObject;
import com.orbitz.consul.Consul;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.ConfigModelDo;
import com.wanji.indicators.model.event.congestion.CongestionAreaDo;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author ke.han
 * @Date 2020/4/16 19:03
 **/
public class ConsulAreaSource extends RichParallelSourceFunction<HashMap<String, Map<String, CongestionAreaDo>>> {

    private static final Logger log = LoggerFactory.getLogger(ConsulAreaSource.class);

    private Consul client;

    private String host = "192.168.11.161";

    private int port = 8500;

    private int[] typeCodeArr;

    private volatile boolean running = true;

    public ConsulAreaSource(int[] typeCodeArr) {
        this.typeCodeArr = typeCodeArr;
    }

    public ConsulAreaSource() {
    }

    @Override
    public void open(Configuration parameters) throws Exception {

    }

    @Override
    public void run(SourceContext<HashMap<String, Map<String, CongestionAreaDo>>> sourceContext) throws Exception {
        //key:crossId value:<rid,CongestionAreaDo>
        HashMap<String, Map<String, CongestionAreaDo>> outMap = new HashMap<>();
        while (running) {
            try {
                //过滤路口溢出监测区域
                String listJson = "";
                List<CongestionAreaDo> dataList = JSONObject.parseArray(listJson, CongestionAreaDo.class);
                for (CongestionAreaDo data : dataList) {
                    Map<String, CongestionAreaDo> congestionAreaDoMap = outMap.get(data.getCrossId());
                    if (congestionAreaDoMap == null) {
                        congestionAreaDoMap = new HashMap<>();
                    }
                    String wkt = data.getWkt();
                    if (StringUtils.isNotBlank(wkt)) {
                        wkt = wkt.replace("POLYGON((", "").replace("))", "").replace(Constant.COMMA, Constant.SEMICOLON).replace(Constant.SPACE, Constant.COMMA);
                        String[] wktPointArr = wkt.split(Constant.SEMICOLON);
                        List<PtInPolyUtil.Point> points = new ArrayList<>();
                        for (String p : wktPointArr) {
                            String[] xyArr = p.split(Constant.COMMA);
                            PtInPolyUtil.Point point = new PtInPolyUtil.Point(Double.valueOf(xyArr[0]), Double.valueOf(xyArr[1]));
                            points.add(point);
                        }
                        data.setPoints(points);
                    }
                    congestionAreaDoMap.put(data.getRid(), data);
                    outMap.put(data.getCrossId(), congestionAreaDoMap);
                }
                if (outMap.size() > 0) {
                    sourceContext.collect(outMap);
                }
                TimeUnit.MINUTES.sleep(1);
            } catch (Exception e) {
                log.error("查询溢出检测区域: " + e.getMessage(), e);
            }
            TimeUnit.SECONDS.sleep(60);
        }
    }

    @Override
    public void cancel() {
        this.running = false;
//        client.destroy();
    }

    private void putData(HashMap<String, ConfigModel> hashMap, List<ConfigModelDo> data) {
        for (ConfigModelDo configModelDo : data) {
            String crossId = configModelDo.getCrossId();
            ConfigModel crossConfigModel = hashMap.get(crossId);
            if (crossConfigModel == null) {
                crossConfigModel = new ConfigModel();
            }
            String key = configModelDo.getKey();
            String value = configModelDo.getValue();
            if (StringUtils.isNotBlank(key) && StringUtils.isNotBlank(value)) {
                hashMap.put(crossId, crossConfigModel);
            }
        }
    }
}
