package com.wanji.indicators.source;

import com.alibaba.fastjson.JSONObject;
import com.google.common.net.HostAndPort;
import com.orbitz.consul.Consul;
import com.orbitz.consul.KeyValueClient;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.ConfigModelDo;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

/**
 * @Author ke.han
 * @Date 2020/4/16 19:03
 **/
public class ConsulConfigSource extends RichParallelSourceFunction<HashMap<String, ConfigModel>> {

    private static final Logger log = LoggerFactory.getLogger(ConsulConfigSource.class);

    private Consul client;

    private String host = "192.168.11.151";

    private int port = 8500;

    private int[] typeCodeArr;

    private volatile boolean running = true;

    public ConsulConfigSource(int[] typeCodeArr) {
        this.typeCodeArr = typeCodeArr;
    }

    public ConsulConfigSource() {}

    @Override
    public void open(Configuration parameters) throws Exception {
        try {
            PropertiesHelper instance = PropertiesHelper.getInstance();
            Properties properties = instance.getProperties();
            String consul_host_properties = properties.getProperty("consul.host");
            String consul_port_properties = properties.getProperty("consul.port");
            String consul_host_env = System.getenv("CONSUL_HOST");
            String consul_port_env = System.getenv("CONSUL_PORT");
            if (StringUtils.isNotBlank(consul_host_env)) {
                this.host = consul_host_env;
            } else {
                this.host = consul_host_properties;
            }
            if (StringUtils.isNotBlank(consul_port_env)) {
                this.port = Integer.parseInt(consul_port_env);
            } else {
                this.port = Integer.parseInt(consul_port_properties);
            }

        } catch (Exception e) {
            log.error("consul 服务异常 链接服务异常: " + e.getMessage());
        }
    }

    @Override
    public void run(SourceContext<HashMap<String,ConfigModel>> sourceContext) throws Exception {
        HashMap<String, ConfigModel> hashMap = new HashMap<>();
        while (running) {
            try {
                this.client = Consul.builder().withHostAndPort(HostAndPort.fromParts(host, port)).build();
                //通过客户端查询kv值  获取配置
                KeyValueClient keyValueClient = this.client.keyValueClient();
                List<String> keys = keyValueClient.getKeys(Constant.CONSUL_EVENT_SETTINGS);
                for (String key : keys) {
                    if (StringUtils.isNotBlank(key) && !key.contains(Constant.CONSUL_EVENT_TEMPLATE) && key.contains(Constant.CONSUL_EVENT_PARAM_SET)) {
                        try {
                            Optional<String> value = keyValueClient.getValue(key).get().getValueAsString();
                            if (value.isPresent()) {
                                List<ConfigModelDo> data = JSONObject.parseArray(value.get().replaceAll("\\\\",""), ConfigModelDo.class);
                                putData(hashMap, data);
                            }
                        } catch (Exception e) {
                            log.error("配置查询 ==>> consul 查询kv值异常: key: " + key + " 异常: " + e.getMessage() , e);
                        }
                    }
                }
                if (!hashMap.isEmpty()) {
                    sourceContext.collect(hashMap);
                }
                TimeUnit.MINUTES.sleep(1);
            } catch (Exception e) {
                log.error("consul 服务异常 查询配置异常: " + e.getMessage() , e);
            } finally {
                if (this.client != null) {
                    this.client.destroy();
                }
            }
            TimeUnit.SECONDS.sleep(33);
        }

    }

    @Override
    public void cancel() {
        this.running = false;
    }

    private void putData(HashMap<String, ConfigModel> hashMap, List<ConfigModelDo> data) {
        for (ConfigModelDo configModelDo : data) {
            String crossId = configModelDo.getCrossId();
            ConfigModel crossConfigModel = hashMap.get(crossId);
            if (crossConfigModel == null) {
                crossConfigModel = new ConfigModel();
            }
            String key = configModelDo.getKey();
            String value = configModelDo.getValue();
            if (StringUtils.isNotBlank(key) && StringUtils.isNotBlank(value)) {
                hashMap.put(crossId, crossConfigModel);
            }
        }
    }
}
