package com.wanji.indicators.source;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.mapper.BaseLaneInfoMapper;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/03/27 9:11
 */
@Service
public class CrossLaneInfoMapSource extends RichParallelSourceFunction<Map<String, List<BaseLaneInfo>>> {

    private static final Logger log = LoggerFactory.getLogger(LaneInfoSource.class);

    private volatile boolean running = true;

    Properties properties = null;

    private BaseLaneInfoMapper baseLaneInfoMapper;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoMapper = beanConf.getBean(BaseLaneInfoMapper.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

    }

    @Override
    public void run(SourceContext<Map<String, List<BaseLaneInfo>>> sourceContext) throws Exception {

        try {
            while (running) {
                String crossList = properties.getProperty("full.area.cross.list");
                LambdaQueryWrapper<BaseLaneInfo> queryWrapper = new LambdaQueryWrapper<>();
                String[] split = crossList.split(",");
                List<String> strings = Arrays.asList(split);
                // 只查询进口道
                queryWrapper.in(BaseLaneInfo::getCrossId, strings).eq(BaseLaneInfo::getType, 2);
                List<BaseLaneInfo> baseLaneInfos = baseLaneInfoMapper.selectList(queryWrapper);
                if (!CollectionUtils.isEmpty(baseLaneInfos)) {
                    // 设置行人车道号
                    setPedLaneId(baseLaneInfos);
                    Map<String, List<BaseLaneInfo>> map = baseLaneInfos.stream().collect(Collectors.groupingBy(BaseLaneInfo::getCrossId));
                    sourceContext.collect(map);
                }
                TimeUnit.MINUTES.sleep(1);
            }
        } catch (Exception e) {
            log.error("查询车道信息异常: " + e.getMessage(), e);
        }
    }

    private void setPedLaneId(List<BaseLaneInfo> baseLaneInfos) {
        String pedestrianIdStr = properties.getProperty("consume.pedestrian.baseLaneInfo");
        if (!StringUtils.isEmpty(pedestrianIdStr)) {
            String[] pedIdArr = pedestrianIdStr.split(",");
            for (String p : pedIdArr) {
                BaseLaneInfo baseLaneInfo = new BaseLaneInfo();
                baseLaneInfo.setCrossId(p.substring(0, 11));
                baseLaneInfo.setId(p);
                baseLaneInfo.setDir(Integer.parseInt(p.substring(p.length() - 1)));
                baseLaneInfos.add(baseLaneInfo);
            }
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}
