package com.wanji.indicators.source;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.event.CrossShapeDo;
import com.wanji.indicators.model.event.CrossShapeModel;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 查询 rid 转向数据
 */
public class CrossShapeSource extends RichParallelSourceFunction<HashMap<String, CrossShapeModel>> {

    private static final Logger log = LoggerFactory.getLogger(CrossShapeSource.class);

    private volatile boolean running = true;

    @Override
    public void run(SourceContext<HashMap<String, CrossShapeModel>> sourceContext) throws Exception {
        HashMap<String, CrossShapeModel> outMap = new HashMap<>();
        try {
            while (running) {
                String sql = "SELECT aa.rid_in,aa.rid_out,aa.turn_dir,aa.wkt,bb.angle AS angle_out,cc.angle AS angle_in " +
                        " FROM cross_turn_line aa " +
                        " LEFT JOIN rid bb ON aa.rid_out = bb.rid " +
                        " LEFT JOIN rid cc ON aa.rid_in = cc.rid " +
                        " WHERE aa.lane_num > 0";
                List<CrossShapeDo> ridLaneNumDos = null;//new JdbcUtil().excuteQueryToList(CrossShapeDo.class, sql);
                if (ridLaneNumDos != null && !ridLaneNumDos.isEmpty()) {
                    outMap.clear();
                    for (CrossShapeDo data : ridLaneNumDos) {
                        String rid_in = data.getRid_in();
                        String wkt = data.getWkt();
                        int turn_dir = data.getTurn_dir();
                        String rid_out = data.getRid_out();
                        double angle_out = data.getAngle_out();
                        double angle_in = data.getAngle_in();
                        CrossShapeModel m = new CrossShapeModel();
                        m.setAngle(angle_out);
                        m.setAngleIn(angle_in);
                        m.setRidIn(rid_in);
                        m.setRidOut(rid_out);
                        m.setTurnDir(turn_dir);
                        if (StringUtils.isNotBlank(wkt)) {
                            String[] wktArr = wkt.split(";");
                            String[] firstPoint = wktArr[0].split(",");
                            String[] lastPoint = wktArr[wktArr.length - 1].split(",");
                            double distance = PtInPolyUtil.getDistance(Double.parseDouble(firstPoint[0]), Double.parseDouble(firstPoint[1]), Double.parseDouble(lastPoint[0]), Double.parseDouble(lastPoint[1]));
                            m.setLength(distance);
                            m.setPointOutStart(new double[]{Double.parseDouble(lastPoint[0]), Double.parseDouble(lastPoint[1])});
                        }
                        outMap.put(rid_in + Constant.MARK + turn_dir, m);
                        outMap.put(rid_out + Constant.MARK + turn_dir, m);
                    }
                }
                if (outMap.size() > 0) {
                    sourceContext.collect(outMap);
                }
                TimeUnit.MINUTES.sleep(1);
    //            Thread.sleep(1000*30);
            }
        } catch (Exception e) {
            log.error("查询 rid 转向数据异常: " + e.getMessage(), e);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}
