package com.wanji.indicators.source;

import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.mapper.BaseCrossTurnInfoMapper;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 查询 进口道对应右转出口rid，及右转出口RID可以进入的各进口RID
 */
public class FirdsOfRightTridSource extends RichParallelSourceFunction<HashMap<String, Map<String,String>>> {

    private static final Logger log = LoggerFactory.getLogger(FirdsOfRightTridSource.class);

    private volatile boolean running = true;

    BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseCrossTurnInfoMapper = beanConf.getBean(BaseCrossTurnInfoMapper.class);


    }

    //20220401修改只查询有高精数据的路口
    private String getQuerySql() {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("SELECT a.inter_id,a.t_rid,string_agg(concat(a.f_rid,';',a.turn_dir_no),',') f_rids ");
        stringBuilder.append("FROM cross_relation b ");
        stringBuilder.append("JOIN (SELECT DISTINCT inter_id from lane_obj where data_source='2' and inter_id is not null) b ");
        stringBuilder.append("ON a.inter_id=b.inter_id ");
        stringBuilder.append("GROUP BY a.inter_id,a.t_rid");
        return  stringBuilder.toString();
    }

    @Override
    public void run(SourceContext<HashMap<String, Map<String,String>>> sourceContext) throws Exception {
        HashMap<String, Map<String,String>> retMap = new HashMap<>();
        while (running) {

            List<BaseCrossTurnInfo> dataList = baseCrossTurnInfoMapper.findRidTurnInfo(null);
            if (dataList != null && !dataList.isEmpty()) {
                Map<String,List<BaseCrossTurnInfo>> groupByTRid = dataList.stream().collect(Collectors.groupingBy(o->o.getOutRid()));
                for (Map.Entry<String,List<BaseCrossTurnInfo>> entry : groupByTRid.entrySet()) {
                    List<BaseCrossTurnInfo> value = entry.getValue();
                    String crossId = value.get(0).getCrossId();
                    //出口rid
                    String trid = entry.getKey();
                    //进口rid集
                    String frids = value.stream().map(o->o.getInRid()+";"+o.getTurnType()).collect(Collectors.joining(","));

                    Map<String, String> refMap = retMap.get(crossId);
                    if (refMap == null) {
                        refMap = new HashMap<>();
                    }
                    refMap.put(trid,frids);
                    retMap.put(crossId, refMap);
                }
            }
            if (retMap.size() > 0) {
                sourceContext.collect(retMap);
            }
            TimeUnit.MINUTES.sleep(1);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}
