package com.wanji.indicators.source;

import com.wanji.indicators.entity.ForeignDevicePO;
import com.wanji.indicators.mapper.ForeignDeviceMapper;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/07/17 11:22
 */
@Service
public class ForeignDeviceInfoSource extends RichParallelSourceFunction<Map<String, List<ForeignDevicePO>>> {

    private static final Logger log = LoggerFactory.getLogger(ForeignDeviceInfoSource.class);

    private volatile boolean running = true;

    Properties properties = null;

    private ForeignDeviceMapper foreignDeviceMapper;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        foreignDeviceMapper = beanConf.getBean(ForeignDeviceMapper.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

    }

    @Override
    public void run(SourceContext<Map<String, List<ForeignDevicePO>>> sourceContext) throws Exception {

        try {
            while (running) {
                List<ForeignDevicePO> foreignDevicePOS = foreignDeviceMapper.selectAllDevices();
                if (!CollectionUtils.isEmpty(foreignDevicePOS)) {
                    Map<String, List<ForeignDevicePO>> map = foreignDevicePOS.stream().collect(Collectors.groupingBy(ForeignDevicePO::getDeviceNo));
                    sourceContext.collect(map);
                }
                TimeUnit.MINUTES.sleep(1);
            }
        } catch (Exception e) {
            log.error("查询外部设备表信息异常: " + e.getMessage(), e);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}

