package com.wanji.indicators.source;

import com.wanji.indicators.entity.BaseLaneDetector;
import com.wanji.indicators.service.BaseLaneDetectorService;
import com.wanji.indicators.service.impl.BaseLaneDetectorServiceImpl;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 车道信息查询
 */
@Service
public class LaneDetectorSource extends RichParallelSourceFunction<List<BaseLaneDetector>> {

    private static final Logger log = LoggerFactory.getLogger(LaneDetectorSource.class);

    private volatile boolean running = true;

    BaseLaneDetectorService baseLaneDetectorService;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneDetectorService = beanConf.getBean(BaseLaneDetectorServiceImpl.class);
    }


    @Override
    public void run(SourceContext<List<BaseLaneDetector>> sourceContext) throws Exception {
        try {
            while (running) {
                List<BaseLaneDetector> baseLaneDetectorList = baseLaneDetectorService.getSenseLaneDetectors();
                sourceContext.collect(baseLaneDetectorList);
                TimeUnit.MINUTES.sleep(1);
            }
        } catch (Exception e) {
            log.error("查询线圈信息异常: " + e.getMessage(), e);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}
