package com.wanji.indicators.source;

import com.wanji.indicators.base.env.BeanFactory;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.ExecutionConfig;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 车道信息查询
 */
@Service
public class LaneInfoSource extends RichParallelSourceFunction<Map<String, BaseLaneInfo>> {

    private static final Logger log = LoggerFactory.getLogger(LaneInfoSource.class);

    private volatile boolean running = true;

    Properties properties = null;

    BaseLaneInfoService baseLaneInfoService;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

    }


    @Override
    public void run(SourceContext<Map<String, BaseLaneInfo>> sourceContext) throws Exception {

        try {
            while (running) {
                String crossList = properties.getProperty("full.area.cross.list");
                Map<String, BaseLaneInfo> map = baseLaneInfoService.findLaneInfo(crossList);
                sourceContext.collect(map);
                TimeUnit.MINUTES.sleep(1);
                // TimeUnit.SECONDS.sleep(1);
            }
        } catch (Exception e) {
            log.error("查询车道信息异常: " + e.getMessage(), e);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }

    /**
     * 理论冲突相位数据，
     * 相位标号定义 以北方向为基准定义 1:北直行相位 2：东直行相位 3：南直行相位 4：西直行相位 5：北左转相位 6：东左转 7：南左转 8：西左转
     */
    public Map<Integer, List<Integer>> createConflictSlots() {

        Map<Integer, List<Integer>> maplist = new HashMap<Integer, List<Integer>>();

        for (int i = 1; i < 9; i++) {
            Integer sidewalkDir = null;
            List<Integer> conflictList = maplist.get(i);
            if (Objects.isNull(conflictList)) {
                conflictList = new ArrayList<>();
            }
            if (i < 5) { // 直行相位
                // 直行相位的冲突相位点
                // 顺时针直行相位
                int clockwise = (i + 1) % 4 == 0 ? 4 : (i + 1) % 4;
                // 逆时针直行相位
                int antiClockWise = (i - 1) % 4 == 0 ? 4 : (i - 1) % 4;
                // 对向左转冲突点
                int acrossLeft = ((i + 2) % 4 == 0 ? 4 : (i + 2) % 4) + 4;
                // 逆时针左转冲突相位点
                int antiLeft = ((i - 1) % 4 == 0 ? 4 : (i - 1) % 4) + 4;

                conflictList.add(clockwise);
                conflictList.add(antiClockWise);
                conflictList.add(acrossLeft);
                conflictList.add(antiLeft);
            } else {// 左转相位冲突相位

                // 顺时针直行相位
                int antiClockWiseStraight = (i + 1) % 4 == 0 ? 4 : (i + 1) % 4;
                // 对向直行冲突点
                int acrossStraight = (i + 2) % 4 == 0 ? 4 : (i + 2) % 4;
                // 顺时针左转相位
                int clockwiseLeft = ((i + 1) % 4 == 0 ? 4 : (i + 1) % 4) + 4;
                // 逆时针左转相位
                int antiClockWiseLeft = ((i - 1) % 4 == 0 ? 4 : (i - 1) % 4) + 4;

                conflictList.add(antiClockWiseStraight);
                conflictList.add(acrossStraight);
                conflictList.add(clockwiseLeft);
                conflictList.add(antiClockWiseLeft);
            }
            maplist.put(i, conflictList);
        }

        return maplist;

    }

//    private static String getQuerySql1() {
//        StringBuilder stringBuilder = new StringBuilder();
//        stringBuilder.append("SELECT a.inter_id,a.f_rid,a.turn_dir_no,b.dir_4_no, ");
//        stringBuilder.append("( ");
//        stringBuilder.append("CASE ");
//        stringBuilder.append("WHEN b.dir_4_no=3 and turn_dir_no=2 THEN 1 ");
//        stringBuilder.append("WHEN b.dir_4_no=3 and turn_dir_no=1 THEN 5 ");
//        stringBuilder.append("WHEN b.dir_4_no=4 and turn_dir_no=2 THEN 2 ");
//        stringBuilder.append("WHEN b.dir_4_no=4 and turn_dir_no=1 THEN 6 ");
//        stringBuilder.append("WHEN b.dir_4_no=1 and turn_dir_no=2 THEN 3 ");
//        stringBuilder.append("WHEN b.dir_4_no=1 and turn_dir_no=1 THEN 7 ");
//        stringBuilder.append("WHEN b.dir_4_no=2 and turn_dir_no=2 THEN 4 ");
//        stringBuilder.append("WHEN b.dir_4_no=2 and turn_dir_no=1 THEN 8 ");
//        stringBuilder.append("END ");
//        stringBuilder.append(") as slot_no ");
//        stringBuilder.append("FROM cross_relation a ");
//        stringBuilder.append("JOIN rid b on a.f_rid=b.rid ");
//        stringBuilder.append("WHERE  turn_dir_no not in (3,4)");
//        return stringBuilder.toString();
//    }

//    public void run1(SourceContext<Map<String, List<Map<String, Object>>>> sourceContext) throws Exception {
//        //key:rid->turn value:冲突rid及转向
//        Map<String, List<Map<String, Object>>> retMap = new HashMap<>();
//        try {
//            List<HashMap> dataList = new JdbcUtil().excuteQueryToList(HashMap.class, getQuerySql1());
//            Map<Integer, List<Integer>> baseConflictInfo = createConflictSlots();
//
//            if (dataList != null && !dataList.isEmpty()) {
//                retMap.clear();
//                for (HashMap<String, Object> map : dataList) {
//                    String inter_id = map.get("inter_id").toString();
//                    String rid = map.get("f_rid").toString();
//                    String turn_dir_no = map.get("turn_dir_no").toString();
//
//                    //当前相位编号
//                    Integer slot_no = Integer.valueOf(map.get("slot_no").toString());
//                    //冲突相位
//                    List<Integer> conflictSlot = baseConflictInfo.get(slot_no);
//                    //冲突相位关联rid等信息
//                    List<HashMap<String, Object>> conflictSlotList = dataList.stream().filter(m ->Objects.equals(inter_id,m.get("inter_id"))  && conflictSlot.contains(Integer.valueOf(m.get("slot_no").toString()))).collect(Collectors.toList());
//                    if (!conflictSlotList.isEmpty()) {
//                        String key = rid + Constant.MARK + turn_dir_no;
//                        List<Map<String, Object>> tmplist = retMap.get(key);
//                        if (Objects.isNull(tmplist)) {
//                            tmplist = new ArrayList<>();
//                        }
//                        tmplist.addAll(conflictSlotList);
//                        retMap.put(key, tmplist);
//                        sourceContext.collect(retMap);
//                    }
//                }
//
//
//            }
//        } catch (Exception e) {
//            log.error("查询 rid 冲突信息数据异常: " + e.getMessage(), e);
//        }
//    }

}
