package com.wanji.indicators.source;

import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.service.BaseLaneInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 查询 进口道对应右转出口rid，及右转出口RID可以进入的各进口RID
 */
@Slf4j
public class LaneRidLengthSource extends RichParallelSourceFunction<Map<String,List<BaseLaneInfo>>> {

    private volatile boolean running = true;

    @Autowired
    BaseLaneInfoService baseLaneInfoService;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoService = beanConf.getBean(BaseLaneInfoService.class);
    }


    @Override
    public void run(SourceContext<Map<String,List<BaseLaneInfo>>> sourceContext) throws Exception {
        while (running) {
            List<BaseLaneInfo> list = baseLaneInfoService.findLaneRidInfo();

            if (list.size() > 0) {
                Map<String,List<BaseLaneInfo>> group = list.stream().collect(Collectors.groupingBy(o->o.getId()));
                sourceContext.collect(group);
            }
            TimeUnit.MINUTES.sleep(1);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }
}
