package com.wanji.indicators.source;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.asynchttpclient.AsyncHttpClient;
import org.asynchttpclient.BoundRequestBuilder;
import org.asynchttpclient.Dsl;
import org.asynchttpclient.ws.WebSocket;
import org.asynchttpclient.ws.WebSocketListener;
import org.asynchttpclient.ws.WebSocketUpgradeHandler;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import javax.xml.bind.DatatypeConverter;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;

/**
 * @author fengyi
 * @date 2023/10/22
 * @description
 */
@Slf4j
public class LightStatusKafkaSource extends RichSourceFunction<String> {
    private volatile boolean running = true;

    KafkaSource<String> lightStatusSource = null;
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        PropertiesHelper instance = PropertiesHelper.getInstance();
        lightStatusSource = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "false")
                .setBootstrapServers(instance.getProperties().getProperty("bootstrap.servers"))
                .setTopics("cross_lights_status")
                .setGroupId(UUID.randomUUID().toString())
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();
    }

    @Override
    public void run(SourceContext<String> ctx) throws Exception {

    }

    @Override
    public void cancel() {
        log.info("cancel function called");
        running = false;
    }

}
