package com.wanji.indicators.source;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;
import org.asynchttpclient.AsyncHttpClient;
import org.asynchttpclient.BoundRequestBuilder;
import org.asynchttpclient.Dsl;
import org.asynchttpclient.ws.WebSocket;
import org.asynchttpclient.ws.WebSocketListener;
import org.asynchttpclient.ws.WebSocketUpgradeHandler;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import javax.xml.bind.DatatypeConverter;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;

/**
 * @author fengyi
 * @date 2023/10/22
 * @description
 */
@Slf4j
public class LightStatusWebSocketSource extends RichSourceFunction<String> {
    private boolean running = true;
    transient AsyncHttpClient client;
    transient List<BoundRequestBuilder> boundRequestBuilderList;
    transient WebSocketUpgradeHandler.Builder webSocketListener;
    private BlockingQueue<String> messages = new ArrayBlockingQueue<>(100);

    Properties properties = null;
    BaseLaneInfoService baseLaneInfoService;

    String signalStatusUrl = null;
    String crossList = null;

    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

        crossList = properties.getProperty("full.area.cross.list");

        String[] sps = crossList.split(",");
        for (String crossId : sps) {
            String signalStatusUrl = properties.getProperty("signal.status.url");
            signalStatusUrl = signalStatusUrl.replace("{crossId}", crossId);
            BoundRequestBuilder boundRequestBuilder = client.prepareGet(signalStatusUrl);
            boundRequestBuilderList.add(boundRequestBuilder);
        }
        client = Dsl.asyncHttpClient();
        webSocketListener = new WebSocketUpgradeHandler.Builder();
    }

    @Override
    public void run(SourceContext<String> ctx) throws Exception {
        WebSocketUpgradeHandler webSocketUpgradeHandler = webSocketListener.addWebSocketListener(
                new WebSocketListener() {

                    private final ObjectMapper myMapper = new ObjectMapper();

                    private String getRsvpId(String payload) {
                        try {
                            Map map = myMapper.readValue(payload, Map.class);
                            Object rsvpId = map.get("rsvp_id");
                            return rsvpId != null ? rsvpId.toString() : "NOT FOUND";
                        } catch (IOException e) {
                            log.error("Mapping failed, returning 'null'", e);
                            return "NULL";
                        }
                    }

                    @Override
                    public void onOpen(WebSocket webSocket) {
                    }

                    @Override
                    public void onClose(WebSocket webSocket, int i, String s) {
                    }

                    @Override
                    public void onError(Throwable throwable) {
                    }

                    @Override
                    public void onTextFrame(String payload, boolean finalFragment, int rsv) {
                        log.debug("onTextFrame({}), rsvp_id={}", hash(payload), getRsvpId(payload));
                        if (payload != null) {
                            try {
                                messages.put(payload);
                            } catch (InterruptedException e) {
                                log.error("Interrupted!", e);
                                Thread.currentThread().interrupt();
                            }
                        }
                    }
                }).build();

        for (BoundRequestBuilder boundRequestBuilder : boundRequestBuilderList) {
            boundRequestBuilder.execute(webSocketUpgradeHandler).get();
        }

        while (running) {
            ctx.collect(messages.take());
        }
        running = false;
    }

    @Override
    public void cancel() {
        log.info("cancel function called");
        running = false;
    }


    private String hash(String input) {
        if (input == null) {
            return "-- NULL --";
        }

        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(input.getBytes());
            byte[] digest = md.digest();
            return DatatypeConverter.printHexBinary(digest).toUpperCase();
        } catch (NoSuchAlgorithmException e) {
            log.error("Cound not instantiate MD5", e);
            return "--NOT CALCULATED--";
        }
    }
}
