package com.wanji.indicators.source.debug;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.EsDateIndexUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.source.SourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;


public class DebugSourceCarCrossFrameDateFile implements SourceFunction<CrossFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(DebugSourceCarCrossFrameDateFile.class);

    private volatile boolean running = true;

    private String path;

    public DebugSourceCarCrossFrameDateFile(String path) {
        this.path = path;
    }

    //获取文件完整路径
    private String[] getFilePath() {
        Date startTime = DateUtil.stringToDate("2023-10-19 00:00:00",DateUtil.YYYY_MM_DD_HH_MM_SS);
        Date endTime = DateUtil.stringToDate("2023-10-19 23:59:00",DateUtil.YYYY_MM_DD_HH_MM_SS);

        String[] fileList  = EsDateIndexUtil.getFileListByTime("d://flink","compacted",startTime.getTime(),endTime.getTime(),"2");

        return fileList;
    }

    @Override
    public void run(SourceContext<CrossFrameModel> sourceContext) throws Exception {
        Properties props = System.getProperties();
        String osName = props.getProperty("os.name");
        long x = 0;
        //清除文件缓存
        ConcurrentHashMap<String, CarTrackModel> vehicleInfoCache = new ConcurrentHashMap();
//        while (running) {
            File files = getFile();
            if (files != null) {
                TimeUnit.MILLISECONDS.sleep(100);
                List<String> sortList = Arrays.asList(getFilePath());
                for (String str : sortList) {
                    if (StringUtils.isNotBlank(osName) && !osName.contains("Windows")) {
                        str = "/" + str;
                    }
                    long l = System.currentTimeMillis();
                    boolean isDelete = false;
                    File file = new File(path + str.trim());
                    if (file.exists()) {
                        isDelete = true;

                        BufferedReader read = null;
                        String s = "";
                        try {
                            read = new BufferedReader(new InputStreamReader(new FileInputStream(file), "UTF-8"), 10 * 1024 * 1024);
                            log.info("Read file: *****| " + str + " |*****");
                            while ((s = (read.readLine())) != null) {
                                if (StringUtils.isNotBlank(s)) {
                                    JSONObject jsonObject = JSONObject.parseObject(s);
                                    //FrameModel frameModel = JSONObject.parseObject(s, FrameModel.class);

                                    long dataTime = jsonObject.getLongValue("globalTimeStamp");
                                    JSONArray track = jsonObject.getJSONArray("e1FrameParticipant");

                                    if (!track.isEmpty()) {
                                        x++;
                                        Map<String,CrossFrameModel> crossFrameModelMap = new HashMap<>();

                                        for (Object obj : track) {
                                            CarTrackModel carTrack = JSONObject.parseObject(obj.toString(), CarTrackModel.class);
                                            carTrack.setGlobalTimeStamp(dataTime);
                                            //补充路口内从哪个进口进入到路口
                                            cacheData(carTrack,vehicleInfoCache);
                                            if (carTrack.getRoadnet().getLaneId().equals("null")) {
                                                continue;
                                            }

                                            String crossId = carTrack.getRoadnet().getCrossId();
                                            String laneId = carTrack.getRoadnet().getLaneId();
                                            if (laneId.equals("null") && !crossId.equals("null")){
                                                carTrack.getRoadnet().setInCrossFlag(1);
                                            }
                                            if (Objects.equals("null",crossId)){
                                                crossId = Constant.EMPTY+"_NULL";
                                            }
                                            CrossFrameModel  crossFrameModel = crossFrameModelMap.get(crossId);
                                            if (Objects.isNull(crossFrameModel)) {
                                                crossFrameModel = new CrossFrameModel();
                                            }
                                            crossFrameModel.setGlobalTimeStamp(dataTime);
                                            crossFrameModel.setCrossId(crossId);
                                            crossFrameModel.getTrackList().add(carTrack);

                                            crossFrameModelMap.put(crossId,crossFrameModel);
                                        }
                                        for (Map.Entry<String,CrossFrameModel> entry : crossFrameModelMap.entrySet()){
                                            sourceContext.collect(entry.getValue());
                                        }

//                                        log.info(" <<=================||--=  一共计算数据  "+x+"  行  ThreadId: "+Thread.currentThread().getId()+" time:" + frameModel.getTime());
                                    }
                                }
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                            isDelete = false;
                            TimeUnit.SECONDS.sleep(1);
                        } finally {
                            try {
                                if (read != null) {
                                    read.close();
                                }
                            } catch (IOException e) {}
                        }
                    }
                    if (isDelete) {
                        log.info(" <<=================||--=  一共计算数据  " + x + "  行 =--||=================>> ");
                        //deleteFile(l, str, file);
                    }
                }
            }
//        }
    }

    @Override
    public void cancel() {
        running = false;
    }



    private boolean deleteFile(long l , String str , File file) throws InterruptedException {
        boolean delete = file.delete();
        if (delete) {
            long e = System.currentTimeMillis();
            log.info(" time consuming: " + (e - l) / 1000 + " s " + (e - l) % 1000 + " ms ==> The file is over: " + str + " clean up.\n");
            TimeUnit.MILLISECONDS.sleep(200);
            return true;
        } else {
            return false;
        }
    }

    private List<String> getFileNameArr(File files) {
        List<String> fileList = new ArrayList<>();
        String[] fileNames = files.list();
        if(fileNames != null) {
            for (String name : fileNames) {
                fileList.add(name);
            }
        }
        return fileList;
    }


    private File getFile() {
        File files = null;
        try {
            files = new File(path);
        } catch (Exception e) {
            try {
                Thread.sleep(3000L);
            } catch (InterruptedException exception) {
            }
        }
        return files;
    }



    private void cacheData(CarTrackModel carTrack, ConcurrentHashMap<String, CarTrackModel> vehicleInfoCache) {
        String crossId = carTrack.getRoadnet().getCrossId();
        int trackId = carTrack.getId();
        CarTrackModel cache = vehicleInfoCache.get(crossId + trackId);

        if (cache==null && !carTrack.getRoadnet().getLaneId().equals("null")) {
                carTrack.getRoadnet().setRid(carTrack.getRoadnet().getRid());
                carTrack.getRoadnet().setLaneId(carTrack.getRoadnet().getLaneId());
                carTrack.getRoadnet().setCrossId(carTrack.getRoadnet().getCrossId());
                vehicleInfoCache.put(crossId + trackId, carTrack);
        }

        if (cache != null) {
            if (carTrack.getRoadnet().getLaneId().equals("null")) {
                carTrack.getRoadnet().setRid(cache.getRoadnet().getRid());
                carTrack.getRoadnet().setLaneId(cache.getRoadnet().getLaneId());
                carTrack.getRoadnet().setCrossId(cache.getRoadnet().getCrossId());
                carTrack.getRoadnet().setInCrossFlag(1);
                vehicleInfoCache.put(crossId + trackId, carTrack);
            }
        }
    }

    /**
     * 排序
     *
     * @param list
     * @return
     */
    public static List<String> getSortList(List<String> list) {
        try {
            Collections.sort(list, new Comparator<String>() {
                @Override
                public int compare(String o1, String o2) {
                    long o1x = Long.valueOf(o1.substring(o1.lastIndexOf("_") + 1, o1.indexOf(".")));
                    long o2x = Long.valueOf(o2.substring(o2.lastIndexOf("_") + 1, o2.indexOf(".")));
                    if (o1x > o2x) {
                        return 1;
                    }
                    if (o1x == o2x) {
                        return 0;
                    }
                    return -1;
                }
            });
        } catch (Exception e) {
            log.error("SortListException:" + e);
            return list;
        }
        return list;
    }


    public static long getCreationTime(File file) {
        if (file == null) {
            return System.currentTimeMillis();
        }
        BasicFileAttributes attr = null;
        try {
            Path path = file.toPath();
            attr = Files.readAttributes(path, BasicFileAttributes.class);
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 创建时间
        Instant instant = attr.creationTime().toInstant();
        // 更新时间
//        Instant instant = attr.lastModifiedTime().toInstant();
        // 上次访问时间
//        Instant instant = attr.lastAccessTime().toInstant();
        String format = DateTimeFormatter.ofPattern(DateUtil.YYYY_MM_DD_HH_MM_SS).withZone(ZoneId.systemDefault()).format(instant);
        return DateUtil.StringToMillis(format);
    }

}
