package com.wanji.indicators.source.debug;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.export.KafkaRecordModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.EsDateIndexUtil;
import com.wanji.indicators.util.FileUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.source.SourceFunction;
import org.apache.kafka.common.record.TimestampType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 生成JDH模拟轨迹数据格式
 */
public class HistoryTrackFileMockConvert {

    private static final Logger log = LoggerFactory.getLogger(HistoryTrackFileMockConvert.class);

    private volatile boolean running = true;


    public static void main(String[] args) {
        //加载配置项 并获取工作路径
        String inPath = "D:\\WanJi-Work\\07 项目\\02 济南\\数据\\";
        String outPath = "d:/JNMatchResultMiniData";
        String startTime = "2023-12-17 08:00:00";
        String endTime = "2023-12-17 08:05:00";

        HistoryTrackFileMockConvert obj = new HistoryTrackFileMockConvert(inPath, startTime, endTime, outPath);
        obj.run();
    }

    private String path;
    private String outPath;
    String startTime;
    String endTime;

    public HistoryTrackFileMockConvert(String path, String startTime, String endTime, String outPath) {
        this.path = path;
        this.startTime = startTime;
        this.endTime = endTime;
        this.outPath = outPath;
    }

    //获取文件完整路径
    private String[] getFilePath() {
        Date startTime = DateUtil.stringToDate(this.startTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        Date endTime = DateUtil.stringToDate(this.endTime, DateUtil.YYYY_MM_DD_HH_MM_SS);

        String[] fileList = EsDateIndexUtil.getFileListByTime(path, "compacted", startTime.getTime(), endTime.getTime(), "2");

        return fileList;
    }

    public void run() {
        Properties props = System.getProperties();
        String osName = props.getProperty("os.name");
        long x = 0;
//        while (running) {
        File files = getFile();
//            if (files != null) {
        List<String> sortList = Arrays.asList(getFilePath());
        sortList = sortList.stream().sorted().collect(Collectors.toList());
        for (String str : sortList) {
            if (StringUtils.isNotBlank(osName) && !osName.contains("Windows")) {
                str = "/" + str;
            }
            long l = System.currentTimeMillis();
            boolean isDelete = false;
            File file = new File(str.trim());
            if (file.exists()) {
                isDelete = true;

                BufferedReader read = null;
                String s = "";
                try {
                    read = new BufferedReader(new InputStreamReader(new FileInputStream(file), "UTF-8"), 10 * 1024 * 1024);
                    //log.info("Read file: *****| " + str + " |*****");
                    while ((s = (read.readLine())) != null) {
                        if (StringUtils.isNotBlank(s)) {
                            s = s.replaceAll("鲁", "万");
                            JSONObject jsonObject = JSONObject.parseObject(s);
                            jsonObject.put("timeStamp", jsonObject.getString("frameTime"));

                            long globalTime = jsonObject.getLongValue("globalTimeStamp");
                            if (globalTime > DateUtil.StringToMillis(this.endTime, "yyyy-MM-dd HH:mm:ss")) {
                                break;
                            }

                            KafkaRecordModel model = new KafkaRecordModel();
                            model.setTimestamp(globalTime);
                            model.setTimestampType(TimestampType.CREATE_TIME);
                            model.setValues(jsonObject);
                            model.setTopic("JNMatchResultMiniData");

                            x++;
                            FileUtil.writeApend(this.outPath, JSONObject.toJSONString(model));
                            //sourceContext.collect(s);
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    isDelete = false;
                } finally {
                    try {
                        if (read != null) {
                            read.close();
                        }
                    } catch (IOException e) {
                    }
                }
            }
            if (isDelete) {
                //log.info(" <<=================||--=  一共计算数据  " + x + "  行 =--||=================>> ");
                //deleteFile(l, str, file);
            }
        }
//            }
//        }
    }


    private boolean deleteFile(long l, String str, File file) throws InterruptedException {
        boolean delete = file.delete();
        if (delete) {
            long e = System.currentTimeMillis();
            log.info(" time consuming: " + (e - l) / 1000 + " s " + (e - l) % 1000 + " ms ==> The file is over: " + str + " clean up.\n");
            TimeUnit.MILLISECONDS.sleep(200);
            return true;
        } else {
            return false;
        }
    }

    private List<String> getFileNameArr(File files) {
        List<String> fileList = new ArrayList<>();
        String[] fileNames = files.list();
        if (fileNames != null) {
            for (String name : fileNames) {
                fileList.add(name);
            }
        }
        return fileList;
    }


    private File getFile() {
        File files = null;
        try {
            files = new File(path);
        } catch (Exception e) {
            try {
                Thread.sleep(3000L);
            } catch (InterruptedException exception) {
            }
        }
        return files;
    }


    private void cacheData(CarTrackModel carTrack, ConcurrentHashMap<String, CarTrackModel> vehicleInfoCache) {
        String crossId = carTrack.getRoadnet().getCrossId();
        int trackId = carTrack.getId();
        CarTrackModel cache = vehicleInfoCache.get(crossId + trackId);

        if (cache == null && !carTrack.getRoadnet().getLaneId().equals("null")) {
            carTrack.getRoadnet().setRid(carTrack.getRoadnet().getRid());
            carTrack.getRoadnet().setLaneId(carTrack.getRoadnet().getLaneId());
            carTrack.getRoadnet().setCrossId(carTrack.getRoadnet().getCrossId());
            vehicleInfoCache.put(crossId + trackId, carTrack);
        }

        if (cache != null) {
            if (carTrack.getRoadnet().getLaneId().equals("null")) {
                carTrack.getRoadnet().setRid(cache.getRoadnet().getRid());
                carTrack.getRoadnet().setLaneId(cache.getRoadnet().getLaneId());
                carTrack.getRoadnet().setCrossId(cache.getRoadnet().getCrossId());
                carTrack.getRoadnet().setInCrossFlag(1);
                vehicleInfoCache.put(crossId + trackId, carTrack);
            }
        }
    }

    /**
     * 排序
     *
     * @param list
     * @return
     */
    public static List<String> getSortList(List<String> list) {
        try {
            Collections.sort(list, new Comparator<String>() {
                @Override
                public int compare(String o1, String o2) {
                    long o1x = Long.valueOf(o1.substring(o1.lastIndexOf("_") + 1, o1.indexOf(".")));
                    long o2x = Long.valueOf(o2.substring(o2.lastIndexOf("_") + 1, o2.indexOf(".")));
                    if (o1x > o2x) {
                        return 1;
                    }
                    if (o1x == o2x) {
                        return 0;
                    }
                    return -1;
                }
            });
        } catch (Exception e) {
            log.error("SortListException:" + e);
            return list;
        }
        return list;
    }


    public static long getCreationTime(File file) {
        if (file == null) {
            return System.currentTimeMillis();
        }
        BasicFileAttributes attr = null;
        try {
            Path path = file.toPath();
            attr = Files.readAttributes(path, BasicFileAttributes.class);
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 创建时间
        Instant instant = attr.creationTime().toInstant();
        // 更新时间
//        Instant instant = attr.lastModifiedTime().toInstant();
        // 上次访问时间
//        Instant instant = attr.lastAccessTime().toInstant();
        String format = DateTimeFormatter.ofPattern(DateUtil.YYYY_MM_DD_HH_MM_SS).withZone(ZoneId.systemDefault()).format(instant);
        return DateUtil.StringToMillis(format);
    }

}
