package com.wanji.indicators.source.debug;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.export.KafkaRecordModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.EsDateIndexUtil;
import com.wanji.indicators.util.FileUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.kafka.common.record.TimestampType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 生成JDH模拟轨迹数据格式
 */
public class SortHistoryTrackFileMockConvert {

    private static final Logger log = LoggerFactory.getLogger(SortHistoryTrackFileMockConvert.class);

    private volatile boolean running = true;


    public static void main(String[] args) {
        //加载配置项 并获取工作路径
        String inPath = "D:\\WanJi-Work\\07 项目\\02 济南\\数据\\";
        String outPath = "d:/JNMatchResultMiniData";
        String startTime = "2024-02-25 08:00:00";
        String endTime = "2024-02-25 08:05:00";
        SortHistoryTrackFileMockConvert obj = new SortHistoryTrackFileMockConvert(inPath, startTime, endTime, outPath);
        obj.run();
    }

    private String path;
    private String outPath;
    String startTime;
    String endTime;

    public SortHistoryTrackFileMockConvert(String path, String startTime, String endTime, String outPath) {
        this.path = path;
        this.startTime = startTime;
        this.endTime = endTime;
        this.outPath = outPath;
    }

    //获取文件完整路径
    private String[] getFilePath() {
        Date startTime = DateUtil.stringToDate(this.startTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        Date endTime = DateUtil.stringToDate(this.endTime, DateUtil.YYYY_MM_DD_HH_MM_SS);

        String[] fileList = EsDateIndexUtil.getFileListByTime(path, "compacted", startTime.getTime(), endTime.getTime(), "2");

        return fileList;
    }

    public void run() {
        Properties props = System.getProperties();
        String osName = props.getProperty("os.name");
        long x = 0;
        List<String> sortList = Arrays.asList(getFilePath());
        sortList = sortList.stream().sorted().collect(Collectors.toList());
        for (String str : sortList) {
            if (StringUtils.isNotBlank(osName) && !osName.contains("Windows")) {
                str = "/" + str;
            }
            long l = System.currentTimeMillis();
            boolean isDelete = false;
            File file = new File(str.trim());
            if (file.exists()) {
                isDelete = true;

                BufferedReader read = null;
                String s = "";
                try {
                    read = new BufferedReader(new InputStreamReader(new FileInputStream(file), "UTF-8"), 10 * 1024 * 1024);

                    Map<Long, KafkaRecordModel> timeLineMap = new TreeMap<>();
                    String s1 = null;
                    int ln = 0;
                    while ((s1 = (read.readLine())) != null) {
                        s1 = s1.replaceAll("鲁", "万");
                        JSONObject jsonObject = JSONObject.parseObject(s1);
                        jsonObject.put("timeStamp", jsonObject.getString("frameTime"));
                        ln++;
                        long ts = jsonObject.getLongValue("globalTimeStamp");
                        if (ts < DateUtil.StringToMillis(endTime, "yyyy-MM-dd HH:mm:ss")) {
                            KafkaRecordModel model = new KafkaRecordModel();
                            model.setTimestamp(ts);
                            model.setTimestampType(TimestampType.CREATE_TIME);
                            model.setValues(jsonObject);
                            model.setTopic("JNMatchResultMiniData");
                            timeLineMap.put(ts, model);
                        }
                    }

                    for (Map.Entry<Long, KafkaRecordModel> entry : timeLineMap.entrySet()) {
                        FileUtil.writeApend(this.outPath, JSONObject.toJSONString(entry.getValue()));
                    }

                    System.out.println("Finished:"+file.getName());
                } catch (Exception e) {
                    e.printStackTrace();
                    isDelete = false;
                } finally {
                    try {
                        if (read != null) {
                            read.close();
                        }

                    } catch (IOException e) {
                    }
                }
            }
            if (isDelete) {
                //log.info(" <<=================||--=  一共计算数据  " + x + "  行 =--||=================>> ");
                //deleteFile(l, str, file);
            }
        }
//            }
//        }
    }


}
