package com.wanji.indicators.task;

import com.wanji.indicators.event.cross.conflictpoint.func.ConflictLaneInfoCoFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.LanePeriodicDataModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadLockStatusModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.LaneTrafficStateMain;
import com.wanji.indicators.task.trafficstate.func.ConnectLaneInfoCoFlatMap;
import com.wanji.indicators.task.trafficstate.func.LanePeriodDataFlatMap;
import com.wanji.indicators.task.trafficstate.func.LaneProcessWindow;
import com.wanji.indicators.task.trafficstate.model.LanePeriodData;
import com.wanji.indicators.task.trafficstate.model.LaneTrafficStateResult;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.AscendingTimestampsWatermarks;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.SlidingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;

import java.util.List;
import java.util.Objects;
import java.util.Properties;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Slf4j
public class BasePeriodIndicatorMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static  final String GROUP_ID= "FlinkConsumerGroup-V56-PERIOD";

    public static void main(String[] args) {

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        String topic = properties.getProperty("period.topic");

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        String groupId  =GROUP_ID;
        String groupChangeFlag = properties.getProperty("consumer.group.change.flag");
        if (Boolean.parseBoolean(groupChangeFlag)){
            groupId += DateUtil.toDateTime(System.currentTimeMillis(),"yyyyMMdd");
        }

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("enable.auto.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("auto.offset.reset", "latest")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(groupId+"_Period_Data"+ UUID.randomUUID().toString())
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "period-data-source");

        SingleOutputStreamOperator<LanePeriodicDataModel> lanePeriodicDataStream =
                stream
                        .flatMap(new LanePeriodDataFlatMap())
                        .setParallelism(1)
                        .name("车道周期数据-JsonToObject");

        SingleOutputStreamOperator<LanePeriodData> joinLaneStream = lanePeriodicDataStream
                .connect(env.addSource(new LaneInfoSource()).broadcast())
                .flatMap(new ConnectLaneInfoCoFlatMap())
                .setParallelism(1)
                .name("车道周期数据-车道属性数据绑定");


        try {
            //车道路况计算
            LaneTrafficStateMain.init(env,null,false).run(joinLaneStream);
            env.execute("基于车道周期指标计算-工作流");
        } catch (Exception e) {
            e.printStackTrace();
            log.error("基于车道周期指标计算异常 :   " + e);
        }
    }
}
