package com.wanji.indicators.task;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.LanePeriodicDataModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.source.debug.DebugPeriodSourceData;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrame;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.LaneTrafficStateMain;
import com.wanji.indicators.task.trafficstate.func.ConnectLaneInfoCoFlatMap;
import com.wanji.indicators.task.trafficstate.func.LanePeriodDataFlatMap;
import com.wanji.indicators.task.trafficstate.model.LanePeriodData;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 离线计算入口  com.mapabc.project.event.cross.stream.OfflineTestMain
 * @author ke.han
 */
public class OfflinePeriodIndicatorTaskMain {

    private static final Logger log = LoggerFactory.getLogger(OfflinePeriodIndicatorTaskMain.class);

    public static void main(String[] args) {
        //初始化环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        env.setParallelism(1);

        //加载配置项 并获取工作路径
        String path =   "d:/flink/workspace1/";
        String outPath = "d:/flink/out/";
        //读取离线数据
        DataStreamSource<String> streamSource = env.addSource(new DebugPeriodSourceData(path));

        SingleOutputStreamOperator<LanePeriodicDataModel> lanePeriodicDataStream =
                streamSource
                        .flatMap(new LanePeriodDataFlatMap())
                        .setParallelism(1)
                        .name("车道周期数据-JsonToObject");

        SingleOutputStreamOperator<LanePeriodData> joinLaneStream = lanePeriodicDataStream
                .connect(env.addSource(new LaneInfoSource()).broadcast())
                .flatMap(new ConnectLaneInfoCoFlatMap())
                .setParallelism(1)
                .name("车道周期数据-车道属性数据绑定");

        LaneTrafficStateMain.init(env,outPath,false).run(joinLaneStream);
        try {
            env.execute("离线调试测试");
        } catch (Exception e) {
            log.error("事件测试 - 计算任务异常 : " + e);
        }
    }
}
