package com.wanji.indicators.task;

import com.wanji.indicators.cache.PedestrianDataCache;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.source.AreaFreeSpeedSource;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrame;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrameDtFile;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.LaneNonMotorAnalysisMainNew;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.area.service.func.RidFreeSpeedCoFlatMap;
import com.wanji.indicators.task.motor.MotorEfficiencyMainNew;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.RidTrafficStateMain;
import com.wanji.indicators.task.turnIndicators.CrossTurnIndicatorMainNew;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 离线计算入口
 */
public class OfflineTestIndicatorTaskMain {

    private static final Logger log = LoggerFactory.getLogger(OfflineTestIndicatorTaskMain.class);

    public static void main(String[] args) {
        //初始化环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(5);
        
        PedestrianDataCache.getInstance();

        env.setParallelism(1);

        //加载配置项 并获取工作路径
        String inPath = "d:/flink/workspace";
        String outPath = "d:/flink/out/";
        String startTime = "2024-01-02_00:00:00";
        String endTime = "2024-01-02_23:00:00";


        ParameterTool parameter = ParameterTool.fromArgs(args);
        inPath = parameter.get("inPath","D:\\WanJi-Work\\07 项目\\02 济南\\数据");
        outPath = parameter.get("outPath","d:/flink/out/");
        startTime = parameter.get("startTime","2024-05-31_16:20:00");
        endTime = parameter.get("endTime","2024-05-31_16:30:00");
       // inPath = "d:/flink/workspace";

        //读取离线数据
        DataStreamSource<String> streamSource = env.addSource(new DebugSourceGlobalFrameDtFile(inPath,startTime,endTime));

        SingleOutputStreamOperator<FrameModel> frameModelStream =
                streamSource
                        .flatMap(new FrameFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据-JsonToObject");

        SingleOutputStreamOperator<CarTrackModel> carTrackModelStream =
                frameModelStream.
                        flatMap(new CarTrackFlatMap(false))
                        .setParallelism(1)
                        .name("轨迹帧数据解析-ToCarTrackModel");

        SingleOutputStreamOperator<CrossFrameModel> crossDataStream =
                streamSource
                        .flatMap(new CrossFrameFlatMap())
                        .setParallelism(1)
                        .name("全域轨迹帧数据-按路口分组输出");


        try {
            SingleOutputStreamOperator<CarTrackModel> joinLaneStream = carTrackModelStream
                    //.connect(env.addSource(new AreaFreeSpeedSource()).broadcast())
                    .flatMap(new RidFreeSpeedCoFlatMap())
                    .setParallelism(1)
                    .name("全域轨迹帧数据-车道属性数据绑定");

            //路段路况
            RidTrafficStateMain.init(env, outPath,true).run(joinLaneStream);

            //CrossTurnIndicatorMainNew.init(env,outPath,true).run(crossDataStream);
//            LaneNonMotorAnalysisMainNew.init(env,outPath,true).run(carTrackModelStream);
            //批量轨迹封装
//            TrackStoreMainNew.init(env,outPath,true).run(carTrackModelStream);
            //车辆行驶路径分析
//            CarRoutePathMainNew.init(env,outPath,true).run(carTrackModelStream);

            env.execute("离线调试测试");
        } catch (Exception e) {
            log.error("事件测试 - 计算任务异常 : " + e);
        }
    }
}
