package com.wanji.indicators.task;

import com.wanji.indicators.event.stream.TrafficEventMain;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.LaneTrafficStateMain;
import com.wanji.indicators.task.trafficstate.RidTrafficStateMain;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.restartstrategy.RestartStrategies;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.runtime.state.filesystem.FsStateBackend;
import org.apache.flink.runtime.state.filesystem.FsStateBackendFactory;
import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
import org.apache.flink.streaming.api.CheckpointingMode;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;
import org.joda.time.DateTimeUtils;

import java.time.format.DateTimeFormatter;
import java.util.Objects;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

public class TrackIndicatorMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final String GROUP_ID = "FlinkConsumerGroup-V56-TRACK";

    public static void main(String[] args) {
        //批量轨迹es index,逗号分割：esIndexName,windowTime
        String indexName = "car-track-30s,30";
        String indexName1 = "car-route-path";

        ParameterTool parameter = ParameterTool.fromArgs(args);
        boolean isMockPlateNumber = parameter.getBoolean("isMockPlateNumber",false);

        if (parameter.get("trackEsName") != null)
            indexName = parameter.get("trackEsName");

        if (parameter.get("routeEsName") != null)
            indexName1 = parameter.get("routeEsName");

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        String topic = properties.getProperty("consumer.topic");

        String groupId = properties.getProperty("consumer.group.id");
        if (StringUtils.isEmpty(groupId)) {
            groupId = GROUP_ID + "_run_route_new";
        }
        String groupChangeFlag = properties.getProperty("consumer.group.change.flag");
        if (Boolean.parseBoolean(groupChangeFlag)){
            groupId += DateUtil.toDateTime(System.currentTimeMillis(),"yyyyMMdd");
        }

        //根据指定时间消费
        long startTimeStampOffset = 0;
        long endTimeStampOffset = 0;
        //yyyyMMdd_HHmmss
        String consumerStartDate = parameter.get("consumerStartDate");
        String consumerEndDate = parameter.get("consumerEndDate");
        if (StringUtils.isNotEmpty(consumerStartDate)) {
            startTimeStampOffset = DateUtil.stringToDate(consumerStartDate, "yyyyMMdd_HHmmss").getTime();
        }
        if (StringUtils.isNotEmpty(consumerEndDate)) {
            endTimeStampOffset = DateUtil.stringToDate(consumerEndDate, "yyyyMMdd_HHmmss").getTime();
        }

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        //env.setParallelism(4);
//        env.getCheckpointConfig().disableCheckpointing();

//        env.enableCheckpointing(3 * 60 * 1000);
//        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);
//        env.getCheckpointConfig().setCheckpointTimeout(10 * 60 * 1000);
//        env.getCheckpointConfig().setCheckpointStorage(new FileSystemCheckpointStorage(properties.getProperty("check.point.uri")));
//        env.setStateBackend(new FsStateBackend(properties.getProperty("check.point.uri")));
//        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.DELETE_ON_CANCELLATION);
//        //env.getCheckpointConfig().setMinPauseBetweenCheckpoints(1 * 60 * 1000);
//        env.getCheckpointConfig().setMaxConcurrentCheckpoints(3);
//        env.getCheckpointConfig().setCheckpointingMode(CheckpointingMode.EXACTLY_ONCE);
        //设置重启策略,30s钟重启一次，最多重启3次
        //env.setRestartStrategy(RestartStrategies.fixedDelayRestart(3, Time.of(30, TimeUnit.SECONDS)));

        DateTime currentDateTime = new DateTime();
        KafkaSource<String> source = null;
        //从指定时间开始消费
        if (startTimeStampOffset > 0) {
            source = KafkaSource.<String>builder()
                    .setProperties(instance.getConsumerProperties())
//                    .setProperty("enable.auto.commit", "true")
//                    .setProperty("auto.commit.interval.ms", "1000")
                    .setProperty("auto.offset.reset", "latest")
                    .setProperty("commit.offsets.on.checkpoint", "true")
                    .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                    .setTopics(topic)
                    .setGroupId(groupId)
                    // 从大于等于此时间戳开始的偏移量开始
                    .setStartingOffsets(OffsetsInitializer.timestamp(startTimeStampOffset))
                    .setBounded(OffsetsInitializer.timestamp(endTimeStampOffset))
                    .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                    .build();

        } else {
            source = KafkaSource.<String>builder()
                    .setProperties(instance.getConsumerProperties())
                    .setProperty("enable.auto.commit", "true")
                    .setProperty("auto.commit.interval.ms", "1000")
                    .setProperty("auto.offset.reset", "latest")
//                    .setProperty("commit.offsets.on.checkpoint", "true")
                    .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                    .setTopics(topic)
                    .setGroupId(groupId)
                    .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                    .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                    .build();
            ;
        }

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-route");

        SingleOutputStreamOperator<FrameModel> frameModelStream =
                stream
                        .flatMap(new FrameFlatMap())
                        .setParallelism(1)
                        .name("路径计算-轨迹帧数据-JsonToObject")

                ;

        SingleOutputStreamOperator<CarTrackModel> carTrackModelStream =
                frameModelStream.
                        flatMap(new CarTrackFlatMap(isMockPlateNumber))
                        .setParallelism(1)
                        .name("路径计算-轨迹帧数据解析-ToCarTrackModel")
                ;

        try {
            //批量轨迹封装
            TrackStoreMainNew.init(env, indexName).run(carTrackModelStream);

            //车辆行驶路径分析
            CarRoutePathMainNew.init(env, indexName1).run(carTrackModelStream);

            env.execute("轨迹路径分析-工作流");
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
