package com.wanji.indicators.task.area.service;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.config.RedisConfig;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.event.cross.accident.AccidentEventMainNew;
import com.wanji.indicators.event.cross.conflictpoint.func.ConflictLaneInfoCoFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.sink.AccidentEventFileSink;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.source.AreaFreeSpeedSource;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.task.track.service.key.KeySelectorGlobalIdPlateNo;
import com.wanji.indicators.task.area.service.func.*;
import com.wanji.indicators.task.area.service.model.AreaIndexAnalysis;
import com.wanji.indicators.task.area.service.model.TravelTimeResultModel;
import com.wanji.indicators.task.area.service.sink.TravelRedisMapper;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.streaming.connectors.redis.RedisSink;
import org.apache.flink.util.Collector;

import java.io.Serializable;
import java.time.Duration;
import java.util.Objects;
import java.util.Properties;

@Slf4j
public class AreaIndexAnalysisMainNew implements Serializable {
    private   StreamExecutionEnvironment env;

    private   PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName = "area.realtime.indicator";

    private int windowTime = 10;

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private AreaIndexAnalysisMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }
    public static AreaIndexAnalysisMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new AreaIndexAnalysisMainNew(env, path,isDebug);
    }

    public void run(SingleOutputStreamOperator<CarTrackModel> filterStream) {
        try {

            SingleOutputStreamOperator<CarTrackModel> motorFilterStream =
                    filterStream.
                            filter(new FilterFunction<CarTrackModel>() {
                                @Override
                                public boolean filter(CarTrackModel value) throws Exception {
                                    return Objects.equals(value.getCategory(),1);
                                }
                            })
                            .setParallelism(1)
                            .name("筛选机动车轨迹");

            SingleOutputStreamOperator<TravelTimeResultModel> groupByPlateStream =
                    motorFilterStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(5))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                            .keyBy(new KeySelectorGlobalIdPlateNo())
                            .window(TumblingEventTimeWindows.of(Time.minutes(this.windowTime)))
                            //.aggregate(new TravelAggregateFunction(),new TravelProcessWindow())
                            .process(new AreaCarProcessWindow())
                            .name("区域指标-单辆车旅行时间、速度计算");

            SingleOutputStreamOperator<AreaIndexAnalysis> areaStream =
                    groupByPlateStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<TravelTimeResultModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
                            .keyBy(o->o.getId()+"_"+o.getOriginalType())
                            .windowAll(TumblingEventTimeWindows.of(Time.seconds(10)))
                            //.aggregate(new AreaTravelAggregateFunction(),new AreaTravelProcessWindow())
                            .process(new AreaOutProcessWindow())
                            .name("区域指标-旅行时间、速度计算");

            SingleOutputStreamOperator<String> toJsonStream = areaStream.flatMap(new FlatMapFunction<AreaIndexAnalysis, String>() {
                @Override
                public void flatMap(AreaIndexAnalysis value, Collector<String> out) throws Exception {
                    out.collect(JSONObject.toJSONString(value));
                }
            });

            if (!isDebug) {
                //最后一条记录存储到redis
                areaStream.addSink(new RedisSink<AreaIndexAnalysis>(new RedisConfig().getRedisConfig(), new TravelRedisMapper(Constant.AREA_REALTIME_INDICATOR + "_10M")));

//                // 初始化kafka生产者
//                FlinkKafkaProducer producer = new FlinkKafkaProducer<>(p.getProperty("area.analysis.data.topic"),
//                        new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
//                        instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);
//                //存储到kafka
//                toJsonStream.addSink(producer).name("区域指标数据-数据发送至kafka").setParallelism(1);

                KafkaSink<String> sink = KafkaSink.<String>builder()
                        .setBootstrapServers(p.getProperty("bootstrap.servers"))
                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                                .setTopic(p.getProperty("area.analysis.data.topic"))
                                .setValueSerializationSchema(new SimpleStringSchema())
                                .build()
                        )
                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                        .build();
                toJsonStream.sinkTo(sink).name("区域指标数据-数据发送至kafka").setParallelism(1);
            } else {
                String path = this.path + "area.analysis.json";
                toJsonStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("区域指标数据-结果输出至文件");
            }
        } catch (Exception e) {
            log.error("区域指标数据计算异常",e);
        }
    }


}
