package com.wanji.indicators.task.area.service;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.task.area.service.func.LaneCarProcessWindow;
import com.wanji.indicators.task.area.service.func.RidOutProcessWindow;
import com.wanji.indicators.task.area.service.key.ExactCategoryKeySelector;
import com.wanji.indicators.task.area.service.model.LaneCarCountAnalysisOutputModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisOutputModel;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.util.Collector;

import java.time.Duration;
import java.util.Objects;
import java.util.Properties;

/**
 * 车道非机动车数量分析
 */
@Slf4j
public class LaneNonMotorAnalysisMainNew {
    private StreamExecutionEnvironment env;

    private PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;


    private int windowTime = 15;

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private LaneNonMotorAnalysisMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    public static LaneNonMotorAnalysisMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new LaneNonMotorAnalysisMainNew(env, path, isDebug);
    }

    public void run(SingleOutputStreamOperator<CarTrackModel> filterStream) {

        try {

            SingleOutputStreamOperator<CarTrackModel> nonMotorFilterStream =
                    filterStream.
                            filter(new FilterFunction<CarTrackModel>() {
                                @Override
                                public boolean filter(CarTrackModel value) throws Exception {
                                    return Objects.equals(value.getCategory(),2);
                                }
                            })
                            .setParallelism(1)
                            .name("车道非机动车数量计算-筛选非机动车轨迹");


            SingleOutputStreamOperator<LaneCarCountAnalysisOutputModel> laneStream =
                    nonMotorFilterStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                            .keyBy(new ExactCategoryKeySelector())
                            .window(TumblingEventTimeWindows.of(Time.minutes(this.windowTime)))
                            .process(new LaneCarProcessWindow())
                            .name("车道非机动车数量计算-按车道分组统计");

//            SingleOutputStreamOperator<ParticipantAnalysisOutputModel> areaStream =
//                    filterStream.assignTimestampsAndWatermarks(
//                            WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
//                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
//                            .keyBy(o->(o.getRoadnet().getRid()))
//                            .window(TumblingEventTimeWindows.of(Time.minutes(this.windowTime)))
//                            //.aggregate(new AreaTravelAggregateFunction(),new AreaTravelProcessWindow())
//                            .process(new RidOutProcessWindow())
//                            .name("行人非机动车对机动车速度影响计算-按RID分组计算");


            //最后一条记录存储到redis
            //areaStream.addSink( new RedisSink<ParticipantAnalysisOutputModel>(new RedisConfig().getRedisConfig(), new TravelRedisMapper(Constant.RID_REALTIME_INDICATOR+"_10M")));

            SingleOutputStreamOperator<String> toJsonStream = laneStream.flatMap(new FlatMapFunction<LaneCarCountAnalysisOutputModel, String>() {
                @Override
                public void flatMap(LaneCarCountAnalysisOutputModel value, Collector<String> out) throws Exception {
                    out.collect(JSONObject.toJSONString(value));
                }
            }).name("车道非机动车数量计算-转换为json");

            if (!isDebug) {
                // 初始化kafka生产者
                KafkaSink<String> sink = KafkaSink.<String>builder()
                        .setBootstrapServers(p.getProperty("bootstrap.servers"))
                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                                .setTopic("analysis.lane.car.number")
                                .setValueSerializationSchema(new SimpleStringSchema())
                                .build()
                        )
                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                        .build();
                toJsonStream.sinkTo(sink).name("车道非机动车数量计算-数据发送至kafka").setParallelism(1);

            } else {
                String path = this.path + "analysis.lane.car.number";
                toJsonStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("车道非机动车数量计算-结果输出至文件");

            }
        } catch (Exception e) {
            log.error("车道非机动车数量计算", e);
        }
    }


}
