package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.area.service.model.TravelTimeResultModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class AreaCarProcessWindow extends ProcessWindowFunction<CarTrackModel, TravelTimeResultModel, Tuple3<String, Integer, String>, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(AreaCarProcessWindow.class);

    private MapState<String, CarTrackModel> mapState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_Area_Indicators", String.class, CarTrackModel.class));
    }

    @Override
    public void process(Tuple3<String, Integer, String> tupleKey, Context context, Iterable<CarTrackModel> elements, Collector<TravelTimeResultModel> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        TravelTimeResultModel outModel = new TravelTimeResultModel();

        List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
        List<CarTrackModel> sortList = list.stream().sorted(Comparator.comparing(CarTrackModel::getGlobalTimeStamp)).collect(Collectors.toList());


        if (sortList.size() > 1) {
            CarTrackModel firstModel = sortList.get(0);
            CarTrackModel lastModel = sortList.get(sortList.size() - 1);
            //单车行驶时间 单位秒
            Double travelTime = (lastModel.getGlobalTimeStamp() - firstModel.getGlobalTimeStamp()) / 1000.0;
//            LineString lineString = GeomsConvertUtil.getLineString(xys);
//            Double distance = GeomsConvertUtil.duToMeter(lineString.getLength());

            double distance = CommonUtil.getTravelDistance(sortList);
            double avgSpeed = (distance / 1000) / (travelTime / 60 / 60.0);

            outModel.setOriginalType(firstModel.getOriginalType());
            outModel.setPlateNo(firstModel.getPicLicense());
            outModel.setTravelTime(travelTime);
            outModel.setTravelSpeed(avgSpeed);
            outModel.setTimestamp(context.window().maxTimestamp());
            outModel.setGlobalId(firstModel.getOrgCode() + "_" + firstModel.getId() + "_" + firstModel.getPicLicense() + "_" + firstModel.getOriginalType());
            outModel.setStartTimestamp(windowStartTs);
            outModel.setEndTimestamp(windowEndTs);
            outModel.setFreeSpeed(firstModel.getAreaFreeSpeed());

            out.collect(outModel);
        }
    }
}
