package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.task.area.service.model.AreaIndexAnalysis;
import com.wanji.indicators.task.area.service.model.TravelTimeResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessAllWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
@Slf4j
public class AreaOutProcessWindow extends ProcessAllWindowFunction<TravelTimeResultModel, AreaIndexAnalysis, TimeWindow> {

    private Properties properties = new Properties();
    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
    }


    @Override
    public void process(Context context, Iterable<TravelTimeResultModel> elements, Collector<AreaIndexAnalysis> out) throws Exception {

        AreaIndexAnalysis areaIndexAnalysis = new AreaIndexAnalysis();

        List<TravelTimeResultModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        Set<String> sets = new HashSet<>();
        double freeSpeed = 0;
        for (TravelTimeResultModel value : list) {
            Integer originType = value.getOriginalType();
            freeSpeed = value.getFreeSpeed();
            if (freeSpeed==0) {
                freeSpeed = Double.parseDouble(this.properties.getProperty("rid.default.free.speed"));
            }
            //机动车
            if (VehicleTypeEnum.isMotorVehicles(originType.shortValue())) {
                double travelTime = value.getTravelTime();
                //取最大时间
                if (travelTime > areaIndexAnalysis.getTravelTime()) {
                    areaIndexAnalysis.setTravelTime(travelTime);
                }

                String[] sps = value.getGlobalId().split("_");
                String plateNo = sps[2];
                log.info(DateUtil.toDateTime(value.getStartTimestamp(),"yyyy-MM-dd HH:mm:ss")+"=>"+plateNo+"=>"+value.getGlobalId());
                if (plateNo.equals("null")) {
                    //未获取到车牌的当作本地车辆，摒弃无车牌数据
                    //areaIndexAnalysis.setLocalVehicleNum(areaIndexAnalysis.getLocalVehicleNum() + 1);
                    continue;
                }

                //速度累加
                areaIndexAnalysis.setSpeed(value.getTravelSpeed() + areaIndexAnalysis.getSpeed());

                if(plateNo.contains(properties.getProperty("plate.prefix"))) {
                    areaIndexAnalysis.setLocalVehicleNum(areaIndexAnalysis.getLocalVehicleNum() + 1);
                    areaIndexAnalysis.setLocalHavePlateNum(areaIndexAnalysis.getLocalHavePlateNum() + 1);
                    sets.add(plateNo);
                } else {
                    //外地车辆数
                    areaIndexAnalysis.setOutVehicleNum(areaIndexAnalysis.getOutVehicleNum() + 1);
                }

            } else if (VehicleTypeEnum.isPedestrian(originType.shortValue())) {
                //行人数量
                areaIndexAnalysis.setPedestrianNum(areaIndexAnalysis.getPedestrianNum() + 1);
            } else if (VehicleTypeEnum.isNonMotorVehicles(originType.shortValue())) {
                //非机动车数量
                areaIndexAnalysis.setNonMotorNum(areaIndexAnalysis.getNonMotorNum() + 1);
            }
            areaIndexAnalysis.setTimeStamp(value.getEndTimestamp());
            areaIndexAnalysis.setStartTime(DateUtil.toDateTime(value.getStartTimestamp(),"yyyy-MM-dd HH:mm:ss"));
            areaIndexAnalysis.setEndTime(DateUtil.toDateTime(value.getEndTimestamp(),"yyyy-MM-dd HH:mm:ss"));
        }
        //在途车辆数=本地+外地
        areaIndexAnalysis.setInWayVehicleNum(areaIndexAnalysis.getLocalVehicleNum() + areaIndexAnalysis.getOutVehicleNum());
        //区域平均旅行速度
        areaIndexAnalysis.setSpeed(areaIndexAnalysis.getSpeed()/areaIndexAnalysis.getInWayVehicleNum());
        //区域指数
        double traIndex = areaIndexAnalysis.getSpeed()==0?1:(freeSpeed / areaIndexAnalysis.getSpeed());
        areaIndexAnalysis.setTrafficIndex(traIndex);

        String trafficState = "1";
        if (traIndex < 2) {
            trafficState = "1";
        } else if (traIndex < 4) {
            trafficState = "2";
        } else if (traIndex < 6) {
            trafficState = "3";
        } else if (traIndex < 8) {
            trafficState = "4";
        } else {
            trafficState = "5";
        }
        areaIndexAnalysis.setTrafficState(trafficState);

        log.info("区域指数记录："+areaIndexAnalysis);

        out.collect(areaIndexAnalysis);
    }

    public static void main(String[] args) {
        String date = "2023-10-11 07:00:00";
        System.out.println(DateUtil.stringToDate(date,"yyyy-MM-dd HH:mm:ss").getTime());
    }
}
