package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.task.area.service.model.AreaIndexAnalysis;
import com.wanji.indicators.task.area.service.model.TravelTimeResultModel;
import org.apache.flink.api.common.functions.AggregateFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class AreaTravelAggregateFunction implements AggregateFunction<TravelTimeResultModel, AreaIndexAnalysis, AreaIndexAnalysis> {

    private static final Logger log = LoggerFactory.getLogger(AreaTravelAggregateFunction.class);

    @Override
    public AreaIndexAnalysis createAccumulator() {
        AreaIndexAnalysis initModel = new AreaIndexAnalysis();
        initModel.setSpeed(0D);
        initModel.setLocalVehicleNum(0);
        initModel.setOutVehicleNum(0);
        initModel.setTrafficIndex(0D);
        initModel.setInWayVehicleNum(0);
        initModel.setTravelTime(0D);
        initModel.setPedestrianNum(0);
        initModel.setNonMotorNum(0);
        return initModel;
    }

    /**
     * 每进入一个数据就会执行一次
     *
     * @param value       当前进入的数据
     * @param  areaIndexAnalysis 之前计算好的中间结果
     * @return
     */
    @Override
    public AreaIndexAnalysis add(TravelTimeResultModel value, AreaIndexAnalysis areaIndexAnalysis) {
        //记录初始进入时间
        if (areaIndexAnalysis.getStartTime() == null) {
            areaIndexAnalysis.setSpeed(0D);
            areaIndexAnalysis.setLocalVehicleNum(0);
            areaIndexAnalysis.setOutVehicleNum(0);
            areaIndexAnalysis.setTrafficIndex(0D);
            areaIndexAnalysis.setInWayVehicleNum(0);
            areaIndexAnalysis.setTravelTime(0D);
            areaIndexAnalysis.setPedestrianNum(0);
            areaIndexAnalysis.setNonMotorNum(0);
            areaIndexAnalysis.setStartTime(value.getStartTime());
        } else {
            Integer originType = value.getOriginalType();
            //System.out.println("originType:"+originType);

            if (VehicleTypeEnum.isMotorVehicles(originType.shortValue())) {
                double travelTime = value.getTravelTime();
                //取最大时间
                if (travelTime > areaIndexAnalysis.getTravelTime()) {
                    areaIndexAnalysis.setTravelTime(travelTime);
                }
                //在途车辆数
                areaIndexAnalysis.setInWayVehicleNum(areaIndexAnalysis.getInWayVehicleNum() + 1);
                //速度累加
                areaIndexAnalysis.setSpeed(value.getTravelSpeed() + areaIndexAnalysis.getSpeed());
                String[] sps = value.getGlobalId().split("_");
                String plateNo = sps[2];
                if (plateNo.contains("鲁")) {
                    //本地车辆数
                    areaIndexAnalysis.setLocalVehicleNum(areaIndexAnalysis.getLocalVehicleNum() + 1);
                } else {
                    //外地车辆数
                    areaIndexAnalysis.setOutVehicleNum(areaIndexAnalysis.getOutVehicleNum() + 1);
                }
            } else if (VehicleTypeEnum.isPedestrian(originType.shortValue())) {
                //行人数量
                areaIndexAnalysis.setPedestrianNum(areaIndexAnalysis.getPedestrianNum() + 1);
            } else if (VehicleTypeEnum.isNonMotorVehicles(originType.shortValue())) {
                //非机动车数量
                areaIndexAnalysis.setNonMotorNum(areaIndexAnalysis.getNonMotorNum() + 1);
            }

            areaIndexAnalysis.setStartTime(value.getStartTime());
            areaIndexAnalysis.setEndTime(value.getEndTime());

        }

       //log.info("==============="+areaIndexAnalysis);

    //System.out.println("avgSpeed:"+accumulator.getAvgSpeed()+":"+accumulator.getSumSpeed()/accumulator.getCount()+" maxSpeed:"+accumulator.getMaxSpeed());

        return areaIndexAnalysis;
}

    /*
       当window的结束时间到达时，触发这个方法，返回结果
        */
    @Override
    public AreaIndexAnalysis getResult(AreaIndexAnalysis accumulator) {
        log.info("AreaIndexAnalysisAggResult:"+accumulator);
        return accumulator;
    }

    /**
     * 在session窗口才会用到merge，时间窗口其实用不到
     *
     * @param a
     * @param b
     * @return
     */
    @Override
    public AreaIndexAnalysis merge(AreaIndexAnalysis a, AreaIndexAnalysis b) {
        return null;
    }
}
