package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.task.area.service.model.AreaIndexAnalysis;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Iterator;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class AreaTravelProcessWindow extends ProcessWindowFunction<
        AreaIndexAnalysis,
        AreaIndexAnalysis,
        String,
        TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(AreaTravelProcessWindow.class);

    @Override
    public void process(String s, Context context, Iterable<AreaIndexAnalysis> elements, Collector<AreaIndexAnalysis> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        AreaIndexAnalysis areaIndexAnalysis = new AreaIndexAnalysis();
        areaIndexAnalysis.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss"));
        areaIndexAnalysis.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss"));

        Iterator<AreaIndexAnalysis> iterator = elements.iterator();

        double totalSpeed = 0;
        int localNum = 0;
        int outNum = 0;
        double maxTraTime = 0;
        int pedestrianNum = 0;
        int nonMotorNum = 0;

        int size = 0;
        while (iterator.hasNext()) {
            AreaIndexAnalysis result = iterator.next();
            totalSpeed += result.getSpeed();
            size = result.getInWayVehicleNum();
        }

        double avgSpeed = ArithOfBigDecmial.div(totalSpeed, size);
        double freeSpeed = 80;
        double traIndex = avgSpeed==0?1:(freeSpeed / avgSpeed);
        areaIndexAnalysis.setTrafficIndex(traIndex);
        String trafficState = "1";
        if (traIndex < 2) {
            trafficState = "1";
        } else if (traIndex < 4) {
            trafficState = "2";
        } else if (traIndex < 6) {
            trafficState = "3";
        } else if (traIndex < 8) {
            trafficState = "4";
        } else {
            trafficState = "5";
        }
        areaIndexAnalysis.setTrafficState(trafficState);

        areaIndexAnalysis.setSpeed(avgSpeed);
        areaIndexAnalysis.setInWayVehicleNum(size);
        areaIndexAnalysis.setLocalVehicleNum(localNum);
        areaIndexAnalysis.setOutVehicleNum(outNum);

        areaIndexAnalysis.setPedestrianNum(pedestrianNum);
        areaIndexAnalysis.setNonMotorNum(nonMotorNum);

        log.info("Area Data Result:" + areaIndexAnalysis.toString());

        out.collect(areaIndexAnalysis);
    }
}
