package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.area.service.model.LaneCarCountAnalysisOutputModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.java.tuple.Tuple5;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class LaneCarProcessWindow extends ProcessWindowFunction<CarTrackModel, LaneCarCountAnalysisOutputModel, Tuple5<String, String, String, Integer, Integer>, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(LaneCarProcessWindow.class);

    private Properties properties = new Properties();

    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
    }

    @Override
    public void process(Tuple5<String, String, String, Integer, Integer> key, Context context, Iterable<CarTrackModel> elements, Collector<LaneCarCountAnalysisOutputModel> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
        //窗口期内车辆数量
        long count = list.stream().map(CarTrackModel::getId).distinct().count();
        //窗口期路口内数量
        long countInCross = list.stream().filter(o->o.getRoadnet().getInCrossFlag()== Constant.CROSSING).map(CarTrackModel::getId).distinct().count();
        if (count > 0) {
            LaneCarCountAnalysisOutputModel outModel = new LaneCarCountAnalysisOutputModel();

            outModel.setCrossId(key.f0);
            outModel.setRid(key.f1);
            outModel.setLaneId(key.f2);
            outModel.setCategory(key.f3);
            outModel.setOriginalType(key.f4);
            outModel.setNumber(count);
            outModel.setTimestamp(context.window().maxTimestamp());
            outModel.setStartTimestamp(windowStartTs);
            outModel.setEndTimestamp(windowEndTs);
            outModel.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss"));
            outModel.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss"));
            outModel.setAvgSpeed(CommonUtil.getFrameAvgSpeed(list));

            List<Integer> idList =  list.stream().map(CarTrackModel::getId).distinct().collect(Collectors.toList());
            outModel.setIdList(idList);

            outModel.setNumberInCross(countInCross);

            out.collect(outModel);
        }
    }

}
