package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class RidCarProcessWindow extends ProcessWindowFunction<CarTrackModel, ParticipantAnalysisMidResultModel, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(RidCarProcessWindow.class);

    private Properties properties = new Properties();

    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
    }

    @Override
    public void process(String key, Context context, Iterable<CarTrackModel> elements, Collector<ParticipantAnalysisMidResultModel> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();
        String[] sps = key.split("_");

        ParticipantAnalysisMidResultModel outModel = new ParticipantAnalysisMidResultModel();

        List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        long count = list.stream().map(CarTrackModel::getId).distinct().count();
        double speed = 0;//getFrameAvgSpeed(list);

        //机动车
        if (sps[1].equals("1")) {
            double totalSpeed = 0;
            int size = 0;
            List<Double> speedList = new ArrayList<>();
            //Map<String,List<CarTrackModel>> groupByPlateNo = list.stream().filter(o->Objects.nonNull(o.getPicLicense())&&!"null".equals(o.getPicLicense())).collect(Collectors.groupingBy(o->o.getPicLicense()));
            Map<String, List<CarTrackModel>> groupByPlateNo = list.stream().collect(Collectors.groupingBy(o -> o.getId().toString()));
            for (Map.Entry<String, List<CarTrackModel>> entry : groupByPlateNo.entrySet()) {
                List<CarTrackModel> carList = entry.getValue();
                if (carList.size() > 1) {
                    carList = carList.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());
                    double distance = CommonUtil.getTravelDistance(carList);
                    double travelTime = (carList.get(carList.size() - 1).getGlobalTimeStamp() - carList.get(0).getGlobalTimeStamp()) / 1000;
                    if (travelTime > 0) {
                        double travelSpeed = (distance / 1000) / (travelTime / 60 / 60.0);
                        totalSpeed += travelSpeed;
                        size++;
                        speedList.add(travelSpeed);
                    }

                }
            }
            //speed = totalSpeed / size;
            speed = this.getFrameAvgSpeed(speedList);
            Double freeSpeed = list.get(0).getRoadnet().getFreeSpeed();
            if (Objects.isNull(freeSpeed)) {
                freeSpeed = Double.parseDouble(this.properties.getProperty("rid.default.free.speed"));
            }
            double trafficIndex = freeSpeed / speed;
            outModel.setTrafficIndex(trafficIndex);
        }

        //Map<Integer,List<CarTrackModel>> groupByType = list.stream().collect(Collectors.groupingBy(CarTrackModel::getOriginalType));

        outModel.setRid(sps[0]);
        outModel.setCrossId(list.get(0).getRoadnet().getCrossId());
        outModel.setCategory(Integer.parseInt(sps[1]));

        outModel.setCount(count);
        outModel.setSpeed(speed);
        outModel.setTimestamp(context.window().maxTimestamp());
        outModel.setStartTimestamp(windowStartTs);
        outModel.setEndTimestamp(windowEndTs);
        outModel.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss"));
        outModel.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss"));

        out.collect(outModel);
    }


    private int getFrameAvgSpeed(List<Double> list) {
        //当前帧机动车平均速度，所有速度大于5km/h的速度进行排序，去掉低速10%，然后进行平均
        List<Double> speedList = list.stream()
                .filter(speed -> speed > 0).distinct()
                .sorted(Comparator.comparing(Double::doubleValue))
                .collect(Collectors.toList());

        int startIndex = new BigDecimal(speedList.size() * 0.1).intValue();
        int endIndex = (int) ArithOfBigDecmial.round(speedList.size() * 0.9, 0);
        List<Double> subList = speedList.subList(startIndex, endIndex);
        double avgSpeed = subList.stream().collect(Collectors.averagingDouble(Double::doubleValue));
        return (int) ArithOfBigDecmial.round(avgSpeed, 0);
    }
}
