package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisOutputModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessAllWindowFunction;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
@Slf4j
public class RidOutProcessWindow extends ProcessWindowFunction<CarTrackModel, ParticipantAnalysisOutputModel,String, TimeWindow> {

    private Properties properties = new Properties();

    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
    }

    @Override
    public void process(String key, Context context, Iterable<CarTrackModel> elements, Collector<ParticipantAnalysisOutputModel> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
        Map<Integer,List<CarTrackModel>> group =  list.stream().collect(Collectors.groupingBy(o->o.getCategory()));

        Integer[] categorys = new Integer[]{1,2,3};

        Map<Integer,ParticipantAnalysisOutputModel> outMap = new HashMap<>();

        for (Map.Entry<Integer,List<CarTrackModel>> entry : group.entrySet()) {
            Integer category = entry.getKey();
            List<CarTrackModel> value = entry.getValue();

            long number = value.stream().map(CarTrackModel::getId).distinct().count();

            ParticipantAnalysisOutputModel resultModel = new ParticipantAnalysisOutputModel();
            resultModel.setCrossId(value.get(0).getRoadnet().getCrossId());
            resultModel.setRid(value.get(0).getRoadnet().getRid());
            resultModel.setCategory(category);
            resultModel.setNumber(number);
            resultModel.setTimestamp(context.window().maxTimestamp());
            resultModel.setStartTimestamp(windowStartTs);
            resultModel.setEndTimestamp(windowEndTs);
            resultModel.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss"));
            resultModel.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss"));

            double avgSpeed = CommonUtil.getFrameAvgSpeed(value);
            resultModel.setSpeed(avgSpeed);

            Double freeSpeed = list.get(0).getRoadnet().getFreeSpeed();
            if (Objects.isNull(freeSpeed)) {
                freeSpeed = Double.parseDouble(this.properties.getProperty("rid.default.free.speed"));
            }
            double trafficIndex = 1;
            if (avgSpeed != 0) {
                trafficIndex = freeSpeed / avgSpeed;
            }
            resultModel.setTrafficIndex(trafficIndex);

            outMap.put(category,resultModel);
        }
        ParticipantAnalysisOutputModel tmp = null;
        for (Map.Entry<Integer,ParticipantAnalysisOutputModel> entry : outMap.entrySet()) {
            tmp = entry.getValue();
            log.info("行人、非机动车对机动车速度影响结果："+tmp.toString());
            out.collect(entry.getValue());
        }
        //无数据的数量置为0
        for (Integer category : categorys) {
            if (!outMap.containsKey(category)) {
                ParticipantAnalysisOutputModel bean = new ParticipantAnalysisOutputModel();
                BeanUtils.copyProperties(tmp,bean);
                bean.setNumber(0L);
                bean.setSpeed(0d);
                bean.setCategory(category);
                log.info("行人、非机动车对机动车速度影响结果："+bean.toString());
                out.collect(bean);
            }
        }
    }
}
