package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.area.service.model.TravelTimeResultModel;
import com.wanji.indicators.util.GeomsConvertUtil;
import org.apache.flink.api.common.functions.AggregateFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class TravelAggregateFunction implements AggregateFunction<CarTrackModel, TravelTimeResultModel, TravelTimeResultModel> {

    private static final Logger log = LoggerFactory.getLogger(TravelAggregateFunction.class);

    @Override
    public TravelTimeResultModel createAccumulator() {
        TravelTimeResultModel initModel = new TravelTimeResultModel();
        initModel.setTravelSpeed(0D);
        initModel.setTravelTime(0D);
        initModel.setDistance(0D);
        initModel.setOriginalType(0);
        return initModel;
    }

    /**
     * 每进入一个数据就会执行一次
     *
     * @param value       当前进入的数据
     * @param accumulator 之前计算好的中间结果
     * @return
     */
    @Override
    public TravelTimeResultModel add(CarTrackModel value, TravelTimeResultModel accumulator) {
        //记录初始进入时间
        if (accumulator.getTimestamp() == null) {
            accumulator.setTimestamp(value.getGlobalTimeStamp());
            accumulator.setStartTimestamp(value.getGlobalTimeStamp());
            accumulator.setLng(value.getLongitude());
            accumulator.setLat(value.getLatitude());
            accumulator.setOriginalType(value.getOriginalType());
            accumulator.setGlobalId(value.getOrgCode()+"_"+value.getId()+"_"+value.getPicLicense()+"_"+value.getOriginalType());
        } else {
            //从出现到当前帧的经过的时间
            long travelTime = value.getGlobalTimeStamp() - accumulator.getTimestamp();
            //行驶距离 米
            double distance = GeomsConvertUtil.getDistance(accumulator.getLng(), accumulator.getLat(), value.getLongitude(), value.getLatitude());
            //累加行驶时间 毫秒
            accumulator.setTravelTime((accumulator.getTravelTime() + travelTime));
            //累加距离 米
            accumulator.setDistance(accumulator.getDistance() + distance);

            accumulator.setTimestamp(value.getGlobalTimeStamp());
            accumulator.setOriginalType(value.getOriginalType());
            accumulator.setLng(value.getLongitude());
            accumulator.setLat(value.getLatitude());
            accumulator.setGlobalId(value.getOrgCode()+"_"+value.getId()+"_"+value.getPicLicense()+"_"+value.getOriginalType());
            //log.info("[carId:{},midId:{},travelTime:{},nowTime:{},preTime:{}]",value.getOrgCode()+"_"+value.getId()+"_"+value.getPicLicense(),accumulator.getGlobalId(),accumulator.getTravelTime(), DateUtil.toDateTime(value.getGlobalTimeStamp(),"yyyy-MM-dd HH:mm:ss.SSS"),DateUtil.toDateTime(accumulator.getTimestamp(),"yyyy-MM-dd HH:mm:ss.SSS"));

        }

        //log.info(accumulator.toString());
        return accumulator;
    }

    /*
       当window的结束时间到达时，触发这个方法，返回结果
        */
    @Override
    public TravelTimeResultModel getResult(TravelTimeResultModel accumulator) {
        log.info("AggResult:"+accumulator);
        return accumulator;
    }

    /**
     * 在session窗口才会用到merge，时间窗口其实用不到
     *
     * @param a
     * @param b
     * @return
     */
    @Override
    public TravelTimeResultModel merge(TravelTimeResultModel a, TravelTimeResultModel b) {
        return null;
    }
}
