package com.wanji.indicators.task.area.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.task.area.service.model.TravelTimeResultModel;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class TravelProcessWindow extends ProcessWindowFunction<
        TravelTimeResultModel,
        TravelTimeResultModel,
        Tuple3<String, Integer, String>,
        TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(TravelProcessWindow.class);

    @Override
    public void process(Tuple3<String, Integer, String> s, Context context, Iterable<TravelTimeResultModel> elements, Collector<TravelTimeResultModel> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        List<TravelTimeResultModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        while (elements.iterator().hasNext()) {

            TravelTimeResultModel result = elements.iterator().next();
            //result.setTimestamp(context.window().maxTimestamp());
            //速度 km/h
            if (VehicleTypeEnum.isMotorVehicles(result.getOriginalType().shortValue())) {
                double speed = ArithOfBigDecmial.div(result.getDistance() * 1000, result.getTravelTime() / 1000 / 60 / 60);
                result.setTravelSpeed(speed);
            }

            result.setStartTime(DateUtil.toDateTime(windowStartTs,"yyyy-MM-dd HH:mm:ss.SSS"));
            result.setEndTime(DateUtil.toDateTime(windowEndTs,"yyyy-MM-dd HH:mm:ss.SSS"));

           //log.info("Travel Data Result:" + result.toString());

            out.collect(result);
        }
    }
}
