/**
 *
 */
package com.wanji.indicators.task.beihang;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.index.strtree.STRtree;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.mapper.BaseCrossTurnInfoMapper;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;


@Component
@Slf4j
public class BaseGisLaneCache {


    private static BaseGisLaneCache instance;

    static STRtree rtree = new STRtree();

    private static Map<String, CarTrackModel.RoadNet> roadnetMap = new ConcurrentHashMap<>();

    private static Map<String, String> laneRefMap = new ConcurrentHashMap<>();

    private static Map<String, String> crossNameRefMap = new ConcurrentHashMap<>();

    private BaseGisLaneCache() {
    }

    public static   BaseGisLaneCache getInstance() {
        if (instance == null) {
            instance = new BaseGisLaneCache();

            Properties properties = PropertiesHelper.getInstance().getProperties();
            String dataDirectory = properties.getProperty("data.file.path");
            if (StringUtils.isEmpty(dataDirectory)) {
                dataDirectory = "/data/";
            }
            if (!dataDirectory.endsWith("/")) {
                dataDirectory += "/";
            }
            String f4 = dataDirectory + "shangdi-gis-lane.geojson";
            instance.loadLaneData(f4);
            String f5 = dataDirectory + "shangdi-gis-lane-ref.csv";
            laneRefMap.putAll(instance.loadGlobalLaneRef(f5));
            String f6 = dataDirectory + "shangdi-gis-cross.csv";
            crossNameRefMap.putAll(instance.loadCrossNameRef(f6));
        }
        return instance;
    }

    private Map<String, String> loadLaneData(String file) {
        Map<String, String> retMap = new HashMap<>();
        log.info("加载车道数据信息,路径：{},数据：{}", file);
        try (InputStream inputStream = PropertiesHelper.class.getResourceAsStream(file);
             BufferedReader br = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
        ) {
            StringBuilder jsonBuilder = new StringBuilder();
            String str = null;
            // 按行读取字符串
            while ((str = br.readLine()) != null) {
                jsonBuilder.append(str);
            }
            //log.info("加载车道数据信息,路径：{},数据：{}", file, jsonBuilder.toString());
            if (jsonBuilder.length() > 0) {
                JSONObject jsonObject = JSON.parseObject(jsonBuilder.toString());
                JSONArray jsonArray = jsonObject.getJSONArray("features");
                int size = jsonArray.size();
                for (int i = 0; i < size; i++) {
                    JSONObject jsonObject1 = jsonArray.getJSONObject(i);
                    JSONObject propJsonObj = jsonObject1.getJSONObject("properties");
                    String crossId = propJsonObj.getString("crossId");
                    Integer ridDir = propJsonObj.getIntValue("dir");
                    String rid = crossId + "_" + ridDir;
                    String laneId = propJsonObj.getString("id");
                    int laneNo = propJsonObj.getIntValue("laneNo");
                    String turn = propJsonObj.getString("turntype");
                    if (StringUtils.isEmpty(turn)){
                          turn = "2";
                    }
                    String sectionId = propJsonObj.getString("sectionId");
                    String type = "2";
                    if (Objects.equals("100", sectionId)) {
                        type = "3";
                    }
                    JSONObject geometryJsonObj = jsonObject1.getJSONObject("geometry");
                    JSONArray coordinatesArray = geometryJsonObj.getJSONArray("coordinates");

                    StringBuilder stringBuilder = new StringBuilder();
                    for (int j = 0; j < coordinatesArray.size(); j++) {
                        JSONArray pointArr = coordinatesArray.getJSONArray(j);
                        for (int k = 0; k < pointArr.size(); k++) {
                            double lng = pointArr.getJSONArray(k).getDouble(0);
                            double lat = pointArr.getJSONArray(k).getDouble(1);
                            stringBuilder.append(lng + "," + lat);
                            if (k != pointArr.size() - 1) {
                                stringBuilder.append(";");
                            }
                        }
                    }
                    try {
                        JSONObject centerLineJson = propJsonObj.getJSONObject("leftBoundaryLine");
                        JSONArray centerLineArray = centerLineJson.getJSONArray("geometry");
                        StringBuilder stringBuilder1 = new StringBuilder();
                        String startPoint = null;
                        String endPoint = null;
                        for (int j = 0; j < centerLineArray.size(); j++) {
                            JSONObject pointArr = centerLineArray.getJSONObject(j);
                            double lng = pointArr.getDouble("lng");
                            double lat = pointArr.getDouble("lat");
                            stringBuilder1.append(lng + "," + lat);
                            if (j == 0) {
                                startPoint = lng + "," + lat;
                            } else {
                                endPoint = lng + "," + lat;
                            }
                            if (j != centerLineArray.size() - 1) {
                                stringBuilder1.append(";");
                            }
                        }
                        String centerLineWkt = stringBuilder1.toString();
                        String polygonWkt = stringBuilder.toString();

                        Geometry polygon = GeomsConvertUtil.genGeometry(polygonWkt);

                        String laneStartPoint = startPoint;
                        Double fx = Double.parseDouble(laneStartPoint.split(",")[0]);
                        Double fy = Double.parseDouble(laneStartPoint.split(",")[1]);

                        String laneEndPoint = endPoint;
                        Double ex = Double.parseDouble(laneEndPoint.split(",")[0]);
                        Double ey = Double.parseDouble(laneEndPoint.split(",")[1]);

                        double laneAngle = PtInPolyUtil.getAngle(fx, fy, ex, ey);

                        CarTrackModel.RoadNet roadNet = new CarTrackModel.RoadNet();
                        roadNet.setLaneId(laneId);
                        roadNet.setLaneNo(laneNo);
                        roadNet.setCrossId(crossId);
                        roadNet.setRid(rid);
                        roadNet.setRidDir8(ridDir);
                        roadNet.setLaneType(Integer.valueOf(type));
                        roadNet.setTurn(Integer.valueOf(turn));
                        roadNet.setLaneStartPoint(startPoint);
                        roadNet.setLaneEndPoint(endPoint);
                        roadNet.setLaneAngle(laneAngle);
                        if (roadNet.getLaneType() == 2) {
                            roadNet.setInCrossFlag(1);
                        }
                        roadNet.setGeometry(polygon);

                        rtree.insert(polygon.getEnvelopeInternal(), roadNet);

                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return retMap;
    }

    private Map<String, String> loadGlobalLaneRef(String file) {
        Map<String, String> retMap = new HashMap<>();
        log.info("加载车道关系数据信息,路径：{},数据：{}", file);
        try (InputStream inputStream = PropertiesHelper.class.getResourceAsStream(file);
             BufferedReader br = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
        ) {

            String str = br.readLine();
            // 按行读取字符串
            while ((str = br.readLine()) != null) {
                 String globalLaneNo = str.substring(str.lastIndexOf(",")+1);
                 String crossLane = str.substring(0,str.lastIndexOf(","));
                 retMap.put(globalLaneNo,crossLane);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return retMap;
    }


    private Map<String, String> loadCrossNameRef(String file) {
        Map<String, String> retMap = new HashMap<>();
        log.info("加载路口名称关系数据信息,路径：{},数据：{}", file);
        try (InputStream inputStream = PropertiesHelper.class.getResourceAsStream(file);
             BufferedReader br = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
        ) {

            String str = br.readLine();
            // 按行读取字符串
            while ((str = br.readLine()) != null) {
                String name = str.substring(str.lastIndexOf(",")+1);
                String id = str.substring(0,str.lastIndexOf(","));
                retMap.put(id,name);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return retMap;
    }


    public CarTrackModel.RoadNet queryRoadNet(String pxy) {
        Point point = GeomsConvertUtil.createPoint(pxy) ;
        List<CarTrackModel.RoadNet> results = rtree.query(point.getEnvelopeInternal());
        for (CarTrackModel.RoadNet roadNet : results) {
             Geometry geometry = roadNet.getGeometry();
             if (geometry != null) {
                 boolean isWithin = point.within(geometry);
                 if (isWithin)
                     return roadNet;
             }
        }
        return  null;

    }

    public Map<String, CarTrackModel.RoadNet> getRoadnetMap() {
        return roadnetMap;
    }

    public Map<String, String> getLaneRefMap() {
        return laneRefMap;
    }

    public Map<String, String> getCrossNameRefMap() {
        return crossNameRefMap;
    }
}
