package com.wanji.indicators.task.beihang;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.FileFilterUtils;
import org.apache.commons.io.filefilter.IOFileFilter;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.source.SourceFunction;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;


public class BeiHangTrackFrameB3DataFile implements SourceFunction<QueueLengthBean> {

    private static final Logger log = LoggerFactory.getLogger(BeiHangTrackFrameB3DataFile.class);

    private volatile boolean running = true;
    private String directory;
    private long startTime;
    private long endTime;

    public BeiHangTrackFrameB3DataFile(String directory, long startTime, long endTime) {
        this.directory = directory;
        this.startTime = startTime;
        this.endTime = endTime;
    }


    //获取文件完整路径
    private String[] getFilePath() {
        List<String> indexList = new ArrayList<>();

        ///data/track_partitions/dt=2023-08-09/ht=11/mt=00

        DateTime start = new DateTime(this.startTime);
        DateTime end = new DateTime(this.endTime);

        while (start.isBefore(end)) {
            String dt = start.toString("yyyyMMdd");
            String directory = this.directory + "/" + dt;
            if (!new File(directory).exists()) {
                start = start.plusDays(1);
                continue;
            }
            IOFileFilter ioFileFilter = FileFilterUtils.prefixFileFilter("2024");
            List<File> fileList = FileUtils.listFiles(new File(directory), ioFileFilter, null).stream().collect(Collectors.toList());
            fileList = fileList.stream().sorted(Comparator.comparing(File::getName)).collect(Collectors.toList());

            String fileName = null;
            for (File dataFile : fileList) {
                fileName = dataFile.getName();
                String filePath = directory + "/" + fileName;
                if (!indexList.contains(filePath)) {
                    indexList.add(filePath);
                }
            }
            start = start.plusDays(1);
//            System.out.println("start:"+start.toString("yyyy-MM-dd HH:mm:ss"));
        }
//        System.out.println("end:"+start.toString("yyyy-MM-dd HH:mm:ss"));
//        System.out.println(indexList.toString());

        return indexList.toArray(new String[0]);
    }

    @Override
    public void run(SourceContext<QueueLengthBean> sourceContext) throws Exception {

        List<String> sortList = Arrays.asList(getFilePath());
        log.info("文件总数：{},文件列表：{}", sortList.size(), sortList.toString());

        while (running) {
            int size = 0;
            for (String str : sortList) {
                File file = new File(str.trim());
                if (file.exists()) {
                    long st = System.currentTimeMillis();
                    try (ZipInputStream zis = new ZipInputStream(new FileInputStream(file))) {
                        ZipEntry entry;
                        while (zis != null && (entry = zis.getNextEntry()) != null) {
                            try (InputStream is = zis;
                                 BufferedReader reader = new BufferedReader(new InputStreamReader(is))) {
                                String line;
                                int e1total = 0;
                                String lastTime = null;
                                while ((line = reader.readLine()) != null) {
                                    long t1 = System.currentTimeMillis();
                                    if (StringUtils.isNotBlank(line)) {
                                        String[] dataRec = line.split("@");
                                        String timestamp = dataRec[0];
                                        String e1Flag = dataRec[1];
                                        if (!StringUtils.equalsIgnoreCase("b3", e1Flag)) {
                                            continue;
                                        }

                                        lastTime = timestamp;
                                        long time = DateUtil.StringToMillis(timestamp,"yyyy-MM-dd HH:mm:ss.SSS");
                                        String e1Data = dataRec[3];
                                        JSONObject jsonObject = JSONObject.parseObject(e1Data);
                                        JSONArray eventList = jsonObject.getJSONArray("eventList");

                                        for (int i = 0; i < eventList.size(); i++) {
                                            JSONObject jsonObject1 = eventList.getJSONObject(i);
                                            String globalLaneNo = jsonObject1.getString("laneNo");
                                            String crossLane = BaseGisLaneCache.getInstance().getLaneRefMap().get(globalLaneNo);
                                            if (crossLane != null) {
                                                String[] sps = crossLane.split(",");

                                                QueueLengthBean bean = new QueueLengthBean();
                                                bean.setCrossName(sps[0]);
                                                bean.setDir(sps[1]);
                                                bean.setLaneNo(sps[2]);
                                                bean.setQueueLength(jsonObject1.getDoubleValue("queueLength"));
                                                bean.setTime(time);
                                                bean.setGlobalLaneNo(globalLaneNo);
                                                sourceContext.collect(bean);
                                            }
                                        }
                                    }

                                }
                                long et = System.currentTimeMillis();
                                log.info("读取文件：{}，耗时：{}ms，扫描B3帧数：{}，最后一帧时间：{}", str, (et - st), e1total, lastTime);
                            }
                        }
                        long et = System.currentTimeMillis();
                        log.info("读取文件：{}，耗时：{}ms", str, (et - st));
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                size++;
                Thread.sleep(100);
            }
            if (size == sortList.size()) {
                sortList.clear();
                Thread.sleep(60 * 1000);
                break;
            }
        }

    }

    @Override
    public void cancel() {
        running = false;
    }


    private void cacheData(CarTrackModel carTrack, ConcurrentHashMap<String, CarTrackModel> vehicleInfoCache) {
        String crossId = carTrack.getRoadnet().getCrossId();
        int trackId = carTrack.getId();
        CarTrackModel cache = vehicleInfoCache.get(crossId + trackId);

        if (cache == null && !carTrack.getRoadnet().getLaneId().equals("null")) {
            carTrack.getRoadnet().setRid(carTrack.getRoadnet().getRid());
            carTrack.getRoadnet().setLaneId(carTrack.getRoadnet().getLaneId());
            carTrack.getRoadnet().setCrossId(carTrack.getRoadnet().getCrossId());
            vehicleInfoCache.put(crossId + trackId, carTrack);
        }

        if (cache != null) {
            if (carTrack.getRoadnet().getLaneId().equals("null")) {
                carTrack.getRoadnet().setRid(cache.getRoadnet().getRid());
                carTrack.getRoadnet().setLaneId(cache.getRoadnet().getLaneId());
                carTrack.getRoadnet().setCrossId(cache.getRoadnet().getCrossId());
                carTrack.getRoadnet().setInCrossFlag(1);
                vehicleInfoCache.put(crossId + trackId, carTrack);
            }
        }
    }


}
