package com.wanji.indicators.task.beihang;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import com.wanji.indicators.util.Tools;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.FileFilterUtils;
import org.apache.commons.io.filefilter.IOFileFilter;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.functions.source.SourceFunction;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;


public class BeiHangTrackFrameDateFile implements SourceFunction<CrossFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(BeiHangTrackFrameDateFile.class);

    private volatile boolean running = true;
    private String directory;
    private long startTime;
    private long endTime;

    public BeiHangTrackFrameDateFile(String directory,long startTime,long endTime) {
        this.directory = directory;
        this.startTime = startTime;
        this.endTime = endTime;
    }


    //获取文件完整路径
    private String[] getFilePath() {
        List<String> indexList = new ArrayList<>();

        ///data/track_partitions/dt=2023-08-09/ht=11/mt=00

        DateTime start = new DateTime(this.startTime);
        DateTime end = new DateTime(this.endTime);

        while (start.isBefore(end)){
            String dt = start.toString("yyyyMMdd");
            String directory = this.directory +"/"+dt;
            if (!new File(directory).exists()){
                start = start.plusDays(1);
                continue;
            }
            IOFileFilter ioFileFilter = FileFilterUtils.prefixFileFilter("2024");
            List<File> fileList = FileUtils.listFiles(new File(directory),ioFileFilter,null).stream().collect(Collectors.toList());
            fileList = fileList.stream().sorted(Comparator.comparing(File::getName)).collect(Collectors.toList());

            String fileName = null;
            for (File dataFile : fileList) {
                fileName = dataFile.getName();
                String filePath = directory +"/"+fileName;
                if(!indexList.contains(filePath)) {
                    indexList.add(filePath);
                }
            }
            start = start.plusDays(1);
//            System.out.println("start:"+start.toString("yyyy-MM-dd HH:mm:ss"));
        }
//        System.out.println("end:"+start.toString("yyyy-MM-dd HH:mm:ss"));
//        System.out.println(indexList.toString());

        return indexList.toArray(new String[0]);
    }

    @Override
    public void run(SourceContext<CrossFrameModel> sourceContext) throws Exception {

        List<String> sortList = Arrays.asList(getFilePath());
        log.info("文件总数：{},文件列表：{}", sortList.size(), sortList.toString());

        while (running) {
            int size = 0;
            for (String str : sortList) {
                File file = new File(str.trim());
                if (file.exists()) {
                    long st = System.currentTimeMillis();
                    try (ZipInputStream zis = new ZipInputStream(new FileInputStream(file))) {
                        ZipEntry entry;
                        while (zis != null && (entry = zis.getNextEntry()) != null) {
                            try (InputStream is = zis;
                                 BufferedReader reader = new BufferedReader(new InputStreamReader(is))) {
                                String line;
                                int e1total = 0;
                                String lastTime = null;
                                while ((line = reader.readLine()) != null) {
                                    long t1 = System.currentTimeMillis();
                                    if (StringUtils.isNotBlank(line)) {
                                        String[] dataRec = line.split("@");
                                        String timestamp = dataRec[0];
                                        String e1Flag = dataRec[1];
                                        if (!StringUtils.equalsIgnoreCase("e1", e1Flag)) {
                                            continue;
                                        }

                                        lastTime = timestamp;
                                        String e1Data = dataRec[3];
                                        JSONObject jsonObject = JSONObject.parseObject(e1Data);
                                        //FrameModel frameModel = JSONObject.parseObject(s, FrameModel.class);

                                        long dataTime = DateUtil.stringToDate(timestamp, "yyyy-MM-dd HH:mm:ss.SSS").getTime();
                                        JSONArray track = jsonObject.getJSONArray("e1FrameParticipant");

                                        if (track != null && !track.isEmpty()) {
                                            e1total++;
                                            Map<String, CrossFrameModel> crossFrameModelMap = new HashMap<>();
                                            for (int index = 0; index < track.size(); index++) {
                                                JSONObject jsonObject1 = track.getJSONObject(index);
                                                CarTrackModel carTrack = track.getObject(index, CarTrackModel.class);// new CarTrackModel();
                                                carTrack.setOriginalType(jsonObject1.getIntValue("type"));

                                                double lon = carTrack.getLongitude();
                                                double lat = carTrack.getLatitude();

                                                long wst = System.currentTimeMillis();
                                                CarTrackModel.RoadNet roadNet = BaseGisLaneCache.getInstance().queryRoadNet(lon + "," + lat);
                                                long wet = System.currentTimeMillis();
                                                //log.info("within 计算耗时：{}ms", wet - wst);

                                                if (roadNet != null) {
                                                    //log.info("读取E1数据：帧时间：{}，文件：{}", timestamp, str);
                                                    carTrack.setRoadnet(roadNet);
                                                    //混合车道根据航向角判定行驶转向
                                                    if (roadNet.getTurn() > 4) {
                                                        carTrack.setCarTurn(CommonUtil.getCarTurn(carTrack));
                                                    } else {
                                                        carTrack.setCarTurn(roadNet.getTurn());
                                                    }
                                                    String crossId = carTrack.getRoadnet().getCrossId();
                                                    CrossFrameModel crossFrameModel = crossFrameModelMap.get(crossId);
                                                    if (Objects.isNull(crossFrameModel)) {
                                                        crossFrameModel = new CrossFrameModel();
                                                    }
                                                    crossFrameModel.setGlobalTimeStamp(dataTime);
                                                    crossFrameModel.setCrossId(crossId);
                                                    crossFrameModel.getTrackList().add(carTrack);
                                                    crossFrameModelMap.put(crossId, crossFrameModel);
                                                }
                                            }
                                            for (Map.Entry<String, CrossFrameModel> entry1 : crossFrameModelMap.entrySet()) {
                                                sourceContext.collect(entry1.getValue());
                                            }
                                            long t2 = System.currentTimeMillis();
                                            //log.info("单帧记录处理耗时：{} ms，每帧目标数量：{}", (t2 - t1), track.size());
                                        }

                                    }

                                }
                                long et = System.currentTimeMillis();
                                log.info("读取文件：{}，耗时：{}ms，扫描E1帧数：{}，最后一帧时间：{}", str, (et - st), e1total, lastTime);
                            }
                        }
                        long et = System.currentTimeMillis();
                        log.info("读取文件：{}，耗时：{}ms", str, (et - st));
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                size++;
                Thread.sleep(  100);
            }
            if (size == sortList.size()) {
                sortList.clear();
                Thread.sleep(60 * 1000);
                break;
            }
        }

    }

    @Override
    public void cancel() {
        running = false;
    }


    private void cacheData(CarTrackModel carTrack, ConcurrentHashMap<String, CarTrackModel> vehicleInfoCache) {
        String crossId = carTrack.getRoadnet().getCrossId();
        int trackId = carTrack.getId();
        CarTrackModel cache = vehicleInfoCache.get(crossId + trackId);

        if (cache == null && !carTrack.getRoadnet().getLaneId().equals("null")) {
            carTrack.getRoadnet().setRid(carTrack.getRoadnet().getRid());
            carTrack.getRoadnet().setLaneId(carTrack.getRoadnet().getLaneId());
            carTrack.getRoadnet().setCrossId(carTrack.getRoadnet().getCrossId());
            vehicleInfoCache.put(crossId + trackId, carTrack);
        }

        if (cache != null) {
            if (carTrack.getRoadnet().getLaneId().equals("null")) {
                carTrack.getRoadnet().setRid(cache.getRoadnet().getRid());
                carTrack.getRoadnet().setLaneId(cache.getRoadnet().getLaneId());
                carTrack.getRoadnet().setCrossId(cache.getRoadnet().getCrossId());
                carTrack.getRoadnet().setInCrossFlag(1);
                vehicleInfoCache.put(crossId + trackId, carTrack);
            }
        }
    }


}
