package com.wanji.indicators.task.beihang;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.task.laneIndicators.CrossLaneIndicatorMainNew;
import com.wanji.indicators.task.laneIndicators.func.LaneDataProcessWindow;
import com.wanji.indicators.task.laneIndicators.model.CrossLaneResultModel;
import com.wanji.indicators.task.laneIndicators.model.LaneResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.jdbc.JdbcConnectionOptions;
import org.apache.flink.connector.jdbc.JdbcExecutionOptions;
import org.apache.flink.connector.jdbc.JdbcSink;
import org.apache.flink.connector.jdbc.JdbcStatementBuilder;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.Duration;
import java.util.List;
import java.util.Properties;

/**
 * 离线计算入口
 */
public class OfflineBeiHangQueueLengthTaskMain {

    private static final Logger log = LoggerFactory.getLogger(OfflineBeiHangQueueLengthTaskMain.class);

    public static void main(String[] args) {

        Properties properties = PropertiesHelper.getInstance().getProperties();


        //初始化环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        //加载配置项 并获取工作路径
        String inPath = "d:/flink/workspace";
        String outPath = "d:/flink/out/";
        boolean isDebug = false;

        ParameterTool parameter = ParameterTool.fromArgs(args);
        inPath = parameter.get("inPath","D:\\WanJi-Work\\07 项目\\02 济南\\数据");
        outPath = parameter.get("outPath","d:/flink/out/");
        String st = parameter.get("startTime","20240412");
        String et = parameter.get("endTime","20240414");
        // --inPath D:\上地西路数据\后厂村+九街20240412\20240412
//        inPath = "D:\\上地西路数据\\七街20240412\\";
        // inPath = "D:\\上地西路数据\\后厂村+九街20240412\\20240412";
//        inPath = "D:\\上地西路数据\\hj202405";
//        st = "20240513";
//        et = "20240518";
        long startTime = DateUtil.StringToMillis(st,"yyyyMMdd");
        long endTime = DateUtil.StringToMillis(et,"yyyyMMdd");
        int parallelism = parameter.getInt("parallelism",1);
        env.setParallelism(parallelism);
        //读取离线数据
        DataStreamSource<QueueLengthBean> streamSource = env.addSource(new BeiHangTrackFrameB3DataFile(inPath,startTime,endTime));
       try {

           SingleOutputStreamOperator<QueueLengthBean> laneStream = streamSource
                   .assignTimestampsAndWatermarks(
                           WatermarkStrategy.<QueueLengthBean>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                   .withTimestampAssigner((event, timestamp) -> event.getTime()))
                   .keyBy(o -> o.getLaneNo())
                   .window(TumblingEventTimeWindows.of(Time.minutes(1)))
                   .maxBy("queueLength")
                   .process(new ProcessFunction<QueueLengthBean, QueueLengthBean>() {
                       @Override
                       public void processElement(QueueLengthBean value, Context ctx, Collector<QueueLengthBean> out) throws Exception {
//                           value.setStartTime(DateUtil.toDateTime(ctx.timestamp(),"yyyy-MM-dd HH:00:00"));
//                           value.setEndTime(DateUtil.toDateTime(ctx.timestamp()+60*60*1000,"yyyy-MM-dd HH:00:00"));

                           value.setStartTime(DateUtil.toDateTime(ctx.timestamp()-1*60*1000,"yyyy-MM-dd HH:mm:00"));
                           value.setEndTime(DateUtil.toDateTime(ctx.timestamp(),"yyyy-MM-dd HH:mm:00"));

                           out.collect(value);
                       }
                   })
                   .name("车道排队指标计算").setParallelism(1);


           if (isDebug) {
               SingleOutputStreamOperator<String> resultStream = laneStream.flatMap(new FlatMapFunction<QueueLengthBean, String>() {
                   @Override
                   public void flatMap(QueueLengthBean value, Collector<String> out) throws Exception {
                       String s = value.getCrossName()+","+value.getDir()+","+value.getLaneNo()+","+value.getQueueLength()+","+value.getStartTime()+","+value.getEndTime()+","+value.getGlobalLaneNo();
                       out.collect(s);
                   }
               }).setParallelism(1);
               String tpath = outPath + "排队长度.csv";
               resultStream.addSink(new JsonStringFileSink(tpath)).setParallelism(1).name("车道排队指标计算-结果输出至文件");
           } else {
               JdbcExecutionOptions jdbcExecutionOptions = JdbcExecutionOptions.builder()
                       .withMaxRetries(3) // 异常重试次数
                       .withBatchSize(1) // 批量大小设置为1，表示一条数据都会写入mysql
                       .withBatchIntervalMs(0) // 批次的间隔为0，表示关闭定时写入
                       .build();
               JdbcConnectionOptions jdbcBuild = new JdbcConnectionOptions.JdbcConnectionOptionsBuilder()
                       .withUrl(properties.getProperty("jdbc.url"))
                       .withUsername(properties.getProperty("jdbc.username"))
                       .withPassword(properties.getProperty("jdbc.password"))
                       .withDriverName(properties.getProperty("jdbc.driver"))
                       .withConnectionCheckTimeoutSeconds(60)
                       .build();

               queueLengthToMysql(jdbcExecutionOptions, jdbcBuild, laneStream);
           }
            env.execute("排队长度-离线汇聚计算");
        } catch (Exception e) {
            log.error("事件测试 - 计算任务异常 : " + e);
        }
    }


    private static void queueLengthToMysql(JdbcExecutionOptions jdbcExecutionOptions, JdbcConnectionOptions jdbcBuild, SingleOutputStreamOperator<QueueLengthBean> laneCsvStream) {
        laneCsvStream.addSink(JdbcSink.sink(
                "insert into t_analysis_cross_queue_length (cross_name,rid_dir,lane_no,max_queue_length,window_start_time,window_end_time,global_lane_no)" +
                        " values(?,?,?,?,?,?,?) " +
                        "on duplicate key update " +
                        "cross_name=values(cross_name)," +
                        "rid_dir=values(rid_dir)," +
                        "lane_no=values(lane_no)," +
                        "max_queue_length=values(max_queue_length)," +
                        "window_start_time=values(window_start_time)," +
                        "window_end_time=values(window_end_time)," +
                        "global_lane_no=values(global_lane_no)"
                ,
                new JdbcStatementBuilder<QueueLengthBean>() {
                    @Override
                    public void accept(PreparedStatement preparedStatement, QueueLengthBean resultModel) throws SQLException {
                        preparedStatement.setString(1, resultModel.getCrossName());
                        preparedStatement.setString(2, resultModel.getDir());
                        preparedStatement.setString(3, resultModel.getLaneNo());
                        preparedStatement.setDouble(4, resultModel.getQueueLength());
                        preparedStatement.setTimestamp(5, new Timestamp(DateUtil.StringToMillis(resultModel.getStartTime())));
                        preparedStatement.setTimestamp(6, new Timestamp(DateUtil.StringToMillis(resultModel.getEndTime())));
                        preparedStatement.setString(7, resultModel.getGlobalLaneNo());
                        preparedStatement.addBatch();
                    }
                },
                jdbcExecutionOptions,
                jdbcBuild
        )).setParallelism(1).name("queue length to mysql");
    }
}
