package com.wanji.indicators.task.citybrain.devicestatus;

import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.DeviceStatusVO;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.streaming.api.functions.windowing.AllWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.TumblingProcessingTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.types.Row;
import org.apache.flink.util.Collector;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

/**
 * @author Kent HAN
 * @date 2024/3/25 13:12
 */
@Slf4j
public class DeviceStatusMain {

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();

    // 城市大脑 URL 路径
    public static final String urlPath;

    static {
        urlPath = properties.getProperty("city.brain.device.status");
    }

    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        DataStream<Row> deviceStream = env.addSource(new PeriodicJdbcReader());

        deviceStream
                .windowAll(TumblingProcessingTimeWindows.of(Time.seconds(5)))
                .apply(new AllWindowFunction<Row, List<Row>, TimeWindow>() {
                    @Override
                    public void apply(TimeWindow window, Iterable<Row> values, Collector<List<Row>> out) {
                        List<Row> rows = new ArrayList<>();
                        for (Row value : values) {
                            rows.add(value);
                        }
                        out.collect(rows);
                    }
                }).addSink(new SinkFunction<List<Row>>() {
                    @Override
                    public void invoke(List<Row> value, Context context) {

                        List<DeviceStatusVO> deviceStatusVOList = new ArrayList<>();
                        for (Row row : value) {
                            DeviceStatusVO deviceStatusVO = new DeviceStatusVO();
                            deviceStatusVO.setData_time(new Date().getTime() + "");
                            deviceStatusVO.setDeviceNo((String) row.getField(0)); // row 只有一个字段
                            deviceStatusVOList.add(deviceStatusVO);
                        }

                        // 将收集到的数据发送至城市大脑
                        HttpUtil.sendData(urlPath, deviceStatusVOList);
                    }
                });

        // 启动Flink作业
        env.execute("推送设备在线状态");
    }
}
