package com.wanji.indicators.task.citybrain.devicestatus;

import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.typeinfo.Types;
import org.apache.flink.api.java.typeutils.RowTypeInfo;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.connector.jdbc.JdbcInputFormat;
import org.apache.flink.core.io.InputSplit;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;
import org.apache.flink.types.Row;

import java.util.Properties;

// 自定义数据源
class PeriodicJdbcReader extends RichSourceFunction<Row> {
    private volatile boolean isRunning = true;
    private JdbcInputFormat jdbcInputFormat;

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    public static final String mysqlIp;
    public static final String mysqlDbName;
    // 全域编号
    public static final String orgCode;

    static {
        mysqlIp = properties.getProperty("mysql.ip");
        mysqlDbName = properties.getProperty("mysql.db.name");
        orgCode = properties.getProperty("org.code");
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        // 初始化 JDBC 连接和查询
        jdbcInputFormat = JdbcInputFormat.buildJdbcInputFormat()
                .setDrivername("com.mysql.cj.jdbc.Driver")
                .setDBUrl("jdbc:mysql://" + mysqlIp + ":3306/" + mysqlDbName + "?serverTimezone=Asia/Shanghai&useUnicode=true&characterEncoding=UTF-8")
                .setUsername("root")
                .setPassword("Wanji300552")
                .setQuery("SELECT equip_code FROM t_device_camera " +
                        "UNION ALL SELECT equip_code FROM t_device_radar " +
                        "UNION ALL SELECT equip_code FROM t_base_lane_detector " +
                        "UNION ALL SELECT DISTINCT cross_id as equip_code FROM t_device_radar " +
                        "UNION ALL SELECT '" + orgCode + "' as equip_code;")
                .setRowTypeInfo(new RowTypeInfo(Types.STRING))
                .finish();
    }

    @Override
    public void run(SourceContext<Row> ctx) throws Exception {
        while (isRunning) {
            // 在这里执行查询
            jdbcInputFormat.openInputFormat();
            InputSplit[] inputSplits = jdbcInputFormat.createInputSplits(1);
            for (InputSplit split : inputSplits) {
                jdbcInputFormat.open(split);
                while (!jdbcInputFormat.reachedEnd()) {
                    Row row = jdbcInputFormat.nextRecord(new Row(1));
                    if (row != null) {
                        ctx.collect(row);
                    }
                }
            }
            jdbcInputFormat.close();
            jdbcInputFormat.closeInputFormat();
            // 等待5秒
            Thread.sleep(5000);
        }
    }

    @Override
    public void cancel() {
        isRunning = false;
    }
}