package com.wanji.indicators.task.citybrain.event.parkevent;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.constant.citybrain.DirConvertSectionEnum;
import com.wanji.indicators.constant.citybrain.VehicleColorConvertEnum;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.event.accident.AccidentShowModel;
import com.wanji.indicators.task.citybrain.vo.LaneParkEventVO;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2024/03/28 10:15
 */
@Service
public class LaneParkEventCoFlatMap implements CoFlatMapFunction<String, Map<String, List<BaseLaneInfo>>, List<LaneParkEventVO>> {

    private static final Logger log = LoggerFactory.getLogger(LaneParkEventCoFlatMap.class);
    private static final Map<String, List<BaseLaneInfo>> baseLaneInfoMap = new HashMap<>();

    @Override
    public void flatMap1(String value, Collector<List<LaneParkEventVO>> out) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            JsonNode jsonNode = instance.readTree(value);
            String eventType = jsonNode.get("eventType").asText();
            // 只获取疑似事故401
            if (!StringUtils.equalsIgnoreCase(EventAbnormalEnum.ACCIDENT_YS.getType(), eventType)) {
                return;
            }
            AccidentShowModel accidentShowModel = instance.readValue(value, AccidentShowModel.class);

            LaneParkEventVO laneParkEventVO = new LaneParkEventVO();
            laneParkEventVO.setData_time(String.valueOf(accidentShowModel.getStartTime()));
            // parking：停车事件，jam：拥堵事件，accident 事故事件
            laneParkEventVO.setEventType("accident");
            laneParkEventVO.setDeviceNo(accidentShowModel.getCrossId());

            List<BaseLaneInfo> baseLaneInfos = baseLaneInfoMap.get(accidentShowModel.getCrossId());
            if (!CollectionUtils.isEmpty(baseLaneInfos)) {
                for (BaseLaneInfo baseLaneInfo : baseLaneInfos) {
                    if (StringUtils.equalsIgnoreCase(baseLaneInfo.getId(), accidentShowModel.getLaneId())
                     && StringUtils.equalsIgnoreCase(accidentShowModel.getCrossId(), baseLaneInfo.getCrossId())) {
                        BaseLaneInfo currentBaseLaneInfo = baseLaneInfo;
                        Integer dir = currentBaseLaneInfo.getDir();
                        laneParkEventVO.setSectionNo(String.valueOf(DirConvertSectionEnum.getSection(dir)));
                        laneParkEventVO.setEuuid(String.valueOf(accidentShowModel.getId()));
                        laneParkEventVO.setId(accidentShowModel.getPlateNumber()); // todo 目标id 目前车牌
                        String laneId = accidentShowModel.getLaneId();
                        laneParkEventVO.setLaneId(laneId.substring(laneId.length() - 1));
                        double x = accidentShowModel.getX();
                        double y = accidentShowModel.getY();
                        String geography = "[" + x + "," + y + "," + "0" + "]";
                        laneParkEventVO.setGeography(geography);
                        laneParkEventVO.setType("abnormal");
                        laneParkEventVO.setState("1");
                        String vehicleType = accidentShowModel.getVehicleType();
                        laneParkEventVO.setVehicleType("1");
                        byte vehicleColor = (byte) accidentShowModel.getVehicleColor();
                        laneParkEventVO.setVehicleColor(VehicleColorConvertEnum.getCode(vehicleColor));
                        laneParkEventVO.setPlateNumber(accidentShowModel.getPlateNumber());
                    }
                }
            }

            out.collect(Collections.singletonList(laneParkEventVO));
        } catch (Exception e) {
            log.error("城市大脑处理停车事件数据异常", e);
            throw new Exception(e);
        }
    }

    @Override
    public void flatMap2(Map<String, List<BaseLaneInfo>> value, Collector<List<LaneParkEventVO>> out) throws Exception {
        baseLaneInfoMap.putAll(value);
    }
}
