package com.wanji.indicators.task.citybrain.event.parkevent;

import com.wanji.indicators.source.CrossLaneInfoMapSource;
import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.LaneParkEventVO;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Properties;

/**
 * @author duanruiming
 * @date 2024/03/28 10:08
 */
@SuppressWarnings("all")
public class LaneParkEventMain {

    private static final Logger log = LoggerFactory.getLogger(LaneParkEventMain.class);

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    public static final String urlPath;

    static {
        urlPath = properties.getProperty("city.brain.park.event");
    }


    public static void main(String[] args) throws Exception {
        try {
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

            PropertiesHelper instance = PropertiesHelper.getInstance();
            Properties properties = instance.getProperties();
            properties.setProperty("group.id", "pushBrainLaneParkData");

            String topic = properties.getProperty("cross.event.data.topic");
            FlinkKafkaConsumer<String> myConsumer = new FlinkKafkaConsumer<>(topic, new SimpleStringSchema(), properties);

            DataStream<String> periodDataStream = env.addSource(myConsumer);

            DataStream<List<LaneParkEventVO>> resultStream = periodDataStream
                    .connect(env.addSource(new CrossLaneInfoMapSource()).broadcast())
                    .flatMap(new LaneParkEventCoFlatMap())
                    .name("城市大脑-推送停车事件数据");

            resultStream.addSink(new SinkFunction<List<LaneParkEventVO>>() {
                @Override
                public void invoke(List<LaneParkEventVO> value, Context context) throws Exception {
                    // 将收集到的数据发送至城市大脑
                    if (!CollectionUtils.isEmpty(value)) {
                        HttpUtil.sendData(urlPath, value);
                    }
                }
            });

            env.executeAsync("pushBrainLaneParkData");
        } catch (Exception e) {
            log.error("城市大脑-推送停车事件数据异常", e);
            throw new Exception(e);
        }
    }
}
