package com.wanji.indicators.task.citybrain.event.spilloverevent;

import com.wanji.indicators.constant.citybrain.DirConvertSectionEnum;
import com.wanji.indicators.constant.citybrain.SpilloverStateEnum;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.task.citybrain.vo.SpilloverEventVO;
import com.wanji.indicators.task.trajectory.pojo.LaneSnapshotDataDTO;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/03/28 14:11
 */
@Service
public class SpilloverEventCoFlatMap implements CoFlatMapFunction<String, Map<String, List<BaseLaneInfo>>, List<SpilloverEventVO>> {
    private final static Logger log = LoggerFactory.getLogger(SpilloverEventCoFlatMap.class);
    private static final Map<String, List<BaseLaneInfo>> baseLaneInfoMap = new HashMap<>();
    private static final Map<String, String> spilloverLaneMap = new HashMap<>();

    @Override
    public void flatMap1(String s, Collector<List<SpilloverEventVO>> out) throws Exception {
        try {
            List<SpilloverEventVO> spilloverEventVOS = new ArrayList<>();
            LaneSnapshotDataDTO laneSnapshotDataDTO = JacksonUtils.getInstance().readValue(s, LaneSnapshotDataDTO.class);
            String timeStamp = laneSnapshotDataDTO.getTimeStamp();
            Date date = DateUtil.parse(timeStamp, com.wanji.indicators.util.DateUtil.YYYY_MM_DD_HH_MM_SS_SSS);
            long dateLong = date.getTime();
            List<LaneSnapshotDataDTO.EventList> eventList = laneSnapshotDataDTO.getEventList();
            if (!CollectionUtils.isEmpty(eventList)) {
                for (Map.Entry<String, List<BaseLaneInfo>> entry : baseLaneInfoMap.entrySet()) {
                    String crossId = entry.getKey();
                    List<BaseLaneInfo> baseLaneInfos = entry.getValue();
                    if (!CollectionUtils.isEmpty(baseLaneInfos)) {
                        Map<Integer, List<BaseLaneInfo>> dirMap = baseLaneInfos.stream().collect(Collectors.groupingBy(BaseLaneInfo::getDir));
                        for (Map.Entry<Integer, List<BaseLaneInfo>> dirEntry : dirMap.entrySet()) {
                            Integer dir = dirEntry.getKey();

                            // 溢出事件方向上报
                            SpilloverEventVO spilloverEventVO = new SpilloverEventVO();
                            spilloverEventVO.setData_time(timeStamp);
                            spilloverEventVO.setEventType("spillback");
                            spilloverEventVO.setDeviceNo(crossId);
                            spilloverEventVO.setSectionNo(String.valueOf(DirConvertSectionEnum.getSection(dir)));
                            spilloverEventVO.setEuuid(crossId.concat(String.valueOf(dir)).concat(String.valueOf(dateLong)));
                            spilloverEventVO.setAreaId(crossId.concat(String.valueOf(dir)));

                            List<BaseLaneInfo> dirList = dirEntry.getValue();
                            int dirOverflowNums = 0;
                            double dirMeanV = 0.0;
                            int count = 0;
                            double dirSpaceOccupancy = 0.0;
                            if (!CollectionUtils.isEmpty(dirList)) {
                                for (BaseLaneInfo baseLaneInfo : dirList) {
                                    String id = baseLaneInfo.getId();
                                    for (LaneSnapshotDataDTO.EventList event : eventList) {
                                        String laneId = event.getLaneId();
                                        if (StringUtils.equalsIgnoreCase(laneId, id)) {
                                            ++count;
                                            dirOverflowNums += event.getOverflowNums();
                                            dirMeanV += event.getMeanV();
                                            dirSpaceOccupancy += event.getVehicleLengthRatio();

                                            setSpilloverState(spilloverEventVO, event, laneId);
                                        }
                                    }
                                }
                            }
                            if (count > 0) {
                                spilloverEventVO.setVehicleNum(String.valueOf(dirOverflowNums / count));
                                spilloverEventVO.setSpeedAverage(String.valueOf(dirMeanV / count));
                                spilloverEventVO.setSpaceOccupancy(String.valueOf(dirSpaceOccupancy / count));
                            }
                            if (Objects.isNull(spilloverEventVO.getState())) {
                                spilloverEventVO.setState("0");
                            }
                            spilloverEventVOS.add(spilloverEventVO);
                        }
                    }
                }
            }
            out.collect(spilloverEventVOS);
        } catch (Exception e) {
            log.error("城市大脑溢出事件数据处理异常：", e);
            throw new Exception(e);
        }
    }

    private static void setSpilloverState(SpilloverEventVO spilloverEventVO, LaneSnapshotDataDTO.EventList event, String laneId) {
        // 是否溢出
        if (event.isOverflow()) {
            spilloverEventVO.setState(SpilloverStateEnum.START.getCode());
            if (!spilloverLaneMap.isEmpty() && StringUtils.equalsIgnoreCase(spilloverLaneMap.get(laneId), SpilloverStateEnum.START.getCode())) {
                spilloverEventVO.setState(SpilloverStateEnum.CONTINUE.getCode());
                spilloverLaneMap.putIfAbsent(laneId, SpilloverStateEnum.CONTINUE.getCode());
            }
            spilloverLaneMap.putIfAbsent(laneId, SpilloverStateEnum.START.getCode());
        } else {
            spilloverEventVO.setState(SpilloverStateEnum.FINISH.getCode());
            spilloverLaneMap.remove(laneId);
        }
    }

    @Override
    public void flatMap2(Map<String, List<BaseLaneInfo>> value, Collector<List<SpilloverEventVO>> out) throws Exception {
        baseLaneInfoMap.putAll(value);
    }
}
