package com.wanji.indicators.task.citybrain.event.spilloverevent;

import com.wanji.indicators.source.CrossLaneInfoMapSource;
import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.SpilloverEventVO;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Properties;

/**
 * @author duanruiming
 * @date 2024/03/28 14:02
 */
public class SpilloverEventMain {

    private static final Logger log = LoggerFactory.getLogger(SpilloverEventMain.class);

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    public static final String urlPath;
    public static String state = "0";

    static {
        urlPath = properties.getProperty("city.brain.spillover.event");
    }

    public static void main(String[] args) throws Exception {
        try {
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

            PropertiesHelper instance = PropertiesHelper.getInstance();
            Properties properties = instance.getProperties();
            properties.setProperty("group.id", "pushBrainLaneSpilloverData");

            String topic = properties.getProperty("lane.snapshot.data");
            KafkaSource<String> source = KafkaSource.<String>builder()
                    .setProperties(instance.getConsumerProperties())
                    .setProperty("enable.auto.commit", "true")
                    .setProperty("auto.commit.interval.ms", "1000")
                    .setProperty("auto.offset.reset", "latest")
                    .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                    .setTopics(topic)
                    .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                    .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                    .build();
            DataStream<String> snapshotStream = env.fromSource(source, WatermarkStrategy.noWatermarks(), topic);

            DataStream<List<SpilloverEventVO>> resultStream = snapshotStream
                    .connect(env.addSource(new CrossLaneInfoMapSource()).broadcast())
                    .flatMap(new SpilloverEventCoFlatMap())
                    .setParallelism(1)
                    .name("城市大脑-推送溢出事件数据");

            resultStream.addSink(new SinkFunction<List<SpilloverEventVO>>() {
                @Override
                public void invoke(List<SpilloverEventVO> value, Context context) throws Exception {
                    // 将收集到的数据发送至城市大脑
                    if (!CollectionUtils.isEmpty(value)) {
                        HttpUtil.sendData(urlPath, value);
                    }
                }
            });

            env.executeAsync("城市大脑-推送溢出事件数据");
        } catch (Exception e) {
            log.error("城市大脑-推送溢出事件数据异常", e);
            throw new Exception(e);
        }
    }
}
