package com.wanji.indicators.task.citybrain.lanestatistics;

import com.wanji.indicators.task.citybrain.vo.LaneSnapShotOverFlowAreaCarNumsDTO;
import com.wanji.indicators.task.trajectory.pojo.LaneSnapshotDataDTO;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import net.wanji.common.utils.tool.JacksonUtils;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.util.Collector;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.Duration;
import java.util.Date;
import java.util.List;
import java.util.Properties;

/**
 * @author duanruiming
 * @date 2024/04/01 13:36
 */
public class LaneSnapShotDataMainTest {

    private static final Logger log = LoggerFactory.getLogger(LaneSnapShotDataMainTest.class);

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();

    public static void main(String[] args) throws Exception {
        try {
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

            PropertiesHelper instance = PropertiesHelper.getInstance();
            Properties properties = instance.getProperties();

            String topic = properties.getProperty("lane.snapshot.data");
            KafkaSource<String> source = KafkaSource.<String>builder()
                    .setProperties(instance.getConsumerProperties())
                    .setProperty("enable.auto.commit", "true")
                    .setProperty("auto.commit.interval.ms", "1000")
                    .setProperty("auto.offset.reset", "latest")
                    .setGroupId("1111")
                    .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                    .setTopics(topic)
                    //.setGroupId(GROUP_ID+"_Period_Data"+ UUID.randomUUID().toString())
                    .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                    .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                    .build();

            DataStream<String> dataStream = env
                    .fromSource(source, WatermarkStrategy.noWatermarks(), topic);

            SingleOutputStreamOperator<LaneSnapShotOverFlowAreaCarNumsDTO> laneSnapshotStream =
                    dataStream.flatMap(new FlatMapFunction<String, LaneSnapShotOverFlowAreaCarNumsDTO>() {
                        @Override
                        public void flatMap(String value, Collector<LaneSnapShotOverFlowAreaCarNumsDTO> out) throws Exception {
                            LaneSnapshotDataDTO laneSnapshotDataDTO = JacksonUtils.getInstance().readValue(value, LaneSnapshotDataDTO.class);
                            String timeStamp = laneSnapshotDataDTO.getTimeStamp();
                            List<LaneSnapshotDataDTO.EventList> eventList = laneSnapshotDataDTO.getEventList();
                            for (LaneSnapshotDataDTO.EventList event : eventList) {
                                LaneSnapShotOverFlowAreaCarNumsDTO carNumsDTO = new LaneSnapShotOverFlowAreaCarNumsDTO();
                                carNumsDTO.setLaneId(event.getLaneId());
                                carNumsDTO.setOverflowNums(event.getOverflowNums() + 1);
                                carNumsDTO.setTimeStamp(timeStamp);
                                Date date = net.wanji.common.utils.tool.DateUtil.parse(timeStamp, DateUtil.YYYY_MM_DD_HH_MM_SS_SSS);
                                carNumsDTO.setTimeStampLong(date.getTime());
                                out.collect(carNumsDTO);
                            }
                        }
                    });

            SingleOutputStreamOperator<LaneSnapShotOverFlowAreaCarNumsDTO> carNumsResultStream = laneSnapshotStream
                    .assignTimestampsAndWatermarks(
                            WatermarkStrategy.<LaneSnapShotOverFlowAreaCarNumsDTO>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((carNumsDTO, timeStampLong) -> carNumsDTO.getTimeStampLong()))
                    .keyBy(LaneSnapShotOverFlowAreaCarNumsDTO::getLaneId)
                    .window(TumblingEventTimeWindows.of(Time.minutes(1)))
                    .aggregate(new OverflowCarNumsAggregateFunction(), new OverflowCarNumsProcessWindow())
                    .name("城市大脑-计算1分钟溢出区车辆数");

            carNumsResultStream.print("城市大脑-计算5分钟溢出区车辆数===>");


            env.executeAsync("城市大脑-推送车道统计数据推送");
        } catch (Exception e) {
            log.error("城市大脑-推送车道统计数据异常", e);
            throw new Exception(e);
        }
    }
}
