package com.wanji.indicators.task.citybrain.lanestatistics;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.LanePeriodicDataModel;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/03/27 13:47
 */
@Service
public class LaneStatisticsDataCoFlatMap implements CoFlatMapFunction<String, Map<String, List<BaseLaneInfo>>, List<LaneStatisticsDataVO>> {

    private static final Logger log = LoggerFactory.getLogger(LaneStatisticsDataCoFlatMap.class);
    private static final Map<String, List<BaseLaneInfo>> baseLaneInfoMap = new HashMap<>();

    @Override
    public void flatMap1(String value, Collector<List<LaneStatisticsDataVO>> out) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            LanePeriodicDataModel lanePeriodicDataModel = instance.readValue(value, LanePeriodicDataModel.class);
            List<LanePeriodicDataModel.EventList> eventList = lanePeriodicDataModel.getEventList();
            Map<String, List<LanePeriodicDataModel.EventList>> periodLaneIdMap = Collections.emptyMap();
            if (!CollectionUtils.isEmpty(eventList)) {
                periodLaneIdMap = eventList.stream().collect(Collectors.groupingBy(LanePeriodicDataModel.EventList::getLaneId));
            }
            String crossId = "";
            int dir = 0;
            String timeStamp = lanePeriodicDataModel.getTimeStamp();
            Date date = DateUtil.parse(timeStamp, com.wanji.indicators.util.DateUtil.YYYY_MM_DD_HH_MM_SS_SSS);

            List<LaneStatisticsDataVO> laneStatisticsDataVOS = new ArrayList<>();
            for (Map.Entry<String, List<BaseLaneInfo>> entry : baseLaneInfoMap.entrySet()) {
                LaneStatisticsDataVO laneStatisticsDataVO = new LaneStatisticsDataVO();
                crossId = entry.getKey();
                List<BaseLaneInfo> baseLaneInfos = entry.getValue();
                laneStatisticsDataVO.setData_time(String.valueOf(date.getTime()));
                laneStatisticsDataVO.setLaneNum(lanePeriodicDataModel.getLaneNum());
                laneStatisticsDataVO.setDeviceNo(crossId);
                laneStatisticsDataVO.setCycle(5 * 60);
                // 每个方向一个线圈
                List<Integer> dirs = baseLaneInfos.stream().map(BaseLaneInfo::getDir).collect(Collectors.toList());
                laneStatisticsDataVO.setCoilNum(dirs.size());

                List<LaneStatisticsDetail> laneStatisticsDetails = new ArrayList<>();
                for (BaseLaneInfo baseLaneInfo : baseLaneInfos) {
                    dir = baseLaneInfo.getDir();
                    Integer turn = baseLaneInfo.getTurn();
                    LaneStatisticsDetail laneStatisticsDetail = new LaneStatisticsDetail();
                    String laneId = baseLaneInfo.getId();
                    List<LanePeriodicDataModel.EventList> eventLists = periodLaneIdMap.get(laneId);
                    if (CollectionUtils.isEmpty(eventLists)) {
                        continue;
                    }
                    int volumePerson = 0;
                    if (laneId.contains(Constant.MIDDLE_LINE)) {
                        volumePerson = eventLists.get(0).getTrafficFlow();
                        laneStatisticsDetail.setVolumePerson(volumePerson);
                    }
                    laneStatisticsDetail.setLaneNo(laneId.substring(laneId.length() - 1));
                    laneStatisticsDetail.setCoilNo(String.valueOf(dir));
                    // 目前机动车 + 非机动车 + 行人
                    laneStatisticsDetail.setVolumeSum(eventLists.get(0).getNonMotorFlow() + eventLists.get(0).getTrafficFlow() + volumePerson);
                    laneStatisticsDetail.setVolumePerson(0); // 目前无行人流量
                    laneStatisticsDetail.setVolumeCarNon(eventLists.get(0).getNonMotorFlow());
                    laneStatisticsDetail.setVolumeCarSmall(eventLists.get(0).getTrafficFlowC());
                    laneStatisticsDetail.setVolumeCarMiddle(eventLists.get(0).getTrafficFlowB());
                    laneStatisticsDetail.setVolumeCarBig(eventLists.get(0).getTrafficFlowA());
                    laneStatisticsDetail.setSpeedAvg(String.valueOf(eventLists.get(0).getMeanV()));
                    laneStatisticsDetail.setOccupancy(String.valueOf(eventLists.get(0).getTimeOccupancy()));
                    laneStatisticsDetail.setHeadway(String.valueOf(eventLists.get(0).getTimeHeadwaySectionMean()));
                    laneStatisticsDetail.setGap("0"); // todo 车身间距S，使用车头间距，但是在快照
                    laneStatisticsDetail.setSpeed85(String.valueOf(eventLists.get(0).getV85()));
                    laneStatisticsDetail.setWArrivalFlow(String.valueOf(eventLists.get(0).getTrafficFlow()));
                    laneStatisticsDetail.setFSpaceOccupyRation(String.valueOf(eventLists.get(0).getVehicleNumsRatioMean())); // 负荷比
                    laneStatisticsDetail.setByQueueLen(String.valueOf(eventLists.get(0).getDynamicQueueLengthMax()));
                    laneStatisticsDetail.setWDelay(String.valueOf(eventLists.get(0).getMeanDelay()));
                    laneStatisticsDetail.setByStoppingTimes(String.valueOf(eventLists.get(0).getMeanStopsNumber()));

                    if (turn == 1) {
                        laneStatisticsDetail.setDwLeftVehNum(eventLists.get(0).getTrafficFlow());
                    }
                    if (turn == 2) {
                        laneStatisticsDetail.setDwRightVehNum(eventLists.get(0).getTrafficFlow());
                    }
                    if (turn == 3) {
                        laneStatisticsDetail.setDwStraightVehNum(eventLists.get(0).getTrafficFlow());
                    }

                    laneStatisticsDetails.add(laneStatisticsDetail);
                }
                laneStatisticsDataVO.setSectionNo(String.valueOf(dir));
                laneStatisticsDataVO.setData_list(laneStatisticsDetails);
                laneStatisticsDataVOS.add(laneStatisticsDataVO);
            }
            out.collect(laneStatisticsDataVOS);
        } catch (Exception e) {
            log.error("城市大脑处理车道统计数据异常", e);
            throw new Exception(e);
        }
    }

    @Override
    public void flatMap2(Map<String, List<BaseLaneInfo>> value, Collector<List<LaneStatisticsDataVO>> out) throws Exception {
        baseLaneInfoMap.putAll(value);
    }
}
