package com.wanji.indicators.task.citybrain.lanestatistics;

import com.wanji.indicators.task.citybrain.vo.LaneSnapShotOverFlowAreaCarNumsDTO;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author duanruiming
 * @date 2024/04/01 17:00
 */
public class OverflowCarNumsProcessWindow extends ProcessWindowFunction<
        LaneSnapShotOverFlowAreaCarNumsDTO,
        LaneSnapShotOverFlowAreaCarNumsDTO,
        String,
        TimeWindow> {
    private static final Logger log = LoggerFactory.getLogger(OverflowCarNumsProcessWindow.class);

    @Override
    public void process(String s, ProcessWindowFunction<LaneSnapShotOverFlowAreaCarNumsDTO, LaneSnapShotOverFlowAreaCarNumsDTO, String, TimeWindow>.Context context, Iterable<LaneSnapShotOverFlowAreaCarNumsDTO> elements, Collector<LaneSnapShotOverFlowAreaCarNumsDTO> out) throws Exception {
        try {
            long windowStartTs = context.window().getStart();
            long windowEndTs = context.window().getEnd();

            if (elements.iterator().hasNext()) {

                LaneSnapShotOverFlowAreaCarNumsDTO result = elements.iterator().next();
                result.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss.SSS"));
                result.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss.SSS"));

                log.info("OverflowCarNums Result:" + result.toString());

                out.collect(result);
            }
        } catch (Exception e) {
            log.error("OverflowCarNumsProcessWindow error:", e);
            throw new RuntimeException(e);
        }
    }
}
