package com.wanji.indicators.task.citybrain.laneturnratio;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.citybrain.DirConvertSectionEnum;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.LanePeriodicDataModel;
import com.wanji.indicators.task.citybrain.lanestatistics.LaneStatisticsDataCoFlatMap;
import com.wanji.indicators.task.citybrain.vo.LaneTurnRatioDataVO;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/03/27 15:43
 */
@Service
public class LaneTurnRatioDataCoFlatMap implements CoFlatMapFunction<String, Map<String, List<BaseLaneInfo>>, List<LaneTurnRatioDataVO>> {

    private static final Logger log = LoggerFactory.getLogger(LaneStatisticsDataCoFlatMap.class);
    private static final Map<String, List<BaseLaneInfo>> baseLaneInfoMap = new HashMap<>();

    @Override
    public void flatMap1(String value, Collector<List<LaneTurnRatioDataVO>> out) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            LanePeriodicDataModel lanePeriodicDataModel = instance.readValue(value, LanePeriodicDataModel.class);
            List<LanePeriodicDataModel.EventList> eventList = lanePeriodicDataModel.getEventList();
            Map<String, List<LanePeriodicDataModel.EventList>> periodLaneIdMap = Collections.emptyMap();
            if (!CollectionUtils.isEmpty(eventList)) {
                periodLaneIdMap = eventList.stream().collect(Collectors.groupingBy(LanePeriodicDataModel.EventList::getLaneId));
            }
            String crossId = "";
            int dir = 0;
            String timeStamp = lanePeriodicDataModel.getTimeStamp();
            Date date = DateUtil.parse(timeStamp, com.wanji.indicators.util.DateUtil.YYYY_MM_DD_HH_MM_SS_SSS);

            int allFlow = eventList.stream().map(LanePeriodicDataModel.EventList::getTrafficFlow).mapToInt(Integer::intValue).sum();
            OptionalDouble average = eventList.stream().map(LanePeriodicDataModel.EventList::getMeanV).mapToDouble(Double::doubleValue).average();
            double meanV = 0.0;
            if (average.isPresent()) {
                meanV = average.getAsDouble();
            }
            List<LaneTurnRatioDataVO> laneTurnRatioDataVOS = new ArrayList<>();
            for (Map.Entry<String, List<BaseLaneInfo>> entry : baseLaneInfoMap.entrySet()) {
                crossId = entry.getKey();
                List<BaseLaneInfo> baseLaneInfos = entry.getValue();
                Map<Integer, List<BaseLaneInfo>> dirList = baseLaneInfos.stream().collect(Collectors.groupingBy(BaseLaneInfo::getDir));
                for (Map.Entry<Integer, List<BaseLaneInfo>> entryList : dirList.entrySet()) {
                    LaneTurnRatioDataVO laneTurnRatioDataVO = new LaneTurnRatioDataVO();
                    laneTurnRatioDataVO.setData_time(String.valueOf(date.getTime()));
                    laneTurnRatioDataVO.setDeviceNo(crossId);
                    laneTurnRatioDataVO.setCycle(5 * 60);
                    laneTurnRatioDataVO.setVolumeSum(allFlow);
                    laneTurnRatioDataVO.setSpeedAvg(String.valueOf(meanV));
                    dir = entryList.getKey();
                    List<BaseLaneInfo> dirBaseLaneInfos = entryList.getValue();
                    List<Integer> leftFlowList = new ArrayList<>();
                    List<Integer> straightFlowList = new ArrayList<>();
                    List<Integer> rightFlowList = new ArrayList<>();
                    List<Double> leftSpeedList = new ArrayList<>();
                    List<Double> straightSpeedList = new ArrayList<>();
                    List<Double> rightSpeedList = new ArrayList<>();

                    for (BaseLaneInfo baseLaneInfo : dirBaseLaneInfos) {
                        String laneId = baseLaneInfo.getId();
                        String turn = String.valueOf(baseLaneInfo.getTurn());

                        List<LanePeriodicDataModel.EventList> eventLists = periodLaneIdMap.get(laneId);
                        if (CollectionUtils.isEmpty(eventLists)) {
                            continue;
                        }
                        LanePeriodicDataModel.EventList eventListEntity = eventLists.get(0);
                        if (StringUtils.equalsIgnoreCase(turn, Constant.LEFT_TURN_CODE)) {
                            leftFlowList.add(eventListEntity.getTrafficFlow());
                            leftSpeedList.add(eventListEntity.getMeanV());
                        }
                        if (StringUtils.equalsIgnoreCase(turn, Constant.STRAIGHT_LINE_CODE)) {
                            straightFlowList.add(eventListEntity.getTrafficFlow());
                            straightSpeedList.add(eventListEntity.getMeanV());
                        }
                        if (StringUtils.equalsIgnoreCase(turn, Constant.RIGHT_TURN_CODE)) {
                            rightFlowList.add(eventListEntity.getTrafficFlow());
                            rightSpeedList.add(eventListEntity.getMeanV());
                        }
                    }
                    int leftFlow = 0;
                    double leftSpeed = 0.0;
                    int straightFlow = 0;
                    double straightSpeed = 0.0;
                    int rightFlow = 0;
                    double rightSpeed = 0.0;
                    if (!CollectionUtils.isEmpty(leftFlowList)) {
                        leftFlow = calculateAverageFlow(leftFlowList);
                        leftSpeed = calculateAverageSpeed(leftSpeedList);
                    }
                    if (!CollectionUtils.isEmpty(straightFlowList)) {
                        straightFlow = calculateAverageFlow(straightFlowList);
                        straightSpeed = calculateAverageSpeed(straightSpeedList);
                    }
                    if (!CollectionUtils.isEmpty(rightFlowList)) {
                        rightFlow = calculateAverageFlow(rightFlowList);
                        rightSpeed = calculateAverageSpeed(rightSpeedList);
                    }
                    laneTurnRatioDataVO.setSectionNo(String.valueOf(DirConvertSectionEnum.getSection(dir)));
                    laneTurnRatioDataVO.setVolumeLeft(leftFlow);
                    laneTurnRatioDataVO.setSpeedLeft(String.valueOf(leftSpeed));
                    laneTurnRatioDataVO.setVolumeStraight(straightFlow);
                    laneTurnRatioDataVO.setSpeedStraight(String.valueOf(straightSpeed));
                    laneTurnRatioDataVO.setVolumeRight(rightFlow);
                    laneTurnRatioDataVO.setSpeedRight(String.valueOf(rightSpeed));
                    laneTurnRatioDataVOS.add(laneTurnRatioDataVO);
                }

            }
            out.collect(laneTurnRatioDataVOS);
        } catch (Exception e) {
            log.error("城市大脑处理车道转向比数据异常", e);
            throw new Exception(e);
        }
    }

    private int calculateAverageFlow(List<Integer> flowList) {
        if (CollectionUtils.isEmpty(flowList)) {
            return 0;
        }
        return flowList.stream().mapToInt(Integer::intValue).sum() / flowList.size();
    }

    private double calculateAverageSpeed(List<Double> speedList) {
        if (CollectionUtils.isEmpty(speedList)) {
            return 0.0;
        }
        return speedList.stream().mapToDouble(Double::doubleValue).sum() / speedList.size();
    }

    @Override
    public void flatMap2(Map<String, List<BaseLaneInfo>> value, Collector<List<LaneTurnRatioDataVO>> out) throws Exception {
        baseLaneInfoMap.putAll(value);
    }
}
