package com.wanji.indicators.task.citybrain.laneturnratio;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.citybrain.DirConvertSectionEnum;
import com.wanji.indicators.task.citybrain.vo.LaneTurnRatioDataVO;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.OptionalDouble;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/04/09 11:19
 */
@Service
@Slf4j
public class LaneTurnRatioDataCoFlatMapNew implements FlatMapFunction<String, List<LaneTurnRatioDataVO>> {
    @Override
    public void flatMap(String value, Collector<List<LaneTurnRatioDataVO>> out) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CrossTurnResultModel model = mapper.readValue(value, CrossTurnResultModel.class);
            String crossId = model.getCrossId();
            Long timestamp = model.getTimestamp();
            List<LaneTurnRatioDataVO> laneTurnRatioDataVOS = new ArrayList<>();
            List<TurnResultModel> turnDataList = model.getTurnDataList();

            if (!CollectionUtils.isEmpty(turnDataList)) {
                Map<Integer, List<TurnResultModel>> dirMap = turnDataList.stream().collect(Collectors.groupingBy(TurnResultModel::getInRidDir));
                if (!dirMap.isEmpty()) {
                    for (Map.Entry<Integer, List<TurnResultModel>> entry : dirMap.entrySet()) {
                        Integer dir = entry.getKey();

                        LaneTurnRatioDataVO laneTurnRatioDataVO = new LaneTurnRatioDataVO();
                        laneTurnRatioDataVO.setData_time(String.valueOf(timestamp));
                        laneTurnRatioDataVO.setDeviceNo(crossId);
                        laneTurnRatioDataVO.setCycle(5 * 60);

                        laneTurnRatioDataVO.setSectionNo(String.valueOf(DirConvertSectionEnum.getSection(dir)));
                        List<TurnResultModel> dirModels = entry.getValue();
                        int allFlow = dirModels.stream().mapToInt(TurnResultModel::getFlow).sum();
                        OptionalDouble average = dirModels.stream().mapToDouble(TurnResultModel::getSpeed).average();
                        double speedAverage = 0.0;
                        if (average.isPresent()) {
                            speedAverage = average.getAsDouble();
                        }
                        laneTurnRatioDataVO.setVolumeSum(allFlow);
                        laneTurnRatioDataVO.setSpeedAvg(String.valueOf(speedAverage));
                        if (!CollectionUtils.isEmpty(dirModels)) {
                            for (TurnResultModel dirModel : dirModels) {
                                String turn = String.valueOf(dirModel.getTurnDirNo());
                                int flow = dirModel.getFlow();
                                double speed = dirModel.getSpeed();
                                if (StringUtils.equalsIgnoreCase(turn, Constant.LEFT_TURN_CODE)) {
                                    laneTurnRatioDataVO.setVolumeLeft(flow);
                                    laneTurnRatioDataVO.setSpeedLeft(String.valueOf(speed));
                                }
                                if (StringUtils.equalsIgnoreCase(turn, Constant.STRAIGHT_LINE_CODE)) {
                                    laneTurnRatioDataVO.setVolumeStraight(flow);
                                    laneTurnRatioDataVO.setSpeedStraight(String.valueOf(speed));
                                }
                                if (StringUtils.equalsIgnoreCase(turn, Constant.RIGHT_TURN_CODE)) {
                                    laneTurnRatioDataVO.setVolumeRight(flow);
                                    laneTurnRatioDataVO.setSpeedRight(String.valueOf(speed));
                                }
                            }
                            laneTurnRatioDataVOS.add(laneTurnRatioDataVO);
                        }
                    }
                }
            }

            if (!CollectionUtils.isEmpty(laneTurnRatioDataVOS)) {
                out.collect(laneTurnRatioDataVOS);
            }
        } catch (Exception e) {
            log.error("城市大脑处理车道转向比数据异常", e);
            throw new Exception(e);
        }
    }
}
