package com.wanji.indicators.task.citybrain.laneturnratio;

import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.LaneTurnRatioDataVO;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Properties;

/**
 * @author duanruiming
 * @date 2024/04/09 11:15
 */
public class LaneTurnRatioDataMainNew {
    private static final Logger log = LoggerFactory.getLogger(LaneTurnRatioDataMain.class);

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    public static final String urlPath;

    static {
        urlPath = properties.getProperty("city.brain.turn.data");
    }

    public static void main(String[] args) throws Exception {
        try {
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

            PropertiesHelper instance = PropertiesHelper.getInstance();
            Properties properties = instance.getProperties();
            String topic = properties.getProperty("turn.ratio.topic");
            properties.setProperty("group.id", "pushBrainLaneTurnRatioData");
            FlinkKafkaConsumer<String> myConsumer = new FlinkKafkaConsumer<>(topic, new SimpleStringSchema(), properties);

            DataStream<String> periodDataStream = env.addSource(myConsumer);

            DataStream<List<LaneTurnRatioDataVO>> resultStream = periodDataStream
                    //.connect(env.addSource(new CrossLaneInfoMapSource()).broadcast())
                    .flatMap(new LaneTurnRatioDataCoFlatMapNew())
                    .name("城市大脑-推送车道转向比数据");

            resultStream.addSink(new SinkFunction<List<LaneTurnRatioDataVO>>() {
                @Override
                public void invoke(List<LaneTurnRatioDataVO> value, Context context) throws Exception {
                    // 将收集到的数据发送至城市大脑
                    if (!CollectionUtils.isEmpty(value)) {
                        HttpUtil.sendData(urlPath, value);
                    }
                }
            });

            env.executeAsync("pushBrainLaneTurnRatioData");
        } catch (Exception e) {
            log.error("城市大脑-推送车道转向比数据异常", e);
            throw new Exception(e);
        }
    }
}
