package com.wanji.indicators.task.citybrain.snapshot;

import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.wanji.indicators.service.impl.BaseLaneInfoServiceImpl;
import com.wanji.indicators.task.citybrain.vo.DynamicQueueVO;
import com.wanji.indicators.task.trajectory.pojo.LaneSnapshotDataDTO;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.functions.RichMapFunction;
import org.apache.flink.configuration.Configuration;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.text.SimpleDateFormat;
import java.util.*;

public class DynamicEnrichWithStaticData extends RichMapFunction<LaneSnapshotDataDTO, List<DynamicQueueVO>> {

    Properties properties = null;
    BaseLaneInfoService baseLaneInfoService;
    private transient Map<String, BaseLaneInfo> laneInfoMap;

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneInfoService = beanConf.getBean(BaseLaneInfoServiceImpl.class);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        properties = instance.getProperties();

        // 加载静态数据
        laneInfoMap = loadStaticDataFromDatabase();
    }

    @Override
    public List<DynamicQueueVO> map(LaneSnapshotDataDTO laneSnapshotDataDTO) throws Exception {
        // 使用静态数据和流中的元素进行处理
        return enrichFrameModel(laneSnapshotDataDTO, laneInfoMap);
    }

    private Map<String, BaseLaneInfo> loadStaticDataFromDatabase() {
        // 从数据库加载数据
        String crossList = properties.getProperty("full.area.cross.list");
        return baseLaneInfoService.findLaneInfo(crossList);
    }

    private List<DynamicQueueVO> enrichFrameModel(
            LaneSnapshotDataDTO laneSnapshotDataDTO, Map<String, BaseLaneInfo> laneInfoMap) {

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");

        List<DynamicQueueVO> dynamicQueueList = new ArrayList<>();
        // 用 crossId 和 ridDir4 分组 EventElement
        Map<String, List<LaneSnapshotDataDTO.EventList>> groupedEventLists = new HashMap<>();
        for (LaneSnapshotDataDTO.EventList event : laneSnapshotDataDTO.getEventList()) {
            BaseLaneInfo laneInfo = laneInfoMap.get(event.getLaneId());
            if (laneInfo != null) {
                Integer ridDir4 = laneInfo.getRidDir4();
                // 转换方向
                ridDir4 = CommonUtil.convertCityBrainDir(ridDir4);
                String groupKey = laneInfo.getCrossId() + "_" + ridDir4;
                // 如果这个组还没有被创建，就新建一个
                groupedEventLists.computeIfAbsent(groupKey, k -> new ArrayList<>());
                // 添加 event 到相应的组
                groupedEventLists.get(groupKey).add(event);
            }
        }

        for (String groupKey : groupedEventLists.keySet()) {

            List<LaneSnapshotDataDTO.EventList> events = groupedEventLists.get(groupKey);
            DynamicQueueVO dynamicQueueVO = new DynamicQueueVO();
            List<DynamicQueueVO.LaneInfo> laneInfos = new ArrayList<>();

            for (LaneSnapshotDataDTO.EventList event : events) {
                DynamicQueueVO.LaneInfo laneInfo = new DynamicQueueVO.LaneInfo();
                String substring = event.getLaneId().substring(event.getLaneId().length() - 1);
                if (CommonUtil.canConvertToInt(substring)) {
                    laneInfo.setLaneNo(Integer.parseInt(substring));
                }
                laneInfo.setQueueLen(event.getDynamicQueueLength());
                laneInfo.setQueueHead(event.getTeamHeadDistance());
                laneInfo.setQueueTail(event.getTeamTailDistance());
                laneInfo.setQueueCount(event.getQueueNums());
                laneInfos.add(laneInfo);
            }

            try {
                Date date = dateFormat.parse(laneSnapshotDataDTO.getTimeStamp());
                dynamicQueueVO.setData_time(date.getTime());
            } catch (Exception e) {
                e.printStackTrace();
            }

            // 设置 StaticQueueVO 的其他属性
            dynamicQueueVO.setDeviceNo(groupKey.split("_")[0]);
            dynamicQueueVO.setLaneNum(laneInfos.size());
            dynamicQueueVO.setSectionNo(Integer.parseInt(groupKey.split("_")[1]));
            dynamicQueueVO.setData_list(laneInfos);

            dynamicQueueList.add(dynamicQueueVO);
        }

        return dynamicQueueList;
    }
}