package com.wanji.indicators.task.citybrainreceive.devicestatus;

import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.Properties;

/**
 * 接收城市大脑数据
 *
 * @author Kent HAN
 * @date 2024/3/27 9:18
 */
public class DeviceStatusReceiveMain {
    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // 获取配置文件中的kafka消费topic
        String sourceTopic = properties.getProperty("device.status.receive.topic");
        String kafkaServerAddress = properties.getProperty("bootstrap.servers.citybrain.receive");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(sourceTopic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-devicestatus-receive")
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env.fromSource(source, WatermarkStrategy.noWatermarks(),
                "city-brain-devicestatus-receive-data-source");

        SingleOutputStreamOperator<DeviceStatusReceiveModel> frameModelStream = stream
                .map(new DeviceStatusReceiveMap())
                .name("接收城市大脑设备状态数据-JsonToObject");

        frameModelStream.print();


        // 启动 Flink 作业
        env.execute("接收城市大脑设备状态数据");
    }
}
