package com.wanji.indicators.task.citybrainreceive.event.park;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.citybrain.ParkTypeEnum;
import com.wanji.indicators.constant.citybrain.SpilloverStateEnum;
import com.wanji.indicators.entity.EventInfo;
import com.wanji.indicators.task.citybrain.vo.LaneParkEventVO;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.api.common.functions.MapFunction;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;

/**
 * @author duanruiming
 * @date 2024/04/11 9:34
 */
@Service
@Slf4j
public class LaneParkEventReceiveMap implements MapFunction<String, List<EventInfo>> {
    @Override
    public List<EventInfo> map(String value) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            String body = model.getBody();
            String headers = model.getHeaders();
            if (StringUtils.isNotBlank(headers) && headers.contains(Constant.WAN_JI_NAME) || StringUtils.isBlank(body)) {
                return Collections.emptyList();
            }
            List<EventInfo> eventInfos = new ArrayList<>();
            List<LaneParkEventVO> laneParkEventVOS = mapper.readValue(body, new TypeReference<List<LaneParkEventVO>>() {});
            if (!CollectionUtils.isEmpty(laneParkEventVOS)) {
                for (LaneParkEventVO laneParkEventVO : laneParkEventVOS) {

                    String dataTime = laneParkEventVO.getData_time();
                    Instant instant = Instant.ofEpochMilli(Long.parseLong(dataTime));
                    ZonedDateTime zonedDateTime = instant.atZone(ZoneId.of("+8"));
                    LocalDateTime localDateTime = zonedDateTime.toLocalDateTime();

                    Date date = new Date(Long.parseLong(dataTime));
                    String dateStr = DateUtil.format(date, Constants.DATE_FORMAT.E_DATE_FORMAT_MILLIS);
                    EventInfo eventInfo = new EventInfo();
                    eventInfo.setDetectTime(localDateTime);
                    eventInfo.setCategory("1");
                    String type = laneParkEventVO.getType();
                    eventInfo.setType("401");
                    if (StringUtils.equalsIgnoreCase(type, "illegal")) {
                        eventInfo.setType("1");
                    }
                    eventInfo.setStartTime(localDateTime);
                    //eventInfo.setEndTime(localDateTime);
                    eventInfo.setDuration(0);
                    eventInfo.setRuksj(localDateTime);
                    eventInfo.setLaneId(laneParkEventVO.getLaneId());
                    eventInfo.setCrossId(laneParkEventVO.getDeviceNo());
                    eventInfo.setEventSerialNumber(laneParkEventVO.getDeviceNo() + dataTime);
                    eventInfo.setPlateNo(laneParkEventVO.getPlateNumber());
                    String geography = laneParkEventVO.getGeography();
                    String[] split = null;
                    if (StringUtils.isNotBlank(geography)) {
                        split = geography.replaceAll("\\[", "").replaceAll("\\]", "").split(",");
                        eventInfo.setLng(new BigDecimal(split[0]));
                        eventInfo.setLat(new BigDecimal(split[1]));
                    }
                    eventInfo.setSource("5");
                    eventInfo.setRemark(ParkTypeEnum.getMsg(laneParkEventVO.getType()) + SpilloverStateEnum.getMsg(laneParkEventVO.getState()));
                    Map<String, String> map = new HashMap<>();
                    map.put("vehicleType", laneParkEventVO.getVehicleType());
                    map.put("vehicleColor", laneParkEventVO.getVehicleColor());
                    map.put("plateType", laneParkEventVO.getPlateType());
                    map.put("plateNumber", laneParkEventVO.getPlateType());
                    eventInfo.setExtend(mapper.writeValueAsString(map));
                    eventInfos.add(eventInfo);
                }
            }

            return eventInfos;
        } catch (Exception e) {
            log.error("停车事件数据转化异常：", e);
            throw new RuntimeException(e);
        }
    }
}