package com.wanji.indicators.task.citybrainreceive.event.spillover;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.citybrain.SpilloverStateEnum;
import com.wanji.indicators.entity.EventInfo;
import com.wanji.indicators.task.citybrain.vo.SpilloverEventVO;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.api.common.functions.MapFunction;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;

/**
 * @author duanruiming
 * @date 2024/04/11 10:40
 */
@Service
@Slf4j
public class SpilloverEventReceiveMap implements MapFunction<String, List<EventInfo>> {
    @Override
    public List<EventInfo> map(String value) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            String body = model.getBody();
            String headers = model.getHeaders();
            if (StringUtils.isNotBlank(headers) && headers.contains(Constant.WAN_JI_NAME) || StringUtils.isBlank(body)) {
                return Collections.emptyList();
            }
            List<EventInfo> eventInfos = new ArrayList<>();
            List<SpilloverEventVO> spilloverEventVOS = mapper.readValue(body, new TypeReference<List<SpilloverEventVO>>() {});
            if (!CollectionUtils.isEmpty(spilloverEventVOS)) {
                for (SpilloverEventVO spilloverEventVO : spilloverEventVOS) {

                    String dataTime = spilloverEventVO.getData_time();
                    Instant instant = Instant.ofEpochMilli(Long.parseLong(dataTime));
                    ZonedDateTime zonedDateTime = instant.atZone(ZoneId.of("+8"));
                    LocalDateTime localDateTime = zonedDateTime.toLocalDateTime();

                    Date date = new Date(Long.parseLong(dataTime));
                    String dateStr = DateUtil.format(date, Constants.DATE_FORMAT.E_DATE_FORMAT_MILLIS);
                    EventInfo eventInfo = new EventInfo();
                    eventInfo.setDetectTime(localDateTime);
                    eventInfo.setCategory("1");
                    String type = spilloverEventVO.getEventType();
                    String typeMeg = null;
                    if (StringUtils.equalsIgnoreCase(type, "spillback")) {
                        eventInfo.setType("605");
                        typeMeg = "溢出";
                    }
                    eventInfo.setStartTime(localDateTime);
                    //eventInfo.setEndTime(localDateTime);
                    eventInfo.setDuration(0);
                    eventInfo.setRuksj(localDateTime);
                    eventInfo.setLaneId(spilloverEventVO.getEuuid());
                    eventInfo.setCrossId(spilloverEventVO.getDeviceNo());
                    eventInfo.setEventSerialNumber(spilloverEventVO.getDeviceNo() + dataTime);
                    String geography = spilloverEventVO.getGeography();
                    String[] split = null;
                    if (StringUtils.isNotBlank(geography)) {
                        split = geography.replaceAll("\\[", "").replaceAll("\\]", "").split(",");
                        eventInfo.setLng(new BigDecimal(split[0]));
                        eventInfo.setLat(new BigDecimal(split[1]));
                    }
                    //eventInfo.setPlaceDesc(SpilloverEventVO.getType() + "-" + SpilloverEventVO.getState());
                    eventInfo.setSource("5");
                    eventInfo.setRemark( typeMeg + SpilloverStateEnum.getMsg(spilloverEventVO.getState()));

                    Map<String, String> map = new HashMap<>();
                    map.put("vehicleNum", spilloverEventVO.getVehicleNum());
                    map.put("areaId", spilloverEventVO.getAreaId());
                    map.put("speedAverage", spilloverEventVO.getSpeedAverage());
                    map.put("spaceOccupancy", spilloverEventVO.getSpaceOccupancy());
                    eventInfo.setExtend(mapper.writeValueAsString(map));
                    eventInfos.add(eventInfo);

                }
            }

            return eventInfos;
        } catch (Exception e) {
            log.error("溢出事件数据转化异常：", e);
            throw new RuntimeException(e);
        }
    }
}