package com.wanji.indicators.task.citybrainreceive.event.spillover;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.task.citybrain.vo.SpilloverEventVO;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import org.apache.flink.api.common.functions.MapFunction;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;

/**
 * @author duanruiming
 * @date 2024/05/16 10:32
 */
@Service
@Slf4j
public class SpilloverEventTempReceiveMap implements MapFunction<String, List<SpilloverEventVO>> {

    @Override
    public List<SpilloverEventVO> map(String value) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            String body = model.getBody();
            String headers = model.getHeaders();
            if (StringUtils.isNotBlank(headers) && headers.contains(Constant.WAN_JI_NAME) || StringUtils.isBlank(body)) {
                return Collections.emptyList();
            }
            List<SpilloverEventVO> spilloverEventVOS = mapper.readValue(body, new TypeReference<List<SpilloverEventVO>>() {});
            if (!CollectionUtils.isEmpty(spilloverEventVOS)) {
                return spilloverEventVOS;
            }
        } catch (Exception e) {
            log.error("溢出事件数据转化异常：", e);
            throw new RuntimeException(e);
        }
        return Collections.emptyList();
    }
}
