package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import com.wanji.indicators.task.citybrain.vo.SpilloverEventVO;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossDataRealtimePO;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.OptionalDouble;

/**
 * @author duanruiming
 * @date 2024/04/19 9:22
 */
@Service
@Slf4j
public class LaneStatisticsReceive2CrossDataCoFlatMap implements CoFlatMapFunction<String, List<SpilloverEventVO>, String> {

    private static final List<SpilloverEventVO> currentPeriodEventList = new ArrayList<>();

    @Override
    public void flatMap1(String value, Collector<String> out) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            List<CrossDataRealtimePO> results = new ArrayList<>();
            String headers = model.getHeaders();
            String body = model.getBody();
            if (StringUtils.isNotBlank(headers) && (headers.contains(Constant.WAN_JI_NAME) ||
                    StringUtils.isBlank(body)) || !body.contains("data_list")) {
                return;
            }
            List<LaneStatisticsDataVO> laneStatisticsDataVOS = mapper.readValue(body, new TypeReference<List<LaneStatisticsDataVO>>() {});
            if (!CollectionUtils.isEmpty(laneStatisticsDataVOS)) {
                for (LaneStatisticsDataVO inVo : laneStatisticsDataVOS) {
                    String crossId = inVo.getDeviceNo();
                    String startTime = inVo.getData_time();
                    Date dateTime = new Date(Long.parseLong(startTime));
                    String dir = inVo.getSectionNo();
                    List<LaneStatisticsDetail> dataList = inVo.getData_list();
                    if (!CollectionUtils.isEmpty(dataList)) {
                        CrossDataRealtimePO po = new CrossDataRealtimePO();
                        po.setCrossId(crossId);
                        po.setStatus(1);
                        po.setType(1);
                        po.setStartTime(dateTime);
                        po.setDuration(5);
                        po.setIsUnbalance(0);
                        po.setIsCongestion(0);
                        po.setIsSpillover(0);
                        po.setUnbalanceIndex(0.0);
                        po.setCongestionIndex(0.0);
                        po.setSpilloverIndex(0.0);
                        int flowSum = dataList.stream().filter(po1 -> StringUtils.isNotBlank(po1.getWArrivalFlow())).map(LaneStatisticsDetail::getWArrivalFlow).mapToInt(Integer::parseInt).sum();
                        po.setFlow(flowSum);
                        po.setFlowRate((double) (flowSum / 5 * 12));
                        OptionalDouble speedAverage = dataList.stream().map(LaneStatisticsDetail::getSpeedAvg).mapToDouble(Double::parseDouble).average();
                        po.setSpeed(0.0);
                        if (speedAverage.isPresent()) {
                            po.setSpeed(speedAverage.getAsDouble());
                        }
                        if (po.getSpeed() > 0) {
                            po.setTrafficIndex(Math.round(60 / po.getSpeed() / dataList.size() * 100.0) / 100.0);
                        } else {
                            po.setTrafficIndex(1.0);
                        }
                        OptionalDouble queueLength = dataList.stream().map(LaneStatisticsDetail::getByQueueLen).mapToDouble(Double::parseDouble).max();
                        po.setQueueLength(0.0);
                        if (queueLength.isPresent()) {
                            po.setQueueLength(queueLength.getAsDouble());
                        }
                        OptionalDouble stopTimeAverage = dataList.stream().map(LaneStatisticsDetail::getByStoppingTimes).mapToDouble(Double::parseDouble).average();
                        po.setStopTimes(0.0);
                        if (stopTimeAverage.isPresent()) {
                            po.setStopTimes(stopTimeAverage.getAsDouble());
                        }
                        OptionalDouble delayTime = dataList.stream().filter(l -> StringUtils.isNotBlank(l.getWDelay())).map(LaneStatisticsDetail::getWDelay).mapToDouble(Double::parseDouble).average();
                        po.setDelayTime(0);
                        if (delayTime.isPresent()) {
                            po.setDelayTime((int) delayTime.getAsDouble());
                        }
                        po.setClearRate(0.0);
                        po.setNotClearCarNums(0);
                        po.setLoadBalance(0.0);
                        po.setSturation(0.7);
                        po.setEffusionRate(0.0);
                        po.setNoStopRate(0.0);
                        po.setOneStopRate(0.0);
                        po.setTwoStopRate(0.0);
                        po.setThreeStopRate(0.0);
                        po.setBatchTime(Integer.parseInt(startTime.substring(0, 10)));
                        po.setEffusionRate(0.0);
                        po.setGreenLightEfficiency(0.0);
                        po.setEndTime(dateTime);
                        // 处理事件，将事件更新到实时数据
                        String spilloverDirs = "";
                        if (!CollectionUtils.isEmpty(currentPeriodEventList)) {
                            for (SpilloverEventVO spilloverEventVO : currentPeriodEventList) {
                                String deviceNo = spilloverEventVO.getDeviceNo();
                                if (StringUtils.equalsIgnoreCase(deviceNo, crossId)
                                        && StringUtils.equalsIgnoreCase("0", spilloverEventVO.getState())) {
                                    String sectionNo = spilloverEventVO.getSectionNo();
                                    spilloverDirs.concat(sectionNo).concat(",");
                                    String spilloverSpeed = spilloverEventVO.getSpeedAverage();
                                    po.setSpeed(Double.parseDouble(spilloverSpeed));
                                    po.setIsSpillover(1);
                                    po.setStatus(3);
                                    po.setSpilloverIndex(2.0);
                                    currentPeriodEventList.remove(spilloverEventVO);
                                }
                            }
                            po.setSpilloverDirs(spilloverDirs);
                        }
                        results.add(po);
                    }
                }
            }

            //Map<String, List<CrossDataRealtimePO>> map = new HashMap<>();
            //map.put(Constant.REAL_TIME_CROSS_DATA_KEY, results);
            out.collect(mapper.writeValueAsString(results));
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2CrossDataCoFlatMap: ", e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public void flatMap2(List<SpilloverEventVO> value, Collector<String> out) throws Exception {
        currentPeriodEventList.addAll(value);
    }
}
