package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.ForeignDevicePO;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossDataRealtimePO;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/07/19 15:49
 */
@Service
@Slf4j
public class LaneStatisticsReceive2CrossDataCoFlatMapVersion2 implements CoFlatMapFunction<String, Map<String, List<ForeignDevicePO>>, String> {

    private static final Map<String, List<ForeignDevicePO>> foreignDeviceMap = new HashMap<>();

    @Override
    public void flatMap1(String value, Collector<String> out) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            List<CrossDataRealtimePO> results = new ArrayList<>();
            List<CrossDataRealtimePO> paramList = mapper.readValue(value, new TypeReference<List<CrossDataRealtimePO>>() {});
            replaceDeviceNO(paramList);
            Map<String, List<CrossDataRealtimePO>> crossMap = paramList.stream().collect(Collectors.groupingBy(CrossDataRealtimePO::getCrossId));
            if (!crossMap.isEmpty()) {
                for (Map.Entry<String, List<CrossDataRealtimePO>> entry : crossMap.entrySet()) {
                    String crossId = entry.getKey();
                    List<CrossDataRealtimePO> realtimePOS = entry.getValue();
                    CrossDataRealtimePO po = new CrossDataRealtimePO();
                    Integer status = 0;
                    Integer flow = 0;
                    Double speed = 0.0;
                    Double speed85 = 0.0;
                    Double queueLengthMax = 0.0;
                    Double stopTimes = 0.0;
                    int duration = 0;
                    String unbalaDirs = "";
                    String conDirs = "";
                    String spillDirs = "";
                    Integer delayTime = 0;
                    Double sturation = 0.0;
                    Double noStopRate = 0.0;
                    Double oneStopRate = 0.0;
                    Double twoStopRate = 0.0;
                    Double threeStopRate = 0.0;
                    Double effusionRate = 0.0;
                    Double greenLightEfficiency = 0.0;
                    Double timeOccupancy = 0.0;
                    Double vehicleLengthRatioMean = 0.0;
                    int trafficA = 0;
                    int trafficB = 0;
                    int trafficC = 0;
                    int nonMotorFlow = 0;
                    for (CrossDataRealtimePO realtimePO : realtimePOS) {
                        status = status >= realtimePO.getStatus() ? status : realtimePO.getStatus();
                        flow += realtimePO.getFlow();
                        speed += realtimePO.getSpeed();
                        speed85 += realtimePO.getV85();
                        duration = duration >= realtimePO.getDuration() ? duration : realtimePO.getDuration();
                        queueLengthMax = queueLengthMax >= realtimePO.getQueueLength() ? queueLengthMax : realtimePO.getQueueLength();
                        stopTimes = stopTimes >= realtimePO.getStopTimes() ? stopTimes : realtimePO.getStopTimes();
                        delayTime += realtimePO.getDelayTime();
                        sturation += realtimePO.getSturation();
                        noStopRate += realtimePO.getNoStopRate();
                        oneStopRate += realtimePO.getOneStopRate();
                        twoStopRate += realtimePO.getTwoStopRate();
                        threeStopRate += realtimePO.getThreeStopRate();
                        effusionRate += realtimePO.getEffusionRate();
                        timeOccupancy += realtimePO.getTimeOccupancy();
                        vehicleLengthRatioMean += realtimePO.getVehicleLengthRatioMean();
                        if (StringUtils.isNotBlank(realtimePO.getUnbalanceDirs())) {
                            unbalaDirs = unbalaDirs.concat(realtimePO.getUnbalanceDirs());
                        }
                        if (StringUtils.isNotBlank(realtimePO.getCongestionDirs())) {
                            conDirs = conDirs.concat(realtimePO.getCongestionDirs());
                        }
                        if (StringUtils.isNotBlank(realtimePO.getSpilloverDirs())) {
                            spillDirs = spillDirs.concat(realtimePO.getSpilloverDirs());
                        }
                        nonMotorFlow += realtimePO.getNonMotorFlow();
                        trafficA += realtimePO.getTrafficFlowA();
                        trafficB += realtimePO.getTrafficFlowB();
                        trafficC += realtimePO.getTrafficFlowC();
                    }
                    po.setCrossId(crossId);
                    po.setStatus(status);
                    po.setType(1);
                    if (status > 1) {
                        po.setType(2);
                    }
                    double delayIndex = Math.round(60 / speed * 100) / 100.0;
                    po.setTrafficIndex(delayIndex);
                    po.setStartTime(realtimePOS.get(0).getStartTime());
                    po.setDuration(duration);
                    if (status == 1) {
                        po.setIsUnbalance(1);
                    } else if (status == 2) {
                        po.setIsCongestion(1);
                    } else if (status == 3) {
                        po.setIsSpillover(1);
                    }
                    po.setUnbalanceIndex(delayIndex);
                    po.setSpilloverIndex(delayIndex);
                    po.setCongestionIndex(delayIndex);
                    po.setUnbalanceDirs(unbalaDirs);
                    po.setCongestionDirs(conDirs);
                    po.setSpilloverDirs(spillDirs);
                    po.setFlow(flow);
                    po.setFlowRate(Math.round(flow * 12 * 100) / 100.0);
                    po.setSpeed(Math.round(speed / realtimePOS.size() * 100) / 100.0);
                    po.setQueueLength(queueLengthMax);
                    po.setStopTimes(Math.round(stopTimes / realtimePOS.size() * 100) / 100.0);
                    po.setDelayTime(Math.round(delayTime / realtimePOS.size() * 100) / 100);
                    po.setSturation(Math.round(sturation / realtimePOS.size() * 100) / 100.0);
                    po.setBatchTime(realtimePOS.get(0).getBatchTime());
                    po.setGmtCreate(realtimePOS.get(0).getGmtCreate());
                    po.setGmtModified(realtimePOS.get(0).getGmtModified());
                    po.setEndTime(realtimePOS.get(0).getEndTime());
                    po.setLoadBalance(Math.round(Math.sqrt(sturation / realtimePOS.size()) * 100) / 100.0);
                    po.setNoStopRate(Math.round(noStopRate / realtimePOS.size() * 100) / 100.0);
                    po.setOneStopRate(Math.round(oneStopRate / realtimePOS.size() * 100) / 100.0);
                    po.setTwoStopRate(Math.round(twoStopRate / realtimePOS.size() * 100) / 100.0);
                    po.setThreeStopRate(Math.round(threeStopRate / realtimePOS.size() * 100) / 100.0);
                    po.setNonMotorFlow(nonMotorFlow);
                    po.setV85(Math.round(speed85 / realtimePOS.size() * 100) / 100.0);
                    po.setTrafficFlowA(trafficA);
                    po.setTrafficFlowB(trafficB);
                    po.setTrafficFlowC(trafficC);
                    po.setTimeOccupancy(Math.round(timeOccupancy / realtimePOS.size() * 100) / 100.0);
                    results.add(po);
                }
            }

            if (!CollectionUtils.isEmpty(results)) {
                Map<String, List<CrossDataRealtimePO>> map = new HashMap<>();
                map.put(Constant.REAL_TIME_CROSS_DATA_KEY, results);
                out.collect(mapper.writeValueAsString(map));
            }
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2CrossDataCoFlatMapVersion2-异常: ", e);
            throw new RuntimeException(e);
        }
    }

    private void replaceDeviceNO(List<CrossDataRealtimePO> paramList) {
        if (!CollectionUtils.isEmpty(paramList)) {
            List<ForeignDevicePO> foreignDevicePOS = foreignDeviceMap.get(paramList.get(0).getCrossId());
            paramList.forEach(po -> {
                if (!CollectionUtils.isEmpty(foreignDevicePOS)) {
                    po.setCrossId(foreignDevicePOS.get(0).getCrossId());
                }
            });
        }
    }

    @Override
    public void flatMap2(Map<String, List<ForeignDevicePO>> value, Collector<String> out) throws Exception {
        foreignDeviceMap.putAll(value);
    }
}