package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossDataRealtimePO;
import org.apache.flink.api.common.functions.MapFunction;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author duanruiming
 * @date 2024/04/16 13:40
 */
@Service
@Slf4j
public class LaneStatisticsReceive2CrossDataMap implements MapFunction<String, String> {
    @Override
    public String map(String value) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            List<CrossDataRealtimePO> results = new ArrayList<>();
            String headers = model.getHeaders();
            String body = model.getBody();
            if (StringUtils.isNotBlank(headers) && (headers.contains(Constant.WAN_JI_NAME) || StringUtils.isBlank(body))) {
                return null;
            }
            List<LaneStatisticsDataVO> laneStatisticsDataVOS = mapper.readValue(body, new TypeReference<List<LaneStatisticsDataVO>>() {
            });
            if (!CollectionUtils.isEmpty(laneStatisticsDataVOS)) {
                for (LaneStatisticsDataVO inVo : laneStatisticsDataVOS) {
                    String crossId = inVo.getDeviceNo();
                    String startTime = inVo.getData_time();
                    Date dateTime = new Date(Long.parseLong(startTime));
                    String dir = inVo.getSectionNo();
                    List<LaneStatisticsDetail> dataList = inVo.getData_list();
                    if (!CollectionUtils.isEmpty(dataList)) {
                        CrossDataRealtimePO po = new CrossDataRealtimePO();
                        po.setCrossId(crossId);
                        po.setStatus(1);
                        po.setType(1);
                        po.setTrafficIndex(1.0);
                        po.setStartTime(dateTime);
                        po.setDuration(5);
                        po.setIsUnbalance(0);
                        po.setIsCongestion(0);
                        po.setIsSpillover(0);
                        po.setUnbalanceIndex(0.0);
                        po.setCongestionIndex(0.0);
                        po.setSpilloverIndex(0.0);
                        int flowSum = dataList.stream().filter(po1 -> StringUtils.isNotBlank(po1.getWArrivalFlow())).map(LaneStatisticsDetail::getWArrivalFlow).mapToInt(Integer::parseInt).sum();
                        po.setFlow(flowSum);
                        po.setFlowRate((double) (flowSum / 5 * 12));
                        OptionalDouble speedAverage = dataList.stream().map(LaneStatisticsDetail::getSpeedAvg).mapToDouble(Double::parseDouble).average();
                        po.setSpeed(0.0);
                        if (speedAverage.isPresent()) {
                            po.setSpeed(speedAverage.getAsDouble());
                        }
                        OptionalDouble queueLength = dataList.stream().map(LaneStatisticsDetail::getByQueueLen).mapToDouble(Double::parseDouble).max();
                        po.setQueueLength(0.0);
                        if (queueLength.isPresent()) {
                            po.setQueueLength(queueLength.getAsDouble());
                        }
                        OptionalDouble stopTimeAverage = dataList.stream().map(LaneStatisticsDetail::getByStoppingTimes).mapToDouble(Double::parseDouble).average();
                        po.setStopTimes(0.0);
                        if (stopTimeAverage.isPresent()) {
                            po.setStopTimes(stopTimeAverage.getAsDouble());
                        }
                        OptionalDouble delayTime = dataList.stream().filter(l -> StringUtils.isNotBlank(l.getWDelay())).map(LaneStatisticsDetail::getWDelay).mapToDouble(Double::parseDouble).average();
                        po.setDelayTime(0);
                        if (delayTime.isPresent()) {
                            po.setDelayTime((int) delayTime.getAsDouble());
                        }
                        po.setClearRate(0.0);
                        po.setNotClearCarNums(0);
                        po.setLoadBalance(0.0);
                        po.setSturation(0.7);
                        po.setEffusionRate(0.0);
                        po.setNoStopRate(0.0);
                        po.setOneStopRate(0.0);
                        po.setTwoStopRate(0.0);
                        po.setThreeStopRate(0.0);
                        po.setBatchTime(Integer.parseInt(startTime.substring(0, 10)));
                        po.setEffusionRate(0.0);
                        po.setGreenLightEfficiency(0.0);
                        po.setEndTime(dateTime);
                        results.add(po);
                    }
                }
            }

            Map<String, List<CrossDataRealtimePO>> map = new HashMap<>();
            map.put(Constant.REAL_TIME_CROSS_DATA_KEY, results);
            return mapper.writeValueAsString(map);
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2CrossDataMap ", e);
            throw new RuntimeException(e);
        }
    }
}