package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.ForeignDevicePO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossDirDataRealtimePO;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author duanruiming
 * @date 2024/07/19 14:38
 */
@Service
@Slf4j
public class LaneStatisticsReceive2DirDataMapVersion2 implements CoFlatMapFunction<String, Map<String, List<ForeignDevicePO>>, String> {

    private static final Map<String, List<ForeignDevicePO>> foreignDeviceMap = new HashMap<>();

    @Override
    public void flatMap1(String value, Collector<String> out) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            List<CrossDirDataRealtimePO> results = new ArrayList<>();
            String headers = model.getHeaders();
            String body = model.getBody();
            if (StringUtils.isNotBlank(headers) && (headers.contains(Constant.WAN_JI_NAME)
                    || StringUtils.isBlank(body)) || !body.contains("data_list")) {
                return;
            }
            List<LaneStatisticsDataVO> laneStatisticsDataVOS = mapper.readValue(body, new TypeReference<List<LaneStatisticsDataVO>>() {});
            if (!CollectionUtils.isEmpty(laneStatisticsDataVOS)) {
                for (LaneStatisticsDataVO inVo : laneStatisticsDataVOS) {
                    String deviceNo = inVo.getDeviceNo();
                    String startTime = inVo.getData_time();
                    Date dateTime = new Date(Long.parseLong(startTime));
                    String dir = inVo.getSectionNo();
                    List<LaneStatisticsDetail> dataList = inVo.getData_list();
                    if (!CollectionUtils.isEmpty(dataList)) {

                        ForeignDevicePO foreignDevicePO = getForeignDevicePO(deviceNo, dir, foreignDeviceMap);
                        if (Objects.isNull(foreignDevicePO)) {
                            continue;
                        }
                        String crossId = foreignDevicePO.getCrossId();
                        CrossDirDataRealtimePO po = new CrossDirDataRealtimePO();
                        po.setId(String.join(Constant.MIDDLE_LINE, crossId, dir, "2"));
                        po.setDirType(Integer.parseInt(dir));
                        po.setInOutType(2);
                        po.setCrossId(crossId);
                        po.setLength(0.0);
                        po.setStatus(1);
                        po.setStartTime(dateTime);
                        int flowSum = dataList.stream().filter(po1 -> StringUtils.isNotBlank(po1.getWArrivalFlow())).map(LaneStatisticsDetail::getWArrivalFlow).mapToInt(Integer::parseInt).sum();
                        po.setFlow(flowSum);
                        po.setCapacity(flowSum);
                        po.setDuration(5);
                        OptionalDouble speedAverage = dataList.stream().map(LaneStatisticsDetail::getSpeedAvg).mapToDouble(Double::parseDouble).average();
                        po.setSpeed(0.0);
                        if (speedAverage.isPresent()) {
                            po.setSpeed(speedAverage.getAsDouble());
                        }
                        if (po.getSpeed() > 0) {
                            po.setTrafficIndex(Math.round(60 / po.getSpeed() / dataList.size() * 100.0) / 100.0);
                        } else {
                            po.setTrafficIndex(1.0);
                        }
                        OptionalDouble queueLength = dataList.stream().map(LaneStatisticsDetail::getByQueueLen).mapToDouble(Double::parseDouble).max();
                        po.setQueueLength(0.0);
                        if (queueLength.isPresent()) {
                            po.setQueueLength(queueLength.getAsDouble());
                        }
                        OptionalDouble stopTimeAverage = dataList.stream().map(LaneStatisticsDetail::getByStoppingTimes).mapToDouble(Double::parseDouble).average();
                        po.setStopTimes(0.0);
                        if (stopTimeAverage.isPresent()) {
                            po.setStopTimes(stopTimeAverage.getAsDouble());
                        }
                        OptionalDouble delayTime = dataList.stream().filter(l -> StringUtils.isNotBlank(l.getWDelay())).map(LaneStatisticsDetail::getWDelay).mapToDouble(Double::parseDouble).average();
                        po.setDelayTime(0);
                        if (delayTime.isPresent()) {
                            po.setDelayTime((int) delayTime.getAsDouble());
                        }
                        po.setSturation(0.7);
                        po.setEffusionRate(0.0);
                        po.setNoStopRate(0.0);
                        po.setOneStopRate(0.0);
                        po.setTwoStopRate(0.0);
                        po.setThreeStopRate(0.0);
                        po.setBatchTime(Integer.parseInt(startTime.substring(0, 10)));
                        po.setEffusionRate(0.0);
                        po.setGreenLightEfficiency(0.0);
                        results.add(po);
                    }
                }
            }

            if (!CollectionUtils.isEmpty(results)) {
                Map<String, List<CrossDirDataRealtimePO>> map = new HashMap<>();
                map.put(Constant.REAL_TIME_DIR_DATA_KEY, results);
                out.collect(mapper.writeValueAsString(map));
            }
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2DirDataMap-异常", e);
            throw new RuntimeException(e);
        }
    }

    private ForeignDevicePO getForeignDevicePO(String deviceNo, String dir, Map<String, List<ForeignDevicePO>> foreignDeviceMap) {
        if (!foreignDeviceMap.isEmpty()) {
            List<ForeignDevicePO> foreignDevicePOS = foreignDeviceMap.get(deviceNo);
            if (!CollectionUtils.isEmpty(foreignDevicePOS)) {
                for (ForeignDevicePO foreignDevicePO : foreignDevicePOS) {
                    Integer type = foreignDevicePO.getType();
                    String currentDir = String.valueOf(foreignDevicePO.getDir());
                    String laneId = foreignDevicePO.getLaneId();
                    String sort = laneId.substring(laneId.length() -1);
                    if (Objects.equals(2, type) && StringUtils.equalsIgnoreCase(dir, currentDir)) {
                        return foreignDevicePO;
                    }
                }
            }
        }
        return null;
    }

    @Override
    public void flatMap2(Map<String, List<ForeignDevicePO>> value, Collector<String> out) throws Exception {
        foreignDeviceMap.putAll(value);
    }
}
