package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.exc.MismatchedInputException;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.ForeignDevicePO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossLaneDataRealTimePO;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author duanruiming
 * @date 2024/07/17 13:51
 */
@Service
@Slf4j
public class LaneStatisticsReceive2LaneDataMapVersion2 implements CoFlatMapFunction<String, Map<String, List<ForeignDevicePO>>, String> {

    private static final Map<String, List<ForeignDevicePO>> foreignDeviceMap = new HashMap<>();

    @Override
    public void flatMap1(String value, Collector<String> out) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            List<CrossLaneDataRealTimePO> results = new ArrayList<>();
            String headers = model.getHeaders();
            String body = model.getBody();
            if (StringUtils.isNotBlank(headers) && (headers.contains(Constant.WAN_JI_NAME)
                    || StringUtils.isBlank(body)) || !body.contains("data_list")) {
                return;
            }
            List<LaneStatisticsDataVO> laneStatisticsDataVOS = null;
            try {
                laneStatisticsDataVOS = mapper.readValue(body, new TypeReference<List<LaneStatisticsDataVO>>() {});
            } catch (MismatchedInputException e) {
                log.error("==============:" + body);
            }
            if (!CollectionUtils.isEmpty(laneStatisticsDataVOS)) {
                for (LaneStatisticsDataVO inVo : laneStatisticsDataVOS) {
                    String deviceNo = inVo.getDeviceNo();
                    String startTime = inVo.getData_time();
                    Date dateTime = new Date(Long.parseLong(startTime));
                    String dir = inVo.getSectionNo();
                    List<LaneStatisticsDetail> dataList = inVo.getData_list();
                    if (!CollectionUtils.isEmpty(dataList)) {
                        for (LaneStatisticsDetail detail : dataList) {
                            String laneNo = detail.getLaneNo();
                            ForeignDevicePO foreignDevicePO = getWjLaneId(deviceNo, dir, laneNo, foreignDeviceMap);
                            if (Objects.isNull(foreignDevicePO)) {
                                continue;
                            }
                            CrossLaneDataRealTimePO po = new CrossLaneDataRealTimePO();
                            // todo
                            po.setId(foreignDevicePO.getLaneId());
                            //po.setId(String.join(Constant.MIDDLE_LINE, crossId, dir, laneNo));
                            po.setCrossId(foreignDevicePO.getCrossId());
                            po.setStartTime(dateTime);
                            String wArrivalFlow = detail.getWArrivalFlow();
                            wArrivalFlow = StringUtils.isBlank(wArrivalFlow) ? "0" : wArrivalFlow;
                            po.setFlow((int) Double.parseDouble(wArrivalFlow));
                            po.setSpeed(Double.parseDouble(detail.getSpeedAvg()));
                            po.setInSpeed(Double.parseDouble(detail.getSpeedAvg()));
                            po.setOutSpeed(Double.parseDouble(detail.getSpeedAvg()));
                            po.setQueueLength(Double.parseDouble(detail.getByQueueLen()));
                            po.setStopTimes(Double.parseDouble(detail.getByStoppingTimes()));
                            String wDelay = detail.getWDelay();
                            wDelay = StringUtils.isNotBlank(wDelay) ? wDelay : "0";
                            po.setDelayTime((int) Double.parseDouble(wDelay));
                            po.setCapacity((int) Double.parseDouble(wArrivalFlow));
                            po.setVehheadDist(Double.parseDouble(detail.getGap()));
                            po.setVehheadTime(Double.parseDouble(detail.getHeadway()));
                            po.setQuality(0.6);
                            po.setNoStopRate(0.0);
                            po.setOneStopRate(0.0);
                            po.setTwoStopRate(0.0);
                            po.setThreeStopRate(0.0);
                            po.setSturation(0.7);
                            po.setBatchTime(Integer.parseInt(startTime.substring(0, 10)));
                            po.setEffusionRate(0.0);
                            po.setGreenLightEfficiency(0.0);
                            results.add(po);
                        }
                    }
                }
            }

            if (!CollectionUtils.isEmpty(results)) {
                Map<String, List<CrossLaneDataRealTimePO>> map = new HashMap<>();
                map.put(Constant.REAL_TIME_LANE_DATA_KEY, results);
                out.collect(mapper.writeValueAsString(map));
            }
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2LaneDataMap ", e);
            throw new RuntimeException(e);
        }
    }

    private ForeignDevicePO getWjLaneId(String deviceNo, String dir, String laneNo, Map<String, List<ForeignDevicePO>> foreignDeviceMap) {
        if (!foreignDeviceMap.isEmpty()) {
            List<ForeignDevicePO> foreignDevicePOS = foreignDeviceMap.get(deviceNo);
            if (!CollectionUtils.isEmpty(foreignDevicePOS)) {
                for (ForeignDevicePO foreignDevicePO : foreignDevicePOS) {
                    Integer type = foreignDevicePO.getType();
                    String currentDir = String.valueOf(foreignDevicePO.getDir());
                    String laneId = foreignDevicePO.getLaneId();
                    String sort = laneId.substring(laneId.length() -1);
                    if (Objects.equals(2, type) && StringUtils.equalsIgnoreCase(dir, currentDir)
                            && StringUtils.equalsIgnoreCase(sort, laneNo) ) {
                        return foreignDevicePO;
                    }
                }
            }
        }
        return null;
    }

    @Override
    public void flatMap2(Map<String, List<ForeignDevicePO>> value, Collector<String> out) throws Exception {
        foreignDeviceMap.putAll(value);
    }
}
