package com.wanji.indicators.task.citybrainreceive.laneturnratio;

import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.Properties;

/**
 * @author duanruiming
 * @date 2024/04/10 14:36
 */
@Slf4j
public class LaneTurnRatioReveiveMain {
    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    public static void main(String[] args) throws Exception {

        try {
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

            // 获取配置文件中的kafka消费topic
            String sourceTopic = properties.getProperty("lane.turn.ratio.receive.topic");
            String kafkaServerAddress = properties.getProperty("bootstrap.servers.citybrain.receive");

            KafkaSource<String> source = KafkaSource.<String>builder()
                    .setProperties(instance.getConsumerProperties())
                    .setProperty("auto.offset.commit", "true")
                    .setProperty("auto.commit.interval.ms", "1000")
                    .setProperty("commit.offsets.on.checkpoint", "true")
                    .setBootstrapServers(kafkaServerAddress)
                    .setTopics(sourceTopic)
                    .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-laneTurnRatio-receive")
                    .setStartingOffsets(OffsetsInitializer.latest())
                    .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                    .build();

            DataStream<String> stream = env.fromSource(source, WatermarkStrategy.noWatermarks(),
                    "city-brain-laneTurnRatio-receive-data-source");

            SingleOutputStreamOperator<CommonReceiveModel> frameModelStream = stream
                    .map(new LaneTurnRatioReceiveMap())
                    .name("接收城市大脑车道转向比数据-JsonToObject");

            // 启动 Flink 作业
            env.execute("接收城市大脑车道转向比数据");
        } catch (Exception e) {
            log.error("接收城市大脑车道转向比数据异常", e);
            throw new RuntimeException(e);
        }
    }
}