package com.wanji.indicators.task.citybrainreceive.snapshot;

import com.wanji.indicators.entity.ForeignDevicePO;
import com.wanji.indicators.mapper.ForeignDeviceMapper;
import com.wanji.indicators.util.CityBrainReceiveUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.co.RichCoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author Kent HAN
 * @date 2024/4/17 13:51
 */
public class QueueAreaFlatMap extends RichCoFlatMapFunction<SnapshotModel, AreaDataReceiveModel, SnapshotModel> {

    // key：lanId
    private static final ConcurrentHashMap<String, AreaDataReceiveModel.Body.LaneInfo>
            areaDataMap = new ConcurrentHashMap<>();

    ForeignDeviceMapper foreignDeviceMapper;
    private transient List<ForeignDevicePO> foreignDevicePOList;

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        foreignDeviceMapper = beanConf.getBean(ForeignDeviceMapper.class);
        // 加载静态数据
        foreignDevicePOList = loadStaticDataFromDatabase();
    }

    private List<ForeignDevicePO> loadStaticDataFromDatabase() {
        return foreignDeviceMapper.selectAllDevices();
    }

    @Override
    public void flatMap1(SnapshotModel value, Collector<SnapshotModel> out) throws Exception {
        SnapshotModel.Header headers = value.getHeaders();
        String companyName = headers.getFrom();
        List<SnapshotModel.Body> bodies = value.getBody();

        updateBody(bodies, companyName);
        out.collect(value);
    }

    private void updateBody(List<SnapshotModel.Body> bodies, String companyName) {
        for (SnapshotModel.Body body : bodies) {
            updateDataList(body.getData_list());
        }
    }

    private void updateDataList(List<SnapshotModel.Body.LaneInfo> dataList) {
        for (SnapshotModel.Body.LaneInfo laneInfo : dataList) {
            String laneId = laneInfo.getLaneId();
            AreaDataReceiveModel.Body.LaneInfo areaData = areaDataMap.get(laneId);
            if (areaData != null) {
                laneInfo.setTargetCount(areaData.getTargetCount());
                laneInfo.setOccupancy(areaData.getOccupancy());
                laneInfo.setSpeedAvg(areaData.getSpeedAvg());
                laneInfo.setPareto(areaData.getPareto());
                laneInfo.setSpeedHead(areaData.getSpeedHead());
                laneInfo.setSpeedTail(areaData.getSpeedTail());
                laneInfo.setPosHead(areaData.getPosHead());
                laneInfo.setPosTail(areaData.getPosTail());
            }
        }
    }

    @Override
    public void flatMap2(AreaDataReceiveModel value, Collector<SnapshotModel> out) throws Exception {
        AreaDataReceiveModel.Header headers = value.getHeaders();
        String companyName = headers.getFrom();

        List<AreaDataReceiveModel.Body> bodies = value.getBody();
        for (AreaDataReceiveModel.Body body : bodies) {
            String deviceNo = body.getDeviceNo();
            Integer sectionNo = body.getSectionNo();
            // deviceNo 转万集路口ID
            String crossId = CityBrainReceiveUtil.convertDeviceNo(deviceNo, foreignDevicePOList);
            String laneIdPrefix = companyName + "-" + crossId + "-" + sectionNo;

            List<AreaDataReceiveModel.Body.LaneInfo> dataList = body.getData_list();
            for (AreaDataReceiveModel.Body.LaneInfo laneInfo : dataList) {
                String laneNo = laneInfo.getLaneNo();
                String laneId = laneIdPrefix + "-" + laneNo;
                areaDataMap.put(laneId, laneInfo);
            }
        }

    }
}
