package com.wanji.indicators.task.citybrainreceive.snapshot;

import com.wanji.indicators.entity.ForeignDevicePO;
import com.wanji.indicators.mapper.ForeignDeviceMapper;
import com.wanji.indicators.util.CityBrainReceiveUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.co.RichCoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author Kent HAN
 * @date 2024/4/17 13:51
 */
public class StaticDynamicFlatMap extends RichCoFlatMapFunction<QueueReceiveModel, QueueReceiveModel, SnapshotModel> {

    // key：lanId
    private static final ConcurrentHashMap<String, QueueReceiveModel.Body.LaneInfo>
            dynamicQueueMap = new ConcurrentHashMap<>();

    ForeignDeviceMapper foreignDeviceMapper;
    private transient List<ForeignDevicePO> foreignDevicePOList;

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        foreignDeviceMapper = beanConf.getBean(ForeignDeviceMapper.class);
        // 加载静态数据
        foreignDevicePOList = loadStaticDataFromDatabase();
    }

    private List<ForeignDevicePO> loadStaticDataFromDatabase() {
        return foreignDeviceMapper.selectAllDevices();
    }

    @Override
    public void flatMap1(QueueReceiveModel value, Collector<SnapshotModel> out) throws Exception {

        SnapshotModel snapshotModel = new SnapshotModel();

        QueueReceiveModel.Header headers = value.getHeaders();
        snapshotModel.setHeaders(buildHeaders(headers));

        String companyName = headers.getFrom();
        List<QueueReceiveModel.Body> bodies = value.getBody();
        snapshotModel.setBody(buildBody(bodies, companyName));

        out.collect(snapshotModel);
    }

    private List<SnapshotModel.Body> buildBody(List<QueueReceiveModel.Body> bodies, String companyName) {
        List<SnapshotModel.Body> res = new ArrayList<>();

        for (QueueReceiveModel.Body body : bodies) {
            SnapshotModel.Body bodyElement = new SnapshotModel.Body();
            bodyElement.setData_time(body.getData_time());
            String deviceNo = body.getDeviceNo();
            bodyElement.setDeviceNo(deviceNo);
            bodyElement.setLaneNum(body.getLaneNum());
            Integer sectionNo = body.getSectionNo();
            bodyElement.setSectionNo(sectionNo);

            // deviceNo 转万集路口ID
            String crossId = CityBrainReceiveUtil.convertDeviceNo(deviceNo, foreignDevicePOList);
            String laneIdPrefix = companyName + "-" + crossId + "-" + sectionNo;
            bodyElement.setData_list(buildDataList(body.getData_list(), laneIdPrefix));

            res.add(bodyElement);
        }
        return res;
    }

    private List<SnapshotModel.Body.LaneInfo> buildDataList(
            List<QueueReceiveModel.Body.LaneInfo> dataList, String laneIdPrefix) {

        List<SnapshotModel.Body.LaneInfo> res =  new ArrayList<>();

        for (QueueReceiveModel.Body.LaneInfo laneInfo : dataList) {
            SnapshotModel.Body.LaneInfo laneInfoElement = new SnapshotModel.Body.LaneInfo();

            String laneNo = laneInfo.getLaneNo();
            String laneId = laneIdPrefix + "-" + laneNo;
            laneInfoElement.setLaneId(laneId);

            laneInfoElement.setLaneNo(laneNo);
            laneInfoElement.setStaticQueueLen(laneInfo.getQueueLen());

            QueueReceiveModel.Body.LaneInfo dynamicLaneInfo = dynamicQueueMap.get(laneId);
            if (dynamicLaneInfo != null) {
                laneInfoElement.setDynamicQueueLen(dynamicLaneInfo.getQueueLen());
            } else {
                laneInfoElement.setDynamicQueueLen(0D);
            }

            laneInfoElement.setQueueHead(laneInfo.getQueueHead());
            laneInfoElement.setQueueTail(laneInfo.getQueueTail());
            laneInfoElement.setQueueCount(laneInfo.getQueueCount());
            laneInfoElement.setTargetCount(0);
            laneInfoElement.setOccupancy(0D);
            laneInfoElement.setSpeedAvg(0D);
            laneInfoElement.setPareto(0D);
            laneInfoElement.setSpeedHead(0D);
            laneInfoElement.setSpeedTail(0D);
            laneInfoElement.setPosHead(0D);
            laneInfoElement.setPosTail(0D);

            res.add(laneInfoElement);
        }
        return res;
    }

    private SnapshotModel.Header buildHeaders(QueueReceiveModel.Header headers) {
        SnapshotModel.Header header = new SnapshotModel.Header();
        header.setName(headers.getName());
        header.setFrom(headers.getFrom());
        header.setTaskId(headers.getTaskId());
        return header;
    }

    @Override
    public void flatMap2(QueueReceiveModel value, Collector<SnapshotModel> out) throws Exception {
        QueueReceiveModel.Header headers = value.getHeaders();
        String companyName = headers.getFrom();

        List<QueueReceiveModel.Body> bodies = value.getBody();
        for (QueueReceiveModel.Body body : bodies) {
            String deviceNo = body.getDeviceNo();
            Integer sectionNo = body.getSectionNo();
            // deviceNo 转万集路口ID
            String crossId = CityBrainReceiveUtil.convertDeviceNo(deviceNo, foreignDevicePOList);
            String laneIdPrefix = companyName + "-" + crossId + "-" + sectionNo;

            List<QueueReceiveModel.Body.LaneInfo> dataList = body.getData_list();
            for (QueueReceiveModel.Body.LaneInfo laneInfo : dataList) {
                String laneNo = laneInfo.getLaneNo();
                String laneId = laneIdPrefix + "-" + laneNo;
                dynamicQueueMap.put(laneId, laneInfo);
            }
        }

    }
}
