package com.wanji.indicators.task.citybrainreceive.targetinfo;

import lombok.Data;
import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.annotation.JsonAlias;

import java.util.List;

@Data
public class TargetInfoReceiveModel {
    private long dataTime;
    private String key = "global";
    private Header headers;
    private List<Body> body;

    // Header内部类
    @Data
    public static class Header {
        private String name;
        private String from;
        private String taskId;
    }

    // Body内部类
    @Data
    public static class Body {
        // 报警设备IPv4地址
        private String ipAddress;

        // MAC地址
        private String macAddress;

        // 断面编号，南向北 1，东向西 2，北向南 3，西向东 4
        private String sectionNo;

        // 车道编号，与车辆正常行驶方向的垂直方向从左向右依次编号
        private Integer laneNo;

        // 数据时间, 毫秒级时间戳
        @JsonAlias({"data_time", "d_time"})
        private Long data_time;

        // 设备编号
        private String deviceNo;

        // 目标车辆编号
        private Integer id;

        // 目标车辆长度(m)，可选
        private Double length;

        // 目标宽度(m)，可选
        private Double width;

        // 目标高度(m)，可选
        private Double height;

        // 目标车辆X坐标(m)，可选
        private Double posX;

        // 目标车辆Y坐标(m)，可选
        private Double posY;

        // 目标车辆Z坐标(m)，可选
        private Double posZ;

        // 目标即时速度(km/h)，可选
        private Double speed;

        // 目标车辆X方向速度(km/h)，可选
        private Double speedX;

        // 目标车辆Y方向速度(km/h)，可选
        private Double speedY;

        // 目标Z方向速度(km/h)，可选
        private Double speedZ;

        // 目标类型，可选
        private Integer type;

        // 目标运动方向加速度，可选
        private Double acs;

        // 目标航向角，可选
        private Double angle;

        // 目标点海拔，可选
        private Double altitude;

        // 目标坐标点经度，可选
        private Double longitude;

        // 目标坐标点纬度，可选
        private Double latitude;

        // 目标散射截面积，可选
        private Double rcs;

        // 置信度，可选
        private Double trust;

        // 车牌，可选
        private String brand;

        // 车牌类型，可选
        @JsonAlias({"PlateType", "plateType"})
        private String plateType;

        // 车辆类型，可选
        private String carType;

        // 车辆颜色，可选
        private String carColor;

        // 车辆品牌，可选
        private String vehicleLogo;
    }
}
