package com.wanji.indicators.task.crossstate.stream;

import com.wanji.indicators.entity.CrossStateData;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.crossstate.service.func.CrossStateFromJsonFlatMap;
import com.wanji.indicators.task.crossstate.service.sink.CrossStateSinkFunction;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.List;
import java.util.Properties;
@Slf4j
public class CrossStateDataSynchronizeMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final String GROUP_ID = "CrossStateDataSynchronizeMain";

    public static void main(String[] args) {

        ParameterTool parameter = ParameterTool.fromArgs(args);
        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        String topic = "cross_real_time_data";

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("enable.auto.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("auto.offset.reset", "earliest")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(GROUP_ID)
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.EARLIEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        try {

            DataStream<String> stream = env
                    .fromSource(source, WatermarkStrategy.noWatermarks(), "CrossStateDataSynchronizeMain");

            SingleOutputStreamOperator<List<CrossStateData>> frameModelStream =
                    stream
                            .flatMap(new CrossStateFromJsonFlatMap())
                            .setParallelism(1)
                            .name("路口状态数据-JsonToObject");

            frameModelStream.addSink(new CrossStateSinkFunction());

            env.execute("路口状态数据同步到Mysql-工作流");

        } catch (Exception e) {
            log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
