package com.wanji.indicators.task.freeflow.service.func;

import com.wanji.indicators.task.freeflow.service.model.FrameMaxSpeedModel;
import com.wanji.indicators.task.freeflow.service.model.MaxSpeedResultModel;
import org.apache.flink.api.common.functions.AggregateFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public   class SpeedAggregateFunction implements AggregateFunction<FrameMaxSpeedModel,MaxSpeedResultModel,MaxSpeedResultModel> {

    private static final Logger log = LoggerFactory.getLogger(SpeedAggregateFunction.class);

    @Override
    public MaxSpeedResultModel createAccumulator() {
        MaxSpeedResultModel initModel = new MaxSpeedResultModel();
        initModel.setAvgSpeed(0D);
        initModel.setMaxSpeed(0d);
        initModel.setSumSpeed(0D);
        initModel.setCount(0);
        return initModel;
    }
    /**
     * 每进入一个数据就会执行一次
     * @param value 当前进入的数据
     * @param accumulator 之前计算好的中间结果
     * @return
     */
    @Override
    public MaxSpeedResultModel add(FrameMaxSpeedModel value, MaxSpeedResultModel accumulator) {
        accumulator.setMaxSpeed(value.getSpeed()>accumulator.getMaxSpeed()?value.getSpeed():accumulator.getMaxSpeed());
        accumulator.setAvgSpeed((value.getSpeed()+accumulator.getAvgSpeed())/2);
        accumulator.setSumSpeed(value.getSpeed()+accumulator.getSumSpeed());
        accumulator.setCount(accumulator.getCount()+1);
        //System.out.println("avgSpeed:"+accumulator.getAvgSpeed()+":"+accumulator.getSumSpeed()/accumulator.getCount()+" maxSpeed:"+accumulator.getMaxSpeed());

        return accumulator;
    }
    /*
       当window的结束时间到达时，触发这个方法，返回结果
        */
    @Override
    public MaxSpeedResultModel getResult(MaxSpeedResultModel accumulator) {
        //log.info("AggResult:"+accumulator);
        return accumulator;
    }
    /**
     * 在session窗口才会用到merge，时间窗口其实用不到
     * @param a
     * @param b
     * @return
     */
    @Override
    public MaxSpeedResultModel merge(MaxSpeedResultModel a, MaxSpeedResultModel b) {
        return null;
    }
}
