package com.wanji.indicators.task.laneIndicators;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.event.stream.func.RefBaseRidInfoCoFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.task.laneIndicators.func.LaneDataProcessWindow;
import com.wanji.indicators.task.laneIndicators.model.CrossLaneResultModel;
import com.wanji.indicators.task.laneIndicators.model.LaneResultLineModel;
import com.wanji.indicators.task.laneIndicators.model.LaneResultModel;
import com.wanji.indicators.task.turnIndicators.func.TurnDataProcessWindowByAngle;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultLineModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.connector.jdbc.JdbcConnectionOptions;
import org.apache.flink.connector.jdbc.JdbcExecutionOptions;
import org.apache.flink.connector.jdbc.JdbcSink;
import org.apache.flink.connector.jdbc.JdbcStatementBuilder;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.util.Collector;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Properties;

@Slf4j
public class CrossLaneIndicatorMainNew implements Serializable {
    private transient StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties properties = new Properties();

    private String path;

    private String indexName;

    JdbcExecutionOptions jdbcExecutionOptions = null;
    JdbcConnectionOptions jdbcBuild = null;

    private CrossLaneIndicatorMainNew(StreamExecutionEnvironment env, String indexName) {
        this.env = env;
        this.indexName = indexName;
        this.instance = PropertiesHelper.getInstance();
        this.properties = this.instance.getProperties();
    }


    public static CrossLaneIndicatorMainNew init(StreamExecutionEnvironment env, String indexName) {
        return new CrossLaneIndicatorMainNew(env, indexName);
    }

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private CrossLaneIndicatorMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
          jdbcBuild = new JdbcConnectionOptions.JdbcConnectionOptionsBuilder()
                .withUrl(properties.getProperty("jdbc.url"))
                .withUsername(properties.getProperty("jdbc.username"))
                .withPassword(properties.getProperty("jdbc.password"))
                .withDriverName(properties.getProperty("jdbc.driver"))
                .withConnectionCheckTimeoutSeconds(60)
                .build();
        jdbcExecutionOptions = JdbcExecutionOptions.builder()
                .withMaxRetries(3) // 异常重试次数
                .withBatchSize(1) // 批量大小设置为1，表示一条数据都会写入mysql
                .withBatchIntervalMs(0) // 批次的间隔为0，表示关闭定时写入
                .build();
    }

    public static CrossLaneIndicatorMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new CrossLaneIndicatorMainNew(env, path, isDebug);
    }


    public void run(SingleOutputStreamOperator<CrossFrameModel> carTrackModelStream) {

        try {

            SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = carTrackModelStream
                    .flatMap(new RefBaseRidInfoCoFlatMap())
                    .name("全域轨迹帧数据-车道属性数据绑定");

            SingleOutputStreamOperator<CrossFrameModel> filterStream =
                    joinLaneStream.flatMap(new FlatMapFunction<CrossFrameModel, CrossFrameModel>() {
                        @Override
                        public void flatMap(CrossFrameModel value, Collector<CrossFrameModel> out) throws Exception {
                            List<CarTrackModel> trackList = value.getTrackList();
                            List<CarTrackModel> trackFilterList = new ArrayList<>();
                            for (CarTrackModel model : trackList) {
                                boolean isMotor = VehicleTypeEnum.isMotorVehicles(model.getOriginalType().shortValue());
                                int inCrossFlag = model.getRoadnet().getInCrossFlag();
                                if (isMotor && !Objects.equals("null", model.getRoadnet().getCrossId())) {
                                    //筛选路口内车辆、进入到出口道上的车辆
                                    if (Objects.equals(model.getRoadnet().getLaneType(), 2)) {
                                        trackFilterList.add(model);
                                    }
                                }

                            }
                            value.setTrackList(trackFilterList);
                            if (!trackFilterList.isEmpty()) {
                                out.collect(value);
                            }

                            //log.info("路口ID:{}, 数据帧时间：{}，路口内机动车数量：{}",value.getCrossId(), DateUtil.toDateTime(value.getGlobalTimeStamp(),"yyyy-MM-dd HH:mm:ss.SSS"),trackFilterList.size());
                        }
                    }).name("筛选在路口内的机动车");


            SingleOutputStreamOperator<CrossLaneResultModel> laneStream = filterStream
                    .assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CrossFrameModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                    .keyBy(o -> o.getCrossId())
                    .window(TumblingEventTimeWindows.of(Time.minutes(5)))
                    .process(new LaneDataProcessWindow())
                    .name("车道指标计算");

            SingleOutputStreamOperator<CrossTurnResultModel> turnStream = filterStream
                    .assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CrossFrameModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                    .keyBy(o -> o.getCrossId())
                    .window(TumblingEventTimeWindows.of(Time.minutes(5)))
                    .process(new TurnDataProcessWindowByAngle())
                    .name("转向指标计算");


//            SingleOutputStreamOperator<String> toJsonStream = laneStream.flatMap(new FlatMapFunction<CrossLaneResultModel, String>() {
//                @Override
//                public void flatMap(CrossLaneResultModel value, Collector<String> out) throws Exception {
//                    out.collect(JSONObject.toJSONString(value));
//                }
//            }).name("车道指标转JSON");


            if (!isDebug) {
//                KafkaSink<String> sink = KafkaSink.<String>builder()
//                        .setBootstrapServers(properties.getProperty("bootstrap.servers"))
//                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
//                                .setTopic("analysis.lane.indicators")
//                                .setValueSerializationSchema(new SimpleStringSchema())
//                                .build()
//                        )
//                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
//                        .build();
//                toJsonStream.sinkTo(sink).name("车道指标-数据发送至kafka");

                this.laneFlowToMysql(jdbcExecutionOptions,jdbcBuild,laneStream);
                this.turnFlowToMysql(jdbcExecutionOptions,jdbcBuild,turnStream);
            } else {

                SingleOutputStreamOperator<String> turnCsvStream = turnStream.flatMap(new FlatMapFunction<CrossTurnResultModel, String>() {
                    @Override
                    public void flatMap(CrossTurnResultModel value, Collector<String> out) throws Exception {
                        List<TurnResultModel> list = value.getTurnDataList();
                        for (TurnResultModel model : list) {
                            String line = value.getCrossId() + ","+value.getCrossName()+"," + model.getInRid() + "," + model.getInRidDir() + "," + model.getOutRid() + "," +
                                    model.getTurnDirNo() + "," + model.getFlow() + ",5m," + value.getStartTime() + "," + value.getEndTime() + "," +
                                    DateUtil.dateFormat(value.getStartTime(), "yyyyMMdd") + "," + DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss");
                            out.collect(line);
                        }
                    }
                }).name("转向指标转JSON");
                String tpath = this.path + "analysis.turn.indicators.csv";
                turnCsvStream.addSink(new JsonStringFileSink(tpath)).setParallelism(1).name("转向指标-结果输出至文件");

            }
        } catch (Exception e) {
            log.error("机动车指标计算任务异常 :   " + e);
        }

    }

    private void laneFlowToMysql(JdbcExecutionOptions jdbcExecutionOptions, JdbcConnectionOptions jdbcBuild, SingleOutputStreamOperator<CrossLaneResultModel> laneCsvStream) {
        laneCsvStream.addSink(JdbcSink.sink(
                "insert into t_analysis_rid_lane_flow (cross_id,cross_name,rid,rid_dir,lane_id,lane_no,turn_dir_no,flow,granularity,window_start_time,window_end_time,dt,ruksj)" +
                        " values(?,?,?,?,?,?,?,?,?,?,?,?,?) " +
                        "on duplicate key update " +
                        "cross_id = values(cross_id)," +
                        "cross_name=values(cross_name)," +
                        "rid=values(rid)," +
                        "rid_dir=values(rid_dir)," +
                        "lane_id=values(lane_id)," +
                        "lane_no=values(lane_no)," +
                        "turn_dir_no=values(turn_dir_no)," +
                        "flow=values(flow)," +
                        "granularity=values(granularity)," +
                        "window_start_time=values(window_start_time)," +
                        "window_end_time=values(window_end_time)," +
                        "dt=values(dt)," +
                        "ruksj=values(ruksj)"
                ,
                new JdbcStatementBuilder<CrossLaneResultModel>() {
                    @Override
                    public void accept(PreparedStatement preparedStatement, CrossLaneResultModel resultModel) throws SQLException {
                        List<LaneResultModel> list = resultModel.getLaneDataList();
                        for (LaneResultModel model : list) {
                            preparedStatement.setString(1, resultModel.getCrossId());
                            preparedStatement.setString(2, resultModel.getCrossName());
                            preparedStatement.setString(3, model.getInRid());
                            preparedStatement.setInt(4, model.getInRidDir());
                            preparedStatement.setString(5, model.getLaneId());
                            preparedStatement.setString(6, model.getLaneNo());
                            preparedStatement.setInt(7, model.getTurnDirNo());
                            preparedStatement.setInt(8, model.getFlow());
                            preparedStatement.setString(9, "5m");
                            preparedStatement.setTimestamp(10, new Timestamp(DateUtil.StringToMillis(resultModel.getStartTime())));
                            preparedStatement.setTimestamp(11, new Timestamp(DateUtil.StringToMillis(resultModel.getEndTime())));
                            try {
                                preparedStatement.setInt(12, Integer.valueOf(DateUtil.dateFormat(resultModel.getStartTime(), "yyyyMMdd")));
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                            preparedStatement.setTimestamp(13, new Timestamp(System.currentTimeMillis()));
                            preparedStatement.addBatch();
                        }
                    }
                },
                jdbcExecutionOptions,
                jdbcBuild
        )).setParallelism(1).name("lane flow to mysql");
    }


    private void turnFlowToMysql(JdbcExecutionOptions jdbcExecutionOptions, JdbcConnectionOptions jdbcBuild, SingleOutputStreamOperator<CrossTurnResultModel> turnCsvStream) {
        turnCsvStream.addSink(JdbcSink.sink(
                "insert into t_analysis_rid_turn_flow (" +
                        "cross_id,cross_name,f_rid,f_rid_dir,t_rid,turn_dir_no,flow,granularity,window_start_time,window_end_time,dt,ruksj" +
                        ")" +
                        " values(?,?,?,?,?,?,?,?,?,?,?,?) " +
                        "on duplicate key update cross_id = values(cross_id)," +
                        "cross_name=values(cross_name)," +
                        "f_rid=values(f_rid)," +
                        "f_rid_dir=values(f_rid_dir)," +
                        "t_rid=values(t_rid)," +
                        "turn_dir_no=values(turn_dir_no)," +
                        "flow=values(flow)," +
                        "granularity=values(granularity)," +
                        "window_start_time=values(window_start_time)," +
                        "window_end_time=values(window_end_time)," +
                        "dt=values(dt)," +
                        "ruksj=values(ruksj)"
                ,
                new JdbcStatementBuilder<CrossTurnResultModel>() {
                    @Override
                    public void accept(PreparedStatement preparedStatement, CrossTurnResultModel resultModel) throws SQLException {
                        List<TurnResultModel> list = resultModel.getTurnDataList();
                        for (TurnResultModel model : list) {
                            preparedStatement.setString(1, resultModel.getCrossId());
                            preparedStatement.setString(2, resultModel.getCrossName());
                            preparedStatement.setString(3, model.getInRid());
                            preparedStatement.setInt(4, model.getInRidDir());
                            preparedStatement.setString(5, model.getOutRid());
                            preparedStatement.setInt(6, model.getTurnDirNo());
                            preparedStatement.setInt(7, model.getFlow());
                            preparedStatement.setString(8, "5m");
                            preparedStatement.setTimestamp(9, new Timestamp(DateUtil.StringToMillis(resultModel.getStartTime())));
                            preparedStatement.setTimestamp(10, new Timestamp(DateUtil.StringToMillis(resultModel.getEndTime())));
                            try {
                                preparedStatement.setInt(11, Integer.valueOf(DateUtil.dateFormat(resultModel.getStartTime(), "yyyyMMdd")));
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                            preparedStatement.setTimestamp(12, new Timestamp(System.currentTimeMillis()));
                            preparedStatement.addBatch();
                        }
                    }
                },
                jdbcExecutionOptions,
                jdbcBuild
        )).setParallelism(1).name("turn flow to mysql");
    }


}
