package com.wanji.indicators.task.laneIndicators.func;

import com.wanji.indicators.constant.TimeGranularityEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.task.beihang.BaseGisLaneCache;
import com.wanji.indicators.task.laneIndicators.model.CrossLaneResultModel;
import com.wanji.indicators.task.laneIndicators.model.LaneResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class LaneDataProcessWindow extends ProcessWindowFunction<CrossFrameModel, CrossLaneResultModel, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(LaneDataProcessWindow.class);


    private transient MapState<String, Long> distinctIdState;

    @Override
    public void open(Configuration parameters) throws Exception {
        StateTtlConfig ttlConfig = StateTtlConfig.newBuilder(Time.hours(12))//状态过期时间
                .setUpdateType(StateTtlConfig.UpdateType.OnReadAndWrite)
                .cleanupFullSnapshot()
                .setStateVisibility(StateTtlConfig.StateVisibility.ReturnExpiredIfNotCleanedUp)//不返回过期值
                .build();
        MapStateDescriptor mapStateDescriptor = new MapStateDescriptor<>("lane_distinct_id",String.class,Long.class);
        mapStateDescriptor.enableTimeToLive(ttlConfig);
        distinctIdState = getRuntimeContext().getMapState(mapStateDescriptor);
    }

    @Override
    public void process(String groupKey, Context context, Iterable<CrossFrameModel> elements, Collector<CrossLaneResultModel> out) throws Exception {
        List<CrossFrameModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        List<CarTrackModel> trackList = new ArrayList<>();
        for (CrossFrameModel crossFrameModel : list) {
            trackList.addAll(crossFrameModel.getTrackList());
        }
        Map<String, List<CarTrackModel>> groupById = trackList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getLaneId()));
        //=====================================================================================//
        long windowStartTime = context.window().getStart();
        long windowEndTime = context.window().getEnd();
        String startTime = DateUtil.toDateTime(windowStartTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        String endTime = DateUtil.toDateTime(windowEndTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        long globalTime = windowEndTime;

        Map<String, LaneResultModel> turnResultModelMap = new HashMap<>();

        for (Map.Entry<String, List<CarTrackModel>> entry : groupById.entrySet()) {

            List<CarTrackModel> value = entry.getValue();

            /***************************计算过停止线车辆数************************************************************/
            long count = 0;//value.stream().map(o->o.getId()).distinct().count();
            Map<Integer, List<CarTrackModel>> groupByTrackId = value.stream().collect(Collectors.groupingBy(o -> o.getId()));
            for (Map.Entry<Integer, List<CarTrackModel>> entry1 : groupByTrackId.entrySet()) {
                List<CarTrackModel> value1 = entry1.getValue();
                value1 = value1.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());

                double laneLength = value1.get(0).getRoadnet().getLaneLength();

                for (CarTrackModel carTrackModel : value1) {

                    String laneStartPoint = carTrackModel.getRoadnet().getLaneStartPoint();
                    String carPoint = carTrackModel.getLongitude() + "," + carTrackModel.getLatitude();
                    double distance = GeomsConvertUtil.getDistance(laneStartPoint, carPoint);

                    String key = entry.getKey()+"_"+entry1.getKey();

                    if (distance >= (laneLength+1) && !distinctIdState.contains(key)) {
                        //过停止线开始计数
                        count++;
                        distinctIdState.put(key, carTrackModel.getGlobalTimeStamp());
                        break;
                    }
                }
            }
            /******************************************************************************************************/

            CarTrackModel carTrackModel = value.get(0);

            LaneResultModel resultModel = turnResultModelMap.get(carTrackModel.getRoadnet().getLaneId());
            if (resultModel == null) {
                resultModel = new LaneResultModel();
            }
            resultModel.setLaneId(carTrackModel.getRoadnet().getLaneId());
            resultModel.setLaneNo(carTrackModel.getRoadnet().getLaneNo() + "");
            resultModel.setInRid(carTrackModel.getRoadnet().getRid());
            resultModel.setInRidDir(carTrackModel.getRoadnet().getRidDir8());
            resultModel.setTurnDirNo(carTrackModel.getRoadnet().getTurn());
            resultModel.setFlow((int) count);

            turnResultModelMap.put(carTrackModel.getRoadnet().getLaneId(), resultModel);

        }
        //输出流量
        List<LaneResultModel> outList = new ArrayList<>(turnResultModelMap.values());
        if (!list.isEmpty()) {
            CrossLaneResultModel crossTurnResultModel = new CrossLaneResultModel();
            crossTurnResultModel.setStartTime(startTime);
            crossTurnResultModel.setEndTime(endTime);
            crossTurnResultModel.setTimestamp(context.window().getStart());
            crossTurnResultModel.setCrossId(groupKey);
            crossTurnResultModel.setLaneDataList(outList);
            crossTurnResultModel.setGranularity(TimeGranularityEnum.FIVE_MINUTE.getType());
            String name = BaseGisLaneCache.getInstance().getCrossNameRefMap().get(groupKey);
            crossTurnResultModel.setCrossName(name);

            out.collect(crossTurnResultModel);
        }
        //状态过期清理
        Set<String> expireKeys = new HashSet<>();
        Iterator<Map.Entry<String,Long>> iterator =  distinctIdState.iterator();
        while (iterator.hasNext()) {
            Map.Entry<String,Long> entry = iterator.next();
            long timestamp  = entry.getValue();
            if (globalTime - timestamp > 5 * 60 * 1000) {
                //缓存超过
                expireKeys.add(entry.getKey());
            }
        }
        if (!expireKeys.isEmpty()) {
            for (String key : expireKeys) {
                distinctIdState.remove(key);
            }
        }
    }
}
