package com.wanji.indicators.task.lightstatus;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Slf4j
public class CrossLightStatusInfoCoFlatMap implements CoFlatMapFunction<CrossFrameModel, String, CrossFrameModel> {

    String lightStatusJson = null;

    @Override
    public void flatMap1(CrossFrameModel value, Collector<CrossFrameModel> out) throws Exception {
        if (lightStatusJson != null) {
            //解析灯态json
            JSONArray jsonArray = JSON.parseArray(lightStatusJson);
            int size = jsonArray.size();

            Map<String, BaseLaneInfo> laneInfoMap = BaseLaneCache.getInstance().getBaseLaneInfoMap();

            List<CrossRidTurnLampStatusModel> list = new ArrayList<>();

            for (int i = 0; i < size; i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                String crossId = jsonObject.getString("crossId");
                long globalTime = value.getGlobalTimeStamp()  ;
                long lightStatusTime = jsonObject.getLong("timeStamp")  ;
                long difTime = globalTime - lightStatusTime;
                if (globalTime != lightStatusTime) {
                    log.info("轨迹时间：{}，信号时间：{}", DateUtil.toDateTime(globalTime,DateUtil.YYYY_MM_DD_HH_MM_SS), DateUtil.toDateTime(lightStatusTime,DateUtil.YYYY_MM_DD_HH_MM_SS));
                }
                if (!Objects.equals(value.getCrossId(), crossId)) {
                    continue;
                }

                Integer cycleCountDown = jsonObject.getInteger("cycleCountDown");
                Integer cyclePhaseCountDown = jsonObject.getInteger("cyclePhaseCountDown");
                Integer cycleLen = jsonObject.getInteger("cycleLen");

                List<BaseLaneInfo> laneList = laneInfoMap.values().stream().filter(o -> o.getCrossId().equals(crossId)).collect(Collectors.toList());


                JSONArray jsonArray1 = jsonObject.getJSONArray("dirLampGroupMapList");
                for (int k = 0; k < jsonArray1.size(); k++) {
                    JSONObject jsonObject2 = jsonArray1.getJSONObject(k);
                    Integer dir = jsonObject2.getIntValue("dir");
                    JSONArray jsonArray2 = jsonObject2.getJSONArray("turnList");
                    for (int m = 0; m < jsonArray2.size(); m++) {
                        JSONObject jsonObject3 = jsonArray2.getJSONObject(m);
                        String status = jsonObject3.getString("color");
                        String turn = jsonObject3.getString("turn");
                        AtomicInteger atomicInteger = new AtomicInteger(dir);
                        Optional<BaseLaneInfo> optional = laneList.stream().filter(o -> o.getType().equals(2) && o.getDir() == atomicInteger.get()).findAny();
                        //灯态信息
                        CrossRidTurnLampStatusModel statusModel = new CrossRidTurnLampStatusModel();
                        statusModel.setCrossId(crossId);
                        statusModel.setDir(dir);
                        if (optional.isPresent()) {
                            statusModel.setRid(optional.get().getRid());
                        }
                        statusModel.setCycleCountDown(cycleCountDown);
                        statusModel.setCycleLen(cycleLen);
                        statusModel.setCyclePhaseCountDown(cyclePhaseCountDown);
                        statusModel.setLampState(status);
                        statusModel.setLightStatusTime(lightStatusTime);
                        statusModel.setTurn(turn);
                        list.add(statusModel);
                    }
                }

            }
            //绑定灯态
            value.setRidTurnLampList(list);
            out.collect(value);
        }
    }

    @Override
    public void flatMap2(String value, Collector<CrossFrameModel> out) throws Exception {
        lightStatusJson = value;
    }
}
