package com.wanji.indicators.task.lightstatus;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.LightStatusEnum;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.event.cross.conflictpoint.func.ConflictLaneInfoCoFlatMap;
import com.wanji.indicators.event.stream.TrafficEventMain;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.lightstatus.model.VehicleHeadTimeModel;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.RidTrafficStateMain;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
import org.apache.flink.streaming.api.datastream.ConnectedStreams;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;

import java.util.*;
import java.util.stream.Collectors;

public class TrackLightStatusMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final String GROUP_ID = "FlinkConsumerGroup-V56-TRACK";

    public static void main(String[] args) {

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();
        String topic = properties.getProperty("consumer.topic");
        String groupId  =GROUP_ID;
        String groupChangeFlag = properties.getProperty("consumer.group.change.flag");
        if (Boolean.parseBoolean(groupChangeFlag)) {
            groupId += DateUtil.toDateTime(System.currentTimeMillis(),"yyyyMMdd");
        }
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        env.enableCheckpointing(2 * 60 * 1000);
        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);
        env.getCheckpointConfig().setCheckpointTimeout(10 * 60 * 1000);
       // env.getCheckpointConfig().setMaxConcurrentCheckpoints(10);
        env.getCheckpointConfig().setCheckpointStorage(new FileSystemCheckpointStorage(properties.getProperty("check.point.uri")));
        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.DELETE_ON_CANCELLATION);
        //============================================================================================//
        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(groupId + "_car_track")
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();
        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-ped");
        //============================================================================================//
        KafkaSource<String> lightStatusSource = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics("cross_lights_status")
                .setGroupId(groupId + "_cross_lights_status")
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();
        DataStream<String> lightStatusStream = env
                .fromSource(lightStatusSource, WatermarkStrategy.noWatermarks(), "cross_lights_status-data-source");

        //============================================================================================//

        SingleOutputStreamOperator<CrossFrameModel> thinningDataStream =
                stream.flatMap(new CrossFrameFlatMap()).setParallelism(1).name("全域轨迹帧数据-按路口分组输出");

        //连接灯态数据源
        ConnectedStreams<CrossFrameModel, String> connStream = thinningDataStream.connect(lightStatusStream);

        SingleOutputStreamOperator<CrossFrameModel> bindLightStatusStream =
                connStream.flatMap(new CrossLightStatusInfoCoFlatMap()).setParallelism(1).name("全域轨迹帧数据-路口灯态绑定");

        SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = bindLightStatusStream
                .connect(env.addSource(new LaneInfoSource()).broadcast())
                .flatMap(new BindCrossLaneInfoCoFlatMap())
                .setParallelism(1)
                .name("全域轨迹帧数据-车道属性数据绑定");

        SingleOutputStreamOperator<VehicleHeadTimeModel> midResultStream =
                joinLaneStream
                        .keyBy(o -> o.getCrossId())
                        .process(new LaneCarHeadTimeProcessFunction())
                        .name("绿灯时间内，进口道转向车道空当时间计算");

        try {

            midResultStream.print("test-结果").name("结果打印");

            env.execute("test-工作流");
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
