package com.wanji.indicators.task.motor;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class MotorCarProcessWindow extends ProcessWindowFunction<CrossFrameModel, MotorEfficiencyResultModel, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(MotorCarProcessWindow.class);
    //减速度阈值
    private double deceleration = -2;
    //减速度持续时长阈值 单位秒
    private int decelerateDuration = 1;
    //运行不流畅次数阈值
    private int unsmoothCount = 2;

    @Override
    public void open(Configuration parameters) {
        //mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_Area_Indicators", String.class, CarTrackModel.class));
    }

    @Override
    public void process(String tupleKey, Context context, Iterable<CrossFrameModel> elements, Collector<MotorEfficiencyResultModel> out) throws Exception {
        List<CrossFrameModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        List<CarTrackModel> trackList = new ArrayList<>();
        for (CrossFrameModel crossFrameModel : list) {
            trackList.addAll(crossFrameModel.getTrackList());
        }
        Map<Integer, List<CarTrackModel>> groupById = trackList.stream().collect(Collectors.groupingBy(o -> o.getId()));
        //=====================================================================================//
        long windowStartTime = context.window().getStart();
        long windowEndTime = context.window().getEnd();
        String startTime = DateUtil.toDateTime(windowStartTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        String endTime = DateUtil.toDateTime(windowEndTime, DateUtil.YYYY_MM_DD_HH_MM_SS);

        for (Map.Entry<Integer, List<CarTrackModel>> entry : groupById.entrySet()) {
            List<CarTrackModel> value = entry.getValue();
            value = value.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());

            MotorEfficiencyResultModel slowSpeedCar = this.getSlowSpeedCar(value);
            if (slowSpeedCar != null) {
                slowSpeedCar.setStartTime(startTime);
                slowSpeedCar.setEndTime(endTime);
                slowSpeedCar.setTimestamp(windowEndTime);
                out.collect(slowSpeedCar);
            }
            MotorEfficiencyResultModel notSmoothCar = this.getNotSmoothCar(value);
            if (notSmoothCar != null) {
                notSmoothCar.setStartTime(startTime);
                notSmoothCar.setEndTime(endTime);
                notSmoothCar.setTimestamp(windowEndTime);
                out.collect(notSmoothCar);
            }
        }
    }

    private MotorEfficiencyResultModel getSlowSpeedCar(List<CarTrackModel> list) {
        //记录减速度持续时长，单位秒
        long atDuration = 0;
        //标记是否发生过持续减速
        boolean isSlowSpeedCar = false;
        long stime = 0;
        //按秒分组
        Map<Long, List<CarTrackModel>> groupBySecond = new TreeMap<>();
        for (CarTrackModel carTrackModel : list) {
            long globalSecondTime = carTrackModel.getGlobalTimeStamp() / 1000;
            List<CarTrackModel> valueList = groupBySecond.get(globalSecondTime);
            if (valueList == null) {
                valueList = new ArrayList<>();
            }
            valueList.add(carTrackModel);
            groupBySecond.put(globalSecondTime, valueList);
        }
        String happenStartTime = null;
        String happenEndTime = null;
        Map.Entry<Long, List<CarTrackModel>> preEntry = null;
        for (Map.Entry<Long, List<CarTrackModel>> entry : groupBySecond.entrySet()) {
            Long second = entry.getKey();
            List<CarTrackModel> nowValue = entry.getValue();

            if (preEntry != null) {
                List<CarTrackModel> preValue = preEntry.getValue();
                CarTrackModel firstObj = nowValue.get(0);
                double nowSpeed = 0;
                double preSpeed = 0;
                //1秒内最后一刻速度
                if (!nowValue.isEmpty()) {
                    nowSpeed = nowValue.get(nowValue.size() - 1).getSpeed();
                }
                //上一秒开始时刻速度
                if (!preValue.isEmpty()) {
                    preSpeed = preValue.get(0).getSpeed();
                }
                double deltSpeed = nowSpeed - preSpeed;
                long deltTime = second - preEntry.getKey();
                if (deltTime > 0) {
                    //计算减速度
                    double at = (deltSpeed / 3.6) / deltTime;
                    at = ArithOfBigDecmial.round(at,0);
                    if (at < deceleration && !isSlowSpeedCar) {
                        atDuration += deltTime;
                        if (atDuration > decelerateDuration) {
                            isSlowSpeedCar = true;
                            stime = firstObj.getGlobalTimeStamp() - atDuration * 1000;
                            happenStartTime = DateUtil.toDateTime(stime, DateUtil.YYYY_MM_DD_HH_MM_SS);
                            happenEndTime = DateUtil.toDateTime(firstObj.getGlobalTimeStamp(), DateUtil.YYYY_MM_DD_HH_MM_SS);
                            log.info("路口ID:{}, 车辆ID：{},车牌号：{} 被判定为减速度车，减速度为：{}，开始时间：{},结束时间：{}",
                                    firstObj.getRoadnet().getCrossId(), firstObj.getId(), firstObj.getPicLicense(), at,
                                    happenStartTime,
                                    happenEndTime
                            );
                            break;
                        } else {
                            //条件消散
                            isSlowSpeedCar = false;
                            atDuration = 0;
                        }
                    }
                }
            }
            preEntry = entry;
        }
        if (isSlowSpeedCar) {
            MotorEfficiencyResultModel model = new MotorEfficiencyResultModel();
            model.setCrossId(list.get(0).getRoadnet().getCrossId());
            model.setOrgCode(list.get(0).getOrgCode());
            model.setPicLicense(list.get(0).getPicLicense());
            model.setTrackId(list.get(0).getId());
            model.setSlowSpeedCar(isSlowSpeedCar);
            model.setSlowSpeedDuration(atDuration);
            model.setHappenStartTime(happenStartTime);
            model.setHappenEndTime(happenEndTime);
            return model;
        }
        return null;
    }

    //运行不流畅车辆
    private MotorEfficiencyResultModel getNotSmoothCar(List<CarTrackModel> list) {
        double angle1 = -1;
        double angle2 = -1;
        double difAngle1 = -1;
        double difAngle2 = 0;
        //运行不通畅次数
        int count = 0;
        //刚进入路口的第一帧
        CarTrackModel tmp = list.get(0);
        String happenStartTime = null;
        String happenEndTime = null;

        int i = 0;
        for (CarTrackModel carTrackModel : list) {

            double  difAngle = carTrackModel.getCourseAngle() - tmp.getCourseAngle();
            long difTime = carTrackModel.getGlobalTimeStamp() - tmp.getGlobalTimeStamp();
            String rid = carTrackModel.getRoadnet().getRid();
            String tmpRid = tmp.getRoadnet().getRid();

            if (Objects.equals(rid, tmpRid) && difTime < 2 * 60 * 1000) {
                if (difAngle < 0 && difAngle < difAngle1) {//difAngle < difAngle1继续向左偏
                    //与上一帧数据比较角度 向左偏移
                    difAngle1 = difAngle;
                    angle1 = carTrackModel.getCourseAngle();
                } else if (difAngle > 0 && difAngle > difAngle2) {//difAngle > difAngle2 继续向右偏
                    //与上一帧数据比较角度 向右偏移
                    angle2 = carTrackModel.getCourseAngle();
                    difAngle2 = difAngle;
                }
                //左右摇摆
                if (difAngle1 < -1 && difAngle2 > 0 && Math.abs(angle1 - angle2) >= 90) {
                    //轨迹运行不通畅
                    count++;

                    String happenTime = DateUtil.toDateTime(carTrackModel.getGlobalTimeStamp(), DateUtil.YYYY_MM_DD_HH_MM_SS);
                    if (Objects.isNull(happenStartTime)) {
                        happenStartTime = happenTime;
                    }
                    happenEndTime = happenTime;
                    if (count>unsmoothCount) {
                        log.info("index:{}，运行不流畅车辆-路口ID:{}, 车辆ID：{},车牌号：{}  发生次数：{}，进入路口时间：{},发生时间：{}"+
                                  "，刚进入路口角度：{},左偏角度：{}，右偏角度：{}",i,
                                carTrackModel.getRoadnet().getCrossId(), carTrackModel.getId(), carTrackModel.getPicLicense(), count,
                                DateUtil.toDateTime(tmp.getGlobalTimeStamp(), DateUtil.YYYY_MM_DD_HH_MM_SS),
                                happenEndTime,
                                tmp.getCourseAngle(),angle1,angle2
                        );
                    }
                    angle1 = -1;
                    angle2 = -1;
                    difAngle1 = -1;
                    difAngle2 = 0;
                }
            } else {
                //可能为再次进入路口、例如从下个路口掉头回来
                angle1 = -1;
                angle2 = -1;
                difAngle1 = -1;
                difAngle2 = 0;
                tmp = carTrackModel;
            }
            i++;
        }
        Map<String, Object> map = new HashMap<>();
        boolean isUnSmooth = false;
        if (count > unsmoothCount) {
            isUnSmooth = true;

            MotorEfficiencyResultModel model = new MotorEfficiencyResultModel();
            model.setCrossId(list.get(0).getRoadnet().getCrossId());
            model.setOrgCode(list.get(0).getOrgCode());
            model.setPicLicense(list.get(0).getPicLicense());
            model.setTrackId(list.get(0).getId());
            model.setUnSmoothCar(isUnSmooth);
            model.setUnSmoothCount(count);
            model.setHappenStartTime(happenStartTime);
            model.setHappenEndTime(happenEndTime);
            return model;
        }

        return null;
    }

}
