package com.wanji.indicators.task.pedestrian.service.func;

/**
 * @author fengyi
 * @ 2023/10/30
 * @dedatescription
 */

import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.cache.PedestrianDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;

/**
 * 行人轨迹基础
 */
@Slf4j
public class RefPedRidInfoCoFlatMap implements FlatMapFunction<CrossFrameModel,  CrossFrameModel> {

    //private Map<String, BaseLaneInfo> map = new HashMap();

    @Override
    public void flatMap(CrossFrameModel frameModel, Collector<CrossFrameModel> collector) throws Exception {

        Map<String, BaseLaneInfo> map = BaseLaneCache.getInstance().getBaseLaneInfoMap();

        List<CarTrackModel> trackList = frameModel.getTrackList();

        List<CarTrackModel> resList = new ArrayList<>();

        for (CarTrackModel carTrack : trackList) {
            String entryLane = carTrack.getRoadnet().getLaneId();
            BaseLaneInfo baseLaneInfo = map.get(entryLane);

            if (baseLaneInfo != null) {
                //log.info("从单例缓存中进行车道信息匹配laneId:{},info:{}",entryLane,baseLaneInfo);
                int ridDir = baseLaneInfo.getDir();
                String laneLngLat = baseLaneInfo.getWkt();
                String[] xys = laneLngLat.split(";");
                String laneStartPoint = xys[0];
                Double fx = Double.parseDouble(laneStartPoint.split(",")[0]);
                Double fy = Double.parseDouble(laneStartPoint.split(",")[1]);

                String laneEndPoint = xys[xys.length-1];
                Double ex = Double.parseDouble(laneEndPoint.split(",")[0]);
                Double ey = Double.parseDouble(laneEndPoint.split(",")[1]);

                double laneAngle = PtInPolyUtil.getAngle(fx,fy,ex,ey);

                Integer type = baseLaneInfo.getType();
                carTrack.getRoadnet().setLaneType(type);
                carTrack.getRoadnet().setLaneStartPoint(laneStartPoint);
                carTrack.getRoadnet().setLaneEndPoint(laneEndPoint);
                carTrack.getRoadnet().setTurn(baseLaneInfo.getTurn());
                carTrack.getRoadnet().setRidDir8(ridDir);
                carTrack.getRoadnet().setLaneAngle(laneAngle);
            } else {
                //行人数据，融合轨迹数据没有携带路网数据的情况，通过区域匹配设置目标所在的rid
                String rid = PedestrianDataCache.getInstance().getRidByBmxGeomIntersect(carTrack.getLongitude(),carTrack.getLatitude());
                String rid1 = PedestrianDataCache.getInstance().getRidByAqdGeomIntersect(carTrack.getLongitude(),carTrack.getLatitude());

                if (Objects.nonNull(rid)) {
                    //log.info("根据斑马线区域匹配RID:{},carInfo:{}",rid,carTrack.toString());
                    Optional<BaseLaneInfo> optional = map.values().stream().filter(o->o.getRid().equals(rid)).findAny();
                    if (optional.isPresent()) {
                        carTrack.getRoadnet().setRid(rid);
                        carTrack.getRoadnet().setRidDir8(optional.get().getDir());
                    }
                } else if (Objects.nonNull(rid1)) {
                   // log.info("根据安全岛区域匹配RID:{},carInfo:{}",rid,carTrack.toString());
                    Optional<BaseLaneInfo> optional = map.values().stream().filter(o->o.getRid().equals(rid1)).findAny();
                    if (optional.isPresent()) {
                        carTrack.getRoadnet().setRid(rid1);
                        carTrack.getRoadnet().setRidDir8(optional.get().getDir());
                    }
                }
            }


            resList.add(carTrack);
        }

        frameModel.setTrackList(resList);

        collector.collect(frameModel);
    }

//    @Override
//    public void flatMap2(Map<String, BaseLaneInfo> hashMap, Collector<CrossFrameModel> collector) throws Exception {
//        this.map = hashMap;
//    }
}