package com.wanji.indicators.task.pedestrian.service.func;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.cache.CrossLightStatusDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

public class TrafficLightStatusFlatMap implements FlatMapFunction<String, Map<String, List<CrossRidTurnLampStatusModel>>> {
    @Override
    public void flatMap(String value, Collector<Map<String, List<CrossRidTurnLampStatusModel>>> out) throws Exception {


        Map<String, BaseLaneInfo> laneInfoMap = BaseLaneCache.getInstance().getBaseLaneInfoMap();

        Map<String, List<CrossRidTurnLampStatusModel>> groupMap = new HashMap<>();

        //解析灯态json
        JSONArray jsonArray = JSON.parseArray(value);
        int size = jsonArray.size();
        long lightStatusTime = 0;
        for (int i = 0; i < size; i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String crossId = jsonObject.getString("crossId");
            lightStatusTime = jsonObject.getLongValue("timeStamp");
            if (lightStatusTime == 0) {
                lightStatusTime = System.currentTimeMillis();
            }
            Integer cycleCountDown = jsonObject.getInteger("cycleCountDown");
            Integer cyclePhaseCountDown = jsonObject.getInteger("cyclePhaseCountDown");
            Integer cycleLen = jsonObject.getInteger("cycleLen");

            List<BaseLaneInfo> laneList = laneInfoMap.values().stream().filter(o -> o.getCrossId().equals(crossId)).collect(Collectors.toList());

            //JSONObject jsonObject1 = jsonObject.getJSONObject("dirLampGroupMap");

            List<CrossRidTurnLampStatusModel> list = groupMap.get(crossId);
            if (list == null) {
                list = new ArrayList<>();
            }

            JSONArray jsonArray1 = jsonObject.getJSONArray("dirLampGroupMapList");
            for (int k = 0; k < jsonArray1.size(); k++) {
                JSONObject jsonObject2 = jsonArray1.getJSONObject(k);
                Integer dir = jsonObject2.getIntValue("dir");
                JSONArray jsonArray2 = jsonObject2.getJSONArray("turnList");
                for (int m = 0; m < jsonArray2.size(); m++) {
                    JSONObject jsonObject3 = jsonArray2.getJSONObject(m);
                    String status = jsonObject3.getString("color");
                    String turn = jsonObject3.getString("turn");
                    AtomicInteger atomicInteger = new AtomicInteger(dir);
                    Optional<BaseLaneInfo> optional = laneList.stream().filter(o -> o.getType().equals(2) && o.getDir() == atomicInteger.get()).findAny();
                    //灯态信息
                    CrossRidTurnLampStatusModel statusModel = new CrossRidTurnLampStatusModel();
                    statusModel.setCrossId(crossId);
                    statusModel.setDir(dir);
                    if (optional.isPresent()) {
                        statusModel.setRid(optional.get().getRid());
                    }
                    statusModel.setCycleCountDown(cycleCountDown);
                    statusModel.setCycleLen(cycleLen);
                    statusModel.setCyclePhaseCountDown(cyclePhaseCountDown);
                    statusModel.setLampState(status);
                    statusModel.setLightStatusTime(lightStatusTime);
                    statusModel.setTurn(turn);
                    list.add(statusModel);
                }
            }

            groupMap.put(crossId, list);
            //缓存灯态
//            String key = crossId + Constant.MARK + lightStatusTime / 1000;
//            CrossLightStatusDataCache.addCrossTimeLight(crossId, list);
            CrossLightStatusDataCache.getCrossLightStatusMap().put(crossId, list);
        }
        out.collect(groupMap);
    }


    public static void main(String[] args) {
        String value = "[\n" +
                "  {\n" +
                "    \"code\": \"53d3c9960f804395ab161f603f3a9a8d\",\n" +
                "    \"crossId\": \"12F6L08J630\",\n" +
                "    \"manufacturerCode\": \"STATIC\",\n" +
                "    \"runMode\": \"11\",\n" +
                "    \"controlMode\": \"11\",\n" +
                "    \"schemeStartTime\": null,\n" +
                "    \"cycleCountDown\": 166,\n" +
                "    \"cyclePhaseCountDown\": 6,\n" +
                "    \"cycleLen\": 180,\n" +
                "    \"phaseId\": \"1\",\n" +
                "    \"schemeId\": \"3\",\n" +
                "    \"dirLampGroupMapList\": [\n" +
                "      {\n" +
                "        \"dir\": \"1\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"1\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 6\n" +
                "          }\n" +
                "        ]\n" +
                "      },\n" +
                "      {\n" +
                "        \"dir\": \"3\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"1\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 6\n" +
                "          }\n" +
                "        ]\n" +
                "      },\n" +
                "      {\n" +
                "        \"dir\": \"5\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 6\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 6\n" +
                "          }\n" +
                "        ]\n" +
                "      }\n" +
                "    ],\n" +
                "    \"timeStamp\": \"1710565952817\"\n" +
                "  },\n" +
                "  {\n" +
                "    \"code\": \"53d3c9960f804395ab161f603f3a9a8d\",\n" +
                "    \"crossId\": \"12F6S08J8T0\",\n" +
                "    \"manufacturerCode\": \"STATIC\",\n" +
                "    \"runMode\": \"11\",\n" +
                "    \"controlMode\": \"11\",\n" +
                "    \"schemeStartTime\": null,\n" +
                "    \"cycleCountDown\": 166,\n" +
                "    \"cyclePhaseCountDown\": 86,\n" +
                "    \"cycleLen\": 180,\n" +
                "    \"phaseId\": \"1\",\n" +
                "    \"schemeId\": \"5\",\n" +
                "    \"dirLampGroupMapList\": [\n" +
                "      {\n" +
                "        \"dir\": \"1\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"1\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 86\n" +
                "          }\n" +
                "        ]\n" +
                "      },\n" +
                "      {\n" +
                "        \"dir\": \"3\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"1\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 86\n" +
                "          }\n" +
                "        ]\n" +
                "      },\n" +
                "      {\n" +
                "        \"dir\": \"5\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 86\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 86\n" +
                "          }\n" +
                "        ]\n" +
                "      }\n" +
                "    ],\n" +
                "    \"timeStamp\": \"1710565952825\"\n" +
                "  },\n" +
                "  {\n" +
                "    \"code\": \"53d3c9960f804395ab161f603f3a9a8d\",\n" +
                "    \"crossId\": \"12F6S08J7P0\",\n" +
                "    \"manufacturerCode\": \"STATIC\",\n" +
                "    \"runMode\": \"11\",\n" +
                "    \"controlMode\": \"11\",\n" +
                "    \"schemeStartTime\": null,\n" +
                "    \"cycleCountDown\": 166,\n" +
                "    \"cyclePhaseCountDown\": 81,\n" +
                "    \"cycleLen\": 180,\n" +
                "    \"phaseId\": \"1\",\n" +
                "    \"schemeId\": \"1\",\n" +
                "    \"dirLampGroupMapList\": [\n" +
                "      {\n" +
                "        \"dir\": \"1\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"1\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 81\n" +
                "          }\n" +
                "        ]\n" +
                "      },\n" +
                "      {\n" +
                "        \"dir\": \"3\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"1\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"red\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"20\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 81\n" +
                "          }\n" +
                "        ]\n" +
                "      },\n" +
                "      {\n" +
                "        \"dir\": \"5\",\n" +
                "        \"turnList\": [\n" +
                "          {\n" +
                "            \"turn\": \"2\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 81\n" +
                "          },\n" +
                "          {\n" +
                "            \"turn\": \"3\",\n" +
                "            \"color\": \"green\",\n" +
                "            \"countDown\": 81\n" +
                "          }\n" +
                "        ]\n" +
                "      }\n" +
                "    ],\n" +
                "    \"timeStamp\": \"1710565952835\"\n" +
                "  }\n" +
                "]\n";

        List<CrossRidTurnLampStatusModel> list = new ArrayList<>();
        //解析灯态json
        JSONArray jsonArray = JSON.parseArray(value);
        int size = jsonArray.size();
        long lightStatusTime = 0;
        for (int i = 0; i < size; i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String crossId = jsonObject.getString("crossId");
            lightStatusTime = jsonObject.getLongValue("timeStamp");
            if (lightStatusTime == 0) {
                lightStatusTime = System.currentTimeMillis();
            }
            Integer cycleCountDown = jsonObject.getInteger("cycleCountDown");
            Integer cyclePhaseCountDown = jsonObject.getInteger("cyclePhaseCountDown");
            Integer cycleLen = jsonObject.getInteger("cycleLen");

            JSONArray jsonArray1 = jsonObject.getJSONArray("dirLampGroupMapList");
            for (int k = 0; k < jsonArray1.size(); k++) {
                JSONObject jsonObject2 = jsonArray1.getJSONObject(k);
                Integer dir = jsonObject2.getIntValue("dir");
                JSONArray jsonArray2 = jsonObject2.getJSONArray("turnList");
                for (int m = 0; m < jsonArray2.size(); m++) {
                    JSONObject jsonObject3 = jsonArray2.getJSONObject(m);
                    String status = jsonObject3.getString("color");
                    String turn = jsonObject3.getString("turn");
                    AtomicInteger atomicInteger = new AtomicInteger(dir);
                    //灯态信息
                    CrossRidTurnLampStatusModel statusModel = new CrossRidTurnLampStatusModel();
                    statusModel.setCrossId(crossId);
                    statusModel.setDir(dir);

                    statusModel.setCycleCountDown(cycleCountDown);
                    statusModel.setCycleLen(cycleLen);
                    statusModel.setCyclePhaseCountDown(cyclePhaseCountDown);
                    statusModel.setLampState(status);
                    statusModel.setLightStatusTime(lightStatusTime);
                    statusModel.setTurn(turn);
                    list.add(statusModel);
                }
            }
            System.out.println(list.size());
        }
    }
}
