package com.wanji.indicators.task.pedestrian.service.func;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.cache.CrossLightStatusDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class TrafficLightStatusInfoCoFlatMap implements CoFlatMapFunction<CrossFrameModel, Map<String,List<CrossRidTurnLampStatusModel>>, CrossFrameModel> {

    Map<String,List<CrossRidTurnLampStatusModel>> lightStatusJson = new HashMap<>();

    @Override
    public void flatMap1(CrossFrameModel value, Collector<CrossFrameModel> out) throws Exception {
        try {
            if (!value.getCrossId().equals("null")) {
                if (!lightStatusJson.isEmpty()) {

                    List<CrossRidTurnLampStatusModel> list = lightStatusJson.get(value.getCrossId());
                    //log.info("轨迹时间：{}，灯态时间：{}",DateUtil.toDateTime(value.getGlobalTimeStamp(),DateUtil.YYYY_MM_DD_HH_MM_SS),DateUtil.toDateTime(list.get(0).getLightStatusTime(),DateUtil.YYYY_MM_DD_HH_MM_SS));
                    if (list != null) {
                        //绑定灯态
                        value.setRidTurnLampList(list);
                    }
                    out.collect(value);
                } else {
                    //String key = value.getCrossId() + Constant.MARK + value.getGlobalTimeStamp()/1000;
                    List<CrossRidTurnLampStatusModel> lightStatusList = CrossLightStatusDataCache.getCrossLightStatusMap().get(value.getCrossId());
                    if (lightStatusList != null && !lightStatusList.isEmpty()) {
                        long lstime = lightStatusList.get(0).getLightStatusTime();
                        long difTime = Math.abs(value.getGlobalTimeStamp() - lstime) / 1000;
                        if (difTime <= 1) {
                            value.setRidTurnLampList(lightStatusList);
                            out.collect(value);
                        }
                    } else {
                        log.info(value.getCrossId() + "==>" + "未匹配到灯态数据时间：" + DateUtil.toDateTime(value.getGlobalTimeStamp(), DateUtil.YYYY_MM_DD_HH_MM_SS));
                    }
                }
            }
        }catch (Exception e) {
            log.error("",e);
        }
    }

    @Override
    public void flatMap2(Map<String,List<CrossRidTurnLampStatusModel>> value, Collector<CrossFrameModel> out) throws Exception {
        lightStatusJson = value;
    }
}
