package com.wanji.indicators.task.queuelength.service.func;

import com.wanji.indicators.model.SnapshotInfo;
import com.wanji.indicators.task.freeflow.service.model.FrameMaxSpeedModel;
import com.wanji.indicators.task.queuelength.service.model.MaxQueueResultModel;
import org.apache.flink.api.common.functions.AggregateFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public   class QueueAggregateFunction implements AggregateFunction<SnapshotInfo,MaxQueueResultModel, MaxQueueResultModel> {

    private static final Logger log = LoggerFactory.getLogger(QueueAggregateFunction.class);

    @Override
    public MaxQueueResultModel createAccumulator() {
        MaxQueueResultModel initModel = new MaxQueueResultModel();
        initModel.setMaxQueueLength(0D);
        return initModel;
    }
    /**
     * 每进入一个数据就会执行一次
     * @param value 当前进入的数据
     * @param accumulator 之前计算好的中间结果
     * @return
     */
    @Override
    public MaxQueueResultModel add(SnapshotInfo value, MaxQueueResultModel accumulator) {

        accumulator.setMaxQueueLength(value.getDynamicQueueLength()>accumulator.getMaxQueueLength()?value.getDynamicQueueLength():accumulator.getMaxQueueLength());

        //System.out.println("avgSpeed:"+accumulator.getAvgSpeed()+":"+accumulator.getSumSpeed()/accumulator.getCount()+" maxSpeed:"+accumulator.getMaxSpeed());

        return accumulator;
    }
    /*
       当window的结束时间到达时，触发这个方法，返回结果
        */
    @Override
    public MaxQueueResultModel getResult(MaxQueueResultModel accumulator) {
        //log.info("AggResult:"+accumulator);
        return accumulator;
    }
    /**
     * 在session窗口才会用到merge，时间窗口其实用不到
     * @param a
     * @param b
     * @return
     */
    @Override
    public MaxQueueResultModel merge(MaxQueueResultModel a, MaxQueueResultModel b) {
        return null;
    }
}
