package com.wanji.indicators.task.track.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.task.track.service.func.ProcessCarTrackByKey;
import com.wanji.indicators.task.track.service.key.KeySelectorGlobalIdPlateNo;
import com.wanji.indicators.task.track.service.sink.BatchTrackElasticSearchSinkFunction;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.SlidingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.apache.http.HttpHost;
import org.springframework.beans.factory.annotation.Value;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.time.Duration;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public class AreaRealTimeIndicatorMainNew implements Serializable {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName = "area.realtime.indicator";

    private int windowTime = 2;

    private AreaRealTimeIndicatorMainNew(StreamExecutionEnvironment env, String paramInfo) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = this.instance.getProperties();
    }


    public static AreaRealTimeIndicatorMainNew init(StreamExecutionEnvironment env, String paramInfo) {
        return new AreaRealTimeIndicatorMainNew(env,paramInfo);
    }


    public void run(SingleOutputStreamOperator<CarTrackModel> carTrackModelStream) {
        //筛选机动车类型数据
        SingleOutputStreamOperator<CarTrackModel> filterStream =
                carTrackModelStream.filter(new FilterFunction<CarTrackModel>() {
                            @Override
                            public boolean filter(CarTrackModel value) throws Exception {
                                String motorObjectType = Constant.MOTOR_TYPES;
                                String[] sps = motorObjectType.split(",");
                                List<String>  typeList = Arrays.asList(sps);
                                String carType = value.getOriginalType().toString();
                                if (typeList.contains(carType)){
                                    return  true;
                                }
                                return false;
                            }
                }).name("筛选机动车数据");

        SingleOutputStreamOperator<JSONObject> groupByPlateStream =
                filterStream.assignTimestampsAndWatermarks(
                        WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                        .keyBy(CarTrackModel::getOrgCode)
                        .window(SlidingEventTimeWindows.of(Time.minutes(this.windowTime),Time.seconds(5)))
                        .process(new ProcessWindowFunction<CarTrackModel, JSONObject, String, TimeWindow>() {
                            @Override
                            public void process(String s, Context context, Iterable<CarTrackModel> elements, Collector<JSONObject> out) throws Exception {
                                Iterator<CarTrackModel> iterator = elements.iterator();
                                List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
                                //平均速度
                                double avgSpeed = list.stream().filter(o->o.getSpeed()>0).collect(Collectors.averagingDouble(o->o.getSpeed())).doubleValue();
                                //目标数量
                                long count = list.stream().collect(Collectors.groupingBy(o->o.getId())).size();

                                JSONObject jsonObject = new JSONObject();
                                jsonObject.put("vehicleNum",count);
                                jsonObject.put("avgSpeed",avgSpeed);
                                out.collect(jsonObject);

                                long ws = context.window().getStart();
                                long we = context.window().getEnd();

                                System.out.println(DateUtil.toDateTime(ws,"yyyy-MM-dd HH:mm:ss.SSS")+" "+DateUtil.toDateTime(we,"yyyy-MM-dd HH:mm:ss.SSS"));
                            }
                        })
                        .setParallelism(2)
                        .name("车辆轨迹分组-（全域ID-车辆ID-车牌号）");


        //从配置文件中读取 es 的地址
        try {

            groupByPlateStream.print();

            List<HttpHost> esAddresses = ElasticSearchSinkUtil.getEsAddresses(p.getProperty("elasticsearch.server"));

            BatchTrackElasticSearchSinkFunction elasticsearchSinkFunction = new BatchTrackElasticSearchSinkFunction(indexName);

            //ElasticSearchSinkUtil.addSink(esAddresses, 2, groupByPlateStream, elasticsearchSinkFunction);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }


}
