package com.wanji.indicators.task.track.service;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.track.service.func.ProcessCarTrackByKey;
import com.wanji.indicators.task.track.service.key.KeySelectorGlobalIdPlateNo;
import com.wanji.indicators.task.track.service.sink.BatchTrackElasticSearchSinkFunction;
import com.wanji.indicators.task.track.service.sink.PartitionBatchTrackElasticSearchSinkFunction;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.http.HttpHost;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.time.Duration;
import java.util.List;
import java.util.Objects;
import java.util.Properties;

@Slf4j
public class TrackStoreMainNew   {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName ;
    //窗口时间，单位秒
    private Integer windowTime=5;


    private TrackStoreMainNew(StreamExecutionEnvironment env,String paramInfo) {
        this.env = env;
        String[] sps = paramInfo.split(",");
        this.indexName = sps[0];
        if (sps.length>1)
            this.windowTime = Integer.parseInt(sps[1]);
        this.instance = PropertiesHelper.getInstance();
        this.p = this.instance.getProperties();
    }


    public static TrackStoreMainNew init(StreamExecutionEnvironment env,String paramInfo) {
        return new TrackStoreMainNew(env,paramInfo);
    }

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private TrackStoreMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
        this.indexName="car-track-30s";
        this.windowTime = 30;
    }
    public static TrackStoreMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new TrackStoreMainNew(env, path,isDebug);
    }


    public void run(SingleOutputStreamOperator<CarTrackModel> carTrackModelStream) {

        //从配置文件中读取 es 的地址
        try {
//            SingleOutputStreamOperator<CarTrackModel> filterStream =
//                    carTrackModelStream.
//                            filter(new FilterFunction<CarTrackModel>() {
//                                @Override
//                                public boolean filter(CarTrackModel value) throws Exception {
//                                    String date = "2023-10-19 23:59:59";
//                                    long endTs = DateUtil.StringToMillis(date);
//                                    long nowts = value.getGlobalTimeStamp();
//                                    if(nowts<endTs){
//                                        log.info("");
//                                    }
//                                    return nowts<endTs;
//                                }
//                            })
//                            .setParallelism(1)
//                            .name("轨迹分析-筛选带车牌的车辆");

            //根据车辆上报时间每5分钟一个窗口累积
            SingleOutputStreamOperator<SingleCarTrackListModel> groupByPlateStream =
                    carTrackModelStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                            .keyBy(new KeySelectorGlobalIdPlateNo())
                            .window(TumblingEventTimeWindows.of(Time.seconds(this.windowTime)))
                            //.trigger(new CountTriggerWithTimeout<>(Integer.MAX_VALUE,TimeCharacteristic.EventTime))
                            .process(new ProcessCarTrackByKey())
                            .setParallelism(2)
                            .name("车辆轨迹分组-（全域ID-车辆ID-车牌号）");

            List<HttpHost> esAddresses = ElasticSearchSinkUtil.getEsAddresses(p.getProperty("elasticsearch.server"));

            //重点车辆分析时会用到全量index
//            BatchTrackElasticSearchSinkFunction elasticsearchSinkFunction = new BatchTrackElasticSearchSinkFunction(indexName);
//            ElasticSearchSinkUtil.addSink(esAddresses, 2, groupByPlateStream, elasticsearchSinkFunction);


            PartitionBatchTrackElasticSearchSinkFunction elasticsearchSinkFunction1 = new PartitionBatchTrackElasticSearchSinkFunction(indexName);
            ElasticSearchSinkUtil.addSink("Sink Track Point",esAddresses, 5, groupByPlateStream, elasticsearchSinkFunction1);


        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }


}
