package com.wanji.indicators.task.track.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.util.CarNumGenerator;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.util.Collector;
import org.apache.kafka.common.protocol.types.Field;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 帧数据转换
 */
public class CarTrackFlatMap implements FlatMapFunction<FrameModel, CarTrackModel> {

    private static final Logger log = LoggerFactory.getLogger(CarTrackFlatMap.class);

    private boolean isMockPlateNumber = false;
    String platePrefix = null;
    public CarTrackFlatMap(boolean isMockPlateNumber) {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        platePrefix = instance.getProperties().getProperty("plate.prefix");
        this.isMockPlateNumber = isMockPlateNumber;
       // isMockPlateNumber = Boolean.parseBoolean(instance.getProperties().getProperty("is.mock.plate.number"));
    }

    @Override
    public void flatMap(FrameModel frameModel, Collector<CarTrackModel> collector) throws Exception {

        String timeStamp = frameModel.getTimeStamp();
        Long globalTimeStamp = frameModel.getGlobalTimeStamp();
        Integer participantNum = frameModel.getParticipantNum();
        String globalId = frameModel.getOrgCode();

        List<CarTrackModel> list = frameModel.getTrackList();

        if (list != null && !list.isEmpty()) {

            list.forEach(o -> {
                o.setTimeStamp(DateUtil.toDateTime(globalTimeStamp, "yyyy-MM-dd HH:mm:ss.SSS"));
                o.setGlobalTimeStamp(globalTimeStamp);
                o.setOrgCode(globalId);

                Integer id = o.getId();

                short originalType = o.getOriginalType().shortValue();
                int category = 0;
                if (VehicleTypeEnum.isMotorVehicles(originalType)) {
                    category = 1;
                    //对无车牌机动车模拟车牌
                    if (isMockPlateNumber && StringUtils.isEmpty(o.getPicLicense())) {
                        String mockPlate = platePrefix +"A"+o.getId();
                        o.setPicLicense(mockPlate);
                    }
                } else if (VehicleTypeEnum.isNonMotorVehicles(originalType)) {
                    category = 2;
                } else {
                    category = 3;
                }
                o.setCategory(category);

                collector.collect(o);
            });
        }
    }

    /**
     * 获取随机数
     *
     * @param min
     * @param max
     * @return
     */
    public static int getRandom(int min, int max) {
        int floor = (int) Math.floor(Math.random() * (max - min + 1) + min);
        return floor;
    }

}
