package com.wanji.indicators.task.track.service.func;

/**
 * @author fengyi
 * @date 2023/3/1
 * @description
 */

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;

//@Slf4j
public class ProcessCarTrackByKey    extends ProcessWindowFunction<CarTrackModel, SingleCarTrackListModel, Tuple3<String,Integer,String>, TimeWindow> {
    private static final Logger log = LoggerFactory.getLogger(ProcessCarTrackByKey.class);
    @Override
    public void process(Tuple3<String, Integer, String> integerIntegerStringTuple3, Context context, Iterable<CarTrackModel> iterable, Collector<SingleCarTrackListModel> out) throws Exception {
        Iterator<CarTrackModel> iterator = iterable.iterator();

        SingleCarTrackListModel singleCarTrackListModel = new SingleCarTrackListModel();

        List<SingleCarTrackListModel.CarTrackDynamicProperty> trackList = new ArrayList<>();
        int i = 0;
        CarTrackModel tmp = null;
        Set<CarTrackModel.RoadNet> roadNets = new HashSet<>();
        while (iterator.hasNext()) {
            CarTrackModel carTrackModel = iterator.next();
//            if (i==0) {
//                //提取公共属性
//                BeanUtils.copyProperties(carTrackModel,singleCarTrackListModel);
//                //窗口的开始时间
//                long winStartTimeStamp = context.window().getStart();
//                //窗口的截止时间
//                long winEndTimeStamp = context.window().getEnd();
//                singleCarTrackListModel.setStartTime(new Date(winStartTimeStamp));
//                singleCarTrackListModel.setEndTime(new Date(winEndTimeStamp));
//            }

            //提取动态属性
            SingleCarTrackListModel.CarTrackDynamicProperty dynamicProperty = new SingleCarTrackListModel.CarTrackDynamicProperty();
            dynamicProperty.setCourseAngle(carTrackModel.getCourseAngle());
            dynamicProperty.setLongitude(carTrackModel.getLongitude());
            dynamicProperty.setLatitude(carTrackModel.getLatitude());
            dynamicProperty.setSpeed(carTrackModel.getSpeed());
            dynamicProperty.setTimeStamp(carTrackModel.getTimeStamp());
            dynamicProperty.setGlobalTimeStamp(carTrackModel.getGlobalTimeStamp());

            if (!trackList.contains(dynamicProperty)) {
                trackList.add(dynamicProperty);
                roadNets.add(carTrackModel.getRoadnet());
            }

            tmp = carTrackModel;
            i++;
        }
        Date globalStartTime = null;
        Date globalEndTime = null;
        if (!trackList.isEmpty()) {
            //按时间排序
            trackList = trackList.stream().sorted(Comparator.comparing(SingleCarTrackListModel.CarTrackDynamicProperty::getGlobalTimeStamp)).collect(Collectors.toList());

            //提取公共属性
            BeanUtils.copyProperties(tmp, singleCarTrackListModel);

            //时段内第一帧检测时间
            globalStartTime = new Date(trackList.get(0).getGlobalTimeStamp());
            //时段内最后一帧检测时间
            globalEndTime = new Date(trackList.get(trackList.size()-1).getGlobalTimeStamp());
            //窗口的开始时间
            long winStartTimeStamp = context.window().getStart();
            //窗口的截止时间
            long winEndTimeStamp = context.window().getEnd();

            singleCarTrackListModel.setStartTime(new Date(winStartTimeStamp));
            singleCarTrackListModel.setEndTime(new Date(winEndTimeStamp));
            singleCarTrackListModel.setGlobalStartTime(globalStartTime);
            singleCarTrackListModel.setGlobalEndTime(globalEndTime);

        }

        singleCarTrackListModel.setTracks(trackList);
        singleCarTrackListModel.setRoadNets(roadNets);
       // log.info("plateNo:{},startTime:{}，endTime:{}，size:{}",singleCarTrackListModel.getStaticProperty().getPicLicense(),singleCarTrackListModel.getStaticProperty().getStartTime(),singleCarTrackListModel.getStaticProperty().getEndTime(),trackList.size());
         out.collect(singleCarTrackListModel);
    }

    public static void main(String[] args) {
        System.out.println(new Date());
    }

}
