package com.wanji.indicators.task.track.service.sink;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.RuntimeContext;
import org.apache.flink.streaming.connectors.elasticsearch.ElasticsearchSinkFunction;
import org.apache.flink.streaming.connectors.elasticsearch.RequestIndexer;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.Requests;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.common.xcontent.XContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.util.UUID;

/**
 * @author fengyi
 * @date 2023/3/2
 * @description
 */
public class BatchTrackElasticSearchSinkFunction implements ElasticsearchSinkFunction<SingleCarTrackListModel>, Serializable {
    private static Logger log = LoggerFactory.getLogger(BatchTrackElasticSearchSinkFunction.class);

    private String index;

    public BatchTrackElasticSearchSinkFunction(String index) {
        this.index = index;
    }

    public IndexRequest createIndexRequest(SingleCarTrackListModel element) {
        String jsonStr = JSON.toJSONString(element, SerializerFeature.WRITE_MAP_NULL_FEATURES);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr, Feature.OrderedField);
        jsonObject.put("recordTime", DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));

        //String id = element.getOrgCode()+"_"+element.getId()+"_"+element.getPicLicense()+"_"+element.getG;

        return Requests.indexRequest()
                    .index(index)
//                    .type(StringUtils.isEmpty(element.getPicLicense()) ?element.getId().toString():element.getPicLicense())
//                    .id(UUID.randomUUID().toString())
//                    .timeout(TimeValue.timeValueSeconds(60))
                    .source(jsonObject.toJSONString(), XContentType.JSON);
    }

    @Override
    public void process(SingleCarTrackListModel row, RuntimeContext runtimeContext, RequestIndexer requestIndexer) {
        try {
            requestIndexer.add(createIndexRequest(row));
        }catch (Exception e){
            e.printStackTrace();
        }

    }
}

