package com.wanji.indicators.task.track.service.sink;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.RuntimeContext;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.connectors.elasticsearch.ElasticsearchSinkFunction;
import org.apache.flink.streaming.connectors.elasticsearch.RequestIndexer;
import org.apache.http.HttpHost;
import org.apache.ibatis.session.SqlSession;
import org.elasticsearch.action.admin.indices.exists.indices.IndicesExistsRequest;
import org.elasticsearch.action.admin.indices.mapping.put.PutMappingRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.Requests;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.common.xcontent.XContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * @author fengyi
 * @date 2023/3/2
 * @description
 */
public class PartitionBatchTrackElasticSearchSinkFunction implements ElasticsearchSinkFunction<SingleCarTrackListModel>, Serializable {
    private static Logger log = LoggerFactory.getLogger(PartitionBatchTrackElasticSearchSinkFunction.class);

    private String index;

    private static Map<String,Boolean> mappingFlag = new HashMap<>();
    @Override
    public void open() throws Exception {
    }

    public PartitionBatchTrackElasticSearchSinkFunction(String index) {
        this.index = index;
    }

    public IndexRequest createIndexRequest(SingleCarTrackListModel element) {
        String jsonStr = JSON.toJSONString(element, SerializerFeature.WRITE_MAP_NULL_FEATURES);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr, Feature.OrderedField);
        jsonObject.put("recordTime", DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));

        //按天对索引进行分区
        String index = getIndexNameByDate(element);
        ElasticSearchSinkUtil.createIndex(index,true,getMappingInfo());

        //String id = element.getOrgCode()+"_"+element.getId()+"_"+element.getPicLicense()+"_"+element.getG;

        IndexRequest indexRequest = Requests.indexRequest()
                    .index(index)
//                    .type(StringUtils.isEmpty(element.getPicLicense()) ?element.getId().toString():element.getPicLicense())
//                    .id(UUID.randomUUID().toString())
//                    .timeout(TimeValue.timeValueSeconds(60))
                    .source(jsonObject.toJSONString(), XContentType.JSON);

        return indexRequest;
    }

    @Override
    public void process(SingleCarTrackListModel row, RuntimeContext runtimeContext, RequestIndexer requestIndexer) {
        try {

            requestIndexer.add(createIndexRequest(row));

        } catch (Exception e){
            e.printStackTrace();
        }

    }




    /**
     * 创建mapping
     * @param client
     * @throws IOException
     */
    public XContentBuilder getMappingInfo() {
        //创建mapping约束字段
        try {
            XContentBuilder mapping = XContentFactory.jsonBuilder()
                    .startObject()
                    //.startObject(index)
                        //.startObject("mapping")
                        .startObject("properties")
                            .startObject("orgCode").field("type", "text").field("fielddata", true).endObject()
                            .startObject("endTime").field("type", "text").field("fielddata", true).endObject()
                        .endObject()
                    //.endObject()
                    //.endObject()
                    .endObject();
            return  mapping;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private String getIndexNameByDate(SingleCarTrackListModel row){
        String globalDate = DateUtil.toDateTime(row.getGlobalEndTime().getTime(),"yyyyMMdd");
        //按天对索引进行分区
        String index = this.index+"_"+globalDate;
        return  index;
    }
}

