package com.wanji.indicators.task.track.service.sink;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import org.apache.flink.api.common.functions.RuntimeContext;
import org.apache.flink.streaming.connectors.elasticsearch.ElasticsearchSinkFunction;
import org.apache.flink.streaming.connectors.elasticsearch.RequestIndexer;
import org.elasticsearch.action.admin.indices.mapping.put.PutMappingRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.Requests;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.common.xcontent.XContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.Serializable;

/**
 * @author fengyi
 * @date 2023/3/2
 * @description
 */
public class PartitionRoutePathElasticSearchSinkFunction implements ElasticsearchSinkFunction<RoutePathCarTrackModel>, Serializable {
    private static Logger log = LoggerFactory.getLogger(PartitionRoutePathElasticSearchSinkFunction.class);

    private String index;

    public PartitionRoutePathElasticSearchSinkFunction(String index) {
        this.index = index;
    }

    public IndexRequest createIndexRequest(RoutePathCarTrackModel element) {
        String id = element.getTargetId().toString()+"_"+element.getPicLicense()+element.getLineNo();
        String jsonStr = JSON.toJSONString(element, SerializerFeature.WRITE_MAP_NULL_FEATURES);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr, Feature.OrderedField);
        jsonObject.put("recordTime", DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));
        String globalDate = DateUtil.toDateTime(DateUtil.StringToMillis(element.getEndTime(),"yyyy-MM-dd HH:mm:ss.SSS"),"yyyyMMdd");
        //按天对索引进行分区
        String index = this.index+"_"+globalDate;
        ElasticSearchSinkUtil.createIndex(index,true,getMappingInfo());
        //log.info(jsonObject.toJSONString());

        return Requests.indexRequest()
                    .index(index)
                    .source(jsonObject.toJSONString(), XContentType.JSON)
                    ;

    }

    @Override
    public void process(RoutePathCarTrackModel row, RuntimeContext runtimeContext, RequestIndexer requestIndexer) {
        try {
            requestIndexer.add(createIndexRequest(row));
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 创建mapping
     * @param client
     * @throws IOException
     */
    public XContentBuilder getMappingInfo() {
        //创建mapping约束字段
        try {
            XContentBuilder mapping = XContentFactory.jsonBuilder()
                    .startObject()
                    //.startObject(indexName)
                    .startObject("properties")
                    //.startObject("orgCode").field("type", "text").field("fielddata", "true").endObject()
                    .startObject("endTime").field("type", "text").field("fielddata", "true").endObject()
                    //.endObject()
                    .endObject()
                    .endObject();
            return  mapping;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}

