package com.wanji.indicators.task.track.service.sink;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.common.functions.RuntimeContext;
import org.apache.flink.streaming.connectors.elasticsearch.ElasticsearchSinkFunction;
import org.apache.flink.streaming.connectors.elasticsearch.RequestIndexer;
import org.elasticsearch.action.DocWriteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.Requests;
import org.elasticsearch.common.xcontent.XContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;

/**
 * @author fengyi
 * @date 2023/3/2
 * @description
 */
public class RoutePathElasticSearchSinkFunction implements ElasticsearchSinkFunction<RoutePathCarTrackModel>, Serializable {
    private static Logger log = LoggerFactory.getLogger(RoutePathElasticSearchSinkFunction.class);

    private String index;

    public RoutePathElasticSearchSinkFunction(String index) {
        this.index = index;
    }

    public IndexRequest createIndexRequest(RoutePathCarTrackModel element) {
        String id = element.getTargetId().toString()+"_"+element.getPicLicense()+element.getLineNo();
        String jsonStr = JSON.toJSONString(element, SerializerFeature.WRITE_MAP_NULL_FEATURES);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr, Feature.OrderedField);
        jsonObject.put("recordTime", DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));

        //log.info(jsonObject.toJSONString());

        return Requests.indexRequest()
                    .index(index)
                    .source(jsonObject.toJSONString(), XContentType.JSON)
                    ;

    }

    @Override
    public void process(RoutePathCarTrackModel row, RuntimeContext runtimeContext, RequestIndexer requestIndexer) {
        try {
            requestIndexer.add(createIndexRequest(row));
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}

