package com.wanji.indicators.task.trafficstate;

import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.source.AreaFreeSpeedSource;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.LaneNonMotorAnalysisMainNew;
import com.wanji.indicators.task.area.service.func.RidFreeSpeedCoFlatMap;
import com.wanji.indicators.task.motor.MotorEfficiencyMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;

import java.util.Properties;
import java.util.UUID;

public class RidRoadStatusStartMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final String GROUP_ID = "FlinkConsumerGroup-V56-TRACK";

    public static void main(String[] args) {
        //批量轨迹es index,逗号分割：esIndexName,windowTime

        ParameterTool parameter = ParameterTool.fromArgs(args);

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();
        String topic = properties.getProperty("consumer.topic");
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

//        env.enableCheckpointing(2 * 60 * 1000);
//        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);
//        env.getCheckpointConfig().setCheckpointTimeout(20 * 60 * 1000);
//        env.getCheckpointConfig().setCheckpointStorage(new FileSystemCheckpointStorage(properties.getProperty("check.point.uri")));
//        env.setStateBackend(new FsStateBackend(properties.getProperty("check.point.uri")));
//        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.DELETE_ON_CANCELLATION);
//        //env.getCheckpointConfig().setMinPauseBetweenCheckpoints(10 * 60 * 1000);
//        env.getCheckpointConfig().setMaxConcurrentCheckpoints(5);
//        env.getCheckpointConfig().setCheckpointingMode(CheckpointingMode.EXACTLY_ONCE);

        String groupId = GROUP_ID;
        String groupChangeFlag = properties.getProperty("consumer.group.change.flag");
        if (Boolean.parseBoolean(groupChangeFlag)) {
            groupId += DateUtil.toDateTime(System.currentTimeMillis(), "yyyyMMdd");
        }

        DateTime currentDateTime = new DateTime();
        DateTime dateTime = currentDateTime.withMillisOfDay(0).minusHours(-4);
        long startTimeStamp = DateUtil.StringToMillis("2023-10-23 20:00:00");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("enable.auto.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("auto.offset.reset", "latest")
//                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(groupId + "-rid_status_data" + UUID.randomUUID())
                /* 设置起始偏移量有以下几种情况
                       1.从指定的位置消费：OffsetsInitializer.offsets(Map<TopicPartition, Long> offsets)
                       2.从最新位置消费(最后一条处)：OffsetsInitializer.latest()
                       3.从最早位置消费(第一条处)：OffsetsInitializer.earliest()
                       4.从上次提交的位置消费：OffsetsInitializer.committedOffsets()
                       5.新的组，从来没有提交过,再指定一个消费方式：OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST)
                      */
                .setStartingOffsets(OffsetsInitializer.latest())
//        .setStartingOffsets(OffsetsInitializer.latest())
                // 从大于等于此时间戳开始的偏移量开始
//                .setStartingOffsets(OffsetsInitializer.timestamp(startTimeStamp))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-raod-status");


        SingleOutputStreamOperator<FrameModel> frameModelStream =
                stream
                        .flatMap(new FrameFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据-JsonToObject");

        SingleOutputStreamOperator<CarTrackModel> carTrackModelStream =
                frameModelStream.
                        flatMap(new CarTrackFlatMap(false))
                        .setParallelism(1)
                        .name("轨迹帧数据解析-ToCarTrackModel");

        SingleOutputStreamOperator<CarTrackModel> joinLaneStream = carTrackModelStream
                //.connect(env.addSource(new AreaFreeSpeedSource()).broadcast())
                .flatMap(new RidFreeSpeedCoFlatMap())
                .setParallelism(1)
                .name("全域轨迹帧数据-车道属性数据绑定");

        try {
            //路段路况
            RidTrafficStateMain.init(env, null, false).run(joinLaneStream);

            env.execute("基于轨迹-路况计算-工作流");
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
