package com.wanji.indicators.task.trafficstate.func;

import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.LanePeriodicDataModel;
import com.wanji.indicators.task.trafficstate.model.LanePeriodData;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Bean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ConnectLaneInfoCoFlatMap implements CoFlatMapFunction<LanePeriodicDataModel, Map<String, BaseLaneInfo>, LanePeriodData> {

    private Map<String, BaseLaneInfo> map = new HashMap();

    @Override
    public void flatMap1(LanePeriodicDataModel frameModel, Collector<LanePeriodData> collector) throws Exception {

        List<LanePeriodicDataModel.EventList> list = frameModel.getEventList();
        String time = frameModel.getTimeStamp();

        for (LanePeriodicDataModel.EventList event : list) {
            String entryLane = event.getLaneId();
            BaseLaneInfo baseLaneInfo = map.get(entryLane);

            LanePeriodData lanePeriodData = new LanePeriodData();
            BeanUtils.copyProperties(event,lanePeriodData);

            lanePeriodData.setDateTime(time);
            lanePeriodData.setTimestamp(DateUtil.stringToDate(time,"yyyy-MM-dd HH:mm:ss:SSS").getTime());

            if (baseLaneInfo != null) {
                lanePeriodData.setLaneLength(baseLaneInfo.getLength());
                lanePeriodData.setRidLength(baseLaneInfo.getRidLength());
                lanePeriodData.setRid(baseLaneInfo.getRid());
            }
            collector.collect(lanePeriodData);
        }


    }

    @Override
    public void flatMap2(Map<String, BaseLaneInfo> hashMap, Collector<LanePeriodData> collector) throws Exception {
        this.map = hashMap;
    }
}
