package com.wanji.indicators.task.trafficstate.func;

/**
 * @author fengyi
 * @date 2023/3/1
 * @description
 */

import com.wanji.indicators.constant.CongestEnum;
import com.wanji.indicators.task.trafficstate.model.LanePeriodData;
import com.wanji.indicators.task.trafficstate.model.LaneTrafficStateResult;
import com.wanji.indicators.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Slf4j
public class LaneProcessWindow extends ProcessWindowFunction<LanePeriodData, LaneTrafficStateResult, String, TimeWindow> {


    @Override
    public void process(String s, Context context, Iterable<LanePeriodData> elements, Collector<LaneTrafficStateResult> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        List<LanePeriodData> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
        double avgQueueLengthOfGreenStart = list.stream().mapToDouble(LanePeriodData::getLightGreenStartQueueLength).average().getAsDouble();
        double laneLength = list.get(0).getLaneLength();
        double laneIndex = avgQueueLengthOfGreenStart / laneLength;
        double avgStopNumber = list.stream().mapToDouble(LanePeriodData::getMeanStopsNumber).average().getAsDouble();

        LaneTrafficStateResult result = new LaneTrafficStateResult();
        result.setNowTimeStamp(context.currentProcessingTime());
        result.setNowDateTime(DateUtil.toDateTime(result.getNowTimeStamp(),DateUtil.YYYY_MM_DD_HH_MM_SS));

        result.setRid(list.get(0).getRid());
        result.setLaneId(list.get(0).getLaneId());
        result.setLaneLength(laneLength);
        result.setRidLength(list.get(0).getRidLength());
        result.setAvgStopNum(avgStopNumber);
        result.setLaneIndex(laneIndex);

        result.setStartTimestamp(windowStartTs);
        result.setEndTimestamp(windowEndTs);
        result.setStartTime(DateUtil.toDateTime(windowStartTs,DateUtil.YYYY_MM_DD_HH_MM_SS));
        result.setEndTime(DateUtil.toDateTime(windowEndTs, DateUtil.YYYY_MM_DD_HH_MM_SS));

        result.setTrafficState(Integer.valueOf(this.getTrafficState(laneIndex,avgStopNumber)));

        log.info("车道交通状态信息：{}",result.toString());

        out.collect(result);
    }

    /**
     * 计算交通状态
     * @param laneIndex
     * @param avgStopNumber
     * @return
     */
    private String getTrafficState(double laneIndex, double avgStopNumber) {
        if (laneIndex <= 0.9 || avgStopNumber <= 0.8) {
            return CongestEnum.NO_CONGEST.getType();
        } else if (laneIndex > 0.9){
            if (avgStopNumber > 0.8 && avgStopNumber <= 1.5) {
                return CongestEnum.LIGHT_CONGEST.getType();
            } else if (avgStopNumber > 1.5 && avgStopNumber <= 2) {
                return CongestEnum.MODERATE_CONGEST.getType();
            }else if (avgStopNumber > 2) {
                return CongestEnum.HEAVY_CONGEST.getType();
            }
        }
        return CongestEnum.NO_CONGEST.getType();
    }

}
