package com.wanji.indicators.task.trafficstate.func;

import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.constant.CongestEnum;
import com.wanji.indicators.constant.RoadClassMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.task.trafficstate.model.LaneTrafficStateResult;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class RidMotorCarProcessWindow extends ProcessWindowFunction<CarTrackModel, LaneTrafficStateResult, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(RidMotorCarProcessWindow.class);

    private Properties properties = new Properties();

    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
    }

    @Override
    public void process(String key, Context context, Iterable<CarTrackModel> elements, Collector<LaneTrafficStateResult> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
        Double freeSpeed = list.get(0).getRoadnet().getFreeSpeed();
        if (Objects.isNull(freeSpeed)) {
            freeSpeed = Double.parseDouble(this.properties.getProperty("rid.default.free.speed"));
        }

        double speed = 0;
        double segmentTotalSpeed = 0;
        double segmentTotalTime = 0;
        //车辆数量
        int segmentCarSize = 0;
        Map<String, List<CarTrackModel>> groupByLane = list.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getLaneId()));

        List<Integer> trackIdList = new ArrayList<>();

        for (Map.Entry<String, List<CarTrackModel>> entryLane : groupByLane.entrySet()) {
            List<CarTrackModel> laneCarList = entryLane.getValue();
            String laneId = entryLane.getKey();

            Map<Integer, List<CarTrackModel>> groupByPlateNo = laneCarList.stream().filter(o -> Objects.nonNull(o.getId())).collect(Collectors.groupingBy(o -> o.getId()));

            double laneTotalSpeed = 0;
            double laneTotalTime = 0;
            double laneCarSize = 0;

            for (Map.Entry<Integer, List<CarTrackModel>> entry : groupByPlateNo.entrySet()) {
                List<CarTrackModel> carList = entry.getValue();
                double travelSpeed = 0;
                double travelTime = 0;
                if (carList.size() > 1) {
                    carList = carList.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());
                    //米
                    double distance = CommonUtil.getTravelDistance(carList);
                    //秒
                    travelTime = (carList.get(carList.size() - 1).getGlobalTimeStamp() - carList.get(0).getGlobalTimeStamp()) / 1000;
                    if (travelTime > 0) {//单车平均行驶速度km/h
                        travelSpeed = (distance / 1000) / (travelTime / 60 / 60.0);
                    }
                    segmentTotalSpeed += travelSpeed;
                    segmentTotalTime += travelTime;

                    laneTotalSpeed += travelSpeed;
                    segmentTotalTime += travelTime;

                    segmentCarSize++;
                    laneCarSize++;

                }
                // trackIdList.add(entry.getKey());
            }
            double laneIndex = 1;
            if (laneCarSize > 0) {
                double laneAvgSpeed = (laneTotalSpeed / laneCarSize);
                if (laneAvgSpeed > 0) {
                    laneIndex = ArithOfBigDecmial.div(freeSpeed, laneAvgSpeed, 0);
                }
            }
            log.info("渠化车道ID:{},拥堵指数：{}", laneId, laneIndex);
        }


        int ridLevel = list.get(0).getRoadnet().getRidLevel();
        double trafficIndex = 1;
        double avgSpeed = 0;
        if (segmentCarSize > 0) {
//            double travelTime = segmentTotalTime / segmentCarSize;
//            double segmentLength = GeomsConvertUtil.getDistance(list.get(0).getRoadnet().getLaneStartPoint(),list.get(0).getRoadnet().getLaneEndPoint())/1000;
//            double freeTime = segmentLength / freeSpeed;
//            trafficIndex = ArithOfBigDecmial.div(travelTime,freeTime);

            avgSpeed = ArithOfBigDecmial.div(segmentTotalSpeed, segmentCarSize);
            if (avgSpeed > 0) {
                trafficIndex = ArithOfBigDecmial.div(freeSpeed, avgSpeed);
            }
        }

        String trafficState = "1";
        Integer count = BaseLaneCache.getInstance().getSegmentLaneNumberMap().get(key);
        if (count != null && count >= 3) {
            ridLevel = RoadClassMap.MAIN_DAO.getType();
        } else {
            ridLevel = RoadClassMap.CIYAO_DAO.getType();
        }
        //主要道路
        if (ridLevel == RoadClassMap.MAIN_DAO.getType()) {
            if (trafficIndex <= 1.8) {
                trafficState = CongestEnum.NO_CONGEST.getType();
            } else if (trafficIndex > 1.8 && trafficIndex <= 2.5) {
                trafficState = CongestEnum.LIGHT_CONGEST.getType();
            } else if (trafficIndex > 2.5 && trafficIndex <= 3.5) {
                trafficState = CongestEnum.MODERATE_CONGEST.getType();
            } else if (trafficIndex > 3.5) {
                trafficState = CongestEnum.HEAVY_CONGEST.getType();
            }
        } else if (ridLevel == RoadClassMap.CIYAO_DAO.getType()) {
            //次要道路
            if (trafficIndex <= 1.5) {
                trafficState = CongestEnum.NO_CONGEST.getType();
            } else if (trafficIndex > 1.5 && trafficIndex <= 2) {
                trafficState = CongestEnum.LIGHT_CONGEST.getType();
            } else if (trafficIndex > 2 && trafficIndex <= 3) {
                trafficState = CongestEnum.MODERATE_CONGEST.getType();
            } else if (trafficIndex > 3) {
                trafficState = CongestEnum.HEAVY_CONGEST.getType();
            }
        }

        log.info("路段渠化ID:{},拥堵指数：{}，拥堵状态：{}，平均速度：{}", key, trafficIndex, trafficState,avgSpeed);

        LaneTrafficStateResult outModel = new LaneTrafficStateResult();
        outModel.setId("Rid_Traffic_State");
        outModel.setRid(list.get(0).getRoadnet().getRid());
        outModel.setRidDir(list.get(0).getRoadnet().getRidDir8());
        outModel.setSegmentId(list.get(0).getRoadnet().getSegmentId());
        outModel.setTrafficIndex(trafficIndex);
        outModel.setTrafficState(Integer.valueOf(trafficState));
        outModel.setStartTimestamp(windowStartTs);
        outModel.setEndTimestamp(windowEndTs);
        outModel.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss"));
        outModel.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss"));
        outModel.setTrackIdList(trackIdList);

        out.collect(outModel);
    }


    private int getFrameAvgSpeed(List<Double> list) {
        //当前帧机动车平均速度，所有速度大于5km/h的速度进行排序，去掉低速10%，然后进行平均
        List<Double> speedList = list.stream()
                .filter(speed -> speed > 0).distinct()
                .sorted(Comparator.comparing(Double::doubleValue))
                .collect(Collectors.toList());

        int startIndex = new BigDecimal(speedList.size() * 0.1).intValue();
        int endIndex = (int) ArithOfBigDecmial.round(speedList.size() * 0.9, 0);
        List<Double> subList = speedList.subList(startIndex, endIndex);
        double avgSpeed = subList.stream().collect(Collectors.averagingDouble(Double::doubleValue));
        return (int) ArithOfBigDecmial.round(avgSpeed, 0);
    }
}
