package com.wanji.indicators.task.trafficstate.func;

import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.constant.CongestEnum;
import com.wanji.indicators.constant.RoadClassMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.task.citybrain.vo.CarPassVO;
import com.wanji.indicators.task.trafficstate.model.LaneTrafficStateResult;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.TypeHint;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class RidProcessWindow extends ProcessWindowFunction<CarTrackModel, LaneTrafficStateResult, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(RidMotorCarProcessWindow.class);

    private Properties properties = new Properties();

    private transient MapState<String, LaneTrafficStateResult> roadStateMapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
        StateTtlConfig ttlConfig = StateTtlConfig.newBuilder(Time.hours(6))//状态过期时间
                .setUpdateType(StateTtlConfig.UpdateType.OnReadAndWrite)
                .cleanupFullSnapshot()
                .setStateVisibility(StateTtlConfig.StateVisibility.ReturnExpiredIfNotCleanedUp)//不返回过期值
                .build();

        MapStateDescriptor mapStateDescriptor = new MapStateDescriptor<>("REALTIME_ROAD_STATUS",
                String.class,
                LaneTrafficStateResult.class
        );
        mapStateDescriptor.enableTimeToLive(ttlConfig);
        roadStateMapState = getRuntimeContext().getMapState(mapStateDescriptor);
    }


    @Override
    public void process(String key, Context context, Iterable<CarTrackModel> elements, Collector<LaneTrafficStateResult> out) throws Exception {
        long windowStartTs = context.window().getStart();
        long windowEndTs = context.window().getEnd();

        List<CarTrackModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());
        //筛选速度大于1m/s
        list = list.stream().filter(o->o.getSpeed()>3.6).collect(Collectors.toList());

        //进口车道
        // List<CarTrackModel> inLaneList = list.stream().filter(o -> Objects.equals(2, o.getRoadnet().getLaneType())).collect(Collectors.toList());
        //出口道
        // List<CarTrackModel> outLaneList = list.stream().filter(o -> Objects.equals(3, o.getRoadnet().getLaneType())).collect(Collectors.toList());

        //保留路段上的车辆
        list = list.stream().filter(o -> Objects.equals(1, o.getRoadnet().getLaneType())).collect(Collectors.toList());

        Double freeSpeed = null;
        String rid = null;
        Integer ridDir = null;
        double trafficIndex = 1;
        if (!list.isEmpty()) {
            freeSpeed = list.get(0).getRoadnet().getFreeSpeed();
            rid = list.get(0).getRoadnet().getRid();
            ridDir = list.get(0).getRoadnet().getRidDir8();

            if (freeSpeed == null) {
                freeSpeed = Double.parseDouble(this.properties.getProperty("rid.default.free.speed"));
            }
            double avgSpeed = getAvgSpeed(list, freeSpeed);
            double laneCount = 0;
            double totalLength = 0;
            double carSizeThreshold = 0;
            Double[] arr = BaseLaneCache.getInstance().getRidLaneLengthMap().get(key + "_1");
            int count = BaseLaneCache.getInstance().getSegmentLaneNumberMap().get(list.get(0).getRoadnet().getSegmentId());
            if (arr != null) {
                laneCount = arr[1];
                totalLength = arr[0];

                carSizeThreshold = ArithOfBigDecmial.div(totalLength, 7, 0) ;
            }
            long segmentCarSize = list.stream().mapToInt(o -> o.getId()).distinct().count();
            //过滤轨迹覆盖度较低情况
            if (avgSpeed > 0 && segmentCarSize > 10) {
                trafficIndex = ArithOfBigDecmial.div(freeSpeed, avgSpeed);
//                double tti = 0;
//                double ttiFree = 0;
//                Map<String, List<CarTrackModel>> groupBy = list.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getSegmentId()));
//                Map<String, Integer> segmentList = BaseLaneCache.getInstance().getSegmentLaneNumberMap();
//                for (Map.Entry<String, Integer> entry : segmentList.entrySet()) {
//                    List<CarTrackModel> carlist= new ArrayList<>() ;
//                    if (groupBy.containsKey(entry.getKey()))
//                        carlist = groupBy.get(entry.getKey());
//                    //渠化段上所有车辆
//                    long segCarCount = carlist.stream().map(o->o.getId()).distinct().count();
//                    //渠化段车辆平均行驶速度
//                    double avgSpd = this.getAvgSpeed(carlist, freeSpeed);
//                    //渠化段长度
//                    double length = carlist.isEmpty()?0: carlist.get(0).getRoadnet().getLaneLength();
//                    //渠化段ttl计算
//                    double travelTimeIndex = (length/1000 / avgSpd) * segCarCount;
//                    double freeTimeIndex = (length/1000/freeSpeed) * segCarCount;;
//
//                    tti += travelTimeIndex;
//                    ttiFree += freeTimeIndex;
//                }
//                if (ttiFree>0) {
//                    trafficIndex = tti / ttiFree;
//                }
            }


            String trafficState = "1";
            int ridLevel = RoadClassMap.MAIN_DAO.getType();

            if (count >= 3) {
                ridLevel = RoadClassMap.MAIN_DAO.getType();
            } else {
                ridLevel = RoadClassMap.CIYAO_DAO.getType();
            }
            //主要道路
            if (ridLevel == RoadClassMap.MAIN_DAO.getType()) {
                if (trafficIndex <= 1.8) {
                    trafficState = CongestEnum.NO_CONGEST.getType();
                } else if (trafficIndex <= 2.5) {
                    trafficState = CongestEnum.LIGHT_CONGEST.getType();
                } else if (trafficIndex <= 3.5) {
                    trafficState = CongestEnum.MODERATE_CONGEST.getType();
                } else  {
                    trafficState = CongestEnum.HEAVY_CONGEST.getType();
                }
            } else if (ridLevel == RoadClassMap.CIYAO_DAO.getType()) {
                //次要道路
                if (trafficIndex <= 1.5) {
                    trafficState = CongestEnum.NO_CONGEST.getType();
                } else if (trafficIndex <= 2) {
                    trafficState = CongestEnum.LIGHT_CONGEST.getType();
                } else if (trafficIndex <= 3) {
                    trafficState = CongestEnum.MODERATE_CONGEST.getType();
                } else {
                    trafficState = CongestEnum.HEAVY_CONGEST.getType();
                }
            }
            //采用tti判定范围
//            if (trafficIndex <= 1.35) {
//                trafficState = CongestEnum.NO_CONGEST.getType();
//            } else if (trafficIndex < 1.55) {
//                trafficState = CongestEnum.LIGHT_CONGEST.getType();
//            } else if (trafficIndex < 1.85) {
//                trafficState = CongestEnum.MODERATE_CONGEST.getType();
//            } else {
//                trafficState = CongestEnum.HEAVY_CONGEST.getType();
//            }

            LaneTrafficStateResult outModel = new LaneTrafficStateResult();
            outModel.setId("Rid_Traffic_State");
            outModel.setRid(rid);
            outModel.setRidDir(ridDir);
            outModel.setTrafficIndex(trafficIndex);
            outModel.setTrafficState(Integer.valueOf(trafficState));
            outModel.setStartTimestamp(windowStartTs);
            outModel.setEndTimestamp(windowEndTs);
            outModel.setStartTime(DateUtil.toDateTime(windowStartTs, "yyyy-MM-dd HH:mm:ss"));
            outModel.setEndTime(DateUtil.toDateTime(windowEndTs, "yyyy-MM-dd HH:mm:ss"));
            outModel.setRidLength(list.get(0).getRoadnet().getRidLength());
            outModel.setNowDateTime(DateUtil.toDateTime(System.currentTimeMillis(), "yyyy-MM-dd HH:mm:ss"));


            LaneTrafficStateResult agoResult = roadStateMapState.get(outModel.getRid());
            if (agoResult != null) {
                long agoTimeStamp = agoResult.getCreateTime();
                long duration = System.currentTimeMillis() - agoTimeStamp;
                int agoState = agoResult.getTrafficState();
                int nowState = outModel.getTrafficState();
                if (agoState == nowState && duration >= 30 * 1000) {
                    //持续时间x秒以上输出
                    out.collect(outModel);
                    outModel.setCreateTime(System.currentTimeMillis());
                    log.info("路段渠化RID:{},方向：{},持续时间：{}ms，拥堵指数：{}，拥堵状态：{}，平均速度：{}，车辆数：{},路段长度：{}m, 自由流速度：{}，开始时间：{},截止时间：{}", key, outModel.getRidDir(), duration, trafficIndex, trafficState, avgSpeed, segmentCarSize, outModel.getRidLength(), freeSpeed, outModel.getStartTime(), outModel.getEndTime());
                } else if (agoState == nowState && duration < 30 * 1000) {
                    out.collect(agoResult);
                }  else  {
                    outModel.setCreateTime(agoTimeStamp);
                }
            } else {
                //第一次缓存创建时间
                outModel.setCreateTime(System.currentTimeMillis());
            }
            roadStateMapState.put(outModel.getRid(), outModel);
        }


    }


    private double getAvgSpeed(List<CarTrackModel> list, double freeSpeed) {
        Map<Integer, List<CarTrackModel>> groupByPlateNo = list.stream().filter(o -> Objects.nonNull(o.getId())).collect(Collectors.groupingBy(o -> o.getId()));

        double laneTotalSpeed = 0;
        double laneTotalTime = 0;
        double laneCarSize = 0;

        for (Map.Entry<Integer, List<CarTrackModel>> entry : groupByPlateNo.entrySet()) {
            List<CarTrackModel> carList = entry.getValue();
            double travelSpeed = 0;
            double travelTime = 0;
            if (carList.size() > 1) {
                carList = carList.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());
                //米
                double distance = CommonUtil.getTravelDistance(carList);
                //秒
                travelTime = (carList.get(carList.size() - 1).getGlobalTimeStamp() - carList.get(0).getGlobalTimeStamp()) / 1000.0;
                travelTime = ArithOfBigDecmial.round(travelTime, 2);
                if (travelTime > 0) {//单车平均行驶速度km/h
                    travelSpeed = (distance / 1000) / (travelTime / 60 / 60.0);
                }

                laneTotalSpeed += travelSpeed;
                laneCarSize++;

            }
            // trackIdList.add(entry.getKey());
        }
        if (laneCarSize > 0) {
            return ArithOfBigDecmial.div(laneTotalSpeed, laneCarSize);
        } else {
            return freeSpeed;
        }
    }
}
