package com.wanji.indicators.task.turnIndicators;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.event.stream.func.RefBaseRidInfoCoFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.task.turnIndicators.func.TurnDataPeriodProcessWindow;
import com.wanji.indicators.task.turnIndicators.func.TurnDataProcessWindow;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import com.wanji.indicators.task.turnIndicators.sink.TurnIndicationSinkFunction;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.AggregateFunction;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.functions.ReduceFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.util.Collector;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Properties;

@Slf4j
public class CrossTurnIndicatorMainNew {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties properties = new Properties();

    private String path;

    private String indexName;


    private CrossTurnIndicatorMainNew(StreamExecutionEnvironment env, String indexName) {
        this.env = env;
        this.indexName = indexName;
        this.instance = PropertiesHelper.getInstance();
        this.properties = this.instance.getProperties();
    }


    public static CrossTurnIndicatorMainNew init(StreamExecutionEnvironment env, String indexName) {
        return new CrossTurnIndicatorMainNew(env, indexName);
    }

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private CrossTurnIndicatorMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    public static CrossTurnIndicatorMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new CrossTurnIndicatorMainNew(env, path, isDebug);
    }


    public void run(SingleOutputStreamOperator<CrossFrameModel> carTrackModelStream) {

        try {

            SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = carTrackModelStream
                    .flatMap(new RefBaseRidInfoCoFlatMap())
                    .name("全域轨迹帧数据-车道属性数据绑定");

            SingleOutputStreamOperator<CrossFrameModel> filterStream =
                    joinLaneStream.flatMap(new FlatMapFunction<CrossFrameModel, CrossFrameModel>() {
                        @Override
                        public void flatMap(CrossFrameModel value, Collector<CrossFrameModel> out) throws Exception {
                            List<CarTrackModel> trackList = value.getTrackList();
                            List<CarTrackModel> trackFilterList = new ArrayList<>();
                            for (CarTrackModel model : trackList) {
                                boolean isMotor = VehicleTypeEnum.isMotorVehicles(model.getOriginalType().shortValue());
                                int inCrossFlag = model.getRoadnet().getInCrossFlag();
                                if (isMotor && !Objects.equals("null",model.getRoadnet().getCrossId())) {
                                    //筛选路口内车辆、进入到出口道上的车辆
                                    if (inCrossFlag==Constant.CROSSING || Objects.equals(model.getRoadnet().getLaneType(),2) || Objects.equals(model.getRoadnet().getLaneType(),3)) {
                                        trackFilterList.add(model);
                                    }
                                }

                            }
                            value.setTrackList(trackFilterList);
                            if (!trackFilterList.isEmpty()) {
                                out.collect(value);
                            }

                            //log.info("路口ID:{}, 数据帧时间：{}，路口内机动车数量：{}",value.getCrossId(), DateUtil.toDateTime(value.getGlobalTimeStamp(),"yyyy-MM-dd HH:mm:ss.SSS"),trackFilterList.size());
                        }
                    }).name("筛选在路口内的机动车");


            SingleOutputStreamOperator<CrossTurnResultModel> processStream = filterStream
                    .assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CrossFrameModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                    .keyBy(o -> o.getCrossId())
                    .window(TumblingEventTimeWindows.of(Time.minutes(5)))
                    .process(new TurnDataProcessWindow())
                    .name("转向指标计算");


            SingleOutputStreamOperator<String> toJsonStream = processStream.flatMap(new FlatMapFunction<CrossTurnResultModel, String>() {
                @Override
                public void flatMap(CrossTurnResultModel value, Collector<String> out) throws Exception {
                    out.collect(JSONObject.toJSONString(value));
                }
            }).name("转向指标转JSON");

            if (!isDebug) {
                KafkaSink<String> sink = KafkaSink.<String>builder()
                        .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                                .setTopic("analysis.turn.indicators")
                                .setValueSerializationSchema(new SimpleStringSchema())
                                .build()
                        )
                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                        .build()
                        ;

                toJsonStream.sinkTo(sink).name("转向指标-数据发送至kafka");

//                processStream.addSink(new TurnIndicationSinkFunction()).name("转向指标-5分钟周期流量插入更新到mysql表");

                //15分钟周期计算
//                SingleOutputStreamOperator<CrossTurnResultModel> process15mStream = processStream
//                        .assignTimestampsAndWatermarks(
//                                WatermarkStrategy.<CrossTurnResultModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
//                                        .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
//                        .keyBy(o -> o.getCrossId())
//                        .window(TumblingEventTimeWindows.of(Time.minutes(15)))
//                        .process(new TurnDataPeriodProcessWindow())
//                        .setParallelism(1).name("转向指标-15分钟周期计算");
//                process15mStream.addSink(new TurnIndicationSinkFunction()).name("转向指标-15分钟周期流量插入更新到mysql表");
//
//                //15分钟周期计算
//                SingleOutputStreamOperator<CrossTurnResultModel> process30mStream = processStream
//                        .assignTimestampsAndWatermarks(
//                                WatermarkStrategy.<CrossTurnResultModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
//                                        .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
//                        .keyBy(o -> o.getCrossId())
//                        .window(TumblingEventTimeWindows.of(Time.minutes(30)))
//                        .process(new TurnDataPeriodProcessWindow())
//                        .setParallelism(1).name("转向指标-15分钟周期计算");
//                process30mStream.addSink(new TurnIndicationSinkFunction()).name("转向指标-30分钟周期流量插入更新到mysql表");
//
//                //60分钟周期计算
//                SingleOutputStreamOperator<CrossTurnResultModel> process60mStream = processStream
//                        .assignTimestampsAndWatermarks(
//                                WatermarkStrategy.<CrossTurnResultModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
//                                        .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
//                        .keyBy(o -> o.getCrossId())
//                        .window(TumblingEventTimeWindows.of(Time.minutes(60)))
//                        .process(new TurnDataPeriodProcessWindow())
//                        .setParallelism(1).name("转向指标-60分钟周期计算");
//                process60mStream.addSink(new TurnIndicationSinkFunction()).name("转向指标-60分钟周期流量插入更新到mysql表");

            } else {
                SingleOutputStreamOperator<String> toCsvStream = processStream.flatMap(new FlatMapFunction<CrossTurnResultModel, String>() {
                    @Override
                    public void flatMap(CrossTurnResultModel value, Collector<String> out) throws Exception {
                        List<TurnResultModel> list = value.getTurnDataList();
                        for (TurnResultModel model : list) {
                            String line = value.getCrossId()+",,"+model.getInRid()+","+model.getInRidDir()+","+model.getOutRid()+","+
                                    model.getTurnDirNo()+","+model.getFlow()+",5m,"+value.getStartTime()+","+value.getEndTime()+","+
                                    DateUtil.dateFormat(value.getStartTime(),"yyyyMMdd")+","+DateUtil.getNowTime("yyyy-MM-dd HH:mm:ss");
                            out.collect(line);
                        }
                    }
                }).name("转向指标转JSON");

                String path = this.path + "analysis.turn.indicators.csv";
                toCsvStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("转向指标-结果输出至文件");

                String jsonpath = this.path + "analysis.turn.indicators.json";
                toJsonStream.addSink(new JsonStringFileSink(jsonpath)).setParallelism(1).name("转向指标-结果输出至文件");
            }
        } catch (Exception e) {
            log.error("机动车指标计算任务异常 :   " + e);
        }

    }

}
