package com.wanji.indicators.task.turnIndicators.func;

import com.wanji.indicators.constant.TimeGranularityEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.joda.time.DateTime;
import org.joda.time.DateTimeUtils;
import org.joda.time.Minutes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class TurnDataPeriodProcessWindow extends ProcessWindowFunction<CrossTurnResultModel, CrossTurnResultModel, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(TurnDataPeriodProcessWindow.class);


    @Override
    public void open(Configuration parameters) {
        //mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_Area_Indicators", String.class, CarTrackModel.class));
    }

    @Override
    public void process(String groupKey, Context context, Iterable<CrossTurnResultModel> elements, Collector<CrossTurnResultModel> out) throws Exception {
        List<CrossTurnResultModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        //=====================================================================================//
        long windowStartTime = context.window().getStart();
        long windowEndTime = context.window().getEnd();
        DateTime startDateTime = new DateTime(windowStartTime);
        DateTime endDateTime = new DateTime(windowEndTime);
        int minute = Minutes.minutesBetween(startDateTime, endDateTime).getMinutes();
        String granularity = minute + "m";
        if (minute == 60) {
            granularity = "1h";
        }

        String startTime = DateUtil.toDateTime(windowStartTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        String endTime = DateUtil.toDateTime(windowEndTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        List<TurnResultModel> turnDataList = new ArrayList<>();
        CrossTurnResultModel outModel = null;
        for (CrossTurnResultModel model : list) {
            turnDataList.addAll(model.getTurnDataList());
            outModel = model;
        }

        Map<String, List<TurnResultModel>> group = turnDataList.stream().collect(Collectors.groupingBy(o -> o.getInRid() + "_" + o.getTurnDirNo()));

        List<TurnResultModel> aggList = new ArrayList<>();
        for (Map.Entry<String, List<TurnResultModel>> entry : group.entrySet()) {
            List<TurnResultModel> value = entry.getValue();
            TurnResultModel result = value.get(0);

            int flow = value.stream().mapToInt(o -> o.getFlow()).sum();
            double speed = value.stream().mapToDouble(o -> o.getSpeed()).average().orElse(0);
            result.setFlow(flow);
            result.setSpeed(speed);

            aggList.add(result);
        }

        outModel.setStartTime(startTime);
        outModel.setEndTime(endTime);
        outModel.setTimestamp(windowStartTime);
        outModel.setGranularity(granularity);
        outModel.setTurnDataList(aggList);

        out.collect(outModel);
    }
}
