package com.wanji.indicators.task.turnIndicators.func;

import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TimeGranularityEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class TurnDataProcessWindow extends ProcessWindowFunction<CrossFrameModel, CrossTurnResultModel, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(TurnDataProcessWindow.class);


    @Override
    public void open(Configuration parameters) {
        //mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_Area_Indicators", String.class, CarTrackModel.class));
    }

    @Override
    public void process(String groupKey, Context context, Iterable<CrossFrameModel> elements, Collector<CrossTurnResultModel> out) throws Exception {
        List<CrossFrameModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        List<CarTrackModel> trackList = new ArrayList<>();
        for (CrossFrameModel crossFrameModel : list) {
            trackList.addAll(crossFrameModel.getTrackList());
        }
        Map<Integer, List<CarTrackModel>> groupById = trackList.stream().collect(Collectors.groupingBy(o -> o.getId()));
        //=====================================================================================//
        long windowStartTime = context.window().getStart();
        long windowEndTime = context.window().getEnd();
        String startTime = DateUtil.toDateTime(windowStartTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        String endTime = DateUtil.toDateTime(windowEndTime, DateUtil.YYYY_MM_DD_HH_MM_SS);

        Map<String, TurnResultModel> turnResultModelMap = new HashMap<>();
        //检测每辆车从哪个进口道驶入路口，而后进入到出口道时，作为转向的一个流量
        for (Map.Entry<Integer, List<CarTrackModel>> entry : groupById.entrySet()) {
            Integer id = entry.getKey();

            List<CarTrackModel> value = entry.getValue();
            value = value.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());
            CarTrackModel tmp = null;
            for (CarTrackModel model : value) {
//                if (model.getId()==7213438  && model.getGlobalTimeStamp() >= DateUtil.StringToMillis("2024-05-12 08:02:58")) {
//                     System.out.println();
//                }
                String rid = model.getRoadnet().getRid();
                Integer type = model.getRoadnet().getLaneType();
                if (tmp != null) {
                    //********************************************************************************************//
                    if (Objects.equals(tmp.getRoadnet().getInCrossFlag(), Constant.CROSSING)
                            && Objects.equals(tmp.getRoadnet().getLaneType(), 3)
                            && !Objects.equals(tmp.getRoadnet().getRidDir8(), model.getRoadnet().getRidDir8())
                            && !Objects.equals(model.getRoadnet().getLaneId(), tmp.getRoadnet().getLaneId())
                    ) {
                        String outRid = tmp.getRoadnet().getRid();
                        //异常轨迹1：车辆从出口道驶出到路口内情况，再驶出到另一个出口到情况
                        Map<String, List<BaseLaneInfo>> outLaneListMap = BaseLaneCache.getInstance().getRidSameDirOutLaneMap();
                        if (outLaneListMap.containsKey(outRid)) {
                            List<BaseLaneInfo> laneList = outLaneListMap.get(outRid).stream().sorted(Comparator.comparing(o -> o.getLaneId())).collect(Collectors.toList());
                            //在出口道逆行行驶附上最进口道的最内侧车道
                            tmp.getRoadnet().setLaneId(laneList.get(0).getLaneId());
                            tmp.getRoadnet().setLaneType(laneList.get(0).getType());
                            tmp.getRoadnet().setTurn(laneList.get(0).getTurn());
                            tmp.getRoadnet().setRid(laneList.get(0).getRid());
                            tmp.getRoadnet().setRidDir8(laneList.get(0).getDir());
                        }
                    }
                    //********************************************************************************************//
                    if (Objects.equals(tmp.getRoadnet().getLaneType(), 2) && Objects.equals(type, 2)) {
                        //异常轨迹2：从一个进口道驶出到另一个进口道逆行
                        if (!Objects.equals(tmp.getRoadnet().getRidDir8(), model.getRoadnet().getRidDir8()) &&
                                !Objects.equals(tmp.getRoadnet().getLaneId(), model.getRoadnet().getLaneId())) {
                            Map<String, List<BaseLaneInfo>> outLaneListMap = BaseLaneCache.getInstance().getRidSameDirOutLaneMap();
                            if (outLaneListMap.containsKey(rid)) {
                                List<BaseLaneInfo> laneList = outLaneListMap.get(rid);
                                model.getRoadnet().setLaneId(laneList.get(0).getLaneId());
                                model.getRoadnet().setLaneType(laneList.get(0).getType());
                                model.getRoadnet().setRid(laneList.get(0).getRid());
                                Map<String, List<BaseCrossTurnInfo>> turnRelationMap = BaseLaneCache.getInstance().getCrossTurnInfoMap();

                                type = laneList.get(0).getType();
                                rid = laneList.get(0).getRid();
                                List<BaseCrossTurnInfo> relMap = turnRelationMap.get(rid);
                                model.getRoadnet().setTurnRelationList(relMap);
                            }
                        } else if (Objects.equals(tmp.getRoadnet().getInCrossFlag(), Constant.CROSSING) &&
                                !Objects.equals(model.getRoadnet().getInCrossFlag(), Constant.CROSSING)){
                            //异常轨迹3：从停止线驶出后路口范围外消失，未转到出口道情况
                            //type = 3;
                            if (model.getId()==3610633  && model.getGlobalTimeStamp() >= 1714267058878L) {
                              //  System.out.println();
                            }
                        }
                    }
                    //********************************************************************************************//

                    //********************************************************************************************//
                    //tmp为上个位置点，type为当前轨迹位置=》【车辆从进口道驶出经过路口内位置到达出口类型时开始计算】
                    if (Objects.equals(tmp.getRoadnet().getInCrossFlag(), Constant.CROSSING)
                            && Objects.equals(tmp.getRoadnet().getLaneType(), 2)
                            && Objects.equals(model.getRoadnet().getLaneType(), 3)) {

//                        if (model.getId()==7213438  && model.getGlobalTimeStamp() >= DateUtil.StringToMillis("2024-05-12 08:02:58")) {
//                            System.out.println();
//                        }

                        //从路口范围进入到出口道
                        String inRid = tmp.getRoadnet().getRid();
                        String outRid = rid;
                        Integer turn = tmp.getCarTurn();

                        List<BaseCrossTurnInfo> turnList = model.getRoadnet().getTurnRelationList();
                        if (turnList != null) {
                            //可进入到当前出口道的进口道RID集合
                            Optional<BaseCrossTurnInfo> optional = turnList.stream().filter(o -> o.getInRid().equals(inRid)).findFirst();
                            if (optional.isPresent()) {
                                String turnType = optional.get().getTurnType();
                                turn = Integer.valueOf(turnType);
                            }
                        }
                        //进口道+转向，否则没有匹配到转向关系的，按车辆车道转向计算流量，如左转车道上的流量计入左转流量
                        String key = inRid + "_" + turn;
//                    if (key.equals("13NF80B5QN013NGH0B5RC00_1")) {
//                        System.out.println();
//                    }
                        TurnResultModel resultModel = turnResultModelMap.get(key);
                        if (resultModel == null) {
                            resultModel = new TurnResultModel();
                        }
                        resultModel.setInRid(tmp.getRoadnet().getRid());
                        resultModel.setInRidDir(tmp.getRoadnet().getRidDir8());
                        resultModel.setOutRid(outRid);
                        resultModel.setTurnDirNo(turn);
                        resultModel.setFlow(resultModel.getFlow() + 1);
                        resultModel.setSpeed((resultModel.getSpeed() + model.getSpeed()) / resultModel.getFlow());

                        resultModel.getTrackIdList().add(entry.getKey());

                        turnResultModelMap.put(key, resultModel);
                        //break;【注释掉，同一辆车在5分钟内通过路口，掉头回来再次通过同一路口，会被少算】
                    }
                }

                tmp = model;
            }
        }
        //输出转向流量
        List<TurnResultModel> outList = new ArrayList<>(turnResultModelMap.values());
        if (!list.isEmpty()) {
            CrossTurnResultModel crossTurnResultModel = new CrossTurnResultModel();
            crossTurnResultModel.setStartTime(startTime);
            crossTurnResultModel.setEndTime(endTime);
            crossTurnResultModel.setTimestamp(context.window().getStart());
            crossTurnResultModel.setCrossId(groupKey);
            crossTurnResultModel.setTurnDataList(outList);
            crossTurnResultModel.setGranularity(TimeGranularityEnum.FIVE_MINUTE.getType());
            out.collect(crossTurnResultModel);
        }
    }
}
