package com.wanji.indicators.task.turnIndicators.sink;

import com.wanji.indicators.constant.TimeGranularityEnum;
import com.wanji.indicators.entity.CrossStateData;
import com.wanji.indicators.entity.TurnIndicatorData;
import com.wanji.indicators.mapper.TurnIndicatorsDataMapper;
import com.wanji.indicators.service.CrossStateDataService;
import com.wanji.indicators.service.impl.CrossStateDataServiceImpl;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import com.wanji.indicators.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.ArrayList;
import java.util.List;

/**
 * @author fengyi
 * @date 2024/1/23
 * @description
 */
@Slf4j
public class TurnIndicationSinkFunction extends RichSinkFunction<CrossTurnResultModel> {

    TurnIndicatorsDataMapper turnIndicatorsDataMapper;

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        turnIndicatorsDataMapper = beanConf.getBean(TurnIndicatorsDataMapper.class);
    }

    @Override
    public void invoke(CrossTurnResultModel value, Context context) throws Exception {
        try {
            List<TurnResultModel> turnDataList = value.getTurnDataList();

            List<TurnIndicatorData> outDataList = new ArrayList<>();

            for (TurnResultModel model : turnDataList) {

                TurnIndicatorData data = new TurnIndicatorData();
                data.setCrossId(value.getCrossId());
                data.setFRid(model.getInRid());
                data.setFRidDir(model.getInRidDir());
                data.setTRid(model.getOutRid());
                data.setTurnDirNo(model.getTurnDirNo());
                data.setGranularity(value.getGranularity());
                data.setWindowStartTime(DateUtil.stringToDate(value.getStartTime(),"yyyy-MM-dd HH:mm:ss"));
                data.setWindowEndTime(DateUtil.stringToDate(value.getEndTime(),"yyyy-MM-dd HH:mm:ss"));
                data.setFlow(model.getFlow());
                data.setSpeed(model.getSpeed());
                String dt = value.getStartTime().substring(0,value.getStartTime().indexOf(" "));
                dt = dt.replaceAll("-","");
                data.setDt(Integer.valueOf(dt));

                outDataList.add(data);
            }

            turnIndicatorsDataMapper.batchInsert(outDataList);

        } catch (Exception e) {
            log.error(this.getClass().getName() + ".invoke()执行失败", e);
        }
    }
}
