package com.wanji.indicators.util;

import com.wanji.indicators.entity.ForeignDevicePO;
import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.core.JsonProcessingException;
import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.databind.JsonNode;
import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.databind.node.ObjectNode;

import java.util.List;
import java.util.Optional;
import java.util.Properties;

public class CityBrainReceiveUtil {

    private static final ObjectMapper objectMapper = new ObjectMapper();
    public static final Properties properties = PropertiesHelper.getInstance().getProperties();

    /**
     * 处理收到的城市大脑 JSON 字符串
     *
     * @author Kent HAN
     * @date 2024/4/18 9:11
     */
    public static String processJson(String json) throws JsonProcessingException {

        JsonNode rootNode = objectMapper.readTree(json);

        // 提取headers和body部分，并处理字符串
        String headersJson = rootNode.get("headers").asText().replace("\\\"", "\"");
        String bodyJson = rootNode.get("body").asText().replace("\\\"", "\"");
        if (headersJson.contains("yigou")) {
            bodyJson = "[" + bodyJson + "]";
        }

        // 将处理后的字符串转换回JsonNode
        JsonNode headersNode = objectMapper.readTree(headersJson);
        JsonNode bodyNode = objectMapper.readTree(bodyJson);

        // 创建一个新的JSON对象并组合headers和body
        ObjectNode combinedJson = objectMapper.createObjectNode();
        combinedJson.set("headers", headersNode);
        combinedJson.set("body", bodyNode);

        return objectMapper.writeValueAsString(combinedJson);
    }

    /**
     * 城市大脑 deviceNo 转万集路口ID
     *
     * @author Kent HAN
     * @date 2024/4/18 9:11
     */
    public static String convertDeviceNo(String deviceNo, List<ForeignDevicePO> foreignDevicePOList) {
        if (foreignDevicePOList == null || deviceNo == null) {
            return deviceNo;
        }

        Optional<ForeignDevicePO> matchingDevice = foreignDevicePOList.stream()
                .filter(device -> deviceNo.equals(device.getDeviceNo()))
                .findFirst();

        return matchingDevice.map(ForeignDevicePO::getCrossId).orElse(deviceNo);
    }

    /**
     * 城市大脑 laneId 转万集 laneID
     *
     * @author Kent HAN
     * @date 2024/4/18 9:11
     */
    public static String convertLaneId(String laneId, List<ForeignDevicePO> foreignDevicePOList) {
        if (laneId == null || laneId.isEmpty()) {
            return laneId;
        }

        String[] split = laneId.split("-");
        if (split.length < 4) {  // 确保 split 数组中至少有四个部分
            return laneId;
        }

        String crossId = split[1];
        int sectionNo;
        int sort;

        try {
            sectionNo = Integer.parseInt(split[2]);
            sort = Integer.parseInt("1" + split[3].substring(split[3].length() - 1));  // 提取最后一个字符前的部分并加1
        } catch (NumberFormatException e) {
            return laneId;  // 如果 sectionNo 或 sort 的转换失败
        }

        for (ForeignDevicePO device : foreignDevicePOList) {
            if (device.getCrossId().equals(crossId) &&
                    device.getSectionNo() != null && device.getSectionNo() == sectionNo &&
                    device.getSort() != null && device.getSort() == sort) {
                return device.getLaneId();
            }
        }

        return laneId;  // 如果没有找到匹配的记录
    }

}
