package com.wanji.indicators.util;

import com.vividsolutions.jts.algorithm.Angle;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Polygon;
import com.wanji.indicators.cache.PedestrianDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.LightStatusEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.event.WaitingAreaDo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.stream.Collectors;

public class CommonUtil {

    private static final Logger log = LoggerFactory.getLogger(CommonUtil.class);

    /**
     * 绑定车道扩展属性
     * @param carTrack
     * @param baseLaneInfo 车道数据
     * @param turnRelationMap 转向关系
     * @return
     */
    public  static void boundLaneExtProperty(CarTrackModel carTrack, BaseLaneInfo baseLaneInfo, Map<String, List<BaseCrossTurnInfo>> turnRelationMap){
        if (baseLaneInfo == null)
            return;

        int ridDir = baseLaneInfo.getDir();
        String laneLngLat = baseLaneInfo.getWkt();
        String[] xys = laneLngLat.split(";");
        String laneStartPoint = xys[0];
        Double fx = Double.parseDouble(laneStartPoint.split(",")[0]);
        Double fy = Double.parseDouble(laneStartPoint.split(",")[1]);

        String laneEndPoint = xys[xys.length - 1];
        Double ex = Double.parseDouble(laneEndPoint.split(",")[0]);
        Double ey = Double.parseDouble(laneEndPoint.split(",")[1]);

        double laneAngle = PtInPolyUtil.getAngle(fx, fy, ex, ey);
        //2进口道 3出口道
        Integer type = baseLaneInfo.getType();
        carTrack.getRoadnet().setLaneType(type);
        carTrack.getRoadnet().setLaneStartPoint(laneStartPoint);
        carTrack.getRoadnet().setLaneEndPoint(laneEndPoint);
        carTrack.getRoadnet().setTurn(baseLaneInfo.getTurn());
        carTrack.getRoadnet().setRidDir8(ridDir);
        carTrack.getRoadnet().setLaneAngle(laneAngle);
        carTrack.getRoadnet().setCrossId(baseLaneInfo.getCrossId());
        carTrack.getRoadnet().setLaneLength(baseLaneInfo.getLength());
        carTrack.getRoadnet().setRidLength(baseLaneInfo.getRidLength());
        //出口道对应的哪些进口转向可以进入
        if (type.equals(3)) {
            List<BaseCrossTurnInfo> turnRelationList = turnRelationMap.get(carTrack.getRoadnet().getRid());
            carTrack.getRoadnet().setTurnRelationList(turnRelationList);
        }
        //混合车道根据航向角判定行驶转向
        if (carTrack.getRoadnet().getTurn() > 4) {
            carTrack.setCarTurn(CommonUtil.getCarTurn(carTrack));
        } else {
            carTrack.setCarTurn(carTrack.getRoadnet().getTurn());
        }
    }

    /**
     * 判断车辆在车道行驶直线方向的左边，还是右边
     *
     * @param carTrack
     * @return 返回车辆转向 1左转 2直行 3右转 4掉头
     */
    public static int getCarTurn(CarTrackModel carTrack) {

        String laneStartPoint = carTrack.getRoadnet().getLaneStartPoint();
        Double fx = Double.parseDouble(laneStartPoint.split(",")[0]);
        Double fy = Double.parseDouble(laneStartPoint.split(",")[1]);

        String laneEndPoint = carTrack.getRoadnet().getLaneEndPoint();
        Double ex = Double.parseDouble(laneEndPoint.split(",")[0]);
        Double ey = Double.parseDouble(laneEndPoint.split(",")[1]);

        double laneAngle = carTrack.getRoadnet().getLaneAngle();

        //小于0在左侧，大于0在右侧
        double turn = Angle.angleBetweenOriented(new Coordinate(fx, fy), new Coordinate(ex, ey), new Coordinate(carTrack.getLongitude(), carTrack.getLatitude()));

        double diffAngle = Math.abs(laneAngle - carTrack.getCourseAngle());

        int carTurn = 0;
        if (diffAngle > 30) {
            if (turn < 0) {
                carTurn = 1;
                if (diffAngle > 160.0 && diffAngle < 200.0)  {
                    carTurn = 4;
                }
            } else if (turn > 0) {
                carTurn = 3;
            } else {
                carTurn = 2;
            }
        } else {
            carTurn = 2;
        }

//        log.info("路口内混合车道车辆转向判定,路口ID：{},车道ID：{},车道功能：{}，" +
//                        "车辆ID：{}，车辆转向：{}，时间：{},是否为机动车：{}," +
//                        "车道角度：{},车辆角度：{},速度：{}",
//                roadNet.getCrossId(), roadNet.getLaneId(), roadNet.getTurn(), carTrack.getId(), carTurn,
//                DateUtil.toDateTime(carTrack.getGlobalTimeStamp(), "yyyy-MM-dd HH:mm:ss.SSS"), VehicleTypeEnum.isMotorVehicles(carTrack.getOriginalType().shortValue()),
//                laneAngle, carTrack.getCourseAngle(), carTrack.getSpeed()
//        );

        return carTurn;
    }

    /**
     * 车辆视野范围区域
     *
     * @param carTrack
     * @return
     */
    public static Polygon getCarFrontSector(CarTrackModel carTrack) {
        double lng = carTrack.getLongitude();
        double lat = carTrack.getLatitude();
        double angle = carTrack.getCourseAngle();
        double langle = angle - 50;
        double rangle = angle + 50;
        double radius = 10;
        Polygon polygon = GeomsConvertUtil.createSector(lng, lat, radius, langle, rangle);
        return polygon;
    }

    /**
     * 判断车辆是否在等待区
     *
     * @param rid
     * @param carTrack
     * @return
     */
    public static boolean isInWaitTurnArea(String rid, CarTrackModel carTrack) {
        boolean isInWaitArea = false;
        List<WaitingAreaDo> waitAreaList = PedestrianDataCache.getInstance().getWaitingTurnAreaMap().get(rid);
        if (Objects.nonNull(waitAreaList)) {
            for (WaitingAreaDo model : waitAreaList) {
                String geom = model.getGeomWkt();
                isInWaitArea = GeomsConvertUtil.withinGeo(carTrack.getLongitude() + "," + carTrack.getLatitude(), geom);
                if (isInWaitArea) {
                    break;
                }
            }
        }
        return isInWaitArea;
    }

    /**
     * 所有速度大于5km/h的速度进行排序，去掉低速10%，然后进行平均
     *
     * @param list
     * @return
     */
    public static double getFrameAvgSpeed(List<CarTrackModel> list) {
        List<Double> speedList = list.stream()
                .filter(o -> o.getSpeed() > 5)
                .sorted(Comparator.comparing(CarTrackModel::getSpeed))
                .map(CarTrackModel::getSpeed).distinct()
                .collect(Collectors.toList());

        int startIndex = new BigDecimal(speedList.size() * 0.1).intValue();
        int endIndex = (int) ArithOfBigDecmial.round(speedList.size() * 0.9, 0);
        List<Double> subList = speedList.subList(startIndex, endIndex);
        double avgSpeed = subList.stream().collect(Collectors.averagingDouble(Double::doubleValue));
        return ArithOfBigDecmial.round(avgSpeed, 0);
    }

    /**
     * 验证灯态是否为绿
     *
     * @param lampStateList
     * @return
     */
    public static boolean isGreen(List<CrossRidTurnLampStatusModel> lampStateList) {
        if (lampStateList != null) {
            for (CrossRidTurnLampStatusModel lampState : lampStateList) {
                String lamp = lampState.getLampState();
                if (LightStatusEnum.GREEN.getType().equals(lamp)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean isInCrossing(CarTrackModel carTrackModel) {
        CarTrackModel.RoadNet roadNet = carTrackModel.getRoadnet();
        String crossId = roadNet.getCrossId();
        String rid = roadNet.getRid();
        String laneId = roadNet.getLaneId();
        String segmentId = roadNet.getSegmentId();

        if (!Objects.equals("null", crossId) && Objects.equals("null", rid) && Objects.equals("null", laneId)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 过滤拥堵场景下的事故上报
     * @param rid
     * @param trackIds
     * @return
     */
//    public static int ridIsCongested(String rid, String trackIds, ConcurrentHashMap<String, EventCross> cache){
//        int stopRidNum = 0;
//        if(StringUtils.isNotBlank(rid)){
//            for(EventCross e : cache.values()){
//                if(rid.equals(e.getRid())){
//                    if(!trackIds.contains(String.valueOf(e.getTrackId())) && e.getAreaDist() < 13.0){
//                        stopRidNum++;
//                    }
//                }
//            }
//        }
//        return stopRidNum;
//    }

    /**
     * 获取车道功能
     *
     * @param matchLaneName
     * @return
     */
    public static String getLaneFunc(String matchLaneName) {
        if (StringUtils.isBlank(matchLaneName)) {
            return Constant.STRAIGHT_LINE_CODE;
        }
        String[] s = matchLaneName.split(" ");
        String laneFuncChn = "";
        String laneFunc = "";
        if (s.length == 3) {
            laneFuncChn = s[1];
        }
        for (String k : Constant.mapTurn.keySet()) {
            String v = Constant.mapTurn.get(k);
            if (laneFuncChn.contains(v)) {
                laneFunc += k + Constant.COMMA;
            }
        }
        if (StringUtils.isNotBlank(laneFunc)) {
            laneFunc = laneFunc.substring(0, laneFunc.lastIndexOf(Constant.COMMA));
        } else {
            laneFunc = Constant.STRAIGHT_LINE_CODE;
            ;
        }
        return laneFunc;
    }

    /**
     * 排序 -- 时间戳 -- 升序
     * @param list
     * @return
     */
//    public static List<EventCross> getSortList(List<EventCross> list) {
//        try {
//            Collections.sort(list, new Comparator<EventCross>() {
//                @Override
//                public int compare(EventCross o1, EventCross o2) {
//                    long o1x = o1.getTime();
//                    long o2x = o2.getTime();
//                    if (o1x > o2x) {
//                        return 1;
//                    }
//                    if (o1x == o2x) {
//                        return 1;
//                    }
//                    return -1;
//                }
//            });
//            return list;
//        } catch (Exception e) {
//            return list;
//        }
//    }


    /**
     * 加载配置项
     */
    public String getWorkSpasePath() {
        String path = "";
        Properties props = System.getProperties();
        String osName = props.getProperty("os.name");
        String FLINK_WORKSPACE = System.getenv("FLINK_WORKSPACE_PATH");
        log.info("当前操作系统: " + osName + "  环境变量配置: " + FLINK_WORKSPACE);
        if (StringUtils.isNotBlank(osName) && osName.contains("Windows")) {
            path = Constant.PATH_WINDOWS;
            if (StringUtils.isNotBlank(FLINK_WORKSPACE)) {
                path = FLINK_WORKSPACE + "\\";
                log.info("读取 " + osName + " 外部环境变量配置: " + path);
            }
        } else {
            path = Constant.PATH_LINUX;
            if (StringUtils.isNotBlank(FLINK_WORKSPACE)) {
                path = FLINK_WORKSPACE;
                log.info("读取 " + osName + " 外部环境变量配置: " + path);
            }
        }
        return path;
    }

    /**
     * 根据方向角度计算RID 4方位 1：北 2：东 3：南 4：西
     */
    public static int getAngleDir(double angle) {
        int dir = 1;
        if (angle >= 0 && angle < 45) {
            dir = 1;
        } else if (angle >= 45 && angle < 135) {
            dir = 2;
        } else if (angle >= 135 && angle < 225) {
            dir = 3;
        } else if (angle >= 225 && angle < 315) {
            dir = 4;
        } else if (angle >= 315) {
            dir = 1;
        }
        return dir;
    }

    public static int getAngleDirExt(double angle, String crossId) {
        int dir = 1;
        if (angle >= 0 && angle < 45) {
            dir = 1;
        } else if (angle >= 45 && angle < 135) {
            dir = 2;
            // 霞景路特殊处理
            if ("13NF80B5QN0".equals(crossId) && angle >=90 && angle < 130) {
                dir = 3;
            }
        } else if (angle >= 135 && angle < 225) {
            dir = 3;
        } else if (angle >= 225 && angle < 315) {
            dir = 4;
        } else if (angle >= 315) {
            dir = 1;
        }
        return dir;
    }

    /**
     * 根据城市大脑转换南北方向
     */
    public static int convertCityBrainDir(int dir) {
        if (dir == 1) {
            dir = 3;
        } else if (dir == 3) {
            dir = 1;
        }
        return dir;
    }

    public static double getTravelDistance(List<CarTrackModel> carList) {
        String xys = carList.stream().map(o -> o.getLongitude() + "," + o.getLatitude()).collect(Collectors.joining(";"));
        double distance = 0;

        String tempx = "";
        String tempy = "";
        String[] sps = xys.split(";");

        int i = 0;
        for (String xy : sps) {
            String[] sps1 = xy.split(",");
            String x = sps1[0];
            String y = sps1[1];
            if (i > 1) {
                distance += GeomsConvertUtil.getDistance(Double.parseDouble(x), Double.parseDouble(y), Double.parseDouble(tempx), Double.parseDouble(tempy));
            }
            tempx = x;
            tempy = y;
            i++;
        }
        return distance;
    }

    public static boolean canConvertToInt(String str) {
        try {
            Integer.parseInt(str);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }
}
