package com.wanji.indicators.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DateUtil {
  private static final Logger log = LoggerFactory.getLogger(DateUtil.class);
  
  private static final String FORMAT = "yyyy-MM-dd HH:mm:ss";
  
  public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
  
  public static String YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";
  
  public static String YYYY_MM_DD = "yyyy-MM-dd";
  
  public static String YYYYMMDD = "yyyyMMdd";

  public static String YYYY_MM_DD_HH_MM_SS_SSS = "yyyy-MM-dd HH:mm:ss:SSS";
  
  public static int currentTimeSlot(long time) {
    Date date = new Date(time);
    int hh = date.getHours();
    String id = TimeZone.getDefault().getID();
    if (StringUtils.isNotBlank(id) && 
      id.contains("Etc/UTC"))
      hh += 8; 
    if (hh > 6 && hh < 10)
      return 1; 
    if (hh > 16 && hh < 20)
      return 3; 
    if (hh >= 20 || hh < 6)
      return 4; 
    return 2;
  }
  
  public static Date stringToDate(String time, String fmt) {
    SimpleDateFormat sdf = new SimpleDateFormat(fmt);
    try {
      Date parse = sdf.parse(time);
      return parse;
    } catch (ParseException e) {
      e.printStackTrace();
      return null;
    } 
  }
  
  public static String dateFormat(String dateTime, String format) throws Exception {
    Date date = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).parse(dateTime);
    return (new SimpleDateFormat(format)).format(date);
  }
  
  public static String toDateTime(long time, String format) {
    SimpleDateFormat df = new SimpleDateFormat(format);
    Date date = new Date(time);
    return df.format(date);
  }
  
  public static long StringToMillis(String time, String fmt) {
    if (StringUtils.isBlank(fmt))
      fmt = "yyyy-MM-dd HH:mm:ss"; 
    SimpleDateFormat sdf = new SimpleDateFormat(fmt);
    Date date = null;
    try {
      date = sdf.parse(time);
      return date.getTime();
    } catch (ParseException e) {
      e.printStackTrace();
      return 0L;
    } 
  }
  
  public static long StringToMillis(String time) {
    return StringToMillis(time, null);
  }
  
  public static long getNowTime() {
    return System.currentTimeMillis();
  }
  
  public static String getNowTime(String format) {
    if (StringUtils.isBlank(format))
      format = "yyyy-MM-dd HH:mm:ss"; 
    return toDateTime(getNowTime(), format);
  }
  
  public static String getAgoTime(String format, String unit, int time) {
    return getTime(format, unit, time, "ago", new Date());
  }
  
  public static String getAfterTime(String format, String unit, int time) {
    return getTime(format, unit, time, "after", new Date());
  }
  
  public static String getAgoTime(String format, String unit, int time, Date date) {
    return getTime(format, unit, time, "ago", date);
  }
  
  public static String getAfterTime(String format, String unit, int time, Date date) {
    return getTime(format, unit, time, "after", date);
  }
  
  private static String getTime(String format, String unit, int time, String type, Date now) {
    Date now_Time;
    long ss = 1000L;
    long mm = 60L * ss;
    long HH = 60L * mm;
    long dd = 24L * HH;
    long milliseconds = 0L;
    if ("ss".equals(unit)) {
      milliseconds = ss;
    } else if ("mm".equals(unit)) {
      milliseconds = mm;
    } else if ("HH".equals(unit)) {
      milliseconds = HH;
    } else {
      milliseconds = dd;
    } 
    if ("ago".equals(type)) {
      now_Time = new Date(now.getTime() - time * milliseconds);
    } else {
      now_Time = new Date(now.getTime() + time * milliseconds);
    } 
    SimpleDateFormat dateFormat = new SimpleDateFormat(format);
    String agoTime = dateFormat.format(now_Time);
    return agoTime;
  }
  
  public static int getDifferenceBetweenTheTime(String startTime, String endTime, String fmt) {
    long start = StringToMillis(startTime, fmt);
    long end = StringToMillis(endTime, fmt);
    long ms = end - start;
    int ss = 1000;
    int mi = ss * 60;
    int hh = mi * 60;
    int dd = hh * 24;
    long day = ms / dd;
    return Math.abs((int)day);
  }
  
  public static int getDifferenceBetweenTheMi(String startTime, String endTime, String fmt) {
    long start = StringToMillis(startTime, fmt);
    long end = StringToMillis(endTime, fmt);
    long ms = end - start;
    int ss = 1000;
    int mm = ss * 60;
    long mi = ms / mm;
    return Math.abs((int)mi);
  }
  
  public static int getDifferenceBetweenTheS(String startTime, String endTime, String fmt) {
    long start = StringToMillis(startTime, fmt);
    long end = StringToMillis(endTime, fmt);
    long ms = end - start;
    int ss = 1000;
    long si = ms / ss;
    return Math.abs((int)si);
  }
  
  public static String getDamaiDay(String date) {
    return getDamaiDay(stringToDate(date, YYYY_MM_DD));
  }
  
  private static String getDamaiDay(Date date) {
    Calendar calendar = Calendar.getInstance();
    calendar.setTime(date);
    switch (calendar.get(7)) {
      case 2:
        return "星期一";
      case 3:
        return "星期二";
      case 4:
        return "星期三";
      case 5:
        return "星期四";
      case 6:
        return "星期五";
      case 7:
        return "星期六";
      case 1:
        return "星期日";
    } 
    return "";
  }
  
  public static long datecheck(long time) {
    String s = DateFormatUtils.format(time, YYYY_MM_DD_HH_MM_SS);
    String dateStr = s.substring(0, 15);
    int m = Integer.valueOf(s.substring(15, 16)).intValue();
    if (m < 5) {
      dateStr = dateStr + "0:00";
    } else {
      dateStr = dateStr + "5:00";
    } 
    return StringToMillis(dateStr);
  }

  public static void main(String[] args) {
    long ts = DateUtil.StringToMillis("2024-05-12 00:00:00","yyyy-MM-dd HH:mm:ss");
    System.out.println(ts);

    System.out.println(DateUtil.toDateTime(1715600421299l,"yyyy-MM-dd HH:mm:ss.SSS"));
    System.out.println(DateUtil.toDateTime(1715600504510l,"yyyy-MM-dd HH:mm:ss.SSS"));

    System.out.println(DateUtil.toDateTime(1715668005610l,"yyyy-MM-dd HH:mm:ss.SSS"));
  }

}
