package com.wanji.indicators.util;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.FileFilterUtils;
import org.apache.commons.io.filefilter.IOFileFilter;
import org.joda.time.*;
import org.joda.time.format.DateTimeFormat;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

public class EsDateIndexUtil {

    private EsDateIndexUtil() {
    }

    public static final String YMD_HMS_FORMATTER = "yyyy-MM-dd HH:mm:ss";
    public static final String YMDHM_FORMATTER = "yyyyMMddHHmm";
    public static final String YMDHMSFORMATTER = "yyyyMMddHHmmss";
    public static final String YMD_HM_FORMATTER = "yyyy-MM-dd HH:mm";
    public static final String YMD_H_FORMATTER = "yyyyMMddHH";
    public static final String YMD_FORMATTER = "yyyyMMdd";
    public static final String YM_FORMATTER = "yyyyMM";
    public static final String HMS_FORMATTER = "HH:mm:ss";
    public static final String HMS_SSS_FORMATTER = "HH:mm:ss SSS";
    public static final String HM_FORMATTER = "HH:mm";
    public static final String H_FORMATTER = "HH";

    private static final String FORMATSTR = "%s_%s";

    /**
     * 根据开始结束时间匹配对应索引
     *
     * @param indexName 索引前缀
     * @param startTime 开始时间戳
     * @param endTime   截止时间戳
     * @param timeType  1:分钟 2：小时 3：天 4：月
     * @return
     */
    public static String[] getIndexByTime(String indexName, long startTime, long endTime, String timeType) {
        List<String> indexList = new ArrayList<>();

        DateTime start = new DateTime(startTime);
        DateTime end = new DateTime(endTime);
        int diffNum = 0;
        //push单位秒
        int unitTime = 0;
        String DATE_FORMAT = null;
        if (Objects.equals("1", timeType)) {
            DATE_FORMAT = YMDHM_FORMATTER;
            diffNum = Minutes.minutesBetween(start, end).getMinutes();
            unitTime = 60;
        } else if (Objects.equals("2", timeType)) {
            DATE_FORMAT = YMD_H_FORMATTER;
            diffNum = Hours.hoursBetween(start, end).getHours();
            unitTime = 60 * 60;
        } else if (Objects.equals("3", timeType)) {
            DATE_FORMAT = YMD_FORMATTER;
            diffNum = Days.daysBetween(start, end).getDays();
            unitTime = 24 * 60 * 60;
        } else if (Objects.equals("4", timeType)) {
            DATE_FORMAT = YM_FORMATTER;
            diffNum = Months.monthsBetween(start, end).getMonths();
            unitTime = 30 * 24 * 60 * 60;
        }

        for (int i = 0; i <= diffNum; i++) {
            String format = String.format(FORMATSTR, indexName, start.plusSeconds(i * unitTime).toString(DATE_FORMAT));
            indexList.add(format);
        }
        return indexList.toArray(new String[0]);
    }

    /**
     * 根据开始结束时间匹配 文件完整路径
     *
     * @param fileDirectory 文件目录
     * @param startTime 开始时间戳
     * @param endTime   截止时间戳
     * @param timeType  1:分钟 2：小时 3：天 4：月 5；10分钟
     * @return
     */
    public static String[] getFileListByTime(String fileDirectory, String filePrefix,long startTime, long endTime, String timeType) {
        Set<String> indexList = new HashSet<>();

        ///data/track_partitions/dt=2023-08-09/ht=11/mt=00

        DateTime start = new DateTime(startTime);
        DateTime end = new DateTime(endTime);

        while (start.isBefore(end)){


            String dt = start.toString("yyyy-MM-dd");
            dt = "dt="+dt;

            int hour = start.getHourOfDay();
            String ht = Tools.fillZeroFront(hour+"",2);
            ht = "ht="+ht;

            int minute = start.getMinuteOfHour();
            int index = minute/10*10;
            String mt = Tools.fillZeroFront(index+"",2);
            mt = "mt="+mt;

            String directory = fileDirectory +"/"+dt+"/"+ht+"/"+mt;
            if (!new File(directory).exists()){
                start = start.plusMinutes(1);
                continue;
            }


            IOFileFilter ioFileFilter = FileFilterUtils.prefixFileFilter(filePrefix);

             List<File> fileList = FileUtils.listFiles(new File(directory),ioFileFilter,null).stream().collect(Collectors.toList());

            String fileName = null;
            for (File dataFile : fileList) {
                fileName = dataFile.getName();
                String filePath = directory +"/"+fileName;
                if(!indexList.contains(filePath)) {
                    indexList.add(filePath);
                }
            }
            start = start.plusMinutes(1);
//            System.out.println("start:"+start.toString("yyyy-MM-dd HH:mm:ss"));
        }
//        System.out.println("end:"+start.toString("yyyy-MM-dd HH:mm:ss"));
//        System.out.println(indexList.toString());
        return indexList.toArray(new String[0]);
    }

    /**
     * 获取当前时间实时索引
     *
     * @param indexName
     * @return
     */
//    public static String[] getIndexByNow(String indexName) {
//        long start = DateTime.now().plusMonths(-1).withTimeAtStartOfDay().getMillis();
//        long end = DateTime.now().getMillis();
//        return getIndexByTime(indexName, start, end);
//    }
//
//    public static String[] getIndexBySingle(String indexName, long time) {
//        long start = new DateTime(time).plusMonths(-1).withTimeAtStartOfDay().getMillis();
//        return getIndexByTime(indexName, start, time);
//    }
    public static String getInsertIndex(String indexName) {
        return String.format("%s_%s", indexName, DateTime.now().toString(YM_FORMATTER));
    }


    public static List<String> getTimeList(String formatter, long interval, long startTime, long endTime) {
        List<String> timeList = new ArrayList<>();
        for (long i = startTime; i < endTime; i += interval) {
            timeList.add(new DateTime(i).toString(formatter));
        }
        return timeList;
    }


    public static List<String> getMinutesStr() {
        List<String> timeStr = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            String h;
            if (i < 10) {
                h = "0" + i;
            } else {
                h = i + "";
            }
            for (int j = 0; j < 60; j++) {
                String m;
                if (j < 10) {
                    m = "0" + j;
                } else {
                    m = j + "";
                }
                timeStr.add(h + ":" + m);
            }
        }
        return timeStr;
    }

    public static void main(String[] args) {

        System.out.println(DateUtil.toDateTime(1699604136810L,YMD_HMS_FORMATTER));

        System.out.println(DateUtil.stringToDate("2023-11-09 06:00:00",YMD_HMS_FORMATTER).getTime());
        DateTime start  =  DateTime.parse("2023-07-09 10:00:00",DateTimeFormat.forPattern(YMD_HMS_FORMATTER));
        DateTime end  =  DateTime.parse("2023-08-09 10:30:00",DateTimeFormat.forPattern(YMD_HMS_FORMATTER));
        String[] indexByTime = EsDateIndexUtil.getIndexByTime("car-route-path", start.getMillis(), end.getMillis(), "3");

        System.out.println(indexByTime.toString());
        String[] list  = getFileListByTime("d:/track_files","compacted-",start.getMillis(),end.getMillis(),null);

    }


}
