package com.wanji.indicators.util;

import com.alibaba.fastjson.JSONArray;
import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKBReader;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.linearref.LengthLocationMap;
import com.vividsolutions.jts.linearref.LinearLocation;
import com.vividsolutions.jts.linearref.LocationIndexedLine;
import com.vividsolutions.jts.operation.buffer.BufferOp;
import com.vividsolutions.jts.operation.buffer.BufferParameters;
import com.vividsolutions.jts.operation.buffer.OffsetCurveBuilder;
import com.vividsolutions.jts.operation.distance.DistanceOp;
import com.vividsolutions.jts.operation.linemerge.LineMerger;
import com.vividsolutions.jts.util.GeometricShapeFactory;
import org.apache.commons.lang.StringUtils;

import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class GeomsConvertUtil {
    private static GeometryFactory geometryFactory = new GeometryFactory();
    //生成工厂类
    private static GeometricShapeFactory shapeFactory = new GeometricShapeFactory();

    private static int pointCxNum = 10;// 连续抽稀pointCxNum个点
    private static int distance = 2;// 抽稀距离 distance 米
    // 地球的赤道半径是6378137米
    public final static double Rc = 6378137;
    // 地球的极半径是6356725米
    public final static double Rj = 6356725;

    /**
     * 按距离缓冲偏移坐标
     *
     * @param xys      坐标集合，格式：x,y;x1,y1;...,xn,yn
     * @param distance 偏移的距离，单位：米，正数为向左平移，负数时为向右偏移
     * @return 返回偏移后的坐标集合 格式：x,y;x1,y1;...,xn,yn
     */
    public static String offsetCureLine(String xys, double distance) {
        String offsetxy = "";
        /*============定义缓冲样式========================*/
        BufferParameters bufferParameters = new BufferParameters();
        bufferParameters.setEndCapStyle(BufferParameters.CAP_FLAT);
        bufferParameters.setJoinStyle(BufferParameters.JOIN_MITRE);
        bufferParameters.setQuadrantSegments(18);
        //米转换为度单位
        distance = meterToDu(distance);
        LineString lineString = GeomsConvertUtil.getLineString(xys);
        BufferOp bufferOp = new BufferOp(lineString, bufferParameters);
        Geometry offsetGeometry = bufferOp.getResultGeometry(distance);

        try {
            offsetxy = formatWkt(offsetGeometry.toText());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return offsetxy;
    }

    /**
     * 按距离缓冲偏移坐标
     *
     * @param xys      坐标集合，格式：x,y;x1,y1;...,xn,yn
     * @param distance 偏移的距离，单位：米，正数为向左平移，负数时为向右偏移
     * @return 返回偏移后的坐标集合 格式：x,y;x1,y1;...,xn,yn
     */
    public static String offsetCure(String xys, double distance) {
        String offsetxy = "";
        /*============定义缓冲样式========================*/
        BufferParameters bufferParameters = new BufferParameters();
        bufferParameters.setEndCapStyle(BufferParameters.CAP_FLAT);
        bufferParameters.setJoinStyle(BufferParameters.JOIN_MITRE);
        bufferParameters.setQuadrantSegments(18);
        /*=============================================*/
        String[] coords = xys.split(";");
        Coordinate[] inputPts = new Coordinate[coords.length];
        int i = 0;
        for (String xy : coords) {
            String[] pointArr = xy.split(",");
            Coordinate point = new Coordinate(Double.parseDouble(pointArr[0]), Double.parseDouble(pointArr[1]));
            inputPts[i] = point;
            i++;
        }
        //米转换为度单位
        distance = meterToDu(distance);
        /*===============调用偏移方法进行偏移======================*/
        OffsetCurveBuilder offsetCurveBuilder = new OffsetCurveBuilder(geometryFactory.getPrecisionModel(), bufferParameters);
        Coordinate[] outPts = offsetCurveBuilder.getOffsetCurve(inputPts, distance);
        for (Coordinate pt : outPts) {
            if (("" + pt.x).equals("NaN") || ("" + pt.y).equals("NaN")) {
                continue;
            }
            offsetxy += pt.x + "," + pt.y + ";";
        }
        return offsetxy.substring(0, offsetxy.length() - 1);
    }

    /**
     * 设置保留小数位数
     *
     * @param value 输入数据
     * @param num   小数点位数
     */
    public static double getScale(double value, int num) {
        BigDecimal bg = new BigDecimal(value);
        BigDecimal nbg = bg.setScale(num, BigDecimal.ROUND_DOWN);
        double rv = nbg.doubleValue();
        return rv;
    }

    public static Geometry wktToGeometry(String wktPoint) {
        WKTReader fromText = new WKTReader();
        Geometry geom = null;
        try {
            geom = fromText.read(wktPoint);
        } catch (ParseException e) {
            throw new RuntimeException("Not a WKT string:" + wktPoint);
        }
        return geom;
    }


    /**
     * 构建GEOMtry
     *
     * @param areaType 01矩形 02多边形 03圆形
     * @param coords   01时：左上;右下坐标 02时：x,y;x1,y1;...;03时：圆心坐标[;边界坐标]|[半径]
     * @return
     * @throws Exception
     */
    public static Geometry getGeomtry(String areaType, String coords) throws Exception {
        Geometry geom = null;
//		String sql = "select e from "+poName+" e where within(e.POINT, GeomFromText(:filter)) = true";
        if (StringUtils.equals(areaType, "03")) {
            String[] sps = coords.split("\\|");
            double dis = 0;
            if (sps.length == 1) {
                String[] psps = sps[0].split(";");
                String cp = psps[0];
                String ep = psps[1];
                double cpx = Double.parseDouble(cp.split(",")[0]);
                double cpy = Double.parseDouble(cp.split(",")[1]);
//				geom = GeomsConvertUtil.createPoint(cp);

                double epx = Double.parseDouble(ep.split(",")[0]);
                double epy = Double.parseDouble(ep.split(",")[1]);
                dis = getDistance(cpx, cpy, epx, epy);
//			    geom = GeomsConvertUtil.createCircle(cpx, cpy, dis);

            } else {
                dis = Double.parseDouble(sps[1]);
                geom = GeomsConvertUtil.createPoint(sps[0]);
            }
            //(st_distance 计算的结果单位是度，需要乘111195（地球半径6371000*PI/180）是将值转化为米);
//			sql  ="select e from VehiclePo e where st_distance(e.POINT, GeomFromText(:filter)) <= "+dis/111195;

        } else if (StringUtils.equals(areaType, "01")) {
            String[] points = coords.split(";");
            String[] leftUp = points[0].split(",");
            String[] rightDown = points[1].split(",");
            String xys = points[0] + ";" + rightDown[0] + "," + leftUp[1] + ";";
            xys += points[1] + ";" + leftUp[0] + "," + rightDown[1];
            geom = GeomsConvertUtil.genGeometry(xys);
            System.out.println("rectangle coords:" + GeomsConvertUtil.getPolygonString(geom));
        } else if (StringUtils.equals(areaType, "02")) {
            geom = GeomsConvertUtil.genGeometry(coords);
            System.out.println("multi poly coords:" + GeomsConvertUtil.getPolygonString(geom));
        }

        return geom;
    }

    public static String getMultiLinearString(Geometry geomBuf) {
        String multis = "";
        int num = geomBuf.getNumGeometries();
        for (int i = 0; i < num; i++) {
            Geometry polygon = geomBuf.getGeometryN(i);
            String ps = getLinearRingString(polygon);
            if (i != num - 1)
                multis += ps + "|";
            else
                multis += ps;
        }
        return multis;
    }

    /**
     * 点空间坐标转换为字符串
     *
     * @param geomBuf
     * @return
     */
    public static String getPointGeomString(Point geomBuf) {
        String xys = "";
        Coordinate coord = geomBuf.getCoordinate();
        xys = coord.x + "," + coord.y;
        return xys;
    }


    /**
     * 格式化boundary点为x,y;x1,y1;...
     *
     * @param geomBuf
     * @return
     */
    public static String getLinearRingString(Geometry geomBuf) {
        String geomstring = geomBuf.toString();

        geomstring = geomstring.substring(12, geomstring.length() - 1);
        geomstring = geomstring.replaceAll(", ", ";");
        geomstring = geomstring.replaceAll(" ", ",");
        return geomstring;
    }


    /**
     * 格式化boundary点为x,y;x1,y1;...
     *
     * @param geomBuf
     * @return
     */
    public static String getPolygonString(Geometry geomBuf) {
        String geomstring = geomBuf.toString();

        geomstring = geomstring.substring(10, geomstring.length() - 2);
        geomstring = geomstring.replaceAll(", ", ";");
        geomstring = geomstring.replaceAll(" ", ",");
        return geomstring;
    }

    /**
     * create a Circle 创建一个圆，圆心(x,y) 半径RADIUS
     *
     * @param x
     * @param y
     * @param RADIUS
     * @return
     */
    public static LineString createCircle(double x, double y, final double RADIUS) {
        final int SIDES = 32;// 圆上面的点个数
        Coordinate coords[] = new Coordinate[SIDES + 1];

        for (int i = 0; i < SIDES; i++) {
            double angle = i * Math.PI * 2.0 / SIDES;
            double dx = Math.cos(angle) * RADIUS;
            double dy = Math.sin(angle) * RADIUS;
            double dlng = dx / (Rc * Math.cos(y * Math.PI / 180) * Math.PI / 180);
            double dlat = dy / (Rc * Math.PI / 180);
            double newlng = x + dlng;
            double newlat = y + dlat;
            coords[i] = new Coordinate(newlng, newlat);
        }
        coords[SIDES] = coords[0];
        //coords[SIDES+1] = new Coordinate(x, y);
        LineString ring = geometryFactory.createLineString(coords);
        return ring;
    }

    // 毫秒经纬度转换为浮点经纬度，格式为x,y;x,y
    public static String millsToDu(String coords) {
        String cs = "";
        String cdsplit[] = coords.split(" ");
        int i = 0;
        for (String point : cdsplit) {
            String[] s = point.split(";");
            if (i == cdsplit.length - 1)
                cs += Long.parseLong(s[0]) / 3600000d + ","
                        + Long.parseLong(s[1]) / 3600000d;
            else
                cs += Long.parseLong(s[0]) / 3600000d + ","
                        + Long.parseLong(s[1]) / 3600000d + ";";
            i++;
        }

        return cs;
    }


    /**
     * 获取路段geometry
     *
     * @param points 格式：x,y;x,y
     * @return
     */
    public static LineString getLineString(String points) {
        String[] linepoints = points.split(";");
        Coordinate[] coordinates1 = new Coordinate[linepoints.length];
        int i = 0;
        for (String lp : linepoints) {
            String[] sp = lp.split(",");
            Coordinate cd = new Coordinate(Double.parseDouble(sp[0]),
                    Double.parseDouble(sp[1]));
            coordinates1[i] = cd;
            i++;
        }
        LineString lineString1 = geometryFactory.createLineString(coordinates1);
        return lineString1;
    }

    /**
     * 联合多边形
     *
     * @param geoms
     * @return
     */
    public static GeometryCollection createGeomCollection(Geometry[] geoms) {
        GeometryCollection polygonCollection = geometryFactory
                .createGeometryCollection(geoms);
        return polygonCollection;
    }

    public static MultiLineString createMultiLineString(LineString[] geoms) {
        return geometryFactory.createMultiLineString(geoms);
    }

    private static WKTReader reader = new WKTReader(geometryFactory);

    /**
     * 创建点几何图形
     *
     * @param xy 格式：x,y
     * @author 周士广
     * @date 2019年10月12日
     */
    public static Point createPoint(String xy) {
        if (StringUtils.isNotBlank(xy)) {
            String[] xys = StringUtils.split(StringUtils.trim(xy), ",");
            if (xys.length == 2) {
                Coordinate coord = new Coordinate(Double.parseDouble(xys[0]),
                        Double.parseDouble(xys[1]));
                Point point = geometryFactory.createPoint(coord);
                return point;
            }
        }
        return null;
    }

    /**************************************************************************
     *从指定的原点出发，偏移输入角度后，向此方向延伸输入距离，返回此时的位置
     *origin_lon：原点经度
     *origin_lat：原点纬度
     *azimuth：偏移角度
     *distance：延伸距离
     *ret_lon:返回位置的经度
     *ret_lat:返回位置的纬度
     **************************************************************************/
    public static Double[] azimuth_offset(double origin_lon, double origin_lat, Integer azimuth, double distance) {
        Double[] lonlat = new Double[2];
        if (azimuth != null && azimuth.equals(0) && distance > 0) {
            lonlat[0] = origin_lon + distance * Math.sin(azimuth * Math.PI / 180) * 180 / (Math.PI * 6371229 * Math.cos(origin_lat * Math.PI / 180));
            lonlat[1] = origin_lat + distance * Math.cos(azimuth * Math.PI / 180) / (Math.PI * 6371229 / 180);
        } else {
            lonlat[0] = origin_lon;
            lonlat[1] = origin_lat;
        }
        return lonlat;
    }

    /**
     * 根据源经纬度与目标经纬度计算出距离（米）
     *
     * @param long1 经度1
     * @param lat1  维度1
     * @param long2 经度2
     * @param lat2  纬度2
     * @return
     */
    public static double getDistance(double long1, double lat1, double long2, double lat2) {
        double a, b, R;
        R = 6378137; // 地球半径
        lat1 = lat1 * Math.PI / 180.0;
        lat2 = lat2 * Math.PI / 180.0;
        a = lat1 - lat2;
        b = (long1 - long2) * Math.PI / 180.0;
        double d;
        double sa2, sb2;
        sa2 = Math.sin(a / 2.0);
        sb2 = Math.sin(b / 2.0);
        d = 2 * R * Math.asin(Math.sqrt(sa2 * sa2 + Math.cos(lat1) * Math.cos(lat2) * sb2 * sb2));
        return d;
    }

    public static double getDistance(String xy1,String xy2) {
        String[] p1 = xy1.split(",");
        String[] p2 = xy2.split(",");

        double long1 = Double.parseDouble(p1[0]);
        double lat1 = Double.parseDouble(p1[1]);
        double long2 = Double.parseDouble(p2[0]);
        double lat2  = Double.parseDouble(p2[1]);

        double a, b, R;
        R = 6378137; // 地球半径
        lat1 = lat1 * Math.PI / 180.0;
        lat2 = lat2 * Math.PI / 180.0;
        a = lat1 - lat2;
        b = (long1 - long2) * Math.PI / 180.0;
        double d;
        double sa2, sb2;
        sa2 = Math.sin(a / 2.0);
        sb2 = Math.sin(b / 2.0);
        d = 2 * R * Math.asin(Math.sqrt(sa2 * sa2 + Math.cos(lat1) * Math.cos(lat2) * sb2 * sb2));
        return d;
    }

    /**
     * wkt转x,y;x,y#x,y;x,y格式
     *
     * @param wkt
     * @return
     * @throws ParseException
     */
    //MULTILINESTRING((113.715396 30.398834,113.715199 30.398557,113.714889 30.398129,113.714434 30.397481,113.71393 30.396791,113.713338 30.395962,113.71306 30.395573,113.712279 30.394459,113.711772 30.393743,113.71149 30.393354,113.711324 30.393113))
    public static String formatWkt(String wkt) throws ParseException {
        String xys = "";

        WKTReader wktReader = new WKTReader();
        Geometry geoms = wktReader.read(wkt);
        int size = geoms.getNumGeometries();
        for (int i = 0; i < size; i++) {
            Geometry geom = geoms.getGeometryN(i);
            String text = geom.toText();
            text = text.substring(text.indexOf("(") + 1, text.length() - 1);
            text = text.replaceAll(", ", ";");
            text = text.replaceAll(" ", ",");

            if (i != size - 1) {
                xys += text + "#";
            } else {
                xys += text;
            }
        }

        return xys;
    }

    public static JSONArray formatWktToJsonArray(String wkt) throws ParseException {
        JSONArray list = new JSONArray();

        WKTReader wktReader = new WKTReader();
        Geometry geoms = wktReader.read(wkt);
        int size = geoms.getNumGeometries();
        for (int i = 0; i < size; i++) {
            Geometry geom = geoms.getGeometryN(i);
            String text = geom.toText();
            text = text.substring(text.indexOf("(") + 1, text.length() - 1);
            text = text.replaceAll(", ", ";");
            text = text.replaceAll(" ", ",");
            list.add(text);
        }

        return list;
    }

    /**
     * 道路线集合
     *
     * @param geomBuf
     * @return
     */
    public static JSONArray getMultiLineString(Geometry geomBuf) {
        JSONArray list = new JSONArray();
        int num = geomBuf.getNumGeometries();
        for (int i = 0; i < num; i++) {
            Geometry polygon = geomBuf.getGeometryN(i);
            String ps = getLinearRingString(polygon);
            list.add(ps);
        }
        return list;
    }

    public static JSONArray getMultiPolyString(Geometry geomBuf) {
        JSONArray list = new JSONArray();
        int num = geomBuf.getNumGeometries();
        for (int i = 0; i < num; i++) {
            Geometry polygon = geomBuf.getGeometryN(i);
            String ps = getPolygonString(polygon);
            ps = ps.replaceAll("\\)", "").replaceAll("\\(", "");
            list.add(ps);
        }
        return list;
    }

    public static String getMaxAreaPolygon(Geometry geomBuf) {

        int num = geomBuf.getNumGeometries();
        double maxarea = 0;
        Geometry maxPolygon = null;
        for (int i = 0; i < num; i++) {
            Geometry polygon = geomBuf.getGeometryN(i);
            double area = polygon.getArea();
            if (i == 0) {
                maxarea = area;
                maxPolygon = polygon;
            } else {
                if (area > maxarea) {
                    maxarea = area;
                    maxPolygon = polygon;
                }
            }
        }
        if (maxPolygon != null) {
            String ps = getPolygonString(maxPolygon);
            ps = ps.replaceAll("\\)", "").replaceAll("\\(", "");
            return ps;
        }
        return null;
    }

    /**
     * 创建geom
     *
     * @param points eg:x,y;x,y;...
     * @return
     */
    public static Geometry genGeometry(String points) throws Exception {
        String[] linepoints = points.split(";");
        Coordinate[] coordinates1 = new Coordinate[linepoints.length + 1];
        int i = 0;
        for (String lp : linepoints) {
            String[] sp = lp.split(",");
            Coordinate cd = new Coordinate(Double.parseDouble(sp[0]),
                    Double.parseDouble(sp[1]));
            coordinates1[i] = cd;
            i++;
        }
        coordinates1[coordinates1.length - 1] = coordinates1[0];
        Polygon geom = geometryFactory.createPolygon(coordinates1);
        return geom;

    }

    /**
     * 判断坐标的点point(x,y)是否在geometry表示的Polygon中
     *
     * @param poing
     * @param geometry wkt格式
     * @return
     */
    public static boolean withinGeo(String poing, String geometry) {
        try {
            WKTReader reader = new WKTReader(geometryFactory);
            Point point = createPoint(poing);
            Geometry polygon = genGeometry(geometry);
            //polygon.buffer()
            return point.within(polygon);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 单边缓冲
     */
    public static Geometry singleSidedBufferCurve(Geometry geom, double distance) {
        BufferParameters bufParam = new BufferParameters();
        bufParam.setSingleSided(true);
        OffsetCurveBuilder ocb = new OffsetCurveBuilder(
                geom.getFactory().getPrecisionModel(), bufParam
        );
        Coordinate[] pts = ocb.getLineCurve(geom.getCoordinates(), distance * 0.00001141);
        Geometry curve = geom.getFactory().createLineString(pts);
        return curve;
    }

    /**
     * 米勒投影算法经纬度转平面坐标
     *
     * @param lat 纬度
     * @param lon 经度
     * @return
     * @author liyun
     * @date 2019年8月19日
     */
    public static double[] MillierConvertion(double lat, double lon) {
        //double L = 6381372 * Math.PI * 2;//地球周长
        double L = 6378137 * Math.PI * 2;//地球周长
        double W = L;// 平面展开后，x轴等于周长
        double H = L / 2;// y轴约等于周长一半
        double mill = 2.3;// 米勒投影中的一个常数，范围大约在正负2.3之间
        double x = lon * Math.PI / 180;// 将经度从度数转换为弧度
        double y = lat * Math.PI / 180;// 将纬度从度数转换为弧度
        y = 1.25 * Math.log(Math.tan(0.25 * Math.PI + 0.4 * y));// 米勒投影的转换
        // 弧度转为实际距离
        x = (W / 2) + (W / (2 * Math.PI)) * x;
        y = (H / 2) - (H / (2 * mill)) * y;
        double[] result = new double[2];
        result[0] = x;
        result[1] = y;
        return result;
    }

    /**
     * 米勒投影算法 平面坐标转经纬度
     *
     * @param x 坐标
     * @param y 坐标
     * @return
     * @author liyun
     * @date 2019年8月19日
     */
    public static double[] lonLatConvertion(double x, double y) {
        //double L = 6381372 * Math.PI * 2;//地球周长
        double L = 6378137 * Math.PI * 2;//地球周长
        double W = L;// 平面展开后，x轴等于周长
        double H = L / 2;// y轴约等于周长一半
        double mill = 2.3;// 米勒投影中的一个常数，范围大约在正负2.3之间
        double lat;
        lat = ((H / 2 - y) * 2 * mill) / (1.25 * H);
        lat = ((Math.atan(Math.exp(lat)) - 0.25 * Math.PI) * 180) / (0.4 * Math.PI);
        double lon;
        lon = (x - W / 2) * 360 / W;
        double[] result = new double[2];
        result[0] = (double) Math.round(lon * 100000000) / 100000000;
        result[1] = (double) Math.round(lat * 100000000) / 100000000;
        return result;

    }

    /**
     * 求两个空间几何图形最近距离的坐标点
     *
     * @param geom1
     * @param geom2
     * @author 周士广
     * @date 2019年10月12日
     * return  x,y
     */
    public static String getNearestDistancePoint(Geometry geom1, Geometry geom2) {
        DistanceOp t = new DistanceOp(geom1, geom2);
        Coordinate[] a = t.nearestPoints();
        String arr = a[0].x + "," + a[0].y;
        return arr;
    }

    /**
     * 获geom2到geom1的最近距离
     *
     * @author shiguang.zhou
     * @date 2021年1月14日
     */
    public static double getNearestDistance(String linewkt, String point) {
        Geometry geom1 = getLineString(linewkt);
        Geometry geom2 = createPoint(point);
        DistanceOp t = new DistanceOp(geom1, geom2);
        double dis = t.distance();
        return ArithOfBigDecmial.mul(dis, 6371000 * Math.PI / 180);
    }

    /**
     * 绘制圆形
     *
     * @param lng       圆心坐标精度
     * @param lat       圆心坐标点数
     * @param radius    半径 单位米
     * @param numpoints 圆弧点数
     */
    public static String addPolylinescircle(double lng, double lat, double radius, int numpoints) {
        double r = 6371000.79;

        double phase = 2 * Math.PI / numpoints;
        StringBuilder xys = new StringBuilder();
        //画图
        for (int i = 0; i < numpoints; i++) {
            /**
             * 计算坐标点
             */
            double dx = (radius * Math.cos(i * phase));
            double dy = (radius * Math.sin(i * phase));//乘以1.6 椭圆比例

            /**
             * 转换成经纬度
             */
            double dlng = dx / (r * Math.cos(lat * Math.PI / 180) * Math.PI / 180);
            double dlat = dy / (r * Math.PI / 180);
            double newlng = lng + dlng;
            xys.append(newlng);
            xys.append(",");
            xys.append(lat + dlat);
            if (i != numpoints - 1)
                xys.append(";");
        }
        return xys.toString();

    }

    /**
     * 根据曲线百分比位置获取响应的点坐标
     *
     * @param coords   坐标串，格式：x1,y1;x2,y2;...;xn,yn
     * @param segIndex 线段索引号
     * @param fraction 百分比，范围 :[0,1]
     * @author 周士广
     * @date 2019年11月25日
     */
    public static String getCureLinePointByFraction(String coords, int segIndex, double fraction) {
        // 根据曲线位置获取响应的点坐标
        LineString ls = GeomsConvertUtil.getLineString(coords);
        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);
        Coordinate point = indexedLine.extractPoint(new LinearLocation(segIndex, fraction));
        String ptxy = point.x + "," + point.y;
        return ptxy;
    }

    /**
     * 根据给定距离，寻找线段中距离终点的坐标位置
     *
     * @param coords 坐标串，格式：x1,y1;x2,y2;...;xn,yn
     * @param dist   单位米
     * @param loc    定位dist距离位置是在第几个线段，以及终点在该线段中所占的百 分比
     */
    public static Coordinate getLocationByDistance(String coords, Double dist, LinearLocation loc) {

        LineString ls = GeomsConvertUtil.getLineString(coords);

        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);

        // 获取距离终点位置坐标
        Coordinate point = indexedLine.extractPoint(loc);

        return point;
    }

    /**
     * 提取子线段
     *
     * @param coords  x,y;x1,y1;...,xn,yn
     * @param inputPt
     * @param posFlg  1：inputPt参数作为终点的所有线段  2：inputPt参数作为起点到尾部所有线段 3：截取到inputPt点所在的线段
     * @author shiguang.zhou
     * @date 2020年7月15日
     */
    public static Geometry getSubLineString(String coords, Coordinate inputPt, int posFlg) {
        LineString ls = GeomsConvertUtil.getLineString(coords);

        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);
        //线段的起始
        LinearLocation startLoc = indexedLine.getStartIndex();
        //线段的终点索引
        LinearLocation endLoc = indexedLine.getEndIndex();
        //查找点在第几个线段
        LinearLocation indexLoc = indexedLine.indexOf(inputPt);
        if (posFlg == 2) {
            startLoc = indexLoc;
            indexLoc = endLoc;
        } else if (posFlg == 3) {
            startLoc = indexLoc;
            //截取到inputPt所在线段终点
            indexLoc = new LinearLocation(indexLoc.getSegmentIndex(), 1);
        }

        double frac = indexLoc.getSegmentFraction();
        //截取子线段
        Geometry geom = indexedLine.extractLine(startLoc, indexLoc);

        return geom;
    }

    /**
     * 获取从开始点到终止点的线段坐标
     *
     * @param coords   x,y;x1,y1;...,xn,yn
     * @param indexLoc 开始点
     * @param type     1：从开始点延划线方向向后取（indexLoc作为起点） 2：从开始点延划线方向向前取(indexLoc作为终点)
     * @author shiguang.zhou
     * @date 2020年7月15日
     */
    public static String getSubLineStringByLoc(String coords, LinearLocation indexLoc, int type) {
        String retwkt = null;
        LineString ls = GeomsConvertUtil.getLineString(coords);

        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);

        LinearLocation startLoc = indexedLine.getStartIndex();
        LinearLocation endLoc = indexedLine.getEndIndex();

        try {
            Geometry subgeom = null;
            if (type == 1) {
                subgeom = indexedLine.extractLine(indexLoc, endLoc);
            } else if (type == 2) {
                subgeom = indexedLine.extractLine(startLoc, indexLoc);
            }
            retwkt = formatWkt(subgeom.toText());
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return retwkt;
    }

    /**
     * 查找点在第几个线段及在这个线段中占据的百分比
     *
     * @author shiguang.zhou
     * @date 2020年9月29日
     */
    public static LinearLocation getFraction(String coords, String xy) {
        String[] sps = xy.split(",");
        Coordinate pt = new Coordinate(Double.valueOf(sps[0]), Double.valueOf(sps[1]));
        LineString ls = GeomsConvertUtil.getLineString(coords);
        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);
        //查找点在第几个线段
        LinearLocation loc = indexedLine.indexOf(pt);
        return loc;
    }

    public static Geometry getSubLineString(String coords, Coordinate startPt, Coordinate endPt) {
        LineString ls = GeomsConvertUtil.getLineString(coords);

        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);

        LinearLocation indexLocTmp = null;
        //查找点在第几个线段
        LinearLocation startindexLoc = indexedLine.indexOf(startPt);
        //查找点在第几个线段
        LinearLocation endindexLoc = indexedLine.indexOf(endPt);

        if (endindexLoc.getSegmentIndex() < startindexLoc.getSegmentIndex()) {
            indexLocTmp = startindexLoc;
            startindexLoc = endindexLoc;
            endindexLoc = indexLocTmp;
        }

        //截取子线段
        Geometry geom = indexedLine.extractLine(startindexLoc, endindexLoc);

        return geom;
    }

    /**
     * 路段长度，单位米
     *
     * @param coords 线段坐标 格式：x1,y1;x2,y2;...
     * @author shiguang.zhou
     * @date 2020年4月14日
     */
    public static double getSegmentTotalLeng(String coords) {
        double totalLen = 0;

        LineString ls = GeomsConvertUtil.getLineString(coords);

//		LocationIndexedLine indexedLine = new LocationIndexedLine(ls);
//
//		LinearLocation startLoc = indexedLine.getStartIndex();
//		LinearLocation endLoc = indexedLine.getEndIndex();
//
//		int startIndex = startLoc.getSegmentIndex();
//		int endIndex = endLoc.getSegmentIndex();
//
//		while (startIndex < endIndex) {
//			LinearLocation loc = new LinearLocation(startIndex, 0);
//			LineSegment lseg = loc.getSegment(ls);
//			double leng = lseg.getLength();// 单位为度
//			leng = ArithOfBigDecmial.mul(leng, 6371000 * Math.PI / 180);// 度转换为米
//			totalLen += leng;
//			startIndex++;
//		}
//		totalLen = ArithOfBigDecmial.round(totalLen, 2);

        totalLen = ArithOfBigDecmial.mul(ls.getLength(), 6371000 * Math.PI / 180);
        totalLen = ArithOfBigDecmial.round(totalLen, 2);
        return totalLen;
    }

    /**
     * 定位dist距离位置是在第几个线段，以及终点在该线段中所占的百 分比
     *
     * @author shiguang.zhou
     * @date 2020年4月14日
     */
    public static LinearLocation getLinearLocation(String coords, Double dist) {

        LineString ls = GeomsConvertUtil.getLineString(coords);

        double degree = dist / (6371000 * Math.PI / 180);// 米转换为度

        // 定位dist距离位置是在第几个线段，以及终点在该线段中所占的百 分比
        LinearLocation loc = LengthLocationMap.getLocation(ls, degree);

        // System.out.println(loc.getSegmentIndex() + "," + loc.getSegmentFraction());
        return loc;
    }

    /**
     * 计算两个几何对象间的距离
     *
     * @return 距离单位米
     * @author shiguang.zhou
     * @date 2020年4月14日
     */
    public static double getDistance(Geometry a, Geometry b) {
        double dist = a.distance(b);// 单位为度
        dist = ArithOfBigDecmial.mul(dist * 6371000, Math.PI);// 6371000*Math.PI/180;//度转换为米
        dist = ArithOfBigDecmial.div(dist, 180);
        return dist;
    }

    /**
     * 度转换为米
     *
     * @author shiguang.zhou
     * @date 2021年1月20日
     */
    public static double duToMeter(double du) {
        return ArithOfBigDecmial.mul(du * 6371000, Math.PI);
    }

    /**
     * 米转换为度
     *
     * @author shiguang.zhou
     * @date 2021年3月9日
     */
    public static double meterToDu(double dist) {
        double degree = dist / (6371000 * Math.PI / 180);// 米转换为度
        return degree;
    }


    public static String getCureLineSegmentByFraction(String coords, int startSegIndex, int endSegIndex,
                                                      double fraction) {
        String subline = null;
        LineString ls = GeomsConvertUtil.getLineString(coords);
        LocationIndexedLine indexedLine = new LocationIndexedLine(ls);
        Geometry eline = indexedLine.extractLine(new LinearLocation(startSegIndex, fraction), new LinearLocation());
        try {
            subline = GeomsConvertUtil.formatWkt(eline.toString());
            int index = subline.indexOf("#");
            if (index > 0)
                subline = subline.substring(0, index);
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return subline;
    }

    /**
     * 判断点是否在区域内
     *
     * @param areaWkt 格式：x1,y1;x2,y2;...;xn,yn
     * @param point   格式：x,y
     * @author shiguang.zhou
     * @date 2021年1月14日
     */
    public static boolean isInArea(String areaWkt, String point) {
        try {
            Geometry areaGeom = genGeometry(areaWkt);
            Point ptgeom = createPoint(point);
            return ptgeom.within(areaGeom);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }

    /**
     * 根据行驶距离计算位置坐标点
     *
     * @author shiguang.zhou
     * @date 2020年8月25日
     */
    public static Double[] getVhcPosPoint(String lanecoord, double distance) {
        LinearLocation lineLoc = GeomsConvertUtil.getLinearLocation(lanecoord, distance);
        Coordinate coord = GeomsConvertUtil.getLocationByDistance(lanecoord, distance,
                lineLoc);
        double x = coord.x;
        double y = coord.y;
        return new Double[]{x, y};
    }

    public static boolean isCanPassCloseArea(String areaWkt, String linexy) {
        try {
            Geometry areaGeom = genGeometry(areaWkt);
            LineString ls = getLineString(linexy);
            return areaGeom.intersects(ls);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }

    public static Geometry unionGeo(Geometry a, Geometry b) {
        return a.union(b);
    }

    /**
     * 按照两条线  距离截取n个面
     *
     * @param lineWkt1 边缘线1
     * @param lineWkt2 边缘线2
     * @param distince 截取的面的长度
     * @param num      截取面的个数
     * @param type     进口需要反转（1进口）
     * @return
     */
    public static List<String> getSubPolygonList(String lineWkt1, String lineWkt2, double distince, int num, int type) {
        if (type == 1) {
            lineWkt1 = reWkt(lineWkt1);
            lineWkt2 = reWkt(lineWkt2);
        }

        String apoint = lineWkt1.split(";")[0];
        String bpoint = lineWkt2.split(";")[0];

        List<String> pList = new ArrayList<>();
        String bpoint1 = "";
        for (int i = 1; i < num; i++) {
            Double[] point = getVhcPosPoint(lineWkt1, distince * i);
            Double[] point2 = getVhcPosPoint(lineWkt2, distince * i);
            String apoint1 = point[0] + "," + point[1];
            if ((point2[0] + "," + point2[1]).equals(bpoint1)) {
                break;
            }
            bpoint1 = point2[0] + "," + point2[1];
            String po = apoint + ";" + apoint1 + ";" + bpoint1 + ";" + bpoint + ";" + apoint;
            pList.add(po);
            apoint = apoint1;
            bpoint = bpoint1;
        }
        return pList;
    }


    public static String reWkt(String wkt) {
        String[] aktArr = wkt.split(";");

        StringBuilder wks = new StringBuilder();
        for (int i = aktArr.length - 1; i >= 0; i--) {
            wks.append(";").append(aktArr[i]);
        }

        if (wks.length() == 0) {
            return "";
        } else {
            return wks.substring(1);
        }
    }

    /**
     * 合并多条线
     *
     * @return
     */
    public List<String> lineMerger(List<Geometry> list) {
        LineMerger lineMerger = new LineMerger();
        lineMerger.add(list);
        Collection<Geometry> mergerLineStrings = lineMerger.getMergedLineStrings();

        List<String> lineList = new ArrayList<>();
        for (Geometry g : mergerLineStrings) {
            lineList.add(g.toText());
        }
        return lineList;
    }

    //判断一个点是否在多边形区域内
    private static boolean isInPolygon(double longitude, double latitude, double[][] points) {
        //将要判断的点的经纬度组成一个点
        Point2D.Double point = new Point2D.Double(longitude, latitude);
        //将区域各个顶点的坐标放到一个点集合里
        ArrayList<Point2D.Double> point2DS = new ArrayList<>();
        for (int i = 0; i < points.length; i++) {
            double polygonPoint_x = points[i][0];//经度
            double polygonPoint_y = points[i][1];//纬度

            Point2D.Double polygonPoint = new Point2D.Double(polygonPoint_x, polygonPoint_y);
            point2DS.add(polygonPoint);

        }

        return check(point, point2DS);
    }

    //points的数据格式是:lon1,lat1;lon2,lat2
    public static boolean isInPolygon(double longitude, double latitude, String points) {
        return withinGeo(longitude + "," + latitude, points);
    }

    private static boolean check(Point2D.Double point, ArrayList<Point2D.Double> polygon) {
        GeneralPath generalPath = new GeneralPath();
        Point2D.Double firstPoint = polygon.get(0);

        generalPath.moveTo(firstPoint.x, firstPoint.y);
        polygon.remove(0);

        for (Point2D.Double polygonPoint : polygon) {
            generalPath.lineTo(polygonPoint.x, polygonPoint.y);
        }

        //将几何多边形封闭
        generalPath.lineTo(firstPoint.x, firstPoint.y);
        generalPath.closePath();

        //测试指定点是否在该区域边界内
        return generalPath.contains(point);
    }


    /**
     * 根据中心点经纬度、半径、起止角度生成扇形
     * @param x 经度
     * @param y 纬度
     * @param radius 半径（米）
     * @param bAngle 起始角度（0~360）
     * @param eAngle 终止角度（0~360）
     * @return
     */
    public static Polygon createSector(double x,double y,double radius,double bAngle,double eAngle){
        List<Coordinate> points = new ArrayList<>();
        points.add(new Coordinate(x,y));
        while (bAngle<eAngle){
            Coordinate p1 = getLatLngPointByDistanceAndAngle(new double[]{x,y},  radius, bAngle);
            points.add(p1);
            bAngle=bAngle+5;
        }
        points.add(new Coordinate(x,y));
        Polygon polygon = geometryFactory.createPolygon(points.toArray(new Coordinate[]{}));
        return polygon;
    }

    /**
     * 求距离此点一定距离、一定夹角的点经纬度
     * https://blog.csdn.net/cdl2008sky/article/details/131963702
     * @param distance
     *            两点之间的距离 单位 m
     * @param angle
     *            两点连线与正北方向的夹角（0~360）
     * @return 算出的点的经纬度
     */
    public static Coordinate getLatLngPointByDistanceAndAngle(double[] startPoint,double distance, double angle) {

        // distance = meterToDu(distance);

        double radLongitude = startPoint[0] * Math.PI / 180.;
        double radLatitude = startPoint[1] * Math.PI / 180.;
        double Ec = Rj + (Rc - Rj) * (90. - startPoint[1]) / 90.;
        double Ed = Ec * Math.cos(radLatitude);

        double dx = distance * Math.sin(Math.toRadians(angle));
        double dy = distance * Math.cos(Math.toRadians(angle));

        double bjd = new BigDecimal((dx /  Ed + radLongitude) * 180. / Math.PI)
                .setScale(7, BigDecimal.ROUND_HALF_UP).doubleValue();
        double bwd = new BigDecimal((dy / Ec + radLatitude) * 180. / Math.PI)
                .setScale(7, BigDecimal.ROUND_HALF_UP).doubleValue();

        return new Coordinate(bjd, bwd);
    }

    public static  Geometry fromHexWkb(String wkbHex)  {

        WKBReader wkbReader = new WKBReader(geometryFactory);
        byte[] wkb = WKBReader.hexToBytes(wkbHex);
        Geometry g2 = null;
        try {
            g2 = wkbReader.read(wkb);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return g2;
    }

    public static void main(String[] args) throws Exception {

        String wkt = "121.424661,31.39226;121.424881,31.39151;121.427819,31.381745;121.427953,31.381296;121.428082,31.380837;121.429196,31.376526;121.429994,31.373874;121.430299,31.372705;121.430537,31.371323;121.430723,31.370427;121.430914,31.369555;121.431103,31.368691;121.43127,31.367942;121.432941,31.362217;121.433261,31.360896;121.433515,31.35981;121.433691,31.358866;121.43398,31.35595;121.434063,31.355082;121.434143,31.354235;121.434701,31.348262;121.434975,31.347014;121.435106,31.346518;121.435187,31.346232;121.435625,31.345065;121.435902,31.344285;121.436606,31.342377;121.437375,31.340518;121.437711,31.339703;121.438044,31.338884;121.439232,31.33604;121.441546,31.33113;121.443878,31.326024;121.445757,31.321816;121.446752,31.31966;121.447063,31.318936;121.447369,31.318237;121.447905,31.316858;121.448035,31.316267;121.448187,31.315546;121.448293,31.314782;121.448497,31.313164;121.448623,31.310281;121.448696,31.307901;121.44866,31.307303;121.448642,31.306604;121.448625,31.305913;121.448931,31.300276;121.448984,31.299198;121.449126,31.298117;121.449373,31.296527;121.450076,31.293463;121.450251,31.292556;121.450429,31.291631;121.450841,31.289895;121.451269,31.287919;121.451429,31.286805;121.451543,31.285801;121.451693,31.284324;121.451924,31.28113;121.452023,31.279895;121.452124,31.278661;121.45231,31.277347;121.452461,31.276862;121.452637,31.276333;121.453007,31.275443;121.453265,31.274885;121.454004,31.273619;121.455329,31.271675;121.45583,31.27095;121.457568,31.268372;121.457939,31.267754;121.458115,31.267362;121.458265,31.26697;121.458421,31.265833;121.458557,31.263704;121.45866,31.26213;121.458817,31.260169;121.45895,31.259749;121.459204,31.258891;121.45946,31.258028;121.459485,31.257836;121.459485,31.257535;121.459496,31.256501;121.459479,31.255648;121.459418,31.254958;121.459025,31.252636;121.458995,31.252406;121.459004,31.252209;121.459112,31.250582;121.459083,31.250372;121.459011,31.250177;121.458846,31.25002;121.458636,31.249863;121.457204,31.249166;121.456868,31.248956;121.456573,31.248734;121.456389,31.248532;121.456215,31.248302;121.456047,31.247995;121.455684,31.247255;121.455313,31.2465;121.455125,31.246055;121.455018,31.245642;121.454982,31.245213;121.454998,31.244865;121.455065,31.244482;121.455181,31.244115;121.455346,31.243768;121.455529,31.243495;121.455743,31.243257;121.456017,31.242991;121.456234,31.242817;121.456465,31.242656;121.456712,31.242519;121.456953,31.242422;121.457267,31.242317;121.457648,31.242207;121.458699,31.241883;121.459971,31.241503;121.460583,31.241306;121.461344,31.24104;121.467182,31.23874;121.468151,31.238373;121.469041,31.238051;121.470426,31.237687;121.471569,31.237297;121.471853,31.237137;121.472116,31.236953;121.472481,31.236665;121.472974,31.236165;121.474106,31.234365;121.475137,31.232781;121.476134,31.231231;121.477547,31.22947;121.478086,31.228782;121.478159,31.228679;121.478189,31.228622;121.478233,31.22853;121.478277,31.228423;121.478386,31.228174;121.47843,31.227951;121.478461,31.227758;121.478468,31.227702;121.478473,31.22761;121.478472,31.22719;121.478448,31.226138;121.478445,31.225997;121.478438,31.225904;121.478417,31.225777;121.478378,31.22561;121.478332,31.225509;121.478246,31.225362;121.478035,31.225136;121.47761,31.224797;121.477185,31.224532;121.476407,31.224124;121.475127,31.223526;121.474212,31.223128;121.473306,31.222745;121.472383,31.222356;121.466695,31.22004;121.461164,31.217746;121.459957,31.217231;121.458745,31.216717;121.452162,31.213851;121.451048,31.213382;121.44972,31.212813;121.449563,31.212682;121.449412,31.212538;121.449314,31.212327;121.449314,31.212087;121.449296,31.211233;121.449211,31.210127;121.448814,31.208214;121.448792,31.208145;121.448652,31.207754;121.448576,31.207569;121.448494,31.207395;121.448466,31.207353;121.447669,31.206248;121.446883,31.205164;121.446424,31.204528;121.445965,31.203891;121.444296,31.201692;121.443138,31.200113;121.442022,31.198663;121.440277,31.196306;121.439249,31.194984;121.439126,31.194794;121.439068,31.194684;121.439012,31.194564;121.438969,31.194472;121.438133,31.191698;121.437404,31.189259;121.437235,31.188728;121.437119,31.188241;121.436961,31.187457;121.436745,31.186035;121.436528,31.184209;121.436272,31.182001;121.436151,31.180969;121.435985,31.179869;121.435917,31.179221;121.435513,31.175943;121.435218,31.173487;121.435111,31.17245;121.435031,31.171169;121.434848,31.169209;121.434702,31.168228;121.434551,31.167266;121.434476,31.166688;121.43411,31.163133;121.434067,31.162858;121.434014,31.162583;121.433201,31.159207;121.432658,31.157705;121.432278,31.157017;121.431802,31.156247;121.430796,31.155095;121.430041,31.154579;121.429217,31.154016;121.428206,31.15342;121.424098,31.151061;121.420987,31.148834;121.417147,31.145353;121.416352,31.144585;121.415073,31.143098;121.414604,31.142675;121.414107,31.142217;121.413622,31.141771;121.409971,31.138218;121.405856,31.133814;121.403605,31.131565;121.40291,31.1309;121.402241,31.130274;121.401049,31.129156;121.400066,31.128214;121.398684,31.126855;121.396102,31.124412;121.395084,31.123459;121.3947,31.123076;121.394223,31.122539;121.393845,31.122059;121.393524,31.121616;121.39302,31.120899;121.392492,31.120155;121.39125,31.118396;121.390298,31.117133;121.38802,31.114151;121.387549,31.113551;121.38586,31.11171;121.385373,31.111152";
        wkt = "117.08968885679263,36.64402151162922;117.08975123301514,36.64404303770529;117.08983574273617,36.64387136708481;117.08980623140542,36.643856836951684;117.08977336651361,36.643865447401225;117.08976330583317,36.64386813816647;117.08968885679263,36.64402151162922";

        boolean isInBmxArea = GeomsConvertUtil.withinGeo("117.08970699999999,36.6440276", wkt);

        System.out.println(isInBmxArea);

        System.out.println(offsetCureLine(wkt, 1));
        System.out.println(offsetCure(wkt, 1));

        Point point = createPoint("121.424661,31.39226");
        Geometry geometry = point.buffer(meterToDu(10));
        System.out.println(formatWkt(geometry.toText()));
		/* LineString ls = GeomsConvertUtil.getLineString(wkt);

        LocationIndexedLine lil = new LocationIndexedLine(ls);
        LinearLocation start = lil.indexOf(new Coordinate(8, 5));
        LinearLocation end = lil.indexOf(new Coordinate(17, 10));
        Geometry result = lil.extractLine(start, end);
        System.out.println(result.toText());*/

//		 Geometry result = GeomsConvertUtil.getSubLineString(wkt, new Coordinate(121.385373,31.111152), new Coordinate(121.393020,31.120899));
//		 System.out.println(result.toText());
//
//		 Geometry result1 = GeomsConvertUtil.getSubLineString(wkt, new Coordinate(121.393025,31.120899), new Coordinate(121.385373,31.111152));
//		 System.out.println(result1.toText());
    }
}
	
