package com.wanji.indicators.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HttpUtil {
  private static final Logger logger = LoggerFactory.getLogger(HttpUtil.class);
  
  public static byte[] httpRequest(String urlStr, String method, String content, String encoding, int timeOut, Map<String, String> headProperty) {
    HttpURLConnection conn = null;
    byte[] btemp = null;
    OutputStreamWriter out = null;
    InputStream input = null;
    ByteArrayOutputStream bout = new ByteArrayOutputStream();
    try {
      URL url = new URL(urlStr);
      conn = (HttpURLConnection)url.openConnection();
      conn.setRequestMethod(method.toUpperCase());
      conn.setConnectTimeout(timeOut * 1000);
      conn.setReadTimeout(timeOut * 1000);
      conn.setDoInput(true);
      conn.setDoOutput(true);
      if (headProperty != null)
        for (String key : headProperty.keySet())
          conn.addRequestProperty(key, headProperty.get(key));  
      conn.connect();
      if (conn.getRequestMethod() == null || 
        !conn.getRequestMethod().equalsIgnoreCase("get")) {
        out = new OutputStreamWriter(conn.getOutputStream(), encoding);
        out.write(content);
        out.close();
      } 
      input = conn.getInputStream();
      byte[] bufferByte = new byte[256];
      int l = -1;
      while ((l = input.read(bufferByte)) > -1) {
        bout.write(bufferByte, 0, l);
        bout.flush();
      } 
      btemp = bout.toByteArray();
      int code = conn.getResponseCode();
      String msg = conn.getResponseMessage();
      Map<String, List<String>> maps = conn.getHeaderFields();
      bout.close();
      input.close();
    } catch (Exception e) {
      logger.error("", e);
    } finally {
      try {
        if (input != null)
          input.close(); 
        if (out != null)
          out.close(); 
        if (conn != null)
          conn.disconnect(); 
      } catch (IOException iOException) {}
    } 
    return btemp;
  }
  
  public static void main(String[] args) {
    Base64.Encoder encoder = Base64.getEncoder();
    String cnt = "elastic:elastic";
    String authorization = encoder.encodeToString(cnt.getBytes(StandardCharsets.UTF_8));
    Map<String, String> headerMap = new HashMap<>();
    headerMap.put("Authorization", "Basic " + authorization);
    headerMap.put("Content-Type", "application/json");
    String url = "http://192.168.11.161:9200/cross.event.state-2022-02/_search?pretty";
    String postParams = "{\n  \"query\": {\n   \"bool\": {\n     \"must\": [\n       {\n         \"match\": {\n           \"typeCode\": \"30\"\n         } \n       },\n       {\n         \"match\": {\n           \"type\": \"cross.event.state\"\n         }\n       }\n     ]\n   }\n  }\n}";
    byte[] ret = httpRequest(url, "post", postParams, "utf-8", 10, headerMap);
    System.out.println(new String(ret));
  }
}
