package com.wanji.indicators.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

public class PtInPolyUtil {
  private static double EARTH_RADIUS = 6378.137D;
  
  private static final double R = 6371000.0D;
  
  private static double rad(double d) {
    return d * Math.PI / 180.0D;
  }
  
  public static double getDistance(double lng1, double lat1, double lng2, double lat2) {
    double radLat1 = rad(lat1);
    double radLat2 = rad(lat2);
    double difference = radLat1 - radLat2;
    double mdifference = rad(lng1) - rad(lng2);
    double distance = 2.0D * Math.asin(Math.sqrt(Math.pow(Math.sin(difference / 2.0D), 2.0D) + 
          Math.cos(radLat1) * Math.cos(radLat2) * 
          Math.pow(Math.sin(mdifference / 2.0D), 2.0D)));
    distance *= EARTH_RADIUS;
    distance *= 1000.0D;
    return distance;
  }
  
  public static double getAngle(double lng_a, double lat_a, double lng_b, double lat_b) {
    double longitude1 = lng_a;
    double longitude2 = lng_b;
    double latitude1 = Math.toRadians(lat_a);
    double latitude2 = Math.toRadians(lat_b);
    double longDiff = Math.toRadians(longitude2 - longitude1);
    double y = Math.sin(longDiff) * Math.cos(latitude2);
    double x = Math.cos(latitude1) * Math.sin(latitude2) - Math.sin(latitude1) * Math.cos(latitude2) * Math.cos(longDiff);
    return (Math.toDegrees(Math.atan2(y, x)) + 360.0D) % 360.0D;
  }
  
  public static double pointToLine(double x1, double y1, double x2, double y2, double x0, double y0) {
    double space = 0.0D;
    double a = getDistance(x1, y1, x2, y2);
    double b = getDistance(x1, y1, x0, y0);
    double c = getDistance(x2, y2, x0, y0);
    if (c + b == a) {
      space = 0.0D;
      return space;
    } 
    if (a <= 1.0E-6D) {
      space = b;
      return space;
    } 
    if (c * c >= a * a + b * b) {
      space = b;
      return space;
    } 
    if (b * b >= a * a + c * c) {
      space = c;
      return space;
    } 
    double p = (a + b + c) / 2.0D;
    double s = Math.sqrt(p * (p - a) * (p - b) * (p - c));
    space = 2.0D * s / a;
    return space;
  }
  
  public static String getCenterOfGravityPoint(String mPoints) {
    String[] arr = mPoints.split(";");
    double x = 0.0D;
    double y = 0.0D;
    int len = arr.length;
    for (int i = 0; i < len; i++) {
      String[] xys = arr[i].split(",");
      x += Double.valueOf(xys[0]).doubleValue();
      y += Double.valueOf(xys[1]).doubleValue();
    } 
    x /= len;
    y /= len;
    return x + "," + y;
  }
  
  public static boolean isPtInPoly(Point point, List<Point> pts) {
    int N = pts.size();
    boolean boundOrVertex = true;
    int intersectCount = 0;
    double precision = 2.0E-10D;
    Point p = point;
    Point p1 = pts.get(0);
    for (int i = 1; i <= N; i++) {
      if (p.equals(p1))
        return boundOrVertex; 
      Point p2 = pts.get(i % N);
      if (p.x < Math.min(p1.x, p2.x) || p.x > Math.max(p1.x, p2.x)) {
        p1 = p2;
      } else {
        if (p.x > Math.min(p1.x, p2.x) && p.x < Math.max(p1.x, p2.x)) {
          if (p.y <= Math.max(p1.y, p2.y)) {
            if (p1.x == p2.x && p.y >= Math.min(p1.y, p2.y))
              return boundOrVertex; 
            if (p1.y == p2.y) {
              if (p1.y == p.y)
                return boundOrVertex; 
              intersectCount++;
            } else {
              double xinters = (p.x - p1.x) * (p2.y - p1.y) / (p2.x - p1.x) + p1.y;
              if (Math.abs(p.y - xinters) < precision)
                return boundOrVertex; 
              if (p.y < xinters)
                intersectCount++; 
            } 
          } 
        } else if (p.x == p2.x && p.y <= p2.y) {
          Point p3 = pts.get((i + 1) % N);
          if (p.x >= Math.min(p1.x, p3.x) && p.x <= Math.max(p1.x, p3.x)) {
            intersectCount++;
          } else {
            intersectCount += 2;
          } 
        } 
        p1 = p2;
      } 
    } 
    if (intersectCount % 2 == 0)
      return false; 
    return true;
  }
  
  public static Map<String, Object> findNeighPosition(double longitude, double latitude) {
    double r = 6371.0D;
    double dis = 5.0D;
    double dlng = 2.0D * Math.asin(Math.sin(dis / 2.0D * r) / Math.cos(latitude * Math.PI / 180.0D));
    dlng = dlng * 180.0D / Math.PI;
    double dlat = dis / r;
    dlat = dlat * 180.0D / Math.PI;
    double minlat = latitude - dlat;
    double maxlat = latitude + dlat;
    double minlng = longitude - dlng;
    double maxlng = longitude + dlng;
    Map<String, Object> map = new HashMap<>();
    map.put("minlat", Double.valueOf(minlat));
    map.put("maxlat", Double.valueOf(maxlat));
    map.put("minlng", Double.valueOf(minlng));
    map.put("maxlng", Double.valueOf(maxlng));
    return map;
  }
  
  public static void main(String[] args) {
    System.out.println(getDir(184.0D));
  }
  
  public static String getDir(double angle) {
    if (angle > 337.5D || angle <= 22.5D)
      return "北方向"; 
    if (angle > 22.5D && angle <= 67.5D)
      return "东北方向"; 
    if (angle > 67.5D && angle <= 112.5D)
      return "东方向"; 
    if (angle > 112.5D && angle <= 157.5D)
      return "东南方向"; 
    if (angle > 157.5D && angle <= 202.5D)
      return "南方向"; 
    if (angle > 202.5D && angle <= 247.5D)
      return "西南方向"; 
    if (angle > 247.5D && angle <= 292.5D)
      return "西方向"; 
    if (angle > 292.5D && angle <= 337.5D)
      return "西北方向"; 
    return "";
  }
  
  public static String getDirEn(double angle) {
    if (angle > 337.5D || angle <= 22.5D)
      return "North direction"; 
    if (angle > 22.5D && angle <= 67.5D)
      return "Northeast direction"; 
    if (angle > 67.5D && angle <= 112.5D)
      return "East direction"; 
    if (angle > 112.5D && angle <= 157.5D)
      return "Southeast direction"; 
    if (angle > 157.5D && angle <= 202.5D)
      return "South direction"; 
    if (angle > 202.5D && angle <= 247.5D)
      return "Southwest direction"; 
    if (angle > 247.5D && angle <= 292.5D)
      return "West direction"; 
    if (angle > 292.5D && angle <= 337.5D)
      return "Northwest direction"; 
    return "";
  }
  
  public static double[] calLocationByDistanceAndLocationAndDirection(double angle, double startLong, double startLat, double distance) {
    double[] result = new double[2];
    double x = distance / 6371000.0D;
    angle = Math.toRadians(angle);
    startLong = Math.toRadians(startLong);
    startLat = Math.toRadians(startLat);
    double lat = Math.asin(Math.sin(startLat) * Math.cos(x) + Math.cos(startLat) * Math.sin(x) * Math.cos(angle));
    double lon = startLong + Math.atan2(Math.sin(angle) * Math.sin(x) * Math.cos(startLat), Math.cos(x) - Math.sin(startLat) * Math.sin(lat));
    lon = Math.toDegrees(lon);
    lat = Math.toDegrees(lat);
    result[0] = lon;
    result[1] = lat;
    return result;
  }
  
  public static int segIntersect(double ax, double ay, double bx, double by, double cx, double cy, double dx, double dy) {
    return segIntersect(new Point(ax, ay), new Point(bx, by), new Point(cx, cy), new Point(dx, dy));
  }
  
  private static int segIntersect(Point A, Point B, Point C, Point D) {
    Point intersection = new Point();
    if (Math.abs(B.getY() - A.getY()) + Math.abs(B.getX() - A.getX()) + Math.abs(D.getY() - C.getY()) + 
      Math.abs(D.getX() - C.getX()) == 0.0D)
      return 0; 
    if (Math.abs(B.getY() - A.getY()) + Math.abs(B.getX() - A.getX()) == 0.0D)
      return 0; 
    if (Math.abs(D.getY() - C.getY()) + Math.abs(D.getX() - C.getX()) == 0.0D)
      return 0; 
    if ((B.getY() - A.getY()) * (C.getX() - D.getX()) - (B.getX() - A.getX()) * (C.getY() - D.getY()) == 0.0D)
      return 0; 
    intersection
      .setX(((B.getX() - A.getX()) * (C.getX() - D.getX()) * (C
        .getY() - A.getY()) - C.getX() * (B
        .getX() - A.getX()) * (C.getY() - D.getY()) + A
        .getX() * (B.getY() - A.getY()) * (C.getX() - D.getX())) / ((B
        .getY() - A.getY()) * (C.getX() - D.getX()) - (B
        .getX() - A.getX()) * (C.getY() - D.getY())));
    intersection
      .setY(((B.getY() - A.getY()) * (C.getY() - D.getY()) * (C
        .getX() - A.getX()) - C.getY() * (B
        .getY() - A.getY()) * (C.getX() - D.getX()) + A
        .getY() * (B.getX() - A.getX()) * (C.getY() - D.getY())) / ((B
        .getX() - A.getX()) * (C.getY() - D.getY()) - (B
        .getY() - A.getY()) * (C.getX() - D.getX())));
    if ((intersection.getX() - A.getX()) * (intersection.getX() - B.getX()) <= 0.0D && (intersection
      .getX() - C.getX()) * (intersection
      .getX() - D.getX()) <= 0.0D && (intersection
      .getY() - A.getY()) * (intersection
      .getY() - B.getY()) <= 0.0D && (intersection
      .getY() - C.getY()) * (intersection
      .getY() - D.getY()) <= 0.0D) {
        //相交
        if ((A.getX() == C.getX() && A.getY() == C.getY()) || (A.getX() == D.getX() && A.getY() == D.getY()) || (B
          .getX() == C.getX() && B.getY() == C.getY()) || (B.getX() == D.getX() && B.getY() == D.getY())) {
          return 2;
        }
      return 1;
    } 
    return -1;
  }
  
  public static class Point {
    public double x;
    
    public double y;
    
    public Point() {}
    
    public Point(double x, double y) {
      this.x = x;
      this.y = y;
    }
    
    public double getX() {
      return this.x;
    }
    
    public void setX(double x) {
      this.x = x;
    }
    
    public double getY() {
      return this.y;
    }
    
    public void setY(double y) {
      this.y = y;
    }
    
    public boolean equals(Object o) {
      if (this == o)
        return true; 
      if (o == null || getClass() != o.getClass())
        return false; 
      Point point = (Point)o;
      return (Double.compare(point.x, this.x) == 0 && 
        Double.compare(point.y, this.y) == 0);
    }
    
    public int hashCode() {
      return Objects.hash(new Object[] { Double.valueOf(this.x), Double.valueOf(this.y) });
    }
    
    public String toString() {
      return "Point [x=" + this.x + ", y=" + this.y + "]";
    }
  }
}
