package com.wanji.indicators.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.shaded.jackson2.org.yaml.snakeyaml.Yaml;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class YmlUtils {
  private static final Logger log = LoggerFactory.getLogger(YmlUtils.class);
  
  public static String bootstrap_file = "C:\\Work\\IDEA\\holo-rsu\\event-compute\\branch-3\\flink-cross-event-compute\\config\\application-prd.yml";
  
  private static Map<String, String> result = new HashMap<>();
  
  public static Map<String, String> getYmlByFileName(String filePath, String... keys) {
    result.clear();
    if (StringUtils.isBlank(filePath) || "null".equals(filePath))
      filePath = bootstrap_file; 
    InputStream in = null;
    try {
      File file = new File(filePath);
      try {
        in = new BufferedInputStream(new FileInputStream(file));
        log.info("读取外部配置文件，文件路径: " + filePath);
      } catch (FileNotFoundException e) {
        in = YmlUtils.class.getResourceAsStream("/application-prd.yml");
        log.info("配置文件不存在,读取默认配置文件: " + e.getMessage());
      } 
      Yaml props = new Yaml();
      Object obj = props.loadAs(in, Map.class);
      Map<String, Object> param = (Map<String, Object>)obj;
      for (Map.Entry<String, Object> entry : param.entrySet()) {
        String key = entry.getKey();
        Object val = entry.getValue();
        if (keys.length != 0 && !keys[0].equals(key))
          continue; 
        if (val instanceof Map) {
          forEachYaml(key, (Map<String, Object>)val, 1, keys);
          continue;
        } 
        result.put(key, val.toString());
      } 
      return result;
    } catch (Exception e) {
      log.error(e.getMessage(), e);
    } finally {
      if (in != null)
        try {
          in.close();
        } catch (IOException e) {
          log.error(e.getMessage(), e);
        }  
    } 
    return null;
  }
  
  public static String getValue(String key) throws FileNotFoundException {
    Map<String, String> map = getYmlByFileName(null, new String[0]);
    if (map == null)
      return null; 
    return map.get(key);
  }
  
  public static Map<String, String> forEachYaml(String key_str, Map<String, Object> obj, int i, String... keys) {
    for (Map.Entry<String, Object> entry : obj.entrySet()) {
      String key = entry.getKey();
      Object val = entry.getValue();
      if (keys.length > i && !keys[i].equals(key))
        continue; 
      String str_new = "";
      if (StringUtils.isNotEmpty(key_str)) {
        str_new = key_str + "." + key;
      } else {
        str_new = key;
      } 
      if (val instanceof Map) {
        forEachYaml(str_new, (Map<String, Object>)val, ++i, keys);
        i--;
        continue;
      } 
      result.put(str_new, val.toString());
    } 
    return result;
  }
  
  public Map<String, String> getLocalYml() {
    Properties properties = new Properties();
    Map<String, String> map = new HashMap<>();
    try {
      InputStream in = YmlUtils.class.getResourceAsStream("/application-prd.yml");
      properties.load(in);
      for (Object k : properties.keySet())
        map.put(k.toString(), properties.getProperty(k.toString())); 
    } catch (IOException e) {
      e.printStackTrace();
    } 
    return map;
  }
  
  public static void main(String[] args) throws FileNotFoundException {
    Map<String, String> ymlByFileName = getYmlByFileName(bootstrap_file, new String[0]);
    Set<Map.Entry<String, String>> entries = ymlByFileName.entrySet();
    for (Map.Entry<String, String> entry : entries)
      System.out.println((String)entry.getKey() + ":" + (String)entry.getValue()); 
    Map<String, String> localYml = (new YmlUtils()).getLocalYml();
    Set<Map.Entry<String, String>> entriess = localYml.entrySet();
    for (Map.Entry<String, String> entry : entriess)
      System.out.println((String)entry.getKey() + ":" + (String)entry.getValue()); 
  }
}
