package com.wanji.indicators.udf;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.source.ConflictTurnSource;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.common.typeinfo.Types;
import org.apache.flink.table.annotation.DataTypeHint;
import org.apache.flink.table.annotation.FunctionHint;
import org.apache.flink.table.functions.TableFunction;
import org.apache.flink.types.Row;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Type;
import java.util.Iterator;
import java.util.List;

/**
 * @author fengyi
 * @date 2023/9/21
 * @description
 */
@FunctionHint(output = @DataTypeHint("ROW<id INT, picLicense STRING, originalType INT, speed DOUBLE,crossId STRING,rid STRING,laneId STRING>"))
public class ParserJsonArray extends TableFunction<Row> {

    private static final Logger log = LoggerFactory.getLogger(ParserJsonArray.class);

    public void eval(String value) {
        try {
            long s = System.currentTimeMillis();

            ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            List<CarTrackModel> list = objectMapper.readValue(value, new TypeReference<List<CarTrackModel>>() {
                @Override
                public Type getType() {
                    return super.getType();
                }
            });

            for (CarTrackModel carTrackModel : list) {
                Integer id = carTrackModel.getId();
                String picLicense = carTrackModel.getPicLicense();
                Integer originalType = carTrackModel.getOriginalType();
                Double speed = carTrackModel.getSpeed();
                CarTrackModel.RoadNet roadNet = carTrackModel.getRoadnet();;
                String crossId = roadNet.getCrossId();
                String rid = roadNet.getRid();
                String laneId = roadNet.getLaneId();
            }
            long e = System.currentTimeMillis();
            System.out.println("耗时："+(e-s)+"ms");

            JSONArray arrays = JSONArray.parseArray(value);
            Iterator<Object> iterator = arrays.iterator();
            while (iterator.hasNext()) {
                JSONObject jsonObject = (JSONObject) iterator.next();
                Integer id = jsonObject.getIntValue("id")   ;
                String picLicense = jsonObject.getString("picLicense");
                Integer originalType =  jsonObject.getIntValue("originalType")   ;
                Double speed = jsonObject.getDoubleValue("speed")  ;
                JSONObject roadnet =  jsonObject.getJSONObject("roadnet");
                String crossId =  roadnet.getString("crossId");
                String rid =  roadnet.getString("rid");
                String laneId =  roadnet.getString("laneId");
                //collect(Row.of(id,picLicense,originalType,speed,crossId,rid,laneId));
            }
            long e1 = System.currentTimeMillis();
            System.out.println("耗时："+(e1-e)+"ms");
        } catch (Exception e) {
            log.error("Parser json failed :" + e.getMessage());
        }
    }


    public static void main(String[] args) {
        String json = "[{\"id\":692652,\"licenseColor\":0,\"picLicense\":\"鲁A35ER0\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08762759999999,\"latitude\":36.642938199999996,\"speed\":56.24,\"altitude\":0.0,\"courseAngle\":55.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0024012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00240\"}},{\"id\":692630,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0961889,\"latitude\":36.644816999999996,\"speed\":1.8,\"altitude\":0.0,\"courseAngle\":314.0,\"baseLocation\":0,\"waitingTime\":23,\"stopNum\":9,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NHC0B5R4013NID0B5RM0090013\",\"rid\":\"13NHC0B5R4013NID0B5RM00\",\"segmentId\":\"13NHC0B5R4013NID0B5RM00900\"}},{\"id\":692631,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09669939999999,\"latitude\":36.644965299999996,\"speed\":2.13,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":25,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NH20B5RH013NID0B5RM0020011\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692632,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0918439,\"latitude\":36.6446373,\"speed\":44.93,\"altitude\":0.0,\"courseAngle\":254.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NGH0B5RC0024013\",\"rid\":\"13NI00B5RM013NGH0B5RC00\",\"segmentId\":\"13NI00B5RM013NGH0B5RC00240\"}},{\"id\":692639,\"licenseColor\":0,\"picLicense\":\"鲁A39S5M\",\"originalColor\":6,\"originalType\":1,\"longitude\":117.0874949,\"latitude\":36.642977099999996,\"speed\":45.94,\"altitude\":0.0,\"courseAngle\":235.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0026012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00260\"}},{\"id\":692645,\"licenseColor\":0,\"picLicense\":\"鲁A3CU96\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0878695,\"latitude\":36.643127799999995,\"speed\":41.04,\"altitude\":0.0,\"courseAngle\":240.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0024011\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00240\"}},{\"id\":692646,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09524239999999,\"latitude\":36.644929999999995,\"speed\":37.23,\"altitude\":0.0,\"courseAngle\":87.0,\"baseLocation\":0,\"waitingTime\":20,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NID0B5RM0010012\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00100\"}},{\"id\":692648,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0966249,\"latitude\":36.644986599999996,\"speed\":3.93,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":20,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NH20B5RH013NID0B5RM0020012\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692651,\"licenseColor\":0,\"picLicense\":\"鲁A6G0B2\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.08633619999999,\"latitude\":36.642184,\"speed\":35.57,\"altitude\":0.0,\"courseAngle\":50.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0010011\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00100\"}},{\"id\":692629,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0964281,\"latitude\":36.645001,\"speed\":12.42,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":21,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NH20B5RH013NID0B5RM0020012\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692654,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.08942789999999,\"latitude\":36.6439154,\"speed\":19.62,\"altitude\":0.0,\"courseAngle\":252.0,\"baseLocation\":0,\"waitingTime\":18,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NGH0B5RC013NF80B5QN0010013\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00100\"}},{\"id\":692655,\"licenseColor\":0,\"picLicense\":\"默A00000\",\"originalColor\":0,\"originalType\":6,\"longitude\":117.0868454,\"latitude\":36.6424421,\"speed\":45.0,\"altitude\":0.0,\"courseAngle\":51.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0020013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00200\"}},{\"id\":692657,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0961619,\"latitude\":36.6447788,\"speed\":2.24,\"altitude\":0.0,\"courseAngle\":314.0,\"baseLocation\":0,\"waitingTime\":11,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NHC0B5R4013NID0B5RM0090011\",\"rid\":\"13NHC0B5R4013NID0B5RM00\",\"segmentId\":\"13NHC0B5R4013NID0B5RM00900\"}},{\"id\":692658,\"licenseColor\":0,\"picLicense\":\"鲁A318WN\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.087513,\"latitude\":36.6428933,\"speed\":53.39,\"altitude\":0.0,\"courseAngle\":55.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0024012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00240\"}},{\"id\":692660,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0853286,\"latitude\":36.6416263,\"speed\":21.68,\"altitude\":0.0,\"courseAngle\":231.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NED0B5Q90010013\",\"rid\":\"13NF80B5QN013NED0B5Q900\",\"segmentId\":\"13NF80B5QN013NED0B5Q900100\"}},{\"id\":692664,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09598559999999,\"latitude\":36.645036999999995,\"speed\":27.33,\"altitude\":0.0,\"courseAngle\":271.0,\"baseLocation\":0,\"waitingTime\":10,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692665,\"licenseColor\":0,\"picLicense\":\"鲁A6N0S6\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.09002149999999,\"latitude\":36.6438693,\"speed\":6.38,\"altitude\":0.0,\"courseAngle\":338.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692604,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0955094,\"latitude\":36.644930699999996,\"speed\":47.45,\"altitude\":0.0,\"courseAngle\":89.0,\"baseLocation\":0,\"waitingTime\":31,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NID0B5RM0090012\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00900\"}},{\"id\":692561,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0962648,\"latitude\":36.6449813,\"speed\":10.77,\"altitude\":0.0,\"courseAngle\":271.0,\"baseLocation\":0,\"waitingTime\":35,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692571,\"licenseColor\":0,\"picLicense\":\"鲁R660WD\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.0863272,\"latitude\":36.6422532,\"speed\":37.8,\"altitude\":0.0,\"courseAngle\":230.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0090012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00900\"}},{\"id\":692575,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09561029999999,\"latitude\":36.6450442,\"speed\":33.92,\"altitude\":0.0,\"courseAngle\":269.0,\"baseLocation\":0,\"waitingTime\":32,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NID0B5RM013NI00B5RM0010013\",\"rid\":\"13NID0B5RM013NI00B5RM00\",\"segmentId\":\"13NID0B5RM013NI00B5RM00100\"}},{\"id\":692589,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09573449999999,\"latitude\":36.6450289,\"speed\":42.56,\"altitude\":0.0,\"courseAngle\":265.0,\"baseLocation\":0,\"waitingTime\":29,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692590,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09637629999999,\"latitude\":36.6449796,\"speed\":6.92,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":30,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NH20B5RH013NID0B5RM0090011\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00900\"}},{\"id\":692596,\"licenseColor\":0,\"picLicense\":\"鲁AZ072Q\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0864763,\"latitude\":36.6423183,\"speed\":45.72,\"altitude\":0.0,\"courseAngle\":230.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0090011\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00900\"}},{\"id\":692597,\"licenseColor\":0,\"picLicense\":\"鲁A68SZ7\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08620839999999,\"latitude\":36.642176,\"speed\":35.25,\"altitude\":0.0,\"courseAngle\":228.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NGH0B5RC013NF80B5QN0090012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00900\"}},{\"id\":692598,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0904463,\"latitude\":36.644238,\"speed\":21.57,\"altitude\":0.0,\"courseAngle\":243.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NI00B5RM013NGH0B5RC0090013\",\"rid\":\"13NI00B5RM013NGH0B5RC00\",\"segmentId\":\"13NI00B5RM013NGH0B5RC00900\"}},{\"id\":692666,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.08993249999999,\"latitude\":36.6437558,\"speed\":20.92,\"altitude\":0.0,\"courseAngle\":152.0,\"baseLocation\":0,\"waitingTime\":12,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NGH0B5RC013NEP0B5QJ0010013\",\"rid\":\"13NGH0B5RC013NEP0B5QJ00\",\"segmentId\":\"13NGH0B5RC013NEP0B5QJ00100\"}},{\"id\":692609,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.096519,\"latitude\":36.6449742,\"speed\":10.44,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":28,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NH20B5RH013NID0B5RM0020011\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692612,\"licenseColor\":0,\"picLicense\":\"鲁A7L65S\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0876517,\"latitude\":36.6430237,\"speed\":41.51,\"altitude\":0.0,\"courseAngle\":236.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0026011\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00260\"}},{\"id\":692613,\"licenseColor\":0,\"picLicense\":\"鲁A052DT\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.090043,\"latitude\":36.6439136,\"speed\":16.38,\"altitude\":0.0,\"courseAngle\":344.0,\"baseLocation\":0,\"waitingTime\":25,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692620,\"licenseColor\":0,\"picLicense\":\"鲁A6Y508\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0896879,\"latitude\":36.6438367,\"speed\":5.22,\"altitude\":0.0,\"courseAngle\":68.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0090014\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00900\"}},{\"id\":692622,\"licenseColor\":0,\"picLicense\":\"鲁A9337Z\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0893905,\"latitude\":36.643755999999996,\"speed\":23.58,\"altitude\":0.0,\"courseAngle\":63.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0030013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00300\"}},{\"id\":692625,\"licenseColor\":4,\"picLicense\":\"鲁ADF6931\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0876761,\"latitude\":36.6430856,\"speed\":49.54,\"altitude\":0.0,\"courseAngle\":236.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0026012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00260\"}},{\"id\":692627,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09650149999999,\"latitude\":36.6449144,\"speed\":38.78,\"altitude\":0.0,\"courseAngle\":94.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NID0B5RM013NH20B5RH0020012\",\"rid\":\"13NID0B5RM013NH20B5RH00\",\"segmentId\":\"13NID0B5RM013NH20B5RH00200\"}},{\"id\":692700,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09621589999999,\"latitude\":36.6447019,\"speed\":19.77,\"altitude\":0.0,\"courseAngle\":133.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NID0B5RM013NHC0B5R40020011\",\"rid\":\"13NID0B5RM013NHC0B5R400\",\"segmentId\":\"13NID0B5RM013NHC0B5R400200\"}},{\"id\":692691,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0886905,\"latitude\":36.643434,\"speed\":49.11,\"altitude\":0.0,\"courseAngle\":60.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0026013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00260\"}},{\"id\":692692,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0965153,\"latitude\":36.6450208,\"speed\":15.23,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NH20B5RH013NID0B5RM0020013\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692694,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08527459999999,\"latitude\":36.6414798,\"speed\":44.28,\"altitude\":0.0,\"courseAngle\":52.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NED0B5Q9013NF80B5QN0024011\",\"rid\":\"13NED0B5Q9013NF80B5QN00\",\"segmentId\":\"13NED0B5Q9013NF80B5QN00240\"}},{\"id\":692695,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09014619999999,\"latitude\":36.6436576,\"speed\":9.94,\"altitude\":0.0,\"courseAngle\":338.0,\"baseLocation\":0,\"waitingTime\":3,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NEP0B5QJ013NGH0B5RC0090012\",\"rid\":\"13NEP0B5QJ013NGH0B5RC00\",\"segmentId\":\"13NEP0B5QJ013NGH0B5RC00900\"}},{\"id\":692696,\"licenseColor\":0,\"picLicense\":\"鲁A5070E\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0899073,\"latitude\":36.6441355,\"speed\":15.52,\"altitude\":0.0,\"courseAngle\":153.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692697,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0898062,\"latitude\":36.6441331,\"speed\":14.22,\"altitude\":0.0,\"courseAngle\":168.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692698,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0866956,\"latitude\":36.6424611,\"speed\":42.95,\"altitude\":0.0,\"courseAngle\":231.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0030011\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00300\"}},{\"id\":692699,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0884938,\"latitude\":36.6434931,\"speed\":53.1,\"altitude\":0.0,\"courseAngle\":240.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0024013\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00240\"}},{\"id\":692689,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0963209,\"latitude\":36.645032,\"speed\":21.6,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NH20B5RH013NID0B5RM0090013\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00900\"}},{\"id\":692701,\"licenseColor\":4,\"picLicense\":\"鲁AD17763\",\"originalColor\":3,\"originalType\":1,\"longitude\":117.08989589999999,\"latitude\":36.6442044,\"speed\":14.19,\"altitude\":0.0,\"courseAngle\":161.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692703,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08695309999999,\"latitude\":36.642652,\"speed\":49.36,\"altitude\":0.0,\"courseAngle\":232.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0030012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00300\"}},{\"id\":692704,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0864172,\"latitude\":36.6422008,\"speed\":52.02,\"altitude\":0.0,\"courseAngle\":50.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0010012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00100\"}},{\"id\":692705,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09662549999999,\"latitude\":36.6450214,\"speed\":16.02,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NH20B5RH013NID0B5RM0020013\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692707,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.0859555,\"latitude\":36.642071099999995,\"speed\":27.94,\"altitude\":0.0,\"courseAngle\":228.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692708,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0861223,\"latitude\":36.6417394,\"speed\":4.32,\"altitude\":0.0,\"courseAngle\":321.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NDT0B5Q9013NF80B5QN0090012\",\"rid\":\"13NDT0B5Q9013NF80B5QN00\",\"segmentId\":\"13NDT0B5Q9013NF80B5QN00900\"}},{\"id\":692709,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0967218,\"latitude\":36.6449871,\"speed\":8.21,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NH20B5RH013NID0B5RM0020012\",\"rid\":\"13NH20B5RH013NID0B5RM00\",\"segmentId\":\"13NH20B5RH013NID0B5RM00200\"}},{\"id\":692582,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09027309999999,\"latitude\":36.644180299999995,\"speed\":7.96,\"altitude\":0.0,\"courseAngle\":245.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NI00B5RM013NGH0B5RC0090013\",\"rid\":\"13NI00B5RM013NGH0B5RC00\",\"segmentId\":\"13NI00B5RM013NGH0B5RC00900\"}},{\"id\":692688,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.0900394,\"latitude\":36.6442748,\"speed\":16.2,\"altitude\":0.0,\"courseAngle\":338.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NGH0B5RC013NEH0B5RJ0010014\",\"rid\":\"13NGH0B5RC013NEH0B5RJ00\",\"segmentId\":\"13NGH0B5RC013NEH0B5RJ00100\"}},{\"id\":692687,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":4,\"longitude\":117.090184,\"latitude\":36.6440589,\"speed\":6.92,\"altitude\":0.0,\"courseAngle\":167.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692686,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.0856589,\"latitude\":36.6418283,\"speed\":51.34,\"altitude\":0.0,\"courseAngle\":228.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NF80B5QN013NED0B5Q90010012\",\"rid\":\"13NF80B5QN013NED0B5Q900\",\"segmentId\":\"13NF80B5QN013NED0B5Q900100\"}},{\"id\":692685,\"licenseColor\":4,\"picLicense\":\"鲁ADL5097\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.085753,\"latitude\":36.6417578,\"speed\":24.52,\"altitude\":0.0,\"courseAngle\":49.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NED0B5Q9013NF80B5QN0090012\",\"rid\":\"13NED0B5Q9013NF80B5QN00\",\"segmentId\":\"13NED0B5Q9013NF80B5QN00900\"}},{\"id\":692684,\"licenseColor\":0,\"picLicense\":\"鲁A6HK00\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.0857333,\"latitude\":36.6417766,\"speed\":20.63,\"altitude\":0.0,\"courseAngle\":50.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NED0B5Q9013NF80B5QN0090011\",\"rid\":\"13NED0B5Q9013NF80B5QN00\",\"segmentId\":\"13NED0B5Q9013NF80B5QN00900\"}},{\"id\":692682,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08632449999999,\"latitude\":36.6422866,\"speed\":35.61,\"altitude\":0.0,\"courseAngle\":230.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0090013\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00900\"}},{\"id\":692681,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08984849999999,\"latitude\":36.6441229,\"speed\":16.38,\"altitude\":0.0,\"courseAngle\":164.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692679,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08760009999999,\"latitude\":36.6428824,\"speed\":28.62,\"altitude\":0.0,\"courseAngle\":55.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0024013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00240\"}},{\"id\":692678,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08947549999999,\"latitude\":36.6437586,\"speed\":24.48,\"altitude\":0.0,\"courseAngle\":65.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0090014\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00900\"}},{\"id\":692677,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0961459,\"latitude\":36.6450329,\"speed\":27.04,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692676,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":4,\"longitude\":117.0897529,\"latitude\":36.6439622,\"speed\":3.35,\"altitude\":0.0,\"courseAngle\":143.0,\"baseLocation\":0,\"waitingTime\":11,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692674,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.09005049999999,\"latitude\":36.6442194,\"speed\":10.66,\"altitude\":0.0,\"courseAngle\":343.0,\"baseLocation\":0,\"waitingTime\":12,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692673,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08978019999999,\"latitude\":36.644063599999996,\"speed\":8.68,\"altitude\":0.0,\"courseAngle\":192.0,\"baseLocation\":0,\"waitingTime\":10,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692672,\"licenseColor\":0,\"picLicense\":\"鲁A926SF\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08624719999999,\"latitude\":36.6420823,\"speed\":43.82,\"altitude\":0.0,\"courseAngle\":51.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0010012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00100\"}},{\"id\":692670,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08970649999999,\"latitude\":36.643996099999995,\"speed\":8.5,\"altitude\":0.0,\"courseAngle\":223.0,\"baseLocation\":0,\"waitingTime\":10,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692361,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08604349999999,\"latitude\":36.641774,\"speed\":1.3,\"altitude\":0.0,\"courseAngle\":318.0,\"baseLocation\":0,\"waitingTime\":91,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NDT0B5Q9013NF80B5QN0090011\",\"rid\":\"13NDT0B5Q9013NF80B5QN00\",\"segmentId\":\"13NDT0B5Q9013NF80B5QN00900\"}},{\"id\":692491,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09548389999999,\"latitude\":36.6449657,\"speed\":41.01,\"altitude\":0.0,\"courseAngle\":89.0,\"baseLocation\":0,\"waitingTime\":44,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NID0B5RM0090011\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00900\"}},{\"id\":692487,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0958627,\"latitude\":36.6450111,\"speed\":22.07,\"altitude\":0.0,\"courseAngle\":271.0,\"baseLocation\":0,\"waitingTime\":56,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692485,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0899091,\"latitude\":36.6439741,\"speed\":20.42,\"altitude\":0.0,\"courseAngle\":162.0,\"baseLocation\":0,\"waitingTime\":58,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692471,\"licenseColor\":0,\"picLicense\":\"鲁A91N59\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0899151,\"latitude\":36.6440504,\"speed\":9.94,\"altitude\":0.0,\"courseAngle\":165.0,\"baseLocation\":0,\"waitingTime\":65,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692460,\"licenseColor\":0,\"picLicense\":\"鲁AZ072Q\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08724129999999,\"latitude\":36.6428262,\"speed\":48.53,\"altitude\":0.0,\"courseAngle\":234.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0028012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00280\"}},{\"id\":692417,\"licenseColor\":0,\"picLicense\":\"鲁A297DP\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0900719,\"latitude\":36.643642199999995,\"speed\":38.31,\"altitude\":0.0,\"courseAngle\":161.0,\"baseLocation\":0,\"waitingTime\":78,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NGH0B5RC013NEP0B5QJ0010011\",\"rid\":\"13NGH0B5RC013NEP0B5QJ00\",\"segmentId\":\"13NGH0B5RC013NEP0B5QJ00100\"}},{\"id\":692410,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.086047,\"latitude\":36.6418731,\"speed\":0.87,\"altitude\":0.0,\"courseAngle\":58.0,\"baseLocation\":0,\"waitingTime\":61,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692402,\"licenseColor\":0,\"picLicense\":\"鲁A05G8R\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08569109999999,\"latitude\":36.6417947,\"speed\":53.28,\"altitude\":0.0,\"courseAngle\":230.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NF80B5QN013NED0B5Q90010011\",\"rid\":\"13NF80B5QN013NED0B5Q900\",\"segmentId\":\"13NF80B5QN013NED0B5Q900100\"}},{\"id\":692395,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0860929,\"latitude\":36.641721499999996,\"speed\":1.26,\"altitude\":0.0,\"courseAngle\":318.0,\"baseLocation\":0,\"waitingTime\":84,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NDT0B5Q9013NF80B5QN0090011\",\"rid\":\"13NDT0B5Q9013NF80B5QN00\",\"segmentId\":\"13NDT0B5Q9013NF80B5QN00900\"}},{\"id\":692379,\"licenseColor\":0,\"picLicense\":\"默A00000\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0860587,\"latitude\":36.6418034,\"speed\":0.04,\"altitude\":0.0,\"courseAngle\":321.0,\"baseLocation\":0,\"waitingTime\":86,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NDT0B5Q9013NF80B5QN0090012\",\"rid\":\"13NDT0B5Q9013NF80B5QN00\",\"segmentId\":\"13NDT0B5Q9013NF80B5QN00900\"}},{\"id\":692376,\"licenseColor\":4,\"picLicense\":\"鲁AFF9132\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08997819999999,\"latitude\":36.6440947,\"speed\":9.08,\"altitude\":0.0,\"courseAngle\":342.0,\"baseLocation\":0,\"waitingTime\":82,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692555,\"licenseColor\":0,\"picLicense\":\"鲁A99G77\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08957459999999,\"latitude\":36.6438205,\"speed\":6.63,\"altitude\":0.0,\"courseAngle\":64.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0090013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00900\"}},{\"id\":692351,\"licenseColor\":0,\"picLicense\":\"鲁A8DK00\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0957116,\"latitude\":36.644903299999996,\"speed\":49.11,\"altitude\":0.0,\"courseAngle\":91.0,\"baseLocation\":0,\"waitingTime\":40,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NI00B5RM013NID0B5RM0090013\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00900\"}},{\"id\":692338,\"licenseColor\":4,\"picLicense\":\"豫AF73056\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0957954,\"latitude\":36.6449554,\"speed\":43.71,\"altitude\":0.0,\"courseAngle\":92.0,\"baseLocation\":0,\"waitingTime\":52,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692323,\"licenseColor\":4,\"picLicense\":\"鲁AD28000\",\"originalColor\":2,\"originalType\":1,\"longitude\":117.0964968,\"latitude\":36.6448855,\"speed\":56.99,\"altitude\":0.0,\"courseAngle\":94.0,\"baseLocation\":0,\"waitingTime\":55,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"13NID0B5RM013NH20B5RH0020013\",\"rid\":\"13NID0B5RM013NH20B5RH00\",\"segmentId\":\"13NID0B5RM013NH20B5RH00200\"}},{\"id\":692302,\"licenseColor\":4,\"picLicense\":\"鲁AD63366\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0961882,\"latitude\":36.6448927,\"speed\":52.17,\"altitude\":0.0,\"courseAngle\":91.0,\"baseLocation\":0,\"waitingTime\":46,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692285,\"licenseColor\":0,\"picLicense\":\"鲁A75DW2\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.0900207,\"latitude\":36.6441325,\"speed\":13.18,\"altitude\":0.0,\"courseAngle\":125.0,\"baseLocation\":0,\"waitingTime\":116,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692269,\"licenseColor\":0,\"picLicense\":\"鲁AW63T5\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.09010769999999,\"latitude\":36.6440738,\"speed\":13.04,\"altitude\":0.0,\"courseAngle\":104.0,\"baseLocation\":0,\"waitingTime\":122,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692265,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0857655,\"latitude\":36.642134399999996,\"speed\":0.26,\"altitude\":0.0,\"courseAngle\":155.0,\"baseLocation\":0,\"waitingTime\":126,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NDG0B5RI013NF80B5QN0033011\",\"rid\":\"13NDG0B5RI013NF80B5QN00\",\"segmentId\":\"13NDG0B5RI013NF80B5QN00330\"}},{\"id\":692256,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08580359999999,\"latitude\":36.642073499999995,\"speed\":1.08,\"altitude\":0.0,\"courseAngle\":145.0,\"baseLocation\":0,\"waitingTime\":127,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NDG0B5RI013NF80B5QN0090011\",\"rid\":\"13NDG0B5RI013NF80B5QN00\",\"segmentId\":\"13NDG0B5RI013NF80B5QN00900\"}},{\"id\":692115,\"licenseColor\":0,\"picLicense\":\"鲁AM7Q60\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.09608619999999,\"latitude\":36.6449211,\"speed\":45.22,\"altitude\":0.0,\"courseAngle\":92.0,\"baseLocation\":0,\"waitingTime\":155,\"stopNum\":2,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692068,\"licenseColor\":0,\"picLicense\":\"鲁M6J789\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08800769999999,\"latitude\":36.6432068,\"speed\":33.81,\"altitude\":0.0,\"courseAngle\":237.0,\"baseLocation\":0,\"waitingTime\":170,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0024011\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00240\"}},{\"id\":691986,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":4,\"longitude\":117.0899694,\"latitude\":36.6443247,\"speed\":0.0,\"altitude\":0.0,\"courseAngle\":339.0,\"baseLocation\":0,\"waitingTime\":192,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692543,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08645039999999,\"latitude\":36.6423394,\"speed\":44.25,\"altitude\":0.0,\"courseAngle\":230.0,\"baseLocation\":0,\"waitingTime\":37,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0090012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00900\"}},{\"id\":692529,\"licenseColor\":4,\"picLicense\":\"鲁AD90217\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0900071,\"latitude\":36.6440123,\"speed\":13.11,\"altitude\":0.0,\"courseAngle\":342.0,\"baseLocation\":0,\"waitingTime\":43,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692533,\"licenseColor\":0,\"picLicense\":\"鲁A5M506\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0867394,\"latitude\":36.6425231,\"speed\":51.45,\"altitude\":0.0,\"courseAngle\":231.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0030012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00300\"}},{\"id\":692537,\"licenseColor\":0,\"picLicense\":\"鲁A278Z1\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08533899999999,\"latitude\":36.641602,\"speed\":45.87,\"altitude\":0.0,\"courseAngle\":231.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NED0B5Q90010012\",\"rid\":\"13NF80B5QN013NED0B5Q900\",\"segmentId\":\"13NF80B5QN013NED0B5Q900100\"}},{\"id\":692538,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":6,\"longitude\":117.0899068,\"latitude\":36.6438075,\"speed\":16.78,\"altitude\":0.0,\"courseAngle\":141.0,\"baseLocation\":0,\"waitingTime\":43,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692539,\"licenseColor\":0,\"picLicense\":\"鲁AU7H50\",\"originalColor\":2,\"originalType\":1,\"longitude\":117.08968139999999,\"latitude\":36.6438614,\"speed\":1.62,\"altitude\":0.0,\"courseAngle\":65.0,\"baseLocation\":0,\"waitingTime\":7,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0090013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00900\"}},{\"id\":692523,\"licenseColor\":0,\"picLicense\":\"鲁A7GB72\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.08823699999999,\"latitude\":36.6433429,\"speed\":46.44,\"altitude\":0.0,\"courseAngle\":238.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NF80B5QN0024012\",\"rid\":\"13NGH0B5RC013NF80B5QN00\",\"segmentId\":\"13NGH0B5RC013NF80B5QN00240\"}},{\"id\":692540,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0953457,\"latitude\":36.6449508,\"speed\":39.89,\"altitude\":0.0,\"courseAngle\":88.0,\"baseLocation\":0,\"waitingTime\":31,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NID0B5RM0020012\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00200\"}},{\"id\":692518,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":3,\"longitude\":117.0899862,\"latitude\":36.6437656,\"speed\":0.62,\"altitude\":0.0,\"courseAngle\":161.0,\"baseLocation\":0,\"waitingTime\":52,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NGH0B5RC013NEP0B5QJ0010012\",\"rid\":\"13NGH0B5RC013NEP0B5QJ00\",\"segmentId\":\"13NGH0B5RC013NEP0B5QJ00100\"}},{\"id\":692544,\"licenseColor\":0,\"picLicense\":\"鲁AL9221\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0899889,\"latitude\":36.6439192,\"speed\":5.08,\"altitude\":0.0,\"courseAngle\":332.0,\"baseLocation\":0,\"waitingTime\":37,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692547,\"licenseColor\":0,\"picLicense\":\"鲁A0PT36\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0895608,\"latitude\":36.643845299999995,\"speed\":14.94,\"altitude\":0.0,\"courseAngle\":64.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0090012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00900\"}},{\"id\":692508,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09522439999999,\"latitude\":36.6449017,\"speed\":38.42,\"altitude\":0.0,\"courseAngle\":87.0,\"baseLocation\":0,\"waitingTime\":35,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NID0B5RM0010013\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00100\"}},{\"id\":692498,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09597009999999,\"latitude\":36.6449874,\"speed\":20.45,\"altitude\":0.0,\"courseAngle\":270.0,\"baseLocation\":0,\"waitingTime\":53,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692551,\"licenseColor\":0,\"picLicense\":\"鲁A86QF1\",\"originalColor\":3,\"originalType\":1,\"longitude\":117.08966939999999,\"latitude\":36.6438862,\"speed\":1.7,\"altitude\":0.0,\"courseAngle\":65.0,\"baseLocation\":0,\"waitingTime\":6,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NGH0B5RC0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0090012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00900\"}},{\"id\":692549,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09625419999999,\"latitude\":36.645008,\"speed\":16.56,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":38,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692504,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09611459999999,\"latitude\":36.6450082,\"speed\":20.31,\"altitude\":0.0,\"courseAngle\":272.0,\"baseLocation\":0,\"waitingTime\":52,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NHC0B5R40\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692300,\"licenseColor\":0,\"picLicense\":\"鲁A5V222\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0949911,\"latitude\":36.644921599999996,\"speed\":31.58,\"altitude\":0.0,\"courseAngle\":87.0,\"baseLocation\":0,\"waitingTime\":46,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NI00B5RM013NID0B5RM0010012\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00100\"}},{\"id\":692240,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09394449999999,\"latitude\":36.6448956,\"speed\":5.69,\"altitude\":0.0,\"courseAngle\":83.0,\"baseLocation\":0,\"waitingTime\":114,\"stopNum\":2,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692693,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09442449999999,\"latitude\":36.6450126,\"speed\":23.12,\"altitude\":0.0,\"courseAngle\":239.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692222,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":10,\"longitude\":117.0945497,\"latitude\":36.6449514,\"speed\":22.97,\"altitude\":0.0,\"courseAngle\":78.0,\"baseLocation\":0,\"waitingTime\":130,\"stopNum\":3,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692309,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09424,\"latitude\":36.6449237,\"speed\":25.67,\"altitude\":0.0,\"courseAngle\":83.0,\"baseLocation\":0,\"waitingTime\":111,\"stopNum\":3,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692207,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0946441,\"latitude\":36.6449905,\"speed\":18.65,\"altitude\":0.0,\"courseAngle\":53.0,\"baseLocation\":0,\"waitingTime\":132,\"stopNum\":3,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692289,\"licenseColor\":0,\"picLicense\":\"鲁AU306L\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.09357519999999,\"latitude\":36.6448544,\"speed\":0.0,\"altitude\":0.0,\"courseAngle\":79.0,\"baseLocation\":0,\"waitingTime\":50,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0025011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00250\"}},{\"id\":692495,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0948123,\"latitude\":36.6449122,\"speed\":29.27,\"altitude\":0.0,\"courseAngle\":86.0,\"baseLocation\":0,\"waitingTime\":45,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692343,\"licenseColor\":0,\"picLicense\":\"鲁A6H380\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0937314,\"latitude\":36.6448731,\"speed\":0.15,\"altitude\":0.0,\"courseAngle\":81.0,\"baseLocation\":0,\"waitingTime\":72,\"stopNum\":2,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0026011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00260\"}},{\"id\":692073,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09469689999999,\"latitude\":36.6451898,\"speed\":23.91,\"altitude\":0.0,\"courseAngle\":0.0,\"baseLocation\":0,\"waitingTime\":172,\"stopNum\":4,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NI00B5RM013NG40B5SK0010011\",\"rid\":\"13NI00B5RM013NG40B5SK00\",\"segmentId\":\"13NI00B5RM013NG40B5SK00100\"}},{\"id\":692284,\"licenseColor\":4,\"picLicense\":\"鲁AFJ1191\",\"originalColor\":2,\"originalType\":1,\"longitude\":117.0936467,\"latitude\":36.644862599999996,\"speed\":0.11,\"altitude\":0.0,\"courseAngle\":81.0,\"baseLocation\":0,\"waitingTime\":56,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0026011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00260\"}},{\"id\":691827,\"licenseColor\":0,\"picLicense\":\"鲁VM7J02\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.09437059999999,\"latitude\":36.6449334,\"speed\":21.03,\"altitude\":0.0,\"courseAngle\":84.0,\"baseLocation\":0,\"waitingTime\":203,\"stopNum\":5,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692388,\"licenseColor\":0,\"picLicense\":\"鲁A7F92M\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.094252,\"latitude\":36.6448718,\"speed\":11.96,\"altitude\":0.0,\"courseAngle\":85.0,\"baseLocation\":0,\"waitingTime\":13,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027013\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692636,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09250229999999,\"latitude\":36.6447074,\"speed\":46.66,\"altitude\":0.0,\"courseAngle\":257.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NI00B5RM013NGH0B5RC0023011\",\"rid\":\"13NI00B5RM013NGH0B5RC00\",\"segmentId\":\"13NI00B5RM013NGH0B5RC00230\"}},{\"id\":692653,\"licenseColor\":0,\"picLicense\":\"鲁Q06069\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0938221,\"latitude\":36.6448348,\"speed\":31.86,\"altitude\":0.0,\"courseAngle\":82.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0026013\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00260\"}},{\"id\":692480,\"licenseColor\":0,\"picLicense\":\"鲁AHH260\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.09313809999999,\"latitude\":36.6447867,\"speed\":30.17,\"altitude\":0.0,\"courseAngle\":78.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0025011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00250\"}},{\"id\":692469,\"licenseColor\":0,\"picLicense\":\"鲁AB699P\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0942693,\"latitude\":36.644899699999996,\"speed\":37.62,\"altitude\":0.0,\"courseAngle\":83.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027012\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692515,\"licenseColor\":0,\"picLicense\":\"鲁A5A70H\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.09388659999999,\"latitude\":36.6448167,\"speed\":50.44,\"altitude\":0.0,\"courseAngle\":82.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0026014\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00260\"}},{\"id\":692457,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.09384,\"latitude\":36.644884399999995,\"speed\":0.29,\"altitude\":0.0,\"courseAngle\":82.0,\"baseLocation\":0,\"waitingTime\":69,\"stopNum\":2,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0026011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00260\"}},{\"id\":692431,\"licenseColor\":0,\"picLicense\":\"鲁A3D9T6\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.09469399999999,\"latitude\":36.6448812,\"speed\":38.09,\"altitude\":0.0,\"courseAngle\":84.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692423,\"licenseColor\":0,\"picLicense\":\"鲁AKW863\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0944116,\"latitude\":36.644886,\"speed\":18.54,\"altitude\":0.0,\"courseAngle\":85.0,\"baseLocation\":0,\"waitingTime\":15,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027013\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692516,\"licenseColor\":0,\"picLicense\":\"鲁A0M719\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0926288,\"latitude\":36.6446623,\"speed\":45.18,\"altitude\":0.0,\"courseAngle\":77.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0024012\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00240\"}},{\"id\":692407,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0949617,\"latitude\":36.6449832,\"speed\":0.11,\"altitude\":0.0,\"courseAngle\":266.0,\"baseLocation\":0,\"waitingTime\":77,\"stopNum\":2,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NID0B5RM013NI00B5RM0090011\",\"rid\":\"13NID0B5RM013NI00B5RM00\",\"segmentId\":\"13NID0B5RM013NI00B5RM00900\"}},{\"id\":692406,\"licenseColor\":0,\"picLicense\":\"鲁AFT608\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.09405849999999,\"latitude\":36.6448575,\"speed\":11.63,\"altitude\":0.0,\"courseAngle\":83.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0027013\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00270\"}},{\"id\":692404,\"licenseColor\":0,\"picLicense\":\"鲁A31M0B\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0945579,\"latitude\":36.6448964,\"speed\":21.24,\"altitude\":0.0,\"courseAngle\":84.0,\"baseLocation\":0,\"waitingTime\":17,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692390,\"licenseColor\":0,\"picLicense\":\"鲁AV05V5\",\"originalColor\":2,\"originalType\":1,\"longitude\":117.09494559999999,\"latitude\":36.6449478,\"speed\":31.86,\"altitude\":0.0,\"courseAngle\":87.0,\"baseLocation\":0,\"waitingTime\":15,\"stopNum\":1,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NI00B5RM013NID0B5RM0010011\",\"rid\":\"13NI00B5RM013NID0B5RM00\",\"segmentId\":\"13NI00B5RM013NID0B5RM00100\"}},{\"id\":692528,\"licenseColor\":0,\"picLicense\":\"鲁A17N7G\",\"originalColor\":9,\"originalType\":1,\"longitude\":117.0946719,\"latitude\":36.6449299,\"speed\":28.88,\"altitude\":0.0,\"courseAngle\":83.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692478,\"licenseColor\":0,\"picLicense\":\"默A00000\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.0948739,\"latitude\":36.6448864,\"speed\":33.59,\"altitude\":0.0,\"courseAngle\":88.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"null\",\"rid\":\"null\",\"segmentId\":\"null\"}},{\"id\":692702,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.0947037,\"latitude\":36.6453683,\"speed\":37.37,\"altitude\":0.0,\"courseAngle\":4.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NG40B5SK0\",\"laneId\":\"13NI00B5RM013NG40B5SK0010011\",\"rid\":\"13NI00B5RM013NG40B5SK00\",\"segmentId\":\"13NI00B5RM013NG40B5SK00100\"}},{\"id\":692637,\"licenseColor\":0,\"picLicense\":\"默A00000\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.08569779999999,\"latitude\":36.6422727,\"speed\":21.75,\"altitude\":0.0,\"courseAngle\":161.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NDG0B5RI013NF80B5QN0031011\",\"rid\":\"13NDG0B5RI013NF80B5QN00\",\"segmentId\":\"13NDG0B5RI013NF80B5QN00310\"}},{\"id\":692680,\"licenseColor\":9,\"picLicense\":null,\"originalColor\":0,\"originalType\":1,\"longitude\":117.08775929999999,\"latitude\":36.642967,\"speed\":29.81,\"altitude\":0.0,\"courseAngle\":55.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NGH0B5RC0024013\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00240\"}},{\"id\":692490,\"licenseColor\":0,\"picLicense\":\"鲁A17AL0\",\"originalColor\":2,\"originalType\":1,\"longitude\":117.0862291,\"latitude\":36.6415836,\"speed\":22.65,\"altitude\":0.0,\"courseAngle\":135.0,\"baseLocation\":0,\"waitingTime\":16,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NF80B5QN013NDT0B5Q90021011\",\"rid\":\"13NF80B5QN013NDT0B5Q900\",\"segmentId\":\"13NF80B5QN013NDT0B5Q900210\"}},{\"id\":692559,\"licenseColor\":0,\"picLicense\":\"鲁AK7338\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.09333509999999,\"latitude\":36.6448227,\"speed\":24.74,\"altitude\":0.0,\"courseAngle\":78.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NI00B5RM0025011\",\"rid\":\"13NGH0B5RC013NI00B5RM00\",\"segmentId\":\"13NGH0B5RC013NI00B5RM00250\"}},{\"id\":692525,\"licenseColor\":0,\"picLicense\":\"鲁AP5926\",\"originalColor\":1,\"originalType\":1,\"longitude\":117.0898222,\"latitude\":36.6445928,\"speed\":19.84,\"altitude\":0.0,\"courseAngle\":347.0,\"baseLocation\":0,\"waitingTime\":30,\"stopNum\":1,\"roadnet\":{\"crossId\":\"null\",\"laneId\":\"13NGH0B5RC013NEH0B5RJ0020011\",\"rid\":\"13NGH0B5RC013NEH0B5RJ00\",\"segmentId\":\"13NGH0B5RC013NEH0B5RJ00200\"}},{\"id\":692668,\"licenseColor\":0,\"picLicense\":\"鲁AX337M\",\"originalColor\":0,\"originalType\":1,\"longitude\":117.086266,\"latitude\":36.6421235,\"speed\":31.79,\"altitude\":0.0,\"courseAngle\":46.0,\"baseLocation\":0,\"waitingTime\":0,\"stopNum\":0,\"roadnet\":{\"crossId\":\"13NF80B5QN0\",\"laneId\":\"13NF80B5QN013NGH0B5RC0010012\",\"rid\":\"13NF80B5QN013NGH0B5RC00\",\"segmentId\":\"13NF80B5QN013NGH0B5RC00100\"}}]"  ;

        ParserJsonArray parserJsonArray = new ParserJsonArray();
        parserJsonArray.eval(json);
    }
}

